package demo.barcharts.category ;

import java.awt.Color ;
import java.awt.GradientPaint ;
import java.awt.GridLayout ;
import java.text.NumberFormat ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.value.StatisticalNumberValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.GradientPaintTransformType ;
import com.ijchart.xychart.constant.VerticalAlignment ;
import com.ijchart.xychart.data.StatisticalChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.label.DefaultLabelGenerator ;
import com.ijchart.xychart.plot.CategoryPlot ;
import com.ijchart.xychart.render.category.StatisticalBarRender ;
import com.ijchart.xychart.tickunit.LongTickUnit ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class StatisticalBarChartDemo1 extends JPanel {
  private ChartOrientation orientation = ChartOrientation.VERTICAL ;

  private NumberFormat format = NumberFormat.getNumberInstance() ;

  public StatisticalBarChartDemo1() {
    format.setMaximumFractionDigits(0) ;
    setLayout(new GridLayout(1, 1)) ;
    addPanel() ;
  }

  private void addPanel() {
    final IMatrixDataset dataset = createDataset() ;
    final IJChart chart = ChartFactory.createStaticalBarChart(
      "Statistical Bar Chart Demo 1", "Type", "Value", dataset, orientation, true, true) ;

    CategoryPlot plot = (CategoryPlot) chart.getPlot() ;
    plot.setDomainGridlineVisible(false) ;
    StatisticalNumberValueAxis numberaxis = (StatisticalNumberValueAxis) plot.getValueAxis() ;
    numberaxis.setTickUnit(new LongTickUnit()) ;
    numberaxis.setAxisRangeIncludeZero(false) ;

    StatisticalBarRender render = (StatisticalBarRender) plot.getRender() ;
    render.setBaseItemLabelVisible(true) ;
    render.setBaseItemLabelPaint(Color.yellow) ;
    render.setBaseItemLabelVerticalAlignment(VerticalAlignment.BOTTOM) ;
    GradientPaint gradientpaint = new GradientPaint(0.0F, 0.0F, Color.blue, 0.0F, 0.0F,
                                                    new Color(0, 0, 64)) ;
    GradientPaint gradientpaint1 = new GradientPaint(0.0F, 0.0F, Color.green, 0.0F, 0.0F,
                                                     new Color(0, 64, 0)) ;
    render.setColumnItemFillPaint(0, gradientpaint) ;
    render.setColumnItemFillPaint(1, gradientpaint1) ;
    render.setAutoSelectFillPaint(false) ;
    render.setBaseItemFillPaintTransformer(GradientPaintTransformType.VERTICAL) ;

    render.setLabelGenerator(new DefaultLabelGenerator() {
      public String generateLabel(IMatrixDataset dataset, int row, int column) {
        StatisticalChartData data = (StatisticalChartData) dataset.getData(row, column) ;
        return format.format(data.getMeanValue()) ;
      }
    }) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private static IMatrixDataset createDataset() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("Column 1") ;
    dataset.addRowKey("Column 2") ;
    dataset.addRowKey("Column 3") ;
    dataset.addRowKey("Column 4") ;

    dataset.addColKey("Row 1") ;
    dataset.addColKey("Row 2") ;

    dataset.addRowData(4) ;
    dataset.addData(0, new StatisticalChartData(10D, 2.3999999999999999D)) ;
    dataset.addData(0, new StatisticalChartData(22D, 2.3999999999999999D)) ;

    dataset.addData(1, new StatisticalChartData(15D, 4.4000000000000004D)) ;
    dataset.addData(1, new StatisticalChartData(18D, 4.4000000000000004D)) ;

    dataset.addData(2, new StatisticalChartData(13D, 2.1000000000000001D)) ;
    dataset.addData(2, new StatisticalChartData(28D, 2.1000000000000001D)) ;

    dataset.addData(3, new StatisticalChartData(7D, 1.3D)) ;
    dataset.addData(3, new StatisticalChartData(17D, 1.3D)) ;

    return dataset ;
  }

}
