package demo.barcharts.xy ;

import java.awt.Color ;
import java.awt.GradientPaint ;
import java.awt.GridLayout ;
import java.text.DecimalFormat ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.value.StackedXYBarValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.GradientPaintTransformType ;
import com.ijchart.xychart.data.XYNumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.format.AppendFormatter ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.render.xy.XYStackedBarRender ;
import com.ijchart.xychart.tickunit.LongTickUnit ;
import com.ijchart.xychart.title.TextTitle ;
import com.ijchart.xychart.tooltip.DefaultTooltipGenerator ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class StackedXYBarChartDemo3 extends JPanel {
  private ChartOrientation orientation = ChartOrientation.VERTICAL ;

  private DecimalFormat format = new DecimalFormat("#,##0") ;

  public StackedXYBarChartDemo3() {
    setLayout(new GridLayout(1, 1)) ;
    addPanel() ;
  }

  private void addPanel() {
    final IMatrixDataset dataset = createDataset() ;
    final IJChart chart = ChartFactory.createStackedXYBarChart(
      "Waste Management", "Date", "Tonnes", dataset, orientation, true, true, false, false) ;

    XYPlot xyplot = (XYPlot) chart.getPlot() ;
//    xyplot.setForegroundAlpha(0.75F);
    StackedXYBarValueAxis domainAxis = (StackedXYBarValueAxis) xyplot.getDomainAxis() ;
    domainAxis.setTickUnit(new LongTickUnit()) ;
    domainAxis.setLowerMargin(0.01D) ;
    domainAxis.setUpperMargin(0.01D) ;

    StackedXYBarValueAxis valueAxis = (StackedXYBarValueAxis) xyplot.getValueAxis() ;
    valueAxis.setTickValueFormatter(new AppendFormatter(null, "%")) ;

    XYStackedBarRender render = (XYStackedBarRender) xyplot.getRender() ;
    render.setMargin(0.29999999999999999D) ;
    render.setRenderAsPercentages(true) ;

    render.setToolTipGenerator(new DefaultTooltipGenerator() {
      public String generateToolTip(IMatrixDataset dataset, int row, int column) {
        Object rowKey = dataset.getRowKey(row) ;
        XYNumberChartData data = (XYNumberChartData) dataset.getData(row, column) ;
        return rowKey + " : " + (int) data.getX() + " = " + format.format(data.getY()) + " tonnes" ;
      }
    }) ;

    chart.addSubtitle(new TextTitle("St Albans City and District Council")) ;
    GradientPaint gradientpaint = new GradientPaint(0.0F, 0.0F, new Color(64, 0, 0), 0.0F, 0.0F,
                                                    Color.red) ;
    GradientPaint gradientpaint1 = new GradientPaint(0.0F, 0.0F, new Color(0, 64, 0), 0.0F, 0.0F,
                                                     Color.green) ;
    render.setRowItemFillPaint(0, gradientpaint) ;
    render.setRowItemFillPaint(1, gradientpaint1) ;
    render.setBaseItemFillPaintTransformer(GradientPaintTransformType.HORIZONTAL) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private static IMatrixDataset createDataset() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("Landfilled") ;
    dataset.addRowKey("Recycled") ;

    double[][] data1 = { {2002, 41287D}, {2003, 41096D}, {2004, 39603D}, {2005, 39693D}, {2006,
                       37227D}
    } ;
    double[][] data2 = { {2002, 7717D}, {2003, 8317D}, {2004, 9493D}, {2005, 11228D}, {2006, 14941D}
    } ;
    dataset.addRowData(2) ;

    for (int i = 0 ; i < data1.length ; i++) {
      dataset.addData(0, new XYNumberChartData(data1[i][0], data1[i][1])) ;
      dataset.addData(1, new XYNumberChartData(data2[i][0], data2[i][1])) ;
    }

    return dataset ;
  }

}
