package demo.barcharts.xy ;

import java.awt.GridLayout ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.value.IntervalXYValueAxis ;
import com.ijchart.xychart.axis.value.NumberValueAxis ;
import com.ijchart.xychart.axis.value.SymbolValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.DateTickUnitType ;
import com.ijchart.xychart.constant.IAngle ;
import com.ijchart.xychart.data.IntervalXYNumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.tickunit.DateTickUnit ;
import com.ijchart.xychart.util.DateUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class XYBarChartDemo7 extends JPanel {
  private ChartOrientation orientation = ChartOrientation.HORIZONTAL ;

  public XYBarChartDemo7() {
    setLayout(new GridLayout(1, 1)) ;
    addPanel() ;
  }

  private void addPanel() {
    final IMatrixDataset dataset = createDataset() ;
    final IJChart chart = ChartFactory.createIntervalXYBar2DChart(
      "XYBarChartDemo7", "Date", "Y", dataset, orientation, true, false, false, true) ;

    XYPlot xyplot = (XYPlot) chart.getPlot() ;
    IntervalXYValueAxis valueAxis = (IntervalXYValueAxis) xyplot.getValueAxis() ;
    valueAxis.setTickUnit(new DateTickUnit(DateTickUnitType.DATE_UNIT_DAY)) ;
    valueAxis.setAxisLabel("Date");

    String[] xSymbols = {"S1", "S2", "S3"} ;
    SymbolValueAxis xAxis = new SymbolValueAxis("Series", (NumberValueAxis) xyplot.getDomainAxis(),
                                                xSymbols) ;
    xAxis.setAxisLabelAngle(IAngle.ANGLE_UP_90);
    xyplot.setDomainAxis(xAxis) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private static IMatrixDataset createDataset() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("S1") ;
    dataset.addRowKey("S2") ;
    dataset.addRowKey("S3") ;

    dataset.addRowData(3) ;
    double offset = -0.45000000000000001D ;
    dataset.addData(0, new IntervalXYNumberChartData(0 - offset, 0 + offset, toMills(12, 6, 2007),
                                                     toMills(14, 6, 2007))) ;
    dataset.addData(0, new IntervalXYNumberChartData(0 - offset, 0 + offset, toMills(15, 6, 2007),
                                                     toMills(16, 6, 2007))) ;
    dataset.addData(1, new IntervalXYNumberChartData(1 - offset, 1 + offset, toMills(12, 6, 2007),
                                                     toMills(18, 6, 2007))) ;
    dataset.addData(2, new IntervalXYNumberChartData(2 - offset, 2 + offset, toMills(14, 6, 2007),
                                                     toMills(17, 6, 2007))) ;

    return dataset ;
  }

  private static long toMills(int day, int month, int year) {
    return DateUtil.dateToTimeInMillis(year, month, day) ;
  }
}
