package demo.miscellaneous.markers ;

import java.awt.BorderLayout ;
import java.awt.Color ;
import java.awt.Font ;

import javax.swing.JPanel ;

import com.ijchart.xychart.annotation.XYPointerAnnotation ;
import com.ijchart.xychart.axis.value.XYNumberValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.HorizontalAlignment ;
import com.ijchart.xychart.constant.TextPosition ;
import com.ijchart.xychart.constant.VerticalAlignment ;
import com.ijchart.xychart.data.XYNumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.format.AbstractChartDataFormatterAdapter ;
import com.ijchart.xychart.format.IChartDataFormatter ;
import com.ijchart.xychart.format.IFormatter ;
import com.ijchart.xychart.format.MillisecondsToDateFormatter ;
import com.ijchart.xychart.marker.ValueMarker ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.render.xy.XYLineAndShapeRender ;
import com.ijchart.xychart.tooltip.DefaultTooltipGenerator ;
import com.ijchart.xychart.ui.Layer ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.util.DateUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class MarkerDemo1 extends JPanel {
  private ChartOrientation orientation = ChartOrientation.VERTICAL ;

  public MarkerDemo1() {
    setLayout(new BorderLayout()) ;
    addPanel1() ;
  }

  private void addPanel1() {
    DefaultMatrixDataset dataset = createDataset() ;
    IJChart chart = ChartFactory.createScatterChart("Marker Demo 1", "Time", "Y", dataset,
                                                    orientation, true, true, true, false) ;
    chart.getLegendTitle().setPosition(RectangleEdge.RIGHT) ;

    XYPlot plot = (XYPlot) chart.getPlot() ;
    XYNumberValueAxis xAxis = (XYNumberValueAxis) plot.getDomainAxis() ;
    xAxis.setLowerMargin(0.03) ;
    xAxis.setUpperMargin(0.20) ;
    final long t1 = DateUtil.dateToTimeInMillis(2003, 5, 22, 1, 10) ;
    final long t2 = DateUtil.dateToTimeInMillis(2003, 5, 22, 2, 20) ;
    xAxis.setUserDefineDatasetRange(t1, t2) ;

    XYNumberValueAxis yAxis = (XYNumberValueAxis) plot.getValueAxis() ;
    yAxis.setAxisRangeIncludeZero(false) ;
    yAxis.setUserDefineDatasetRange(145, 210) ;
    yAxis.setLowerMargin(0.02) ;
    yAxis.setUpperMargin(0.02) ;

    XYLineAndShapeRender render = (XYLineAndShapeRender) plot.getRender() ;
    IChartDataFormatter formatter = new AbstractChartDataFormatterAdapter() {
      private IFormatter format = new MillisecondsToDateFormatter("yyyy-MM-dd HH:mm") ;

      public String format(IMatrixDataset dataset, int row, int column) {
        XYNumberChartData data = (XYNumberChartData) dataset.getData(row, column) ;
        return "(" + format.format(data.getXData(), row) + "," + data.getY() + ")" ;
      }

    } ;
    DefaultTooltipGenerator tooltip = new DefaultTooltipGenerator(formatter) ;
    render.setToolTipGenerator(tooltip) ;

    ValueMarker valuemarker = new ValueMarker(146D) ;
    valuemarker.setBorderPaint(Color.green) ;
    valuemarker.setText("Bid Start Price") ;
    valuemarker.setTextHorizontalAlign(HorizontalAlignment.RIGHT) ;
    yAxis.addValueMarker(Layer.FOREGROUND, valuemarker) ;

    ValueMarker valuemarker1 = new ValueMarker(175D) ;
    valuemarker1.setBorderPaint(Color.red) ;
    valuemarker1.setText("Target Price") ;
    valuemarker1.setTextHorizontalAlign(HorizontalAlignment.RIGHT) ;
    yAxis.addValueMarker(Layer.FOREGROUND, valuemarker1) ;

    long time1 = DateUtil.dateToTimeInMillis(2003, 5, 22, 2, 0) ;
    ValueMarker valuemarker2 = new ValueMarker(time1) ;
    valuemarker2.setBorderPaint(Color.orange) ;
    valuemarker2.setText("Original Close (02:00)") ;
    valuemarker2.setTextVerticalAlign(VerticalAlignment.TOP) ;
    xAxis.addValueMarker(Layer.BACKGROUND, valuemarker2) ;

    long time2 = DateUtil.dateToTimeInMillis(2003, 5, 22, 2, 15) ;
    ValueMarker valuemarker3 = new ValueMarker(time2) ;
    valuemarker3.setBorderPaint(Color.red) ;
    valuemarker3.setText("Close Date (02:15)") ;
    valuemarker3.setTextVerticalAlign(VerticalAlignment.TOP) ;
    valuemarker3.setTextPosition(TextPosition.OUTSIDE_RIGHT) ;
    xAxis.addValueMarker(Layer.BACKGROUND, valuemarker3) ;

    long time3 = DateUtil.dateToTimeInMillis(2003, 5, 22, 2, 10) ;
    XYPointerAnnotation xypointerannotation = new XYPointerAnnotation("Best Bid", time3, 163D,
                                                                      -2.8561944901923448D) ;
    xypointerannotation.setBaseRadius(35D) ;
    xypointerannotation.setTextFont(new Font("SansSerif", 0, 9)) ;
    xypointerannotation.setTextPaint(Color.blue) ;
    render.addAnnotation(xypointerannotation) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private static DefaultMatrixDataset createDataset() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("Supplier 1") ;
    dataset.addRowKey("Supplier 2") ;
    dataset.addRowData(2) ;

    dataset.addData(0, createData(13, 1, 22, 5, 2003, 200D)) ;
    dataset.addData(0, createData(14, 1, 22, 5, 2003, 195D)) ;
    dataset.addData(0, createData(45, 1, 22, 5, 2003, 190D)) ;
    dataset.addData(0, createData(46, 1, 22, 5, 2003, 188D)) ;
    dataset.addData(0, createData(47, 1, 22, 5, 2003, 185D)) ;
    dataset.addData(0, createData(52, 1, 22, 5, 2003, 180D)) ;

    dataset.addData(1, createData(25, 1, 22, 5, 2003, 185D)) ;
    dataset.addData(1, createData(0, 2, 22, 5, 2003, 175D)) ;
    dataset.addData(1, createData(5, 2, 22, 5, 2003, 170D)) ;
    dataset.addData(1, createData(6, 2, 22, 5, 2003, 168D)) ;
    dataset.addData(1, createData(9, 2, 22, 5, 2003, 165D)) ;
    dataset.addData(1, createData(10, 2, 22, 5, 2003, 163D)) ;

    return dataset ;
  }

  private static XYNumberChartData createData(int minute, int hour, int day, int month, int year,
                                              double value) {
    long time = DateUtil.dateToTimeInMillis(year, month, day, hour, minute) ;
    return new XYNumberChartData(time, value) ;
  }
}
