package demo.others.categorycharts;

import java.awt.GridLayout ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.category.CategoryAxis ;
import com.ijchart.xychart.axis.category.CategoryAxis3D ;
import com.ijchart.xychart.axis.value.AbstractValueAxis ;
import com.ijchart.xychart.axis.value.NumberValueAxis3D ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.AxisLocation ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.HorizontalAlignment ;
import com.ijchart.xychart.constant.IAngle ;
import com.ijchart.xychart.constant.VerticalAlignment ;
import com.ijchart.xychart.data.NumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.format.NumberChartDataFormatter ;
import com.ijchart.xychart.label.DefaultLabelGenerator ;
import com.ijchart.xychart.marker.CategoryMarker ;
import com.ijchart.xychart.marker.IntervalValueMarker ;
import com.ijchart.xychart.marker.ValueMarker ;
import com.ijchart.xychart.plot.Category3DPlot ;
import com.ijchart.xychart.render.category.Bar2DRender ;
import com.ijchart.xychart.ui.IEffect3D ;
import com.ijchart.xychart.ui.Layer ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class CategoryBar3DChartPanel1 extends JPanel {
  private IEffect3D effect = new IEffect3D() {
    public double getXOffset() {
      return 12 ;
    }

    public double getYOffset() {
      return 8 ;
    }

  } ;

  private IMatrixDataset dataset1 = createDataset1() ;

  private IMatrixDataset dataset2 = createDataset2() ;

  private ChartOrientation orientation = ChartOrientation.VERTICAL ;

  public CategoryBar3DChartPanel1() {
    init() ;
  }

  private void init() {
    setLayout(new GridLayout(1, 1)) ;
    addPanel1() ;
  }

  private void addPanel1() {
    CategoryAxis xAxis = createCategoryAxis1() ;
    CategoryMarker marker = new CategoryMarker(dataset1.getRowKey(2)) ;
    marker.setText("Special 1") ;
    marker.setTextVerticalAlign(VerticalAlignment.TOP) ;
    xAxis.addCategoryMarker(Layer.BACKGROUND, marker) ;

    marker = new CategoryMarker(dataset1.getRowKey(4)) ;
    marker.setText("Special 2") ;
    marker.setTextVerticalAlign(VerticalAlignment.TOP) ;
    xAxis.addCategoryMarker(Layer.FOREGROUND, marker) ;

    AbstractValueAxis yAxis = createValueAxis1() ;
    yAxis.setInverted(false) ;

    if (orientation.equals(ChartOrientation.VERTICAL)) {
      ValueMarker valueMarker = new ValueMarker(4) ;
      valueMarker.setText("Special background value 1") ;
      valueMarker.setTextHorizontalAlign(HorizontalAlignment.RIGHT) ;
      yAxis.addValueMarker(Layer.BACKGROUND, valueMarker) ;

      valueMarker = new ValueMarker(12) ;
      valueMarker.setText("Special foreground value 2") ;
      valueMarker.setTextHorizontalAlign(HorizontalAlignment.RIGHT) ;
      valueMarker.setTextVerticalAlign(VerticalAlignment.BOTTOM) ;
      yAxis.addValueMarker(Layer.FOREGROUND, valueMarker) ;
    } else if (orientation.equals(ChartOrientation.HORIZONTAL)) {
      ValueMarker valueMarker = new ValueMarker(4) ;
      valueMarker.setText("Special background value 1") ;
      valueMarker.setTextHorizontalAlign(HorizontalAlignment.LEFT) ;
      valueMarker.setTextVerticalAlign(VerticalAlignment.TOP) ;
      yAxis.addValueMarker(Layer.BACKGROUND, valueMarker) ;

      valueMarker = new ValueMarker(12) ;
      valueMarker.setText("Special foreground value 2") ;
      valueMarker.setTextHorizontalAlign(HorizontalAlignment.RIGHT) ;
      valueMarker.setTextVerticalAlign(VerticalAlignment.TOP) ;
      yAxis.addValueMarker(Layer.FOREGROUND, valueMarker) ;
    }

    IntervalValueMarker intervalMarker = new IntervalValueMarker(5, 10) ;
    intervalMarker.setText("special foreground interval marker 1") ;
    yAxis.addIntervalValueMarker(Layer.FOREGROUND, intervalMarker) ;

    intervalMarker = new IntervalValueMarker(12, 18) ;
    yAxis.addIntervalValueMarker(Layer.BACKGROUND, intervalMarker) ;
    intervalMarker.setText("special background interval marker 2") ;
    yAxis.setVisible(true) ;

    Bar2DRender render = new Bar2DRender(dataset1) ;
    render.setLabelGenerator(new DefaultLabelGenerator(new NumberChartDataFormatter("#0.00"))) ;
    render.setMaximumBarWidth(20) ;

    IJChart chart = ChartFactory.createCategoryBar3DChart(
      "This is a very long title,it has many words and wrap automatically.", "x axis text",
      "y axis text", createDataset1(), orientation, true, true) ;
    Category3DPlot plot = (Category3DPlot) chart.getPlot() ;
    plot.setValueAxis(yAxis) ;

//        plot.getCategoryAxis().setAxisVisible(false) ;
//        plot.getValueAxis().setAxisVisible(false) ;

    plot.setCategoryAxis(1, createCategoryAxis2()) ;
    plot.setCategoryAxisLocation(1, AxisLocation.TOP_OR_RIGHT) ;
    plot.setValueAxis(1, createValueAxis2()) ;
    plot.setValueAxisLocation(1, AxisLocation.BOTTOM_OR_RIGHT) ;

    plot.setCategoryAxis(2, createCategoryAxis3()) ;
    plot.setCategoryAxisLocation(2, AxisLocation.TOP_OR_RIGHT) ;
    plot.setValueAxis(2, createValueAxis3()) ;
    plot.setValueAxisLocation(2, AxisLocation.BOTTOM_OR_RIGHT) ;

    plot.setCategoryAxis(3, createCategoryAxis4()) ;
    plot.setCategoryAxisLocation(3, AxisLocation.BOTTOM_OR_LEFT) ;
    plot.setValueAxis(3, createValueAxis4()) ;
    plot.setValueAxisLocation(3, AxisLocation.TOP_OR_LEFT) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private CategoryAxis createCategoryAxis1() {
    CategoryAxis axis = new CategoryAxis("category label1", dataset1) ;
    if (orientation.equals(ChartOrientation.HORIZONTAL)) {
      axis.setAxisLabelAngle(IAngle.ANGLE_DOWN_90) ;
    }

    return axis ;
  }

  private CategoryAxis createCategoryAxis2() {
    CategoryAxis axis = new CategoryAxis3D("category label2", dataset1, effect) ;
    if (orientation.equals(ChartOrientation.HORIZONTAL)) {
      axis.setAxisLabelAngle(IAngle.ANGLE_DOWN_90) ;
    }

    return axis ;
  }

  private CategoryAxis createCategoryAxis3() {
    CategoryAxis axis = new CategoryAxis3D("category label3", dataset2, effect) ;
    if (orientation.equals(ChartOrientation.HORIZONTAL)) {
      axis.setAxisLabelAngle(IAngle.ANGLE_DOWN_90) ;
    }

    return axis ;
  }

  private CategoryAxis createCategoryAxis4() {
    CategoryAxis axis = new CategoryAxis3D("category label4", dataset2, effect) ;
    if (orientation.equals(ChartOrientation.HORIZONTAL)) {
      axis.setAxisLabelAngle(IAngle.ANGLE_DOWN_90) ;
    }

    return axis ;
  }

  private AbstractValueAxis createValueAxis1() {
    AbstractValueAxis valueAxis = new NumberValueAxis3D("value label1", dataset1, effect) ;
    if (orientation.equals(ChartOrientation.VERTICAL)) {
      valueAxis.setAxisLabelAngle(IAngle.ANGLE_DOWN_90) ;
    }
    valueAxis.setValueLineVisible(true) ;
    valueAxis.setValueLineValue(0) ;

    return valueAxis ;
  }

  private AbstractValueAxis createValueAxis2() {
    AbstractValueAxis valueAxis = new NumberValueAxis3D("value label2", dataset1, effect) ;
    if (orientation.equals(ChartOrientation.VERTICAL)) {
      valueAxis.setAxisLabelAngle(IAngle.ANGLE_DOWN_90) ;
    }
    valueAxis.setValueLineValue(1) ;
    valueAxis.setValueLineVisible(true) ;

    return valueAxis ;
  }

  private AbstractValueAxis createValueAxis3() {
    AbstractValueAxis valueAxis = new NumberValueAxis3D("value label3", dataset2, effect) ;
    if (orientation.equals(ChartOrientation.VERTICAL)) {
      valueAxis.setAxisLabelAngle(IAngle.ANGLE_DOWN_90) ;
    }
    valueAxis.setValueLineValue(20) ;
    valueAxis.setValueLineVisible(true) ;

    return valueAxis ;
  }

  private AbstractValueAxis createValueAxis4() {
    AbstractValueAxis valueAxis = new NumberValueAxis3D("value label4", dataset2, effect) ;
    if (orientation.equals(ChartOrientation.VERTICAL)) {
      valueAxis.setAxisLabelAngle(IAngle.ANGLE_DOWN_90) ;
    }
    valueAxis.setValueLineValue(50) ;
    valueAxis.setValueLineVisible(true) ;

    return valueAxis ;
  }

  private static IMatrixDataset createDataset1() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("123456789011") ;
    dataset.addRowKey("ab\ncd\nef") ;
    dataset.addRowKey("abcde") ;
    dataset.addRowKey("12345678") ;
    dataset.addRowKey("") ;

    for (int i = 0 ; i < 5 ; i++) {
      dataset.addRowData() ;
      for (int j = 0 ; j < 6 ; j++) {
        if (i % 2 == 0 && j == 3) {
          dataset.addData(i, null) ;
        } else if (i == 0 && j % 5 != 0) {
          dataset.addColKey("col key " + j) ;
        } else {
          dataset.addColKey("long long long long col key " + j) ;
        }

        dataset.addData(i, new NumberChartData(Math.abs((i + 1) * -j + 1 + Math.random()))) ;
      }
    }

    return dataset ;
  }

  private static IMatrixDataset createDataset2() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("123456789011") ;
    dataset.addRowKey("abncdnef") ;
    dataset.addRowKey("abcde") ;
    dataset.addRowKey("12345678") ;
    dataset.addRowKey("ABCD") ;

    for (int i = 0 ; i < 5 ; i++) {
      dataset.addRowData() ;
      for (int j = 0 ; j < 5 ; j++) {
        if (i % 2 == 0 && j == 3) {
          dataset.addData(i, null) ;
        } else {
          dataset.addData(i, new NumberChartData((i + 1) * j * 10 + 1)) ;
        }
      }
    }

    return dataset ;
  }

}
