package demo.piecharts ;

import java.awt.BorderLayout ;
import java.awt.FlowLayout ;
import java.awt.event.ActionEvent ;
import java.awt.event.ActionListener ;
import java.text.NumberFormat ;
import java.util.Arrays ;
import java.util.Collections ;
import java.util.List ;

import javax.swing.JButton ;
import javax.swing.JCheckBox ;
import javax.swing.JPanel ;

import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.KeyValueComparatorType ;
import com.ijchart.xychart.data.NumberChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.dataset.SingleRowDataset ;
import com.ijchart.xychart.dataset.data.KeyValuePair ;
import com.ijchart.xychart.plot.NoAxisPlot ;
import com.ijchart.xychart.render.noaxis.PieRender ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class PieChartDemo4 extends JPanel implements ActionListener {
  private NumberFormat format = NumberFormat.getPercentInstance() ;

  private IJChart chart ;

  private boolean ascendingByKey ;

  private boolean ascendingByValue ;

  public PieChartDemo4() {
    format.setMaximumFractionDigits(0) ;

    setLayout(new BorderLayout()) ;
    ascendingByKey = false ;
    ascendingByValue = false ;
    addPanel() ;

    JPanel jpanel = new JPanel(new FlowLayout()) ;
    JButton jbutton = new JButton("By Key") ;
    jbutton.setActionCommand("BY_KEY") ;
    jbutton.addActionListener(this) ;
    JButton jbutton1 = new JButton("By Value") ;
    jbutton1.setActionCommand("BY_VALUE") ;
    jbutton1.addActionListener(this) ;
    JButton jbutton2 = new JButton("Random") ;
    jbutton2.setActionCommand("RANDOM") ;
    jbutton2.addActionListener(this) ;

    JCheckBox jcheckbox = new JCheckBox("Simple Labels") ;
    jcheckbox.setActionCommand("LABELS") ;
    jcheckbox.addActionListener(this) ;

    jpanel.add(jbutton) ;
    jpanel.add(jbutton1) ;
    jpanel.add(jbutton2) ;
    jpanel.add(jcheckbox) ;
    add(jpanel, "South") ;
  }

  private void addPanel() {
    SingleRowDataset dataset = createDataset(getDatas("none")) ;
    chart = ChartFactory.createPieChart("Pie Chart Demo 4", dataset, true, true) ;
    NoAxisPlot plot = (NoAxisPlot) chart.getPlot() ;
    plot.setNoDataMessage("No data available") ;

    PieRender render = (PieRender) plot.getRender() ;
    render.setCircular(false) ;
    render.setLabelGap(0.02D) ;
    render.setExplodePercent(getSectionDIndex(), 0.5D) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private int getSectionDIndex() {
    NoAxisPlot plot = (NoAxisPlot) chart.getPlot() ;
    PieRender render = (PieRender) plot.getRender() ;
    IMatrixDataset dataset = render.getDataset() ;
    for (int i = 0 ; i < dataset.getColumnCount() ; i++) {
      if (dataset.getColKey(i).equals("Section D")) {
        return i ;
      }
    }

    return -1 ;
  }

  public void actionPerformed(ActionEvent actionevent) {
    String s = actionevent.getActionCommand() ;
    KeyValuePair[] datas = getDatas(s) ;

    SingleRowDataset newDataset = createDataset(datas) ;
    NoAxisPlot plot = (NoAxisPlot) chart.getPlot() ;
    PieRender render = (PieRender) plot.getRender() ;
    render.setDataset(newDataset) ;
  }

  private static SingleRowDataset createDataset(KeyValuePair[] datas) {
    SingleRowDataset dataset = new SingleRowDataset() ;
    dataset.addRowData() ;
    for (int i = 0 ; i < datas.length ; i++) {
      dataset.addColKey(datas[i].getKey()) ;
      dataset.addData(0, new NumberChartData((Double) datas[i].getValue())) ;
    }

    return dataset ;
  }

  private KeyValuePair[] getDatas(String actionCommand) {
    KeyValuePair[] datas = new KeyValuePair[6] ;
    datas[0] = new KeyValuePair("Section A", new Double(43.200000000000003D)) ;
    datas[1] = new KeyValuePair("Section B", new Double(10D)) ;
    datas[2] = new KeyValuePair("Section C", new Double(27.5D)) ;
    datas[3] = new KeyValuePair("Section D", new Double(17.5D)) ;
    datas[4] = new KeyValuePair("Section E", new Double(11D)) ;
    datas[5] = new KeyValuePair("Section F", new Double(19.399999999999999D)) ;

    if ("BY_KEY".equals(actionCommand)) {
      for (int i = 0 ; i < datas.length ; i++) {
        datas[i].setComparatorType(KeyValueComparatorType.BY_KEY) ;
      }

      Arrays.sort(datas) ;
      if (!ascendingByKey) {
        ascendingByKey = true ;
      } else {
        ascendingByKey = false ;
        List list = Arrays.asList(datas) ;
        Collections.reverse(list) ;
        datas = (KeyValuePair[]) list.toArray(new KeyValuePair[0]) ;
      }
    } else if ("BY_VALUE".equals(actionCommand)) {
      for (int i = 0 ; i < datas.length ; i++) {
        datas[i].setComparatorType(KeyValueComparatorType.BY_VALUE) ;
      }

      Arrays.sort(datas) ;
      if (!ascendingByValue) {
        ascendingByValue = true ;
      } else {
        ascendingByValue = false ;
        List list = Arrays.asList(datas) ;
        Collections.reverse(list) ;
        datas = (KeyValuePair[]) list.toArray(new KeyValuePair[0]) ;
      }
    } else if ("RANDOM".equals(actionCommand)) {
      List list = Arrays.asList(datas) ;
      Collections.shuffle(list) ;
      datas = (KeyValuePair[]) list.toArray(new KeyValuePair[0]) ;
    } else if ("LABELS".equals(actionCommand)) {
      NoAxisPlot plot = (NoAxisPlot) chart.getPlot() ;
      PieRender render = (PieRender) plot.getRender() ;
      render.setSimpleLabels(!render.isSimpleLabels()) ;
    }

    return datas ;
  }

}
