package demo.timeseriescharts;

import java.awt.BasicStroke ;
import java.awt.BorderLayout ;
import java.awt.Color ;
import java.awt.Font ;
import java.awt.GradientPaint ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.value.XYNumberValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.GradientPaintTransformType ;
import com.ijchart.xychart.constant.HorizontalAlignment ;
import com.ijchart.xychart.constant.VerticalAlignment ;
import com.ijchart.xychart.data.XYNumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.format.AbstractChartDataFormatterAdapter ;
import com.ijchart.xychart.format.IChartDataFormatter ;
import com.ijchart.xychart.format.IFormatter ;
import com.ijchart.xychart.format.MillisecondsToDateFormatter ;
import com.ijchart.xychart.marker.IntervalValueMarker ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.render.xy.XYDifferenceRender ;
import com.ijchart.xychart.title.TextTitle ;
import com.ijchart.xychart.tooltip.DefaultTooltipGenerator ;
import com.ijchart.xychart.ui.DefaultGradientPaintTransformer ;
import com.ijchart.xychart.ui.Layer ;
import com.ijchart.xychart.ui.RectangleInsets ;
import com.ijchart.xychart.util.DateUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class DifferenceChartDemo2 extends JPanel {
  private ChartOrientation orientation = ChartOrientation.VERTICAL ;

  public DifferenceChartDemo2() {
    setLayout(new BorderLayout()) ;
    addPanel1() ;
  }

  private void addPanel1() {
    IMatrixDataset dataset = createDataset() ;
    final IJChart chart = ChartFactory.createXYDifferenceChart("Difference Chart Demo 2", "Time",
                                                               "Time", dataset, orientation, true,
                                                               true, true, true) ;
    chart.addSubtitle(new TextTitle("Data source: http://www.sunrisesunset.com/",
                                    new Font("SansSerif", 0, 12))) ;

    XYPlot xyplot = (XYPlot) chart.getPlot() ;
    XYDifferenceRender render = (XYDifferenceRender) xyplot.getRender() ;
    render.setPositivePaint(Color.BLUE) ;
    render.setNegativePaint(Color.BLUE) ;
    render.setBaseItemBorderStroke(new BasicStroke(2.0F)) ;
    render.setAutoSelectBorderPaint(false) ;
    render.setRowItemBorderPaint(0, Color.yellow) ;
    render.setRowItemBorderPaint(1, Color.red) ;

    IChartDataFormatter formatter = new AbstractChartDataFormatterAdapter() {
      private IFormatter format1 = new MillisecondsToDateFormatter("yyyy-MMM-dd") ;

      private IFormatter format2 = new MillisecondsToDateFormatter("kk:mm") ;

      public String format(IMatrixDataset dataset, int row, int column) {
        XYNumberChartData data = (XYNumberChartData) dataset.getData(row, column) ;
        return "(" + format1.format(data.getXData(), row) + ","
          + format2.format(data.getYData(), row) + ")" ;
      }
    } ;
    DefaultTooltipGenerator tooltip = new DefaultTooltipGenerator(formatter) ;
    render.setToolTipGenerator(tooltip) ;

    render.setBackgroundPaint(Color.lightGray) ;
    xyplot.setDomainGridlinePaint(Color.white) ;
    xyplot.setValueGridlinePaint(Color.white) ;
    xyplot.setAxisOffset(new RectangleInsets(5D, 5D, 5D, 5D)) ;
    xyplot.setDomainCrosshairVisible(true) ;
    xyplot.setValueCrosshairVisible(true) ;

    XYNumberValueAxis dateAxis = (XYNumberValueAxis) xyplot.getDomainAxis() ;
    dateAxis.setLowerMargin(0.0D) ;
    dateAxis.setUpperMargin(0.0D) ;
    xyplot.setForegroundAlpha(0.5F) ;

    GradientPaint gradientpaint = new GradientPaint(0.0F, 0.0F, Color.red, 1.0F, 1.0F, Color.orange) ;
    IntervalValueMarker intervalmarker = new IntervalValueMarker(dayToMills(28, 3, 2004),
                                                                 dayToMills(30, 10, 2004),
                                                                 gradientpaint, gradientpaint,
                                                                 new BasicStroke(2.0F), 1.0F) ;
    intervalmarker.setText("British Summer Time") ;
    intervalmarker.setTextHorizontalAlign(HorizontalAlignment.RIGHT) ;
    intervalmarker.setTextVerticalAlign(VerticalAlignment.BOTTOM) ;
    intervalmarker.setTextFont(new Font("SansSerif", 3, 10)) ;
    intervalmarker.setGradientPaintTransformer(new DefaultGradientPaintTransformer(
      GradientPaintTransformType.HORIZONTAL)) ;
    dateAxis.addIntervalValueMarker(Layer.BACKGROUND, intervalmarker) ;

    XYNumberValueAxis dateaxis1 = (XYNumberValueAxis) xyplot.getValueAxis() ;
    dateaxis1.setLowerMargin(0.04999999999999999D) ;
    dateaxis1.setUpperMargin(0.04999999999999999D) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private static IMatrixDataset createDataset() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("Sunrise") ;
    dataset.addRowKey("Sunset") ;
    dataset.addRowData(2) ;

    dataset.addData(0, new XYNumberChartData(dayToMills(1, 1, 2004), timeToMills(8, 5))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(2, 1, 2004), timeToMills(8, 5))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(3, 1, 2004), timeToMills(8, 5))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(4, 1, 2004), timeToMills(8, 5))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(5, 1, 2004), timeToMills(8, 5))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(6, 1, 2004), timeToMills(8, 4))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(7, 1, 2004), timeToMills(8, 4))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(8, 1, 2004), timeToMills(8, 4))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(9, 1, 2004), timeToMills(8, 3))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(10, 1, 2004), timeToMills(8, 3))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(11, 1, 2004), timeToMills(8, 2))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(12, 1, 2004), timeToMills(8, 1))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(13, 1, 2004), timeToMills(8, 1))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(14, 1, 2004), timeToMills(8, 0))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(15, 1, 2004), timeToMills(7, 59))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(16, 1, 2004), timeToMills(7, 58))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(17, 1, 2004), timeToMills(7, 58))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(18, 1, 2004), timeToMills(7, 57))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(19, 1, 2004), timeToMills(7, 56))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(20, 1, 2004), timeToMills(7, 55))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(21, 1, 2004), timeToMills(7, 54))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(22, 1, 2004), timeToMills(7, 53))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(23, 1, 2004), timeToMills(7, 51))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(24, 1, 2004), timeToMills(7, 50))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(25, 1, 2004), timeToMills(7, 49))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(26, 1, 2004), timeToMills(7, 48))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(27, 1, 2004), timeToMills(7, 46))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(28, 1, 2004), timeToMills(7, 45))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(29, 1, 2004), timeToMills(7, 44))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(30, 1, 2004), timeToMills(7, 42))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(31, 1, 2004), timeToMills(7, 41))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(1, 2, 2004), timeToMills(7, 39))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(2, 2, 2004), timeToMills(7, 38))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(3, 2, 2004), timeToMills(7, 36))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(4, 2, 2004), timeToMills(7, 35))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(5, 2, 2004), timeToMills(7, 33))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(6, 2, 2004), timeToMills(7, 31))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(7, 2, 2004), timeToMills(7, 30))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(8, 2, 2004), timeToMills(7, 28))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(9, 2, 2004), timeToMills(7, 26))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(10, 2, 2004), timeToMills(7, 25))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(11, 2, 2004), timeToMills(7, 23))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(12, 2, 2004), timeToMills(7, 21))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(13, 2, 2004), timeToMills(7, 19))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(14, 2, 2004), timeToMills(7, 17))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(15, 2, 2004), timeToMills(7, 15))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(16, 2, 2004), timeToMills(7, 13))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(17, 2, 2004), timeToMills(7, 11))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(18, 2, 2004), timeToMills(7, 10))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(19, 2, 2004), timeToMills(7, 8))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(20, 2, 2004), timeToMills(7, 6))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(21, 2, 2004), timeToMills(7, 4))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(22, 2, 2004), timeToMills(7, 2))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(23, 2, 2004), timeToMills(6, 59))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(24, 2, 2004), timeToMills(6, 57))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(25, 2, 2004), timeToMills(6, 55))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(26, 2, 2004), timeToMills(6, 53))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(27, 2, 2004), timeToMills(6, 51))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(28, 2, 2004), timeToMills(6, 49))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(29, 2, 2004), timeToMills(6, 47))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(1, 3, 2004), timeToMills(6, 45))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(2, 3, 2004), timeToMills(6, 43))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(3, 3, 2004), timeToMills(6, 40))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(4, 3, 2004), timeToMills(6, 38))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(5, 3, 2004), timeToMills(6, 36))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(6, 3, 2004), timeToMills(6, 34))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(7, 3, 2004), timeToMills(6, 32))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(8, 3, 2004), timeToMills(6, 29))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(9, 3, 2004), timeToMills(6, 27))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(10, 3, 2004), timeToMills(6, 25))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(11, 3, 2004), timeToMills(6, 23))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(12, 3, 2004), timeToMills(6, 20))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(13, 3, 2004), timeToMills(6, 18))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(14, 3, 2004), timeToMills(6, 16))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(15, 3, 2004), timeToMills(6, 14))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(16, 3, 2004), timeToMills(6, 11))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(17, 3, 2004), timeToMills(6, 9))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(18, 3, 2004), timeToMills(6, 7))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(19, 3, 2004), timeToMills(6, 5))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(20, 3, 2004), timeToMills(6, 2))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(21, 3, 2004), timeToMills(6, 0))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(22, 3, 2004), timeToMills(5, 58))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(23, 3, 2004), timeToMills(5, 55))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(24, 3, 2004), timeToMills(5, 53))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(25, 3, 2004), timeToMills(5, 51))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(26, 3, 2004), timeToMills(5, 49))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(27, 3, 2004), timeToMills(5, 46))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(28, 3, 2004), timeToMills(6, 44))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(29, 3, 2004), timeToMills(6, 42))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(30, 3, 2004), timeToMills(6, 39))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(31, 3, 2004), timeToMills(6, 37))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(1, 4, 2004), timeToMills(6, 35))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(2, 4, 2004), timeToMills(6, 33))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(3, 4, 2004), timeToMills(6, 30))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(4, 4, 2004), timeToMills(6, 28))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(5, 4, 2004), timeToMills(6, 26))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(6, 4, 2004), timeToMills(6, 24))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(7, 4, 2004), timeToMills(6, 21))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(8, 4, 2004), timeToMills(6, 19))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(9, 4, 2004), timeToMills(6, 17))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(10, 4, 2004), timeToMills(6, 15))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(11, 4, 2004), timeToMills(6, 12))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(12, 4, 2004), timeToMills(6, 10))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(13, 4, 2004), timeToMills(6, 8))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(14, 4, 2004), timeToMills(6, 6))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(15, 4, 2004), timeToMills(6, 4))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(16, 4, 2004), timeToMills(6, 2))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(17, 4, 2004), timeToMills(5, 59))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(18, 4, 2004), timeToMills(5, 57))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(19, 4, 2004), timeToMills(5, 55))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(20, 4, 2004), timeToMills(5, 53))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(21, 4, 2004), timeToMills(5, 51))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(22, 4, 2004), timeToMills(5, 49))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(23, 4, 2004), timeToMills(5, 47))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(24, 4, 2004), timeToMills(5, 45))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(25, 4, 2004), timeToMills(5, 43))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(26, 4, 2004), timeToMills(5, 41))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(27, 4, 2004), timeToMills(5, 39))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(28, 4, 2004), timeToMills(5, 37))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(29, 4, 2004), timeToMills(5, 35))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(30, 4, 2004), timeToMills(5, 33))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(1, 5, 2004), timeToMills(5, 31))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(2, 5, 2004), timeToMills(5, 29))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(3, 5, 2004), timeToMills(5, 28))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(4, 5, 2004), timeToMills(5, 26))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(5, 5, 2004), timeToMills(5, 24))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(6, 5, 2004), timeToMills(5, 22))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(7, 5, 2004), timeToMills(5, 20))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(8, 5, 2004), timeToMills(5, 19))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(9, 5, 2004), timeToMills(5, 17))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(10, 5, 2004), timeToMills(5, 15))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(11, 5, 2004), timeToMills(5, 14))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(12, 5, 2004), timeToMills(5, 12))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(13, 5, 2004), timeToMills(5, 11))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(14, 5, 2004), timeToMills(5, 9))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(15, 5, 2004), timeToMills(5, 8))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(16, 5, 2004), timeToMills(5, 6))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(17, 5, 2004), timeToMills(5, 5))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(18, 5, 2004), timeToMills(5, 3))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(19, 5, 2004), timeToMills(5, 2))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(20, 5, 2004), timeToMills(5, 1))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(21, 5, 2004), timeToMills(4, 59))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(22, 5, 2004), timeToMills(4, 58))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(23, 5, 2004), timeToMills(4, 57))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(24, 5, 2004), timeToMills(4, 56))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(25, 5, 2004), timeToMills(4, 55))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(26, 5, 2004), timeToMills(4, 53))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(27, 5, 2004), timeToMills(4, 52))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(28, 5, 2004), timeToMills(4, 51))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(29, 5, 2004), timeToMills(4, 50))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(30, 5, 2004), timeToMills(4, 50))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(31, 5, 2004), timeToMills(4, 49))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(1, 6, 2004), timeToMills(4, 48))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(2, 6, 2004), timeToMills(4, 47))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(3, 6, 2004), timeToMills(4, 46))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(4, 6, 2004), timeToMills(4, 46))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(5, 6, 2004), timeToMills(4, 45))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(6, 6, 2004), timeToMills(4, 45))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(7, 6, 2004), timeToMills(4, 44))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(8, 6, 2004), timeToMills(4, 44))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(9, 6, 2004), timeToMills(4, 43))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(10, 6, 2004), timeToMills(4, 43))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(11, 6, 2004), timeToMills(4, 42))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(12, 6, 2004), timeToMills(4, 42))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(13, 6, 2004), timeToMills(4, 42))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(14, 6, 2004), timeToMills(4, 42))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(15, 6, 2004), timeToMills(4, 42))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(16, 6, 2004), timeToMills(4, 42))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(17, 6, 2004), timeToMills(4, 42))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(18, 6, 2004), timeToMills(4, 42))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(19, 6, 2004), timeToMills(4, 42))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(20, 6, 2004), timeToMills(4, 42))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(21, 6, 2004), timeToMills(4, 42))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(22, 6, 2004), timeToMills(4, 42))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(23, 6, 2004), timeToMills(4, 43))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(24, 6, 2004), timeToMills(4, 43))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(25, 6, 2004), timeToMills(4, 43))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(26, 6, 2004), timeToMills(4, 44))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(27, 6, 2004), timeToMills(4, 44))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(28, 6, 2004), timeToMills(4, 45))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(29, 6, 2004), timeToMills(4, 45))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(30, 6, 2004), timeToMills(4, 46))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(1, 7, 2004), timeToMills(4, 47))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(2, 7, 2004), timeToMills(4, 47))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(3, 7, 2004), timeToMills(4, 48))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(4, 7, 2004), timeToMills(4, 49))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(5, 7, 2004), timeToMills(4, 50))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(6, 7, 2004), timeToMills(4, 51))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(7, 7, 2004), timeToMills(4, 51))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(8, 7, 2004), timeToMills(4, 52))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(9, 7, 2004), timeToMills(4, 53))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(10, 7, 2004), timeToMills(4, 54))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(11, 7, 2004), timeToMills(4, 55))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(12, 7, 2004), timeToMills(4, 57))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(13, 7, 2004), timeToMills(4, 58))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(14, 7, 2004), timeToMills(4, 59))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(15, 7, 2004), timeToMills(5, 0))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(16, 7, 2004), timeToMills(5, 1))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(17, 7, 2004), timeToMills(5, 2))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(18, 7, 2004), timeToMills(5, 4))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(19, 7, 2004), timeToMills(5, 5))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(20, 7, 2004), timeToMills(5, 6))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(21, 7, 2004), timeToMills(5, 8))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(22, 7, 2004), timeToMills(5, 9))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(23, 7, 2004), timeToMills(5, 10))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(24, 7, 2004), timeToMills(5, 12))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(25, 7, 2004), timeToMills(5, 13))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(26, 7, 2004), timeToMills(5, 14))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(27, 7, 2004), timeToMills(5, 16))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(28, 7, 2004), timeToMills(5, 17))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(29, 7, 2004), timeToMills(5, 19))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(30, 7, 2004), timeToMills(5, 20))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(31, 7, 2004), timeToMills(5, 22))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(1, 8, 2004), timeToMills(5, 23))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(2, 8, 2004), timeToMills(5, 25))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(3, 8, 2004), timeToMills(5, 26))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(4, 8, 2004), timeToMills(5, 28))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(5, 8, 2004), timeToMills(5, 29))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(6, 8, 2004), timeToMills(5, 31))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(7, 8, 2004), timeToMills(5, 32))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(8, 8, 2004), timeToMills(5, 34))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(9, 8, 2004), timeToMills(5, 36))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(10, 8, 2004), timeToMills(5, 37))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(11, 8, 2004), timeToMills(5, 39))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(12, 8, 2004), timeToMills(5, 40))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(13, 8, 2004), timeToMills(5, 42))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(14, 8, 2004), timeToMills(5, 43))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(15, 8, 2004), timeToMills(5, 45))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(16, 8, 2004), timeToMills(5, 47))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(17, 8, 2004), timeToMills(5, 48))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(18, 8, 2004), timeToMills(5, 50))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(19, 8, 2004), timeToMills(5, 51))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(20, 8, 2004), timeToMills(5, 53))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(21, 8, 2004), timeToMills(5, 55))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(22, 8, 2004), timeToMills(5, 56))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(23, 8, 2004), timeToMills(5, 58))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(24, 8, 2004), timeToMills(5, 59))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(25, 8, 2004), timeToMills(6, 1))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(26, 8, 2004), timeToMills(6, 3))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(27, 8, 2004), timeToMills(6, 4))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(28, 8, 2004), timeToMills(6, 6))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(29, 8, 2004), timeToMills(6, 7))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(30, 8, 2004), timeToMills(6, 9))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(31, 8, 2004), timeToMills(6, 10))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(1, 9, 2004), timeToMills(6, 12))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(2, 9, 2004), timeToMills(6, 14))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(3, 9, 2004), timeToMills(6, 15))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(4, 9, 2004), timeToMills(6, 17))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(5, 9, 2004), timeToMills(6, 18))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(6, 9, 2004), timeToMills(6, 20))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(7, 9, 2004), timeToMills(6, 22))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(8, 9, 2004), timeToMills(6, 23))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(9, 9, 2004), timeToMills(6, 25))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(10, 9, 2004), timeToMills(6, 26))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(11, 9, 2004), timeToMills(6, 28))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(12, 9, 2004), timeToMills(6, 30))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(13, 9, 2004), timeToMills(6, 31))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(14, 9, 2004), timeToMills(6, 33))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(15, 9, 2004), timeToMills(6, 34))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(16, 9, 2004), timeToMills(6, 36))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(17, 9, 2004), timeToMills(6, 38))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(18, 9, 2004), timeToMills(6, 39))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(19, 9, 2004), timeToMills(6, 41))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(20, 9, 2004), timeToMills(6, 42))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(21, 9, 2004), timeToMills(6, 44))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(22, 9, 2004), timeToMills(6, 46))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(23, 9, 2004), timeToMills(6, 47))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(24, 9, 2004), timeToMills(6, 49))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(25, 9, 2004), timeToMills(6, 50))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(26, 9, 2004), timeToMills(6, 52))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(27, 9, 2004), timeToMills(6, 54))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(28, 9, 2004), timeToMills(6, 55))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(29, 9, 2004), timeToMills(6, 57))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(30, 9, 2004), timeToMills(6, 58))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(1, 10, 2004), timeToMills(7, 0))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(2, 10, 2004), timeToMills(7, 2))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(3, 10, 2004), timeToMills(7, 3))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(4, 10, 2004), timeToMills(7, 5))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(5, 10, 2004), timeToMills(7, 7))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(6, 10, 2004), timeToMills(7, 8))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(7, 10, 2004), timeToMills(7, 10))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(8, 10, 2004), timeToMills(7, 12))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(9, 10, 2004), timeToMills(7, 13))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(10, 10, 2004), timeToMills(7, 15))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(11, 10, 2004), timeToMills(7, 17))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(12, 10, 2004), timeToMills(7, 18))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(13, 10, 2004), timeToMills(7, 20))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(14, 10, 2004), timeToMills(7, 22))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(15, 10, 2004), timeToMills(7, 23))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(16, 10, 2004), timeToMills(7, 25))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(17, 10, 2004), timeToMills(7, 27))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(18, 10, 2004), timeToMills(7, 29))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(19, 10, 2004), timeToMills(7, 30))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(20, 10, 2004), timeToMills(7, 32))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(21, 10, 2004), timeToMills(7, 34))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(22, 10, 2004), timeToMills(7, 35))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(23, 10, 2004), timeToMills(7, 37))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(24, 10, 2004), timeToMills(7, 39))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(25, 10, 2004), timeToMills(7, 41))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(26, 10, 2004), timeToMills(7, 42))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(27, 10, 2004), timeToMills(7, 44))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(28, 10, 2004), timeToMills(7, 46))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(29, 10, 2004), timeToMills(7, 48))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(30, 10, 2004), timeToMills(7, 49))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(31, 10, 2004), timeToMills(6, 51))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(1, 11, 2004), timeToMills(6, 53))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(2, 11, 2004), timeToMills(6, 55))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(3, 11, 2004), timeToMills(6, 57))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(4, 11, 2004), timeToMills(6, 58))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(5, 11, 2004), timeToMills(7, 0))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(6, 11, 2004), timeToMills(7, 2))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(7, 11, 2004), timeToMills(7, 4))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(8, 11, 2004), timeToMills(7, 5))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(9, 11, 2004), timeToMills(7, 7))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(10, 11, 2004), timeToMills(7, 9))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(11, 11, 2004), timeToMills(7, 11))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(12, 11, 2004), timeToMills(7, 12))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(13, 11, 2004), timeToMills(7, 14))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(14, 11, 2004), timeToMills(7, 16))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(15, 11, 2004), timeToMills(7, 17))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(16, 11, 2004), timeToMills(7, 19))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(17, 11, 2004), timeToMills(7, 21))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(18, 11, 2004), timeToMills(7, 23))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(19, 11, 2004), timeToMills(7, 24))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(20, 11, 2004), timeToMills(7, 26))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(21, 11, 2004), timeToMills(7, 28))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(22, 11, 2004), timeToMills(7, 29))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(23, 11, 2004), timeToMills(7, 31))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(24, 11, 2004), timeToMills(7, 32))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(25, 11, 2004), timeToMills(7, 34))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(26, 11, 2004), timeToMills(7, 35))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(27, 11, 2004), timeToMills(7, 37))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(28, 11, 2004), timeToMills(7, 38))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(29, 11, 2004), timeToMills(7, 40))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(30, 11, 2004), timeToMills(7, 41))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(1, 12, 2004), timeToMills(7, 43))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(2, 12, 2004), timeToMills(7, 44))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(3, 12, 2004), timeToMills(7, 45))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(4, 12, 2004), timeToMills(7, 47))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(5, 12, 2004), timeToMills(7, 48))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(6, 12, 2004), timeToMills(7, 49))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(7, 12, 2004), timeToMills(7, 51))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(8, 12, 2004), timeToMills(7, 52))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(9, 12, 2004), timeToMills(7, 53))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(10, 12, 2004), timeToMills(7, 54))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(11, 12, 2004), timeToMills(7, 55))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(12, 12, 2004), timeToMills(7, 56))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(13, 12, 2004), timeToMills(7, 57))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(14, 12, 2004), timeToMills(7, 58))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(15, 12, 2004), timeToMills(7, 59))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(16, 12, 2004), timeToMills(7, 59))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(17, 12, 2004), timeToMills(8, 0))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(18, 12, 2004), timeToMills(8, 1))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(19, 12, 2004), timeToMills(8, 2))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(20, 12, 2004), timeToMills(8, 2))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(21, 12, 2004), timeToMills(8, 3))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(22, 12, 2004), timeToMills(8, 3))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(23, 12, 2004), timeToMills(8, 4))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(24, 12, 2004), timeToMills(8, 4))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(25, 12, 2004), timeToMills(8, 4))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(26, 12, 2004), timeToMills(8, 5))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(27, 12, 2004), timeToMills(8, 5))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(28, 12, 2004), timeToMills(8, 5))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(29, 12, 2004), timeToMills(8, 5))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(30, 12, 2004), timeToMills(8, 5))) ;
    dataset.addData(0, new XYNumberChartData(dayToMills(31, 12, 2004), timeToMills(8, 5))) ;

    dataset.addData(1, new XYNumberChartData(dayToMills(1, 1, 2004), timeToMills(16, 0))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(2, 1, 2004), timeToMills(16, 1))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(3, 1, 2004), timeToMills(16, 2))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(4, 1, 2004), timeToMills(16, 3))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(5, 1, 2004), timeToMills(16, 4))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(6, 1, 2004), timeToMills(16, 5))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(7, 1, 2004), timeToMills(16, 7))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(8, 1, 2004), timeToMills(16, 8))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(9, 1, 2004), timeToMills(16, 9))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(10, 1, 2004), timeToMills(16, 11))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(11, 1, 2004), timeToMills(16, 12))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(12, 1, 2004), timeToMills(16, 13))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(13, 1, 2004), timeToMills(16, 15))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(14, 1, 2004), timeToMills(16, 16))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(15, 1, 2004), timeToMills(16, 18))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(16, 1, 2004), timeToMills(16, 19))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(17, 1, 2004), timeToMills(16, 21))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(18, 1, 2004), timeToMills(16, 22))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(19, 1, 2004), timeToMills(16, 24))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(20, 1, 2004), timeToMills(16, 26))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(21, 1, 2004), timeToMills(16, 27))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(22, 1, 2004), timeToMills(16, 29))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(23, 1, 2004), timeToMills(16, 31))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(24, 1, 2004), timeToMills(16, 32))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(25, 1, 2004), timeToMills(16, 34))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(26, 1, 2004), timeToMills(16, 36))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(27, 1, 2004), timeToMills(16, 38))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(28, 1, 2004), timeToMills(16, 39))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(29, 1, 2004), timeToMills(16, 41))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(30, 1, 2004), timeToMills(16, 43))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(31, 1, 2004), timeToMills(16, 45))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(1, 2, 2004), timeToMills(16, 47))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(2, 2, 2004), timeToMills(16, 48))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(3, 2, 2004), timeToMills(16, 50))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(4, 2, 2004), timeToMills(16, 52))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(5, 2, 2004), timeToMills(16, 54))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(6, 2, 2004), timeToMills(16, 56))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(7, 2, 2004), timeToMills(16, 57))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(8, 2, 2004), timeToMills(16, 59))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(9, 2, 2004), timeToMills(17, 1))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(10, 2, 2004), timeToMills(17, 3))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(11, 2, 2004), timeToMills(17, 5))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(12, 2, 2004), timeToMills(17, 7))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(13, 2, 2004), timeToMills(17, 8))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(14, 2, 2004), timeToMills(17, 10))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(15, 2, 2004), timeToMills(17, 12))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(16, 2, 2004), timeToMills(17, 14))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(17, 2, 2004), timeToMills(17, 16))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(18, 2, 2004), timeToMills(17, 17))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(19, 2, 2004), timeToMills(17, 19))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(20, 2, 2004), timeToMills(17, 21))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(21, 2, 2004), timeToMills(17, 23))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(22, 2, 2004), timeToMills(17, 25))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(23, 2, 2004), timeToMills(17, 27))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(24, 2, 2004), timeToMills(17, 28))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(25, 2, 2004), timeToMills(17, 30))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(26, 2, 2004), timeToMills(17, 32))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(27, 2, 2004), timeToMills(17, 34))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(28, 2, 2004), timeToMills(17, 35))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(29, 2, 2004), timeToMills(17, 37))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(1, 3, 2004), timeToMills(17, 39))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(2, 3, 2004), timeToMills(17, 41))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(3, 3, 2004), timeToMills(17, 43))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(4, 3, 2004), timeToMills(17, 44))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(5, 3, 2004), timeToMills(17, 46))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(6, 3, 2004), timeToMills(17, 48))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(7, 3, 2004), timeToMills(17, 50))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(8, 3, 2004), timeToMills(17, 51))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(9, 3, 2004), timeToMills(17, 53))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(10, 3, 2004), timeToMills(17, 55))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(11, 3, 2004), timeToMills(17, 56))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(12, 3, 2004), timeToMills(17, 58))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(13, 3, 2004), timeToMills(18, 0))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(14, 3, 2004), timeToMills(18, 2))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(15, 3, 2004), timeToMills(18, 3))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(16, 3, 2004), timeToMills(18, 5))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(17, 3, 2004), timeToMills(18, 7))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(18, 3, 2004), timeToMills(18, 8))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(19, 3, 2004), timeToMills(18, 10))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(20, 3, 2004), timeToMills(18, 12))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(21, 3, 2004), timeToMills(18, 13))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(22, 3, 2004), timeToMills(18, 15))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(23, 3, 2004), timeToMills(18, 17))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(24, 3, 2004), timeToMills(18, 19))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(25, 3, 2004), timeToMills(18, 20))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(26, 3, 2004), timeToMills(18, 22))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(27, 3, 2004), timeToMills(18, 24))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(28, 3, 2004), timeToMills(19, 25))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(29, 3, 2004), timeToMills(19, 27))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(30, 3, 2004), timeToMills(19, 29))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(31, 3, 2004), timeToMills(19, 30))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(1, 4, 2004), timeToMills(19, 32))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(2, 4, 2004), timeToMills(19, 34))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(3, 4, 2004), timeToMills(19, 35))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(4, 4, 2004), timeToMills(19, 37))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(5, 4, 2004), timeToMills(19, 39))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(6, 4, 2004), timeToMills(19, 40))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(7, 4, 2004), timeToMills(19, 42))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(8, 4, 2004), timeToMills(19, 44))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(9, 4, 2004), timeToMills(19, 45))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(10, 4, 2004), timeToMills(19, 47))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(11, 4, 2004), timeToMills(19, 49))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(12, 4, 2004), timeToMills(19, 50))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(13, 4, 2004), timeToMills(19, 52))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(14, 4, 2004), timeToMills(19, 54))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(15, 4, 2004), timeToMills(19, 55))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(16, 4, 2004), timeToMills(19, 57))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(17, 4, 2004), timeToMills(19, 59))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(18, 4, 2004), timeToMills(20, 0))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(19, 4, 2004), timeToMills(20, 2))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(20, 4, 2004), timeToMills(20, 4))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(21, 4, 2004), timeToMills(20, 5))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(22, 4, 2004), timeToMills(20, 7))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(23, 4, 2004), timeToMills(20, 9))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(24, 4, 2004), timeToMills(20, 10))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(25, 4, 2004), timeToMills(20, 12))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(26, 4, 2004), timeToMills(20, 14))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(27, 4, 2004), timeToMills(20, 15))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(28, 4, 2004), timeToMills(20, 17))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(29, 4, 2004), timeToMills(20, 19))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(30, 4, 2004), timeToMills(20, 20))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(1, 5, 2004), timeToMills(20, 22))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(2, 5, 2004), timeToMills(20, 24))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(3, 5, 2004), timeToMills(20, 25))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(4, 5, 2004), timeToMills(20, 27))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(5, 5, 2004), timeToMills(20, 28))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(6, 5, 2004), timeToMills(20, 30))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(7, 5, 2004), timeToMills(20, 32))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(8, 5, 2004), timeToMills(20, 33))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(9, 5, 2004), timeToMills(20, 35))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(10, 5, 2004), timeToMills(20, 36))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(11, 5, 2004), timeToMills(20, 38))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(12, 5, 2004), timeToMills(20, 40))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(13, 5, 2004), timeToMills(20, 41))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(14, 5, 2004), timeToMills(20, 43))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(15, 5, 2004), timeToMills(20, 44))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(16, 5, 2004), timeToMills(20, 46))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(17, 5, 2004), timeToMills(20, 47))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(18, 5, 2004), timeToMills(20, 49))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(19, 5, 2004), timeToMills(20, 50))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(20, 5, 2004), timeToMills(20, 51))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(21, 5, 2004), timeToMills(20, 53))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(22, 5, 2004), timeToMills(20, 54))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(23, 5, 2004), timeToMills(20, 56))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(24, 5, 2004), timeToMills(20, 57))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(25, 5, 2004), timeToMills(20, 58))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(26, 5, 2004), timeToMills(21, 0))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(27, 5, 2004), timeToMills(21, 1))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(28, 5, 2004), timeToMills(21, 2))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(29, 5, 2004), timeToMills(21, 3))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(30, 5, 2004), timeToMills(21, 4))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(31, 5, 2004), timeToMills(21, 6))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(1, 6, 2004), timeToMills(21, 7))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(2, 6, 2004), timeToMills(21, 8))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(3, 6, 2004), timeToMills(21, 9))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(4, 6, 2004), timeToMills(21, 10))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(5, 6, 2004), timeToMills(21, 11))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(6, 6, 2004), timeToMills(21, 12))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(7, 6, 2004), timeToMills(21, 13))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(8, 6, 2004), timeToMills(21, 13))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(9, 6, 2004), timeToMills(21, 14))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(10, 6, 2004), timeToMills(21, 15))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(11, 6, 2004), timeToMills(21, 16))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(12, 6, 2004), timeToMills(21, 16))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(13, 6, 2004), timeToMills(21, 17))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(14, 6, 2004), timeToMills(21, 18))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(15, 6, 2004), timeToMills(21, 18))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(16, 6, 2004), timeToMills(21, 19))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(17, 6, 2004), timeToMills(21, 19))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(18, 6, 2004), timeToMills(21, 19))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(19, 6, 2004), timeToMills(21, 20))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(20, 6, 2004), timeToMills(21, 20))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(21, 6, 2004), timeToMills(21, 20))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(22, 6, 2004), timeToMills(21, 21))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(23, 6, 2004), timeToMills(21, 21))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(24, 6, 2004), timeToMills(21, 21))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(25, 6, 2004), timeToMills(21, 21))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(26, 6, 2004), timeToMills(21, 21))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(27, 6, 2004), timeToMills(21, 21))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(28, 6, 2004), timeToMills(21, 21))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(29, 6, 2004), timeToMills(21, 21))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(30, 6, 2004), timeToMills(21, 21))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(1, 7, 2004), timeToMills(21, 20))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(2, 7, 2004), timeToMills(21, 20))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(3, 7, 2004), timeToMills(21, 19))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(4, 7, 2004), timeToMills(21, 19))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(5, 7, 2004), timeToMills(21, 18))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(6, 7, 2004), timeToMills(21, 18))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(7, 7, 2004), timeToMills(21, 17))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(8, 7, 2004), timeToMills(21, 17))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(9, 7, 2004), timeToMills(21, 16))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(10, 7, 2004), timeToMills(21, 15))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(11, 7, 2004), timeToMills(21, 14))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(12, 7, 2004), timeToMills(21, 14))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(13, 7, 2004), timeToMills(21, 13))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(14, 7, 2004), timeToMills(21, 12))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(15, 7, 2004), timeToMills(21, 11))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(16, 7, 2004), timeToMills(21, 10))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(17, 7, 2004), timeToMills(21, 9))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(18, 7, 2004), timeToMills(21, 8))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(19, 7, 2004), timeToMills(21, 7))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(20, 7, 2004), timeToMills(21, 5))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(21, 7, 2004), timeToMills(21, 4))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(22, 7, 2004), timeToMills(21, 3))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(23, 7, 2004), timeToMills(21, 2))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(24, 7, 2004), timeToMills(21, 0))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(25, 7, 2004), timeToMills(20, 59))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(26, 7, 2004), timeToMills(20, 58))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(27, 7, 2004), timeToMills(20, 56))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(28, 7, 2004), timeToMills(20, 55))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(29, 7, 2004), timeToMills(20, 53))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(30, 7, 2004), timeToMills(20, 52))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(31, 7, 2004), timeToMills(20, 50))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(1, 8, 2004), timeToMills(20, 48))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(2, 8, 2004), timeToMills(20, 47))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(3, 8, 2004), timeToMills(20, 45))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(4, 8, 2004), timeToMills(20, 43))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(5, 8, 2004), timeToMills(20, 42))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(6, 8, 2004), timeToMills(20, 40))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(7, 8, 2004), timeToMills(20, 38))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(8, 8, 2004), timeToMills(20, 36))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(9, 8, 2004), timeToMills(20, 34))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(10, 8, 2004), timeToMills(20, 33))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(11, 8, 2004), timeToMills(20, 31))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(12, 8, 2004), timeToMills(20, 29))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(13, 8, 2004), timeToMills(20, 27))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(14, 8, 2004), timeToMills(20, 25))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(15, 8, 2004), timeToMills(20, 23))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(16, 8, 2004), timeToMills(20, 21))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(17, 8, 2004), timeToMills(20, 19))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(18, 8, 2004), timeToMills(20, 17))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(19, 8, 2004), timeToMills(20, 15))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(20, 8, 2004), timeToMills(20, 13))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(21, 8, 2004), timeToMills(20, 11))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(22, 8, 2004), timeToMills(20, 9))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(23, 8, 2004), timeToMills(20, 7))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(24, 8, 2004), timeToMills(20, 4))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(25, 8, 2004), timeToMills(20, 2))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(26, 8, 2004), timeToMills(20, 0))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(27, 8, 2004), timeToMills(19, 58))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(28, 8, 2004), timeToMills(19, 56))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(29, 8, 2004), timeToMills(19, 54))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(30, 8, 2004), timeToMills(19, 51))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(31, 8, 2004), timeToMills(19, 49))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(1, 9, 2004), timeToMills(19, 47))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(2, 9, 2004), timeToMills(19, 45))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(3, 9, 2004), timeToMills(19, 42))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(4, 9, 2004), timeToMills(19, 40))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(5, 9, 2004), timeToMills(19, 38))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(6, 9, 2004), timeToMills(19, 36))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(7, 9, 2004), timeToMills(19, 33))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(8, 9, 2004), timeToMills(19, 31))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(9, 9, 2004), timeToMills(19, 29))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(10, 9, 2004), timeToMills(19, 27))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(11, 9, 2004), timeToMills(19, 24))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(12, 9, 2004), timeToMills(19, 22))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(13, 9, 2004), timeToMills(19, 20))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(14, 9, 2004), timeToMills(19, 17))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(15, 9, 2004), timeToMills(19, 15))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(16, 9, 2004), timeToMills(19, 13))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(17, 9, 2004), timeToMills(19, 11))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(18, 9, 2004), timeToMills(19, 8))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(19, 9, 2004), timeToMills(19, 6))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(20, 9, 2004), timeToMills(19, 4))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(21, 9, 2004), timeToMills(19, 1))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(22, 9, 2004), timeToMills(18, 59))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(23, 9, 2004), timeToMills(18, 57))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(24, 9, 2004), timeToMills(18, 54))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(25, 9, 2004), timeToMills(18, 52))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(26, 9, 2004), timeToMills(18, 50))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(27, 9, 2004), timeToMills(18, 47))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(28, 9, 2004), timeToMills(18, 45))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(29, 9, 2004), timeToMills(18, 43))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(30, 9, 2004), timeToMills(18, 41))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(1, 10, 2004), timeToMills(18, 38))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(2, 10, 2004), timeToMills(18, 36))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(3, 10, 2004), timeToMills(18, 34))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(4, 10, 2004), timeToMills(18, 31))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(5, 10, 2004), timeToMills(18, 29))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(6, 10, 2004), timeToMills(18, 27))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(7, 10, 2004), timeToMills(18, 25))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(8, 10, 2004), timeToMills(18, 22))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(9, 10, 2004), timeToMills(18, 20))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(10, 10, 2004), timeToMills(18, 18))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(11, 10, 2004), timeToMills(18, 16))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(12, 10, 2004), timeToMills(18, 14))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(13, 10, 2004), timeToMills(18, 11))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(14, 10, 2004), timeToMills(18, 9))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(15, 10, 2004), timeToMills(18, 7))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(16, 10, 2004), timeToMills(18, 5))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(17, 10, 2004), timeToMills(18, 3))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(18, 10, 2004), timeToMills(18, 1))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(19, 10, 2004), timeToMills(17, 59))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(20, 10, 2004), timeToMills(17, 57))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(21, 10, 2004), timeToMills(17, 55))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(22, 10, 2004), timeToMills(17, 53))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(23, 10, 2004), timeToMills(17, 50))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(24, 10, 2004), timeToMills(17, 48))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(25, 10, 2004), timeToMills(17, 46))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(26, 10, 2004), timeToMills(17, 45))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(27, 10, 2004), timeToMills(17, 43))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(28, 10, 2004), timeToMills(17, 41))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(29, 10, 2004), timeToMills(17, 39))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(30, 10, 2004), timeToMills(17, 37))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(31, 10, 2004), timeToMills(16, 35))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(1, 11, 2004), timeToMills(16, 33))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(2, 11, 2004), timeToMills(16, 31))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(3, 11, 2004), timeToMills(16, 30))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(4, 11, 2004), timeToMills(16, 28))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(5, 11, 2004), timeToMills(16, 26))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(6, 11, 2004), timeToMills(16, 24))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(7, 11, 2004), timeToMills(16, 23))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(8, 11, 2004), timeToMills(16, 21))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(9, 11, 2004), timeToMills(16, 20))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(10, 11, 2004), timeToMills(16, 18))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(11, 11, 2004), timeToMills(16, 16))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(12, 11, 2004), timeToMills(16, 15))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(13, 11, 2004), timeToMills(16, 13))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(14, 11, 2004), timeToMills(16, 12))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(15, 11, 2004), timeToMills(16, 11))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(16, 11, 2004), timeToMills(16, 9))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(17, 11, 2004), timeToMills(16, 8))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(18, 11, 2004), timeToMills(16, 7))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(19, 11, 2004), timeToMills(16, 5))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(20, 11, 2004), timeToMills(16, 4))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(21, 11, 2004), timeToMills(16, 3))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(22, 11, 2004), timeToMills(16, 2))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(23, 11, 2004), timeToMills(16, 1))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(24, 11, 2004), timeToMills(16, 0))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(25, 11, 2004), timeToMills(15, 59))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(26, 11, 2004), timeToMills(15, 58))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(27, 11, 2004), timeToMills(15, 57))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(28, 11, 2004), timeToMills(15, 56))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(29, 11, 2004), timeToMills(15, 56))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(30, 11, 2004), timeToMills(15, 55))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(1, 12, 2004), timeToMills(15, 54))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(2, 12, 2004), timeToMills(15, 54))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(3, 12, 2004), timeToMills(15, 53))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(4, 12, 2004), timeToMills(15, 53))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(5, 12, 2004), timeToMills(15, 52))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(6, 12, 2004), timeToMills(15, 52))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(7, 12, 2004), timeToMills(15, 51))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(8, 12, 2004), timeToMills(15, 51))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(9, 12, 2004), timeToMills(15, 51))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(10, 12, 2004), timeToMills(15, 51))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(11, 12, 2004), timeToMills(15, 50))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(12, 12, 2004), timeToMills(15, 50))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(13, 12, 2004), timeToMills(15, 50))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(14, 12, 2004), timeToMills(15, 50))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(15, 12, 2004), timeToMills(15, 51))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(16, 12, 2004), timeToMills(15, 51))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(17, 12, 2004), timeToMills(15, 51))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(18, 12, 2004), timeToMills(15, 51))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(19, 12, 2004), timeToMills(15, 51))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(20, 12, 2004), timeToMills(15, 52))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(21, 12, 2004), timeToMills(15, 52))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(22, 12, 2004), timeToMills(15, 53))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(23, 12, 2004), timeToMills(15, 53))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(24, 12, 2004), timeToMills(15, 54))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(25, 12, 2004), timeToMills(15, 55))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(26, 12, 2004), timeToMills(15, 55))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(27, 12, 2004), timeToMills(15, 56))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(28, 12, 2004), timeToMills(15, 57))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(29, 12, 2004), timeToMills(15, 58))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(30, 12, 2004), timeToMills(15, 59))) ;
    dataset.addData(1, new XYNumberChartData(dayToMills(31, 12, 2004), timeToMills(16, 0))) ;

    return dataset ;
  }

  private static long dayToMills(int day, int month, int year) {
    return DateUtil.dateToTimeInMillis(year, month, day) ;
  }

  private static long timeToMills(int hour, int second) {
    return DateUtil.dateToTimeInMillis(1970, 1, 1, hour, second) ;
  }

}
