package demo.xycharts ;

import java.awt.Color ;
import java.awt.Font ;
import java.awt.GridLayout ;
import java.awt.Shape ;
import java.awt.geom.Ellipse2D ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.value.XYNumberValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.data.XYNumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.render.xy.XYLineAndShapeRender ;
import com.ijchart.xychart.title.TextTitle ;
import com.ijchart.xychart.ui.supplier.DefaultDrawingSupplier ;
import com.ijchart.xychart.ui.supplier.IDrawingSupplier ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class XYLineAndShapeRenderDemo2 extends JPanel {

  public XYLineAndShapeRenderDemo2() {
    setLayout(new GridLayout(1, 1)) ;
    createPanel1() ;
  }

  public void createPanel1() {
    IJChart chart = createChart() ;
    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private IJChart createChart() {
    IMatrixDataset dataset = createDataset(1, 1.0D) ;
    IJChart chart = ChartFactory.createXYLineAndShapeChart(
      "XYLineAndShapeRender Demo 1", "X", "Y", dataset, ChartOrientation.VERTICAL,
      true, true, false, false) ;
    TextTitle texttitle = new TextTitle(
      "This chart shows various combinations of the useFillPaint and useOutlinePaint flags.") ;
    texttitle.setTextFont(new Font("Dialog", 0, 10)) ;
    chart.addSubtitle(texttitle) ;
    XYPlot plot = (XYPlot) chart.getPlot() ;

    final Ellipse2D double1 = new Ellipse2D.Double( -4D, -4D, 8D, 8D) ;
    IDrawingSupplier supplier = new DefaultDrawingSupplier() {
      public Shape getNextShape() {
        return double1 ;
      }
    } ;
    plot.setDrawingSupplier(supplier) ;

    XYNumberValueAxis domainAxis = (XYNumberValueAxis) plot.getDomainAxis() ;
    domainAxis.setLowerMargin(0.02D) ;
    domainAxis.setUpperMargin(0.02D) ;

    XYNumberValueAxis valueAxis = (XYNumberValueAxis) plot.getValueAxis() ;
    valueAxis.setAxisRangeIncludeZero(false) ;
    valueAxis.setUserDefineDatasetRange(1, 5) ;
    valueAxis.setLowerMargin(0.02D) ;
    valueAxis.setUpperMargin(0.02D) ;

    XYLineAndShapeRender render = (XYLineAndShapeRender) plot.getRender() ;
    render.setBaseItemShapeVisible(true) ;
    render.setAutoSelectBorderPaint(false) ;
    render.setAutoSelectFillPaint(false) ;
    render.setRowItemBorderPaint(0, Color.red) ;
    render.setRowItemFillPaint(0, Color.red) ;

    IMatrixDataset dataset1 = createDataset(2, 2D) ;
    XYLineAndShapeRender render1 = new XYLineAndShapeRender(dataset1) ;
    plot.setRender(1, render1) ;
    render1.setAutoSelectBorderPaint(false) ;
    render1.setAutoSelectFillPaint(false) ;
    render1.setRowItemBorderPaint(0, Color.red) ;
    render1.setRowItemFillPaint(0, Color.yellow) ;

    IMatrixDataset dataset2 = createDataset(3, 3D) ;
    XYLineAndShapeRender render2 = new XYLineAndShapeRender(dataset2) ;
    plot.setRender(2, render2) ;
    render2.setAutoSelectBorderPaint(false) ;
    render2.setAutoSelectFillPaint(false) ;
    render2.setRowItemBorderPaint(0, Color.red) ;
    render2.setRowItemFillPaint(0, Color.red) ;

    IMatrixDataset dataset3 = createDataset(4, 4D) ;
    XYLineAndShapeRender render3 = new XYLineAndShapeRender(dataset3) ;
    plot.setRender(3, render3) ;
    render3.setAutoSelectBorderPaint(false) ;
    render3.setAutoSelectFillPaint(false) ;
    render3.setRowItemBorderPaint(0, Color.red) ;
    render3.setRowItemFillPaint(0, Color.yellow) ;

    IMatrixDataset dataset4 = createDataset(5, 5D) ;
    XYLineAndShapeRender render4 = new XYLineAndShapeRender(dataset4) ;
    plot.setRender(4, render4) ;
    render4.setAutoSelectBorderPaint(false) ;
    render4.setAutoSelectFillPaint(false) ;
    render4.setRowItemBorderPaint(0, Color.red) ;
    render4.setRowItemFillPaint(0, Color.yellow) ;

    return chart ;
  }

  private static IMatrixDataset createDataset(int i, double d) {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;

    dataset.addRowData() ;
    dataset.addRowKey("Series " + i) ;
    dataset.addData(0, new XYNumberChartData(1.0D, d)) ;
    dataset.addData(0, new XYNumberChartData(2D, d)) ;
    dataset.addData(0, new XYNumberChartData(3D, d)) ;

    return dataset ;
  }

}
