package com.ijchart.xychart.annotation ;

import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;

import com.ijchart.xychart.axis.category.CategoryAxis ;
import com.ijchart.xychart.axis.value.AbstractValueAxis ;
import com.ijchart.xychart.chartinfo.AnnotationShapeCollection ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.TextAnchor ;
import com.ijchart.xychart.shape.RectangleTextChartShape ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 * <zh>{@link com.ijchart.xychart.plot.CategoryPlot}ıע(annotation).</zh>
 * <en>A text annotation that can be placed on a {@link com.ijchart.xychart.plot.CategoryPlot}.</en>
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class CategoryTextAnnotation extends AbstractTextAnnotation implements ICategoryAnnotation,
  Cloneable, Serializable {

  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 7690180863186598581L ;

  /**
   * <zh>(category).</zh>
   * <en>The category.</en>
   */
  private Object category ;

  /**
   * <zh>yֵ.</zh>
   * <en>The value.</en>
   */
  private double value ;

  /**
   * <zh>ݷ(category)yֵ(value)ȷʾעλ.</zh>
   * <en>Creates a new annotation to be displayed at the given location.</en>
   *
   * @param text String <zh>ע (Ϊnull).</zh>
   * <en>The text (null not permitted).</en>
   * @param category Object <zh>(category) (Ϊnull).</zh>
   * <en>The category (null not permitted).</en>
   * @param value double <zh>yֵ.</zh>
   * <en>The value.</en>
   */
  public CategoryTextAnnotation(String text, Object category, double value) {
    super(text) ;

    if (category == null) {
      throw new IllegalArgumentException("Null 'category' argument.") ;
    }

    this.category = category ;
    this.value = value ;
  }

  /**
   * <zh>ط(category).</zh>
   * <en>Returns the category.</en>
   *
   * @return <zh>(category) (Ϊnull).</zh>
   * <en>The category (never null).</en>
   */
  public Object getCategory() {
    return this.category ;
  }

  /**
   * <zh>÷(category),м
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent}¼.</zh>
   * <en>Sets the category and sends an
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent} to all registered listeners.</en>
   *
   * @param category Object <zh>(category) (Ϊnull).</zh>
   * <en>The category (null not permitted).</en>
   */
  public void setCategory(Object category) {
    if (category == null) {
      throw new IllegalArgumentException("Null 'category' argument.") ;
    }

    this.category = category ;
    fireAnnotationChanged() ;
  }

  /**
   * <zh>yֵ.</zh>
   * <en>Returns the value.</en>
   *
   * @return double <zh>yֵ.</zh>
   * <en>The value.</en>
   */
  public double getValue() {
    return this.value ;
  }

  /**
   * <zh>yֵ,м
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent}¼.</zh>
   * <en>Sets the value and sends an
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent} to all registered listeners.</en>
   *
   * @param value <zh>yֵ.</zh>
   * <en>The value.</en>
   */
  public void setValue(double value) {
    if (this.value != value) {
      this.value = value ;
      fireAnnotationChanged() ;
    }
  }

  /**
   * <zh>ע(annotation)ͼ.</zh>
   * <en>Draws the annotation.</en>
   *
   * @param categoryAxis CategoryAxis <zh>category (Ϊnull).</zh>
   * <en>The category axis (null not permitted).</en>
   * @param valueAxis AbstractValueAxis <zh>value (Ϊnull).</zh>
   * <en>The value axis (null not permitted).</en>
   * @param valueAxisEdge RectangleEdge <zh>valueλ (Ϊnull).</zh>
   * <en>The value axis edge (null not permitted).</en>
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param orientation ChartOrientation <zh>ͼķ (Ϊnull).</zh>
   * <en>The chart orientation (null not permitted).</en>
   * @param annotationShapes AnnotationShapeCollection <zh>ע(annotation)ͼμ (Ϊnull).</zh>
   * <en>Collects information about the annotation (null not permitted).</en>
   */
  public void drawAnnotation(CategoryAxis categoryAxis, AbstractValueAxis valueAxis,
                             RectangleEdge valueAxisEdge, Rectangle2D dataArea,
                             ChartOrientation orientation,
                             AnnotationShapeCollection annotationShapes) {
    if (categoryAxis == null) {
      throw new IllegalArgumentException("Null 'categoryAxis' argument.") ;
    }
    if (valueAxis == null) {
      throw new IllegalArgumentException("Null 'valueAxis' argument.") ;
    }
    if (valueAxisEdge == null) {
      throw new IllegalArgumentException("Null 'valueAxisEdge' argument.") ;
    }
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (annotationShapes == null) {
      throw new IllegalArgumentException("Null 'annotationShapes' argument.") ;
    }

    TextAnchor anchor = horizontalAlignToAnchor(getTextHorizontalAlign()) ;
    double xPos = categoryAxis.categoryToJava2D(this.category, anchor) ;
    if (xPos < 0) { //categoryҲ
      return ;
    }

    double yPos = valueAxis.valueToJava2D(dataArea, valueAxisEdge, this.value, false) ;
    if (yPos == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
      return ;
    }

    Rectangle2D rect = new Rectangle2D.Double() ;
    if (orientation.equals(ChartOrientation.VERTICAL)) {
      rect = new Rectangle2D.Double(xPos, yPos, 1, 1) ;
    } else if (orientation.equals(ChartOrientation.HORIZONTAL)) {
      rect = new Rectangle2D.Double(yPos, xPos, 1, 1) ;
    }

    RectangleTextChartShape annotation = new RectangleTextChartShape(rect, getText(), getTextFont()) ;
    annotation.setDrawBorder(false) ;
    annotation.setTextPaint(getTextPaint()) ;
    annotation.setFillShape(false) ;
    annotation.setRotate(getRotationAngle()) ;
    annotation.setForceShowAllText(true) ;
    annotation.setTooltipText(getTooltipText()) ;
    annotation.setForceInsideTextNoWrap(true);
    annotation.setForceInsideTextNoWrap(true) ;

    annotation.setTextHorizontalAlign(getTextHorizontalAlign()) ;
    annotation.setTextVerticalAlign(getTextVerticalAlign()) ;
    annotation.setTextPosition(getTextPosition()) ;
    annotation.setTextDirection(getTextDirection()) ;

    annotationShapes.addAnnotationShape(annotation) ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof CategoryTextAnnotation)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    CategoryTextAnnotation that = (CategoryTextAnnotation) obj ;
    if (EqualsUtil.equals(this.category, that.category)) {
      return false ;
    }
    if (this.value != that.value) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    double hashcode = super.hashCode() ;
    hashcode = 37 * hashcode + HashcodeUtil.hashCode(this.category) ;
    hashcode = 37 * hashcode + 100 * this.value ;

    return (int) hashcode ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[category=" + this.category
      + ",value=" + this.value
      + ",text=" + getText()
      + "]" ;
  }

}
