package com.ijchart.xychart.annotation ;

import java.awt.BasicStroke ;
import java.awt.Color ;
import java.awt.Paint ;
import java.awt.Stroke ;
import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;

import com.ijchart.xychart.axis.value.AbstractValueAxis ;
import com.ijchart.xychart.chartinfo.AnnotationShapeCollection ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.shape.LineChartShape ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 * <zh>{@link com.ijchart.xychart.plot.XYPlot}ıע(annotation).</zh>
 * <en>A text annotation that can be placed on a {@link com.ijchart.xychart.plot.XYPlot}.</en>
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class XYLineAnnotation extends AbstractAnnotation implements IXYAnnotation,
  Cloneable, Serializable {

  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 3311190899963418134L ;

  /**
   * <zh>ֱʹõȱʡpaint.</zh>
   * <en>The default line paint.</en>
   */
  public static final Paint DEFAULT_LINE_PAINT = Color.black ;

  /**
   * <zh>ֱʹõȱʡstroke.</zh>
   * <en>The default line stroke.</en>
   */
  public static final Stroke DEFAULT_LINE_STROKE = new BasicStroke(1.0F) ;

  /**
   * <zh>xֵ.</zh>
   * <en>The start x value.</en>
   */
  private double startX ;

  /**
   * <zh>yֵ.</zh>
   * <en>The start y value.</en>
   */
  private double startY ;

  /**
   * <zh>xֵ.</zh>
   * <en>The end x value.</en>
   */
  private double endX ;

  /**
   * <zh>յyֵ.</zh>
   * <en>The end y value.</en>
   */
  private double endY ;

  /**
   * <zh>ֱʹõpaint.</zh>
   * <en>The line paint.</en>
   */
  private Paint paint ;

  /**
   * <zh>ֱʹõstroke.</zh>
   * <en>The line stroke.</en>
   */
  private Stroke stroke ;

  /**
   * <zh>ƴ(startX,startY)յ(endX,endY)ֱ.</zh>
   * <en>Draw a line from (startX,startY) to (endX,endY).</en>
   *
   * @param startX double <zh>xֵ.</zh>
   * <en>The start x value.</en>
   * @param startY double <zh>yֵ.</zh>
   * <en>The start y value.</en>
   * @param endX double <zh>xֵ.</zh>
   * <en>The end x value.</en>
   * @param endY double <zh>յyֵ.</zh>
   * <en>The end y value.</en>
   */
  public XYLineAnnotation(double startX, double startY, double endX, double endY) {
    this(startX, startY, endX, endY, DEFAULT_LINE_PAINT, DEFAULT_LINE_STROKE) ;
  }

  /**
   * <zh>ƴ(startX,startY)յ(endX,endY)ֱ.</zh>
   * <en>Draw a line from (startX,startY) to (endX,endY).</en>
   *
   * @param startX double <zh>xֵ.</zh>
   * <en>The start x value.</en>
   * @param startY double <zh>yֵ.</zh>
   * <en>The start y value.</en>
   * @param endX double <zh>xֵ.</zh>
   * <en>The end x value.</en>
   * @param endY double <zh>յyֵ.</zh>
   * <en>The end y value.</en>
   * @param paint Paint <zh>ֱʹõpaint (Ϊnull).</zh>
   * <en>The line paint (null not permitted).</en>
   * @param stroke Stroke <zh>ֱʹõstroke (Ϊnull).</zh>
   * <en>The line stroke (null not permitted).</en>
   */
  public XYLineAnnotation(double startX, double startY, double endX, double endY,
                          Paint paint, Stroke stroke) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument.") ;
    }
    if (stroke == null) {
      throw new IllegalArgumentException("Null 'stroke' argument.") ;
    }

    this.startX = startX ;
    this.startY = startY ;

    this.endX = endX ;
    this.endY = endY ;

    this.paint = paint ;
    this.stroke = stroke ;
  }

  /**
   * <zh>xֵ.</zh>
   * <en>Returns the start x value.</en>
   *
   * @return double <zh>xֵ.</zh>
   * <en>The start x value.</en>
   */
  public double getStartX() {
    return this.startX ;
  }

  /**
   * <zh>xֵ,м
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent}¼.</zh>
   * <en>Sets the start x value and sends an
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent} to all registered listeners.</en>
   *
   * @param startX double <zh>xֵ.</zh>
   * <en>The start x value.</en>
   */
  public void setStartX(double startX) {
    if (this.startX != startX) {
      this.startX = startX ;
      fireAnnotationChanged() ;
    }
  }

  /**
   * <zh>yֵ.</zh>
   * <en>Returns the start y value.</en>
   *
   * @return double <zh>yֵ.</zh>
   * <en>The start y value.</en>
   */
  public double getStartY() {
    return this.startY ;
  }

  /**
   * <zh>yֵ,м
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent}¼.</zh>
   * <en>Sets the start y value and sends an
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent} to all registered listeners.</en>
   *
   * @param startY double <zh>yֵ.</zh>
   * <en>The start y value.</en>
   */
  public void setStartY(double startY) {
    if (this.startY != startY) {
      this.startY = startY ;
      fireAnnotationChanged() ;
    }
  }

  /**
   * <zh>յxֵ.</zh>
   * <en>Returns the end x value.</en>
   *
   * @return double <zh>յxֵ.</zh>
   * <en>The end x value.</en>
   */
  public double getEndX() {
    return this.endX ;
  }

  /**
   * <zh>յxֵ,м
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent}¼.</zh>
   * <en>Sets the end x value and sends an
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent} to all registered listeners.</en>
   *
   * @param endX double <zh>յxֵ.</zh>
   * <en>The end x value.</en>
   */
  public void setEndX(double endX) {
    if (this.endX != endX) {
      this.endX = endX ;
      fireAnnotationChanged() ;
    }
  }

  /**
   * <zh>յyֵ.</zh>
   * <en>Returns the end y value.</en>
   *
   * @return double <zh>յyֵ.</zh>
   * <en>The end y value.</en>
   */
  public double getEndY() {
    return this.endY ;
  }

  /**
   * <zh>յyֵ,м
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent}¼.</zh>
   * <en>Sets the end y value and sends an
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent} to all registered listeners.</en>
   *
   * @param endY double <zh>յyֵ.</zh>
   * <en>The end y value.</en>
   */
  public void setEndY(double endY) {
    if (this.endY != endY) {
      this.endY = endY ;
      fireAnnotationChanged() ;
    }
  }

  /**
   * <zh>ػֱʹõpaint.</zh>
   * <en>Returns the line paint.</en>
   *
   * @return Paint <zh>Paint (Ϊnull).</zh>
   * <en>The line paint (never null).</en>
   */
  public Paint getPaint() {
    return this.paint ;
  }

  /**
   * <zh>ûֱʹõpaint,м
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent}¼.</zh>
   * <en>Sets the line paint and sends an
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (null not permitted).</en>
   */
  public void setPaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument.") ;
    }

    this.paint = paint ;
    fireAnnotationChanged() ;
  }

  /**
   * <zh>ػֱʹõstroke.</zh>
   * <en>Returns the line stroke.</en>
   *
   * @return Stroke <zh>Stroke (Ϊnull).</zh>
   * <en>The line stroke (never null).</en>
   */
  public Stroke getStroke() {
    return this.stroke ;
  }

  /**
   * <zh>ûֱʹõstroke,м
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent}¼.</zh>
   * <en>Sets the line stroke and sends an
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent} to all registered listeners.</en>
   *
   * @param stroke Stroke <zh>Stroke (Ϊnull).</zh>
   * <en>The line stroke (null not permitted).</en>
   */
  public void setStroke(Stroke stroke) {
    if (stroke == null) {
      throw new IllegalArgumentException("Null 'stroke' argument.") ;
    }

    this.stroke = stroke ;
    fireAnnotationChanged() ;
  }

  /**
   * <zh>ע(annotation)ͼ.</zh>
   * <en>Draws the annotation.</en>
   *
   * @param xAxis AbstractValueAxis <zh>x (Ϊnull).</zh>
   * <en>The x-axis (null not permitted).</en>
   * @param xAxisEdge RectangleEdge <zh>xλ (Ϊnull).</zh>
   * <en>The x-axis edge (null not permitted).</en>
   * @param yAxis AbstractValueAxis <zh>y (Ϊnull).</zh>
   * <en>The y-axis (null not permitted).</en>
   * @param yAxisEdge RectangleEdge <zh>yλ (Ϊnull).</zh>
   * <en>The y-axis edge (null not permitted).</en>
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param orientation ChartOrientation <zh>ͼķ (Ϊnull).</zh>
   * <en>The chart orientation (null not permitted).</en>
   * @param annotationShapes AnnotationShapeCollection <zh>ע(annotation)ͼμ (Ϊnull).</zh>
   * <en>Collects information about the annotation (null not permitted).</en>
   */
  public void drawAnnotation(AbstractValueAxis xAxis, RectangleEdge xAxisEdge,
                             AbstractValueAxis yAxis, RectangleEdge yAxisEdge, Rectangle2D dataArea,
                             ChartOrientation orientation,
                             AnnotationShapeCollection annotationShapes) {
    if (xAxis == null) {
      throw new IllegalArgumentException("Null 'xAxis' argument.") ;
    }
    if (xAxisEdge == null) {
      throw new IllegalArgumentException("Null 'xAxisEdge' argument.") ;
    }
    if (yAxis == null) {
      throw new IllegalArgumentException("Null 'yAxis' argument.") ;
    }
    if (yAxisEdge == null) {
      throw new IllegalArgumentException("Null 'yAxisEdge' argument.") ;
    }
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (orientation == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (annotationShapes == null) {
      throw new IllegalArgumentException("Null 'annotationShapes' argument.") ;
    }

    double startXPos = xAxis.valueToJava2D(dataArea, xAxisEdge, this.startX, false) ;
    if (startXPos == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
      return ;
    }

    double startYPos = yAxis.valueToJava2D(dataArea, yAxisEdge, this.startY, false) ;
    if (startYPos == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
      return ;
    }

    double endXPos = xAxis.valueToJava2D(dataArea, xAxisEdge, this.endX, false) ;
    if (endXPos == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
      return ;
    }

    double endYPos = yAxis.valueToJava2D(dataArea, yAxisEdge, this.endY, false) ;
    if (endYPos == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
      return ;
    }

    LineChartShape lineShape = null ;
    if (orientation.equals(ChartOrientation.VERTICAL)) {
      lineShape = new LineChartShape(startXPos, startYPos, endXPos, endYPos) ;
    } else if (orientation.equals(ChartOrientation.HORIZONTAL)) {
      lineShape = new LineChartShape(startYPos, startXPos, endYPos, endXPos) ;
    }

    if (lineShape != null) {
      lineShape.setBorderPaint(this.paint) ;
      lineShape.setBorderStroke(this.stroke) ;
      lineShape.setDrawBorder(true) ;
      lineShape.setTooltipText(getTooltipText()) ;

      annotationShapes.addAnnotationShape(lineShape) ;
    }
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof XYLineAnnotation)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    XYLineAnnotation that = (XYLineAnnotation) obj ;
    if (this.startX != that.startX) {
      return false ;
    }
    if (this.startY != that.startY) {
      return false ;
    }
    if (this.endX != that.endX) {
      return false ;
    }
    if (this.endY != that.endY) {
      return false ;
    }
    if (EqualsUtil.equals(this.paint, that.paint)) {
      return false ;
    }
    if (EqualsUtil.equals(this.stroke, that.stroke)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    double hashcode = super.hashCode() ;
    hashcode = 37 * hashcode + this.startX ;
    hashcode = 37 * hashcode + this.startY ;
    hashcode = 37 * hashcode + this.endX ;
    hashcode = 37 * hashcode + this.endY ;

    hashcode = 37 * hashcode + HashcodeUtil.hashCode(this.paint) ;
    hashcode = 37 * hashcode + HashcodeUtil.hashCode(this.stroke) ;

    return (int) hashcode ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[startX=" + this.startX
      + ",startY=" + this.startY
      + ",endX=" + this.endX
      + ",endY=" + this.endY
      + ",paint=" + this.paint
      + ",stroke=" + this.stroke
      + "]" ;
  }

}
