package com.ijchart.xychart.axis.category ;

import java.awt.Graphics2D ;
import java.awt.Paint ;
import java.awt.Shape ;
import java.awt.Stroke ;
import java.awt.geom.AffineTransform ;
import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;
import java.util.ArrayList ;
import java.util.Collection ;
import java.util.Collections ;
import java.util.Hashtable ;
import java.util.List ;

import com.ijchart.xychart.axis.AbstractAxis ;
import com.ijchart.xychart.chartinfo.AxisShapeCollection ;
import com.ijchart.xychart.chartinfo.RenderShapeCollection ;
import com.ijchart.xychart.constant.AxisLabelType ;
import com.ijchart.xychart.constant.HorizontalAlignment ;
import com.ijchart.xychart.constant.TextAnchor ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.event.MarkerChangeEvent ;
import com.ijchart.xychart.format.IFormatter ;
import com.ijchart.xychart.listener.IMarkerChangeListener ;
import com.ijchart.xychart.marker.AbstractMarker ;
import com.ijchart.xychart.marker.CategoryMarker ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.shape.LineChartShape ;
import com.ijchart.xychart.shape.RectangleTextChartShape ;
import com.ijchart.xychart.ui.Graphics2DState ;
import com.ijchart.xychart.ui.Layer ;
import com.ijchart.xychart.ui.Rectangle ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.ui.RectangleInsets ;
import com.ijchart.xychart.util.AngleUtil ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.util.StringUtil ;
import com.ijchart.xychart.util.TextUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ʾֵ.</zh>
 * <en>An axis that displays categories (X-Axis).</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class CategoryAxis extends AbstractAxis implements IMarkerChangeListener, Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 8531674170929821783L ;

  /**
   * <zh>ϵбǩĿհ᳤֮ȵıֵȱʡֵ,λΪٷֱ.</zh>
   * <en>The default margin between categories (a percentage of the overall axis length).</en>
   */
  public static final double DEFAULT_CATEGORY_MARGIN = 0.10 ;

  /**
   * <zh>ȱʡˮƽ뷽ʽ.</zh>
   * <en>The default horizontal alignment. </en>
   */
  public static final HorizontalAlignment DEFAULT_HORIZONTAL_ALIGNMENT = HorizontalAlignment.CENTER ;

  /**
   * <zh>ǩȱʡʾ,Ĭȫʾ.</zh>
   * <en>The default maximum number of lines for labels (default is all).</en>
   */
  public static final int DEFAULT_MAX_CATEGORY_LABEL_LINE = Integer.MAX_VALUE ;

  /**
   * <zh>ϵбǩĿհ᳤֮ȵıֵ,λΪٷֱ.</zh>
   * <en>The amount of space reserved between categories.</en>
   */
  private double categoryMargin ;

  /**
   * <zh>ǩ</zh>
   * <en>The maximum number of lines for category labels. </en>
   */
  private int maximumCategoryLabelLines ;

  /**
   * <zh>ǩˮƽ뷽ʽ.</zh>
   * <en>The category label horizontal alignment.</en>
   */
  private HorizontalAlignment categoryLabelHorizontalAlignment ;

  /**
   * <zh>category᷽ǰǼ.</zh>
   * <en>A list containing lists of markers for the category axes.</en>
   */
  private List foregroundCategoryMarkers ;

  /**
   * <zh>category᷽ıǼ</zh>
   * <en>A list containing lists of markers for the category axes.</en>
   */
  private List backgroundCategoryMarkers ;

  /**
   * <zh>categoryǩλ.</zh>
   * <en>The position of categorys.</en>
   */
  protected Hashtable categoryLabelPosition ;

  /**
   * <zh>һϵеĿ.</zh>
   * <en>The width of a category.</en>
   */
  protected double acategoryWidth ;

  /**
   * <zh>ݼ.</zh>
   * <en>Constructs an axis using a dataset.</en>
   *
   * @param dataset IMatrixDataset <zh>άݼ (Ϊnull).</zh>
   * <en>The two dimension matrix dataset (null not permitted).</en>
   */
  public CategoryAxis(IMatrixDataset dataset) {
    this(null, dataset) ;
  }

  /**
   * <zh>ǩݼ.</zh>
   * <en>Constructs an axis using label and a dataset.</en>
   *
   * @param label <zh>ǩ.</zh>
   * <en>The axis label (null permitted).</en>
   * @param dataset IMatrixDataset <zh>άݼ (Ϊnull).</zh>
   * <en>The two dimension matrix dataset (null not permitted).</en>
   */
  public CategoryAxis(String label, IMatrixDataset dataset) {
    super(label, dataset) ;

    this.categoryMargin = DEFAULT_CATEGORY_MARGIN ;
    this.maximumCategoryLabelLines = DEFAULT_MAX_CATEGORY_LABEL_LINE ;
    this.categoryLabelHorizontalAlignment = DEFAULT_HORIZONTAL_ALIGNMENT ;

    this.foregroundCategoryMarkers = new ArrayList() ;
    this.backgroundCategoryMarkers = new ArrayList() ;
    this.categoryLabelPosition = new Hashtable() ;
    this.acategoryWidth = 0.0D ;

    setLowerMargin(0.05) ;
    setUpperMargin(0.05) ;
  }

  /**
   * <zh>ϵбǩĿհ᳤֮ȵıֵ,λΪٷֱ.</zh>
   * <en>Returns the category margin.</en>
   *
   * @return <zh>ϵбǩĿհ᳤֮ȵıֵ,λΪٷֱ.</zh>
   * <en>The margin.</en>
   */
  public double getCategoryMargin() {
    return this.categoryMargin ;
  }

  /**
   * <zh>ϵбǩĿհ᳤֮ȵıֵ,λΪٷֱ,м
   * {@link com.ijchart.xychart.event.AxisChangeEvent}¼.</zh>
   * <en>Sets the category margin and sends an {@link com.ijchart.xychart.event.AxisChangeEvent} to
   * all registered listeners.  The overall category margin is distributed over
   * N-1 gaps, where N is the number of categories on the axis.</en>
   *
   * @param margin <zh>ϵбǩĿհ᳤֮ȵıֵ(λΪٷֱ),:0.05ʾ5%</zh>
   * <en>The margin as a percentage of the axis length (for example, 0.05 is five percent).</en>
   */
  public void setCategoryMargin(double margin) {
    if (this.categoryMargin != margin) {
      this.categoryMargin = margin ;
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>رǩ.</zh>
   * <en>Returns the maximum number of lines to use for each category label.</en>
   *
   * @return <zh>ǩ.</zh>
   * <en>The maximum number of lines.</en>
   */
  public int getMaximumCategoryLabelLines() {
    return this.maximumCategoryLabelLines ;
  }

  /**
   * <zh>ñǩ,м{@link com.ijchart.xychart.event.AxisChangeEvent}¼.</zh>
   * <en>Sets the maximum number of lines to use for each category label and
   * sends an {@link com.ijchart.xychart.event.AxisChangeEvent} to all registered listeners.</en>
   *
   * @param lines <zh>ǩ</zh>
   * <en>The maximum number of lines.</en>
   */
  public void setMaximumCategoryLabelLines(int lines) {
    if (this.maximumCategoryLabelLines != lines) {
      this.maximumCategoryLabelLines = lines ;
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>رǩˮƽ뷽ʽ.</zh>
   * <en>Returns the category label horizontal alignment.</en>
   *
   * @return HorizontalAlignment <zh>ǩˮƽ뷽ʽ(Ϊnull).</zh>
   * <en>The category label horizontal alignment (never null).</en>
   */
  public HorizontalAlignment getCategoryLabelHorizontalAlignment() {
    return categoryLabelHorizontalAlignment ;
  }

  /**
   * <zh>ñǩˮƽ뷽ʽ,м
   * {@link com.ijchart.xychart.event.AxisChangeEvent}¼.</zh>
   * <en>Sets the category label horizontal alignment and sends an
   * {@link com.ijchart.xychart.event.AxisChangeEvent} to all registered listeners.</en>
   *
   * @param align HorizontalAlignment
   * <zh>ǩˮƽ뷽ʽ (Ϊnull).</zh>
   * <en>The category label horizontal alignment (null not permitted).</en>
   */
  public void setCategoryLabelHorizontalAlignment(HorizontalAlignment align) {
    if (align == null) {
      throw new IllegalArgumentException("Null 'align' argument.") ;
    }

    if (!this.categoryLabelHorizontalAlignment.equals(align)) {
      this.categoryLabelHorizontalAlignment = align ;
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>ϵĿ̶.</zh>
   * <en>Draws tick marks in a specified area and edge.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   * @param axisArea Rectangle2D <zh>ʣĿհ.</zh>
   * <en>The empty chart area.</en>
   * @param edge RectangleEdge <zh>λ.</zh>
   * <en>The axis position.</en>
   * @param shapeCollection AxisShapeCollection <zh>ͼμ.</zh>
   * <en>Collects information about the axis (null permitted).</en>
   * @param cursor <zh>λ.</zh>
   * <en>The cursor location.</en>
   *
   * @return double <zh>̶ȹλ.</zh>
   * <en>The cursor location after draw tick mark.</en>
   */
  protected double drawTickMark(Graphics2D g2, Rectangle2D axisArea, RectangleEdge edge,
                                AxisShapeCollection shapeCollection, double cursor) {
    if (!isTickMarkVisible()) {
      return cursor ;
    }

    IMatrixDataset dataset = getDataset() ;
    int tickMarkCount = getAxisLabelType().equals(AxisLabelType.ROW) ? dataset.getRowCount()
                        : dataset.getColumnCount() ;
    if (tickMarkCount == 0) {
      return cursor ;
    }

    double categoryWidth = 0 ; //ÿϵеĿ
    double gapSize = 0 ; //С
    LineChartShape shape = null ;
    String[] formatedLabels = getFormatedTickLabels(g2, axisArea, edge) ;
    if (RectangleEdge.isTopOrBottom(edge)) { //top or bottom axis
      categoryWidth = axisArea.getWidth()
                      * (1 - getLowerMargin() - getUpperMargin() - ((tickMarkCount == 1) ? 0
                                                                    : categoryMargin))
                      / tickMarkCount ;
      if (tickMarkCount > 1) {
        gapSize = axisArea.getWidth() * categoryMargin / (tickMarkCount - 1) ;
      }
      //@todoŻ,Ҫgapsize
      double startX = axisArea.getX() + axisArea.getWidth() * getLowerMargin()
                      + 0.5 * categoryWidth ;
      for (int i = 0 ; i < tickMarkCount ; i++) {
        if (!isDrawEmptyLabelTickMark() && formatedLabels != null && i < formatedLabels.length
            && StringUtil.isEmpty(formatedLabels[i])) {
          continue ;
        }

        double x = startX + i * (categoryWidth + gapSize) ;

        if (edge.equals(RectangleEdge.TOP)) {
          shape = new LineChartShape(x, cursor - getTickMarkOutsideLength(),
                                     x, cursor + getTickMarkInsideLength()) ;
        } else if (edge.equals(RectangleEdge.BOTTOM)) {
          shape = new LineChartShape(x, axisArea.getY() - getTickMarkInsideLength(),
                                     x, axisArea.getY() + getTickMarkOutsideLength()) ;
        }

        shape.setDrawBorder(true) ;
        shape.setBorderStroke(getTickMarkStroke()) ;
        shape.setBorderPaint(getTickMarkPaint()) ;
        shapeCollection.addTickMarkShape(shape) ; //һͼ
      }

      if (edge.equals(RectangleEdge.TOP)) {
        cursor -= getTickMarkOutsideLength() ;
      } else if (edge.equals(RectangleEdge.BOTTOM)) {
        cursor += getTickMarkOutsideLength() ;
      }
    } else if (RectangleEdge.isLeftOrRight(edge)) { //left or right axis
      categoryWidth = axisArea.getHeight()
                      * (1 - getLowerMargin() - getUpperMargin()
                         - ((tickMarkCount == 1) ? 0 : categoryMargin)) / tickMarkCount ;
      if (tickMarkCount > 1) {
        gapSize = axisArea.getHeight() * categoryMargin / (tickMarkCount - 1) ;
      }

      double startY = axisArea.getY() + axisArea.getHeight() * getUpperMargin()
                      + 0.5 * categoryWidth ;
      for (int i = 0 ; i < tickMarkCount ; i++) {
        if (!isDrawEmptyLabelTickMark() && formatedLabels != null && i < formatedLabels.length
            && StringUtil.isEmpty(formatedLabels[i])) {
          continue ;
        }

        double y = startY + i * (categoryWidth + gapSize) ;
        if (edge.equals(RectangleEdge.LEFT)) {
          shape = new LineChartShape(cursor - getTickMarkOutsideLength(), y,
                                     cursor + getTickMarkInsideLength(), y) ;
        } else if (edge.equals(RectangleEdge.RIGHT)) {
          shape = new LineChartShape(cursor - getTickMarkInsideLength(), y,
                                     cursor + getTickMarkOutsideLength(), y) ;
        }

        shape.setDrawBorder(true) ;
        shape.setBorderStroke(getTickMarkStroke()) ;
        shape.setBorderPaint(getTickMarkPaint()) ;
        shapeCollection.addTickMarkShape(shape) ; //һͼ
      }

      if (edge.equals(RectangleEdge.LEFT)) {
        cursor -= getTickMarkOutsideLength() ;
      } else if (edge.equals(RectangleEdge.RIGHT)) {
        cursor += getTickMarkOutsideLength() ;
      }
    }

    return cursor ;
  }

  /**
   * <zh>ϵĿ̶ֵıǩ.</zh>
   * <en>Draws tick mark labels in a specified area and edge.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   * @param plotArea Rectangle2D <zh>ͼplot.</zh>
   * <en>The area within which the plot should be drawn.</en>
   * @param axisArea Rectangle2D <zh>ʣĿհ.</zh>
   * <en>The empty chart area.</en>
   * @param edge RectangleEdge <zh>λ.</zh>
   * <en>The axis position.</en>
   * @param shapeCollection AxisShapeCollection <zh>ͼμ.</zh>
   * <en>Collects information about the axis (null permitted).</en>
   * @param cursor <zh>λ.</zh>
   * <en>The cursor location.</en>
   *
   * @return double <zh>̶ֵλ.</zh>
   * <en>The cursor location after draw tick label.</en>
   */
  protected double drawTickLabel(Graphics2D g2, Rectangle2D plotArea, Rectangle2D axisArea,
                                 RectangleEdge edge, AxisShapeCollection shapeCollection,
                                 double cursor) {
    calculateCategoryPosition(axisArea, edge) ;
    if (!isVisible() || !isTickLabelsVisible()) {
      return cursor ;
    }

    String[] keyArr = getFormatedTickLabels(null, null, null) ;
    if (StringUtil.isEmpty(keyArr, true)) {
      return cursor ;
    }

    int tickMarkCount = keyArr.length ;
    double categoryWidth = 0 ; //ÿϵеĿ
    double gapSize = 0 ; //С
    double tickLabelMaxHeight = getLabelMaxDrawWidth(g2, plotArea, edge) ;
    Rectangle2D tickLabelDrawRect = null ;

    if (RectangleEdge.isTopOrBottom(edge)) { //top or bottom axis
      if (tickMarkCount == 1) { //ֻһ̶ֵ
        categoryWidth = axisArea.getWidth() * (1 - getLowerMargin() - getUpperMargin()) / 2 ;
      } else {
        categoryWidth = axisArea.getWidth()
                        * (1 - getLowerMargin() - getUpperMargin() - categoryMargin)
                        / tickMarkCount ;
      }
      if (tickMarkCount > 1) {
        gapSize = axisArea.getWidth() * categoryMargin / (tickMarkCount - 1) ;
      }

      Rectangle2D labelRect = calculateLabelBound(g2, categoryWidth, tickLabelMaxHeight,
                                                  getTickLabelAngle(), edge, keyArr,
                                                  getTickLabelFont(), RectangleInsets.INSETS_ZERO) ;
      double startX = axisArea.getX() + axisArea.getWidth() * getLowerMargin() ;
      for (int i = 0 ; i < tickMarkCount ; i++) {
        double x = startX + i * (categoryWidth + gapSize) ;
        if (tickMarkCount <= 1) { //ֻһ̶ֵ
          x += categoryWidth / 2 ;
        }

        if (edge.equals(RectangleEdge.TOP)) {
          tickLabelDrawRect = new Rectangle2D.Double(x + labelRect.getX(), cursor + labelRect.getY(),
                                                     labelRect.getWidth(), labelRect.getHeight()) ;

          if (i == tickMarkCount - 1) { //λ
            cursor -= getTickLabelRotateBound(tickLabelDrawRect, getTickLabelAngle()).getHeight() ;
          }
        } else if (edge.equals(RectangleEdge.BOTTOM)) {
          tickLabelDrawRect = new Rectangle2D.Double(x + labelRect.getX(), cursor + labelRect.getY(),
                                                     labelRect.getWidth(), labelRect.getHeight()) ;
          if (i == tickMarkCount - 1) { //λ
            cursor += getTickLabelRotateBound(tickLabelDrawRect, getTickLabelAngle()).getHeight() ;
          }
        }

        shapeCollection.addTickLabelShape(createChartShape(tickLabelDrawRect, keyArr[i], i)) ;
      }
    } else if (RectangleEdge.isLeftOrRight(edge)) { //left or right axis
      if (tickMarkCount == 1) { //ֻһ̶ֵ
        categoryWidth = axisArea.getHeight()
                        * (1 - getLowerMargin() - getUpperMargin()) / 2 ;
      } else {
        categoryWidth = axisArea.getHeight()
                        * (1 - getLowerMargin() - getUpperMargin() - categoryMargin)
                        / tickMarkCount ;
      }

      if (tickMarkCount > 1) {
        gapSize = axisArea.getHeight() * categoryMargin / (tickMarkCount - 1) ;
      }

      Rectangle2D labelRect = calculateLabelBound(g2, categoryWidth, tickLabelMaxHeight,
                                                  getTickLabelAngle(), edge, keyArr,
                                                  getTickLabelFont(), RectangleInsets.INSETS_ZERO) ;
      double startY = axisArea.getY() + axisArea.getHeight() * getUpperMargin() ;
      for (int i = 0 ; i < tickMarkCount ; i++) {
        double y = startY + i * (categoryWidth + gapSize) ;
        if (tickMarkCount <= 1) { //ֻһ̶ֵ
          y += categoryWidth / 2 ;
        }

        if (edge.equals(RectangleEdge.LEFT)) {
          tickLabelDrawRect = new Rectangle2D.Double(cursor + labelRect.getX(), y + labelRect.getY(),
                                                     labelRect.getWidth(), labelRect.getHeight()) ;
          if (i == tickMarkCount - 1) { //λ
            cursor -= getTickLabelRotateBound(tickLabelDrawRect, getTickLabelAngle()).getWidth() ;
          }
        } else if (edge.equals(RectangleEdge.RIGHT)) {
          tickLabelDrawRect = new Rectangle2D.Double(cursor + labelRect.getX(), y + labelRect.getY(),
                                                     labelRect.getWidth(), labelRect.getHeight()) ;
          if (i == tickMarkCount - 1) { //λ
            cursor += getTickLabelRotateBound(tickLabelDrawRect, getTickLabelAngle()).getWidth() ;
          }
        }

        shapeCollection.addTickLabelShape(createChartShape(tickLabelDrawRect, keyArr[i], i)) ;
      }
    }

    return cursor ;
  }

  /**
   * <zh>ǩλ.</zh>
   * <en>Calculate the position of category labels.</en>
   *
   * @param axisArea Rectangle2D <zh>ʣĿհ.</zh>
   * <en>The empty chart area.</en>
   * @param edge RectangleEdge <zh>λ.</zh>
   * <en>The axis position.</en>
   */
  private void calculateCategoryPosition(Rectangle2D axisArea, RectangleEdge edge) {
    IMatrixDataset dataset = getDataset() ;
    this.categoryLabelPosition.clear() ;

    boolean isRow = getAxisLabelType().equals(AxisLabelType.ROW) ;
    int tickMarkCount = isRow ? dataset.getRowCount() : dataset.getColumnCount() ;
    if (tickMarkCount == 0) {
      return ;
    }

    double gapSize = 0 ; //С
    if (RectangleEdge.isTopOrBottom(edge)) { //top or bottom axis
      acategoryWidth = axisArea.getWidth()
                       * (1 - getLowerMargin() - getUpperMargin() - ((tickMarkCount == 1) ? 0
                                                                     : categoryMargin))
                       / tickMarkCount ;
      if (tickMarkCount > 1) {
        gapSize = axisArea.getWidth() * categoryMargin / (tickMarkCount - 1) ;
      }

      double startX = axisArea.getX() + axisArea.getWidth() * getLowerMargin()
                      + 0.5 * acategoryWidth ;
      for (int i = 0 ; i < tickMarkCount ; i++) {
        double x = startX + i * (acategoryWidth + gapSize) ;
        if (isRow) {
          this.categoryLabelPosition.put(StringUtil.nullToString(dataset.getRowKey(i)),
                                         new Double(x)) ;
        } else {
          this.categoryLabelPosition.put(StringUtil.nullToString(dataset.getColKey(i)),
                                         new Double(x)) ;
        }
      }
    } else if (RectangleEdge.isLeftOrRight(edge)) { //left or right axis
      acategoryWidth = axisArea.getHeight()
                       * (1 - getLowerMargin() - getUpperMargin() - ((tickMarkCount == 1) ? 0
                                                                     : categoryMargin))
                       / tickMarkCount ;
      if (tickMarkCount > 1) {
        gapSize = axisArea.getHeight() * categoryMargin / (tickMarkCount - 1) ;
      }

      double startY = axisArea.getY() + axisArea.getHeight() * getUpperMargin()
                      + 0.5 * acategoryWidth ;
      for (int i = 0 ; i < tickMarkCount ; i++) {
        double y = startY + i * (acategoryWidth + gapSize) ;
        if (isRow) {
          this.categoryLabelPosition.put(StringUtil.nullToString(dataset.getRowKey(i)),
                                         new Double(y)) ;
        } else {
          this.categoryLabelPosition.put(StringUtil.nullToString(dataset.getColKey(i)),
                                         new Double(y)) ;
        }
      }
    }
  }

  /**
   * <zh>ݷ(colKey)ֵλ.</zh>
   * <en>Calculate the position of category (colKey).</en>
   *
   * @param category Object <zh>ֵ(colKey).</zh>
   * <en>The category.</en>
   * @param anchor CategoryAnchor <zh>ֶ뷽ʽ (Ϊnull).</zh>
   * <en>The position of a text (null permitted).</en>
   *
   * @return double <zh>(colKey)ֵ,ûҵ-1.</zh>
   * <en>The coordinate,if the category not found,returns -1.</en>
   */
  public double categoryToJava2D(Object category, TextAnchor anchor) {
    Object position = this.categoryLabelPosition.get(category) ;
    if (position != null) {
      if (anchor == null) {
        return ((Double) position).doubleValue() ;
      } else if (anchor.equals(TextAnchor.START)) {
        return ((Double) position).doubleValue() - 0.5 * this.acategoryWidth ;
      } else if (anchor.equals(TextAnchor.MIDDLE)) {
        return ((Double) position).doubleValue() ;
      } else if (anchor.equals(TextAnchor.END)) {
        return ((Double) position).doubleValue() + 0.5 * this.acategoryWidth ;
      } else {
        return ((Double) position).doubleValue() ;
      }
    } else {
      return -1 ;
    }
  }

  /**
   * <zh>һRectangleTextChartShape.</zh>
   * <en>Create a RectangleTextChartShape object.</en>
   *
   * @param tickLabelDrawRect Rectangle2D <zh>̶ֵİΧ.</zh>
   * <en>The tick label bound.</en>
   * @param tickLabel String <zh>̶ֵ.</zh>
   * <en>The tick label.</en>
   * @param rowKeyIndex <zh>ݼrowKeyֵ.</zh>
   * <en>The index of rowKey in dataset. </en>
   *
   * @return IChartShape <zh>ͼζ.</zh>
   * <en>A {@link IChartShape}.</en>
   */
  protected IChartShape createChartShape(Rectangle2D tickLabelDrawRect,
                                         String tickLabel, int rowKeyIndex) {
    IMatrixDataset dataset = getDataset() ;
    Object rowKey = dataset.getRowKey(rowKeyIndex) ;

    RectangleTextChartShape shape = new RectangleTextChartShape(tickLabelDrawRect, tickLabel) ;
    shape.setRotate(AngleUtil.conversionAngle(getTickLabelAngle())) ;
    shape.setTextFont(getTickLabelFont(rowKey)) ;
    shape.setTextPaint(getTickLabelPaint(rowKey)) ;
    shape.setMargin(getTickLabelInsets()) ;
    shape.setTextMaxShowLine(this.maximumCategoryLabelLines) ;
    shape.setDrawBorder(false) ;
    shape.setTooltipText(tickLabel) ;

    shape.setTextHorizontalAlign(getTickLabelHorizontalAlign()) ;
    shape.setTextVerticalAlign(getTickLabelVerticalAlign()) ;

    return shape ;
  }

  /**
   * <zh>ϵıǩҪ,ҲɲmaximumCategoryLabelWidthRatioȷͼ
   * ĿȻ߶.</zh>
   * <en>Calculation the width of painting coordinate axis labels required by the maximum width
   * and not more than maximumCategoryLabelWidthRatio determined by the parameters of
   * the entire region of the chart width or height.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   * @param plotArea Rectangle2D <zh>ͼplot.</zh>
   * <en>The area within which the plot should be drawn.</en>
   * @param edge RectangleEdge <zh>λ.</zh>
   * <en>The axis position.</en>
   *
   * @return double <zh>.</zh>
   * <en>The width.</en>
   */
  protected double getLabelMaxDrawWidth(Graphics2D g2, Rectangle2D plotArea, RectangleEdge edge) {
    if (!isTickLabelsVisible()) {
      return 0 ;
    }

    String[] rowKeyArr = getFormatedTickLabels(null, null, null) ;
    if (StringUtil.isEmpty(rowKeyArr, true)) {
      return 0 ;
    }

    double areaWidth = 0 ;
    if (RectangleEdge.isLeftOrRight(edge)) {
      //ƱǩҪ
      double labelWidth = 0 ;
      if (isAxisLabelVisible()) {
        Rectangle2D labelRect = new Rectangle2D.Double(0, 0, plotArea.getHeight(), 0) ;
        RectangleTextChartShape shape = new RectangleTextChartShape(labelRect, getAxisLabel()) ;
        shape.setMargin(getAxisLabelInsets()) ;
        shape.setTextFont(getAxisLabelFont()) ;
        //תlabel
        Rectangle bound = shape.getMinimumBound(g2) ;
        AffineTransform transform = AffineTransform.getRotateInstance(getAxisLabelAngle()) ;
        labelWidth = transform.createTransformedShape(bound.toRectangle2D()).getBounds().getWidth() ;
      }

      //ȥlabeltick mark
      areaWidth = plotArea.getWidth() * getMaximumCategoryLabelWidthRatio() ;
      areaWidth -= labelWidth ;
      if (isTickMarkVisible()) {
        areaWidth -= getTickMarkOutsideLength() ;
      }
    } else if (RectangleEdge.isTopOrBottom(edge)) {
      //ƱǩҪ߶
      double labelHeight = 0 ;
      if (isAxisLabelVisible()) {
        Rectangle2D labelRect = new Rectangle2D.Double(0, 0, plotArea.getWidth(), 0) ;
        RectangleTextChartShape shape = new RectangleTextChartShape(labelRect, getAxisLabel()) ;
        shape.setMargin(getAxisLabelInsets()) ;
        shape.setTextFont(getAxisLabelFont()) ;
        //תlabel
        Rectangle bound = shape.getMinimumBound(g2) ;
        AffineTransform transform = AffineTransform.getRotateInstance(getAxisLabelAngle()) ;
        labelHeight = transform.createTransformedShape(bound.toRectangle2D()).getBounds().
                      getHeight() ;
      }

      //ȥlabeltick mark߶
      areaWidth = plotArea.getHeight() * getMaximumCategoryLabelWidthRatio() ;
      areaWidth -= labelHeight ;
      if (isTickMarkVisible()) {
        areaWidth -= getTickMarkOutsideLength() ;
      }
    }

    IMatrixDataset dataset = getDataset() ;
    //save old graphics state
    Graphics2DState state = new Graphics2DState(g2) ;
    state.saveOldState() ;
    //ĿȻ߶
    AffineTransform transform = AffineTransform.getRotateInstance(getTickLabelAngle()) ;
    RectangleInsets insets = getTickLabelInsets() ;
    double maxWidth = 0 ;
    for (int i = 0 ; i < rowKeyArr.length ; i++) {
      g2.setFont(getTickLabelFont(dataset.getRowKey(i))) ;
      Rectangle2D bound = TextUtil.getStringBound(g2, rowKeyArr[i]) ;
      if (insets != null) {
        insets.expandRect(bound) ;
      }

      Shape shape = transform.createTransformedShape(bound) ;
      bound = shape.getBounds() ;

      if (RectangleEdge.isLeftOrRight(edge)) {
        maxWidth = Math.max(bound.getWidth(), maxWidth) ;
      } else if (RectangleEdge.isTopOrBottom(edge)) {
        maxWidth = Math.max(bound.getHeight(), maxWidth) ;
      }

    }

    state.restoreOldState() ;

    if (maxWidth > areaWidth) {
      return areaWidth ;
    } else {
      return maxWidth ;
    }
  }

  /**
   * <zh>ظʽʾĿ̶ֵ.</zh>
   * <en>Returns the formated tick labels.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ),Ϊnull.</zh>
   * <en>The graphics device (null permitted).</en>
   * @param maxAxisArea Rectangle2D <zh>,϶СplotArea (Ϊnull).</zh>
   * <en>The maximum area within which the axis should be drawn (null permitted).</en>
   * @param edge RectangleEdge <zh>λ (Ϊnull).</zh>
   * <en>The axis position (null permitted).</en>
   *
   * @return String[] <zh>̶ֵ.</zh>
   * <en>The array of tick labels.</en>
   */
  protected String[] getFormatedTickLabels(Graphics2D g2, Rectangle2D maxAxisArea,
                                           RectangleEdge edge) {
    IMatrixDataset dataset = getDataset() ;

    if (getAxisLabelType().equals(AxisLabelType.ROW)) {
      IFormatter formatter = getRowKeyFormatter() ;
      int rowCount = dataset.getRowCount() ;
      String[] rowKeyArr = new String[rowCount] ;

      for (int i = 0 ; i < rowCount ; i++) {
        Object rowKey = dataset.getRowKey(i) ;

        if (formatter != null) {
          rowKeyArr[i] = formatter.format(rowKey, i) ;
        } else {
          if (rowKey != null) {
            rowKeyArr[i] = rowKey.toString() ;
          }
        }
      }

      return rowKeyArr ;
    } else if (getAxisLabelType().equals(AxisLabelType.COLUMN)) {
      IFormatter formatter = getColKeyFormatter() ;
      int columnCount = dataset.getColumnCount() ;
      String[] columnKeyArr = new String[columnCount] ;

      for (int i = 0 ; i < columnCount ; i++) {
        Object colKey = dataset.getColKey(i) ;

        if (formatter != null) {
          columnKeyArr[i] = formatter.format(colKey, i) ;
        } else {
          if (colKey != null) {
            columnKeyArr[i] = colKey.toString() ;
          }
        }
      }

      return columnKeyArr ;
    }

    return null ;
  }

  /**
   * <zh>һmarker,м{@link com.ijchart.xychart.event.AxisChangeEvent}¼.
   * һmarkerᴹֱcategory.</zh>
   * <en>Adds a marker for display by a particular render.
   * <P>
   * Typically a marker will be drawn by the render as a line perpendicular
   * to a category axis, however this is entirely up to the render.</en>
   *
   * @param layer Layer <zh>Ƶ˳(ǰ򱳾) (Ϊnull).</zh>
   * <en>The layer (foreground or background) (null not permitted).</en>
   * @param marker CategoryMarker <zh>CategoryMarker (Ϊnull).</zh>
   * <en>The marker (null not permitted).</en>
   */
  public void addCategoryMarker(Layer layer, CategoryMarker marker) {
    if (layer == null) {
      throw new IllegalArgumentException("Null 'layer' not permitted.") ;
    }
    if (marker == null) {
      throw new IllegalArgumentException("Null 'marker' not permitted.") ;
    }

    if (layer.equals(Layer.FOREGROUND)) {
      this.foregroundCategoryMarkers.add(marker) ;
    } else if (layer.equals(Layer.BACKGROUND)) {
      this.backgroundCategoryMarkers.add(marker) ;
    }

    marker.addChangeListener(this) ;
    fireAxisChanged() ;
  }

  /**
   * <zh>Layer󷵻еcategory marker.</zh>
   * <en>Returns a collection of category markers for specified layer.</en>
   *
   * @param layer <zh>Layer (Ϊnull).</zh>
   * <en>The layer (null not permitted).</en>
   *
   * @return Collection <zh>category marker󼯺 (Ϊnull).</zh>
   * <en>A collection of markers (possibly null).</en>
   */
  public Collection getCategoryMarkers(Layer layer) {
    if (layer == null) {
      throw new IllegalArgumentException("Null 'layer' not permitted.") ;
    }

    if (layer.equals(Layer.FOREGROUND)) {
      return Collections.unmodifiableCollection(this.foregroundCategoryMarkers) ;
    } else if (layer.equals(Layer.BACKGROUND)) {
      return Collections.unmodifiableCollection(this.backgroundCategoryMarkers) ;
    }

    return null ;
  }

  /**
   * <zh>ָLayerɾmarker.</zh>
   * <en>Removes a marker for a specific layer and sends an
   * {@link com.ijchart.xychart.event.AxisChangeEvent} to all registered listeners.</en>
   *
   * @param layer Layer <zh>layer (Ϊnull).</zh>
   * <en>The layer (foreground or background) (null not permitted).</en>
   * @param marker CategoryMarker <zh>CategoryMarker (Ϊnull).</zh>
   * <en>The marker (null not permitted).</en>
   *
   * @return <zh>Ƿɾ˴˶.</zh>
   * <en>A boolean indicating whether or not the marker was actually removed.</en>
   */
  public boolean removeCategoryMarker(Layer layer, CategoryMarker marker) {
    if (layer == null) {
      throw new IllegalArgumentException("Null 'layer' not permitted.") ;
    }
    if (marker == null) {
      throw new IllegalArgumentException("Null 'marker' not permitted.") ;
    }

    boolean removed = false ;

    if (layer.equals(Layer.FOREGROUND)) {
      removed = this.foregroundCategoryMarkers.remove(marker) ;
    } else if (layer.equals(Layer.BACKGROUND)) {
      removed = this.backgroundCategoryMarkers.remove(marker) ;
    }

    if (removed) {
      marker.removeChangeListener(this) ;
      fireAxisChanged() ;
    }

    return removed ;
  }

  /**
   * <zh>ɾеcategory marker,м{@link com.ijchart.xychart.event.AxisChangeEvent}
   * ¼.</zh>
   * <en>Clears all the category markers for the plot and sends an
   * {@link com.ijchart.xychart.event.AxisChangeEvent} to all registered listeners.</en>
   *
   * @param layer Layer <zh>Ƶ˳(ǰ򱳾) (Ϊnull).</zh>
   * <en>The layer (foreground or background) (null not permitted).</en>
   */
  public void clearCategoryMarkers(Layer layer) {
    if (layer == null) {
      throw new IllegalArgumentException("Null 'layer' not permitted.") ;
    }

    boolean notify = false ;

    if (layer.equals(Layer.FOREGROUND)) {
      for (int i = 0 ; i < this.foregroundCategoryMarkers.size() ; i++) {
        AbstractMarker marker = (AbstractMarker)this.foregroundCategoryMarkers.get(i) ;
        marker.removeChangeListener(this) ;
      }

      notify = this.foregroundCategoryMarkers.size() > 0 ;
      this.foregroundCategoryMarkers.clear() ;
    }

    if (layer.equals(Layer.BACKGROUND)) {
      for (int i = 0 ; i < this.backgroundCategoryMarkers.size() ; i++) {
        AbstractMarker marker = (AbstractMarker)this.backgroundCategoryMarkers.get(i) ;
        marker.removeChangeListener(this) ;
      }

      notify = this.backgroundCategoryMarkers.size() > 0 ;
      this.backgroundCategoryMarkers.clear() ;
    }

    if (notify) {
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>,IAxis.draw()֮.</zh>
   * <en>Draw the grid line,only call after IAxis.draw().</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   * @param dataArea Rectangle2D <zh>ͼdata() (Ϊnull).</zh>
   * <en>The area within which the data should be drawn (null not permitted).</en>
   * @param axisEdge RectangleEdge <zh>λ (Ϊnull).</zh>
   * <en>The axis position (null not permitted).</en>
   * @param shapeCollection RenderShapeCollection <zh>ͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   * @param stroke Stroke <zh>stroke (Ϊnull).</zh>
   * <en>The grid line stroke.</en>
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The grid line paint (null not permitted).</en>
   */
  public void drawGridLine(Graphics2D g2, Rectangle2D dataArea, RectangleEdge axisEdge,
                           RenderShapeCollection shapeCollection, Stroke stroke, Paint paint) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (axisEdge == null) {
      throw new IllegalArgumentException("Null 'edge' argument.") ;
    }
    if (shapeCollection == null) {
      throw new IllegalArgumentException("Null 'shapeCollection' argument.") ;
    }
    if (stroke == null) {
      throw new IllegalArgumentException("Null 'stroke' argument.") ;
    }
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument.") ;
    }

    if (!isGridLineVisible()) {
      return ;
    }

    IMatrixDataset dataset = getDataset() ;
    int tickMarkCount = getAxisLabelType().equals(AxisLabelType.ROW) ? dataset.getRowCount()
                        : dataset.getColumnCount() ;
    if (tickMarkCount == 0) {
      return ;
    }

    double categoryWidth = 0 ; //ÿϵеĿ
    double gapSize = 0 ; //С
    if (RectangleEdge.isTopOrBottom(axisEdge)) { //top or bottom axis
      categoryWidth = dataArea.getWidth()
                      * (1 - getLowerMargin() - getUpperMargin() - ((tickMarkCount == 1) ?
                                                                    0 : categoryMargin))
                      / tickMarkCount ;
      if (tickMarkCount > 1) {
        gapSize = dataArea.getWidth() * categoryMargin / (tickMarkCount - 1) ;
      }
      //@todoŻ,Ҫgapsize
      double startX = dataArea.getX() + dataArea.getWidth() * getLowerMargin()
                      + 0.5 * categoryWidth ;
      for (int i = 0 ; i < tickMarkCount ; i++) {
        double x = startX + i * (categoryWidth + gapSize) ;
        LineChartShape shape = new LineChartShape(x, dataArea.getMinY(), x, dataArea.getMaxY()) ;

        shape.setDrawBorder(true) ;
        shape.setBorderStroke(stroke) ;
        shape.setBorderPaint(paint) ;
        shapeCollection.addDomainGridLineShape(shape) ; //һͼ
      }

    } else if (RectangleEdge.isLeftOrRight(axisEdge)) { //left or right axis
      categoryWidth = dataArea.getHeight()
                      * (1 - getLowerMargin() - getUpperMargin() - ((tickMarkCount == 1) ? 0
                                                                    : categoryMargin))
                      / tickMarkCount ;
      if (tickMarkCount > 1) {
        gapSize = dataArea.getHeight() * categoryMargin / (tickMarkCount - 1) ;
      }

      double startY = dataArea.getY() + dataArea.getHeight() * getUpperMargin()
                      + 0.5 * categoryWidth ;
      for (int i = 0 ; i < tickMarkCount ; i++) {
        double y = startY + i * (categoryWidth + gapSize) ;
        LineChartShape shape = new LineChartShape(dataArea.getMinX(), y,
                                                  dataArea.getMaxX(), y) ;

        shape.setDrawBorder(true) ;
        shape.setBorderStroke(stroke) ;
        shape.setBorderPaint(paint) ;
        shapeCollection.addDomainGridLineShape(shape) ; //һͼ
      }
    }

  }

  /**
   * <zh>յmarker仯¼.</zh>
   * <en>Receives notification of an marker change event.</en>
   *
   * @param event <zh>marker仯¼.</zh>
   * <en>The marker change event.</en>
   */
  public void markerChanged(MarkerChangeEvent event) {
    fireAxisChanged() ;
  }

  /**
   * <zh>¡.</zh>
   * <en>Creates a clone of the axis.</en>
   *
   * @return A clone.
   *
   * @throws CloneNotSupportedException if some component of the axis does
   *         not support cloning.
   */
  public Object clone() throws CloneNotSupportedException {
    CategoryAxis clone = (CategoryAxis)super.clone() ;

    return clone ;
  }

  /**
   * <zh>ԱǷһ.</zh>
   * <en>Tests this axis for equality with an arbitrary object.</en>
   *
   * @param obj <zh>ҪԵĶ (Ϊnull).</zh>
   * <en>The object (null permitted).</en>
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof CategoryAxis)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    CategoryAxis that = (CategoryAxis) obj ;
    if (this.categoryMargin != that.categoryMargin) {
      return false ;
    }
    if (this.maximumCategoryLabelLines != that.maximumCategoryLabelLines) {
      return false ;
    }
    if (!EqualsUtil.equals(this.categoryLabelHorizontalAlignment,
                           that.categoryLabelHorizontalAlignment)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.foregroundCategoryMarkers, that.foregroundCategoryMarkers)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.backgroundCategoryMarkers, that.backgroundCategoryMarkers)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcode.</zh>
   * <en>Returns a hash code for this object.</en>
   *
   * @return <zh>hashcode.</zh>
   * <en>A hash code.</en>
   */
  public int hashCode() {
    int result = super.hashCode() ;
    result = 29 * result + (int) (100 * this.categoryMargin) ;
    result = 29 * result + 29 * this.maximumCategoryLabelLines ;
    result = 29 * result + HashcodeUtil.hashCode(categoryLabelHorizontalAlignment) ;
    result = 29 * result + HashcodeUtil.hashCode(this.foregroundCategoryMarkers) ;
    result = 29 * result + HashcodeUtil.hashCode(this.backgroundCategoryMarkers) ;

    return result ;
  }

}
