package com.ijchart.xychart.axis.value ;

import java.io.Serializable ;

import com.ijchart.xychart.constant.XYValueAxisType ;
import com.ijchart.xychart.data.ErrorChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.exception.InvalidChartDataException ;
import com.ijchart.xychart.range.NumberRange ;
import com.ijchart.xychart.tickunit.DateTickUnit ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>XY plotʹõֵ,ÿһ{@link ErrorChartData}.</zh>
 * <en>The error value axis used for XY plot,every data is {@link ErrorChartData}.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class ErrorValueAxis extends AbstractNumberValueAxisWithType implements Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -8410470857385527244L ;

  /**
   * <zh>ݼ.</zh>
   * <en>Constructs an axis using a dataset.</en>
   *
   * @param dataset IMatrixDataset <zh>άݼ (Ϊnull).</zh>
   * <en>The two dimension matrix dataset (null not permitted).</en>
   * @param valueAxisType XYValueAxisType <zh>value (Ϊnull).</zh>
   * <en>The type of value axis (null not permitted).</en>
   * @param isDateAxis boolean <zh>Ƿ͵.</zh>
   * <en>Whether or not is a date axis.</en>
   */
  public ErrorValueAxis(IMatrixDataset dataset, XYValueAxisType valueAxisType,
                        boolean isDateAxis) {
    this(null, dataset, valueAxisType, isDateAxis) ;
  }

  /**
   * <zh>ǩݼ.</zh>
   * <en>Constructs an axis using label and a dataset.</en>
   *
   * @param label <zh>ǩ.</zh>
   * <en>The axis label (null permitted).</en>
   * @param dataset IMatrixDataset <zh>άݼ (Ϊnull).</zh>
   * <en>The two dimension matrix dataset (null not permitted).</en>
   * @param valueAxisType XYValueAxisType <zh>value (Ϊnull).</zh>
   * <en>The type of value axis (null not permitted).</en>
   * @param isDateAxis boolean <zh>Ƿ͵.</zh>
   * <en>Whether or not is a date axis.</en>
   */
  public ErrorValueAxis(String label, IMatrixDataset dataset, XYValueAxisType valueAxisType,
                        boolean isDateAxis) {
    super(label, dataset, valueAxisType) ;

    if (isDateAxis) {
      this.tickUnit = new DateTickUnit(null) ;
    }
  }

  /**
   * <zh>õݼݷΧ.</zh>
   * <en>Returns the value range of a dataset.</en>
   *
   * @return NumberRange <zh>ݷΧ (Ϊnull).</zh>
   * <en>The value range of a dataset (never null).</en>
   */
  public NumberRange getDatasetRange() {
    if (getUserDefineDatasetRange() != null) {
      return getUserDefineDatasetRange() ;
    }

    double minValue = Double.NaN ;
    double maxValue = Double.NaN ;
    IMatrixDataset dataset = getDataset() ;

    boolean isX = false, isY = false ; //Ƿxyֵ
    if (getValueAxisType().equals(XYValueAxisType.XY_VALUE_AXIS_X)) {
      isX = true ;
    } else if (getValueAxisType().equals(XYValueAxisType.XY_VALUE_AXIS_Y)) {
      isY = true ;
    }

    if (dataset != null) {
      int rows = dataset.getRowCount() ;
      for (int i = 0 ; i < rows ; i++) {
        int currRowCols = dataset.getColumnCount(i) ;

        for (int j = 0 ; j < currRowCols ; j++) {
          Object data = dataset.getData(i, j) ;
          if (data != null) {
            if (!(data instanceof ErrorChartData)) {
              throw new InvalidChartDataException(
                "Invalid chart data type,musts be 'ErrorChartData'.") ;
            } else {
              if (isX) { //ȡxֵ
                ErrorChartData errorData = (ErrorChartData) data ;
                double x = errorData.getXValue() ;
                double xLow = errorData.getXLowValue() ;
                double xHigh = errorData.getXHighValue() ;

                minValue = getMinValue(minValue, x) ;
                minValue = getMinValue(minValue, xLow) ;
                minValue = getMinValue(minValue, xHigh) ;

                maxValue = getMaxValue(maxValue, x) ;
                maxValue = getMaxValue(maxValue, xLow) ;
                maxValue = getMaxValue(maxValue, xHigh) ;
              } else if (isY) { //ȡyֵ
                ErrorChartData errorData = (ErrorChartData) data ;
                double y = errorData.getYValue() ;
                double yLow = errorData.getYLowValue() ;
                double yHigh = errorData.getYHighValue() ;

                minValue = getMinValue(minValue, y) ;
                minValue = getMinValue(minValue, yLow) ;
                minValue = getMinValue(minValue, yHigh) ;

                maxValue = getMaxValue(maxValue, y) ;
                maxValue = getMaxValue(maxValue, yLow) ;
                maxValue = getMaxValue(maxValue, yHigh) ;
              }
            }
          }
        }
      }
    }

    if (Double.isNaN(minValue)) {
      minValue = 0 ;
    }
    if (Double.isNaN(maxValue)) {
      maxValue = 0 ;
    }

    return new NumberRange(minValue, maxValue) ;
  }

  /**
   * <zh>ȡֵеĽСֵ.</zh>
   * <en>Returns the minimum value.</en>
   *
   * @param min double
   * @param value double
   * @return double
   */
  private static double getMinValue(double min, double value) {
    if (Double.isNaN(min)) {
      if (!Double.isNaN(value)) {
        return value ;
      }
    }

    if (Double.isNaN(value)) {
      return min ;
    } else {
      return Math.min(min, value) ;
    }
  }

  /**
   * <zh>ȡֵеĽϴֵ.</zh>
   * <en>Returns the maximum value.</en>
   *
   * @param max double
   * @param value double
   * @return double
   */
  private static double getMaxValue(double max, double value) {
    if (Double.isNaN(max)) {
      if (!Double.isNaN(value)) {
        return value ;
      }
    }

    if (Double.isNaN(value)) {
      return max ;
    } else {
      return Math.max(max, value) ;
    }
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates whether some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument;
   * false otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return false ;
    }
    if (!(obj instanceof ErrorValueAxis)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    return super.hashCode() ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object. </en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "ErrorValueAxis" ;
  }

}
