package com.ijchart.xychart.axis.value ;

import java.io.Serializable ;

import com.ijchart.xychart.data.OHLCChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.dataset.SingleRowDataset ;
import com.ijchart.xychart.exception.InvalidChartDataException ;
import com.ijchart.xychart.range.NumberRange ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>OHLCֵ.</zh>
 * <en> The class for axes that display number of OHLCChartData.</en>
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class OHLCValueAxis extends NumberValueAxis implements Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -8712965106900992718L ;

  /**
   * <zh>ݼ.</zh>
   * <en>Constructs an axis using a dataset.</en>
   *
   * @param dataset SingleRowDataset <zh>άݼ (Ϊnull).</zh>
   * <en>The two dimension matrix dataset (null not permitted).</en>
   */
  public OHLCValueAxis(SingleRowDataset dataset) {
    this(null, dataset) ;
  }

  /**
   * <zh>ǩݼ.</zh>
   * <en>Constructs an axis using label and a dataset.</en>
   *
   * @param label <zh>ǩ.</zh>
   * <en>The axis label (null permitted).</en>
   * @param dataset SingleRowDataset <zh>άݼ (Ϊnull).</zh>
   * <en>The two dimension matrix dataset (null not permitted).</en>
   */
  public OHLCValueAxis(String label, SingleRowDataset dataset) {
    super(label, dataset) ;
  }

  /**
   * <zh>õݼݷΧ.</zh>
   * <en>Returns the value range of a dataset.</en>
   *
   * @return NumberRange <zh>ݷΧ (Ϊnull).</zh>
   * <en>The value range of a dataset (never null).</en>
   */
  public NumberRange getDatasetRange() {
    if (getUserDefineDatasetRange() != null) {
      return getUserDefineDatasetRange() ;
    }

    double minValue = Double.NaN ;
    double maxValue = Double.NaN ;
    IMatrixDataset dataset = getDataset() ;

    if (dataset != null) {
      int rows = dataset.getRowCount() ;

      for (int i = 0 ; i < rows ; i++) {
        int currRowCols = dataset.getColumnCount(i) ;
        for (int j = 0 ; j < currRowCols ; j++) {
          Object data = dataset.getData(i, j) ;
          if (data != null) {
            if (!(data instanceof OHLCChartData)) {
              throw new InvalidChartDataException(
                "Invalid chart data type,musts be 'OHLCChartData'.") ;
            } else {
              OHLCChartData ohlcData = (OHLCChartData) data ;
              double open = ohlcData.getOpen() ;
              double close = ohlcData.getClose() ;
              double high = ohlcData.getHigh() ;
              double low = ohlcData.getLow() ;

              minValue = getMinValue(minValue, open) ;
              minValue = getMinValue(minValue, close) ;
              minValue = getMinValue(minValue, high) ;
              minValue = getMinValue(minValue, low) ;

              maxValue = getMaxValue(maxValue, open) ;
              maxValue = getMaxValue(maxValue, close) ;
              maxValue = getMaxValue(maxValue, high) ;
              maxValue = getMaxValue(maxValue, low) ;

            }
          }
        }
      }
    }

    if (Double.isNaN(minValue)) {
      minValue = 0 ;
    }
    if (Double.isNaN(maxValue)) {
      maxValue = 0 ;
    }

    return new NumberRange(minValue, maxValue) ;
  }

  /**
   * <zh>Сֵ.</zh>
   * <en>Calculate the minimum value.</en>
   *
   * @param min double
   * @param other double
   * @return double
   */
  private static double getMinValue(double min, double other) {
    double result = Double.NaN ;
    if (Double.isNaN(min)) {
      if (!Double.isNaN(other)) {
        result = other ;
      }
    } else {
      if (!Double.isNaN(other)) {
        result = Math.min(min, other) ;
      }
    }

    return result ;
  }

  /**
   * <zh>ֵ.</zh>
   * <en>Calculate the maximum value.</en>
   *
   * @param max double
   * @param other double
   * @return double
   */
  private static double getMaxValue(double max, double other) {
    double result = Double.NaN ;
    if (Double.isNaN(max)) {
      if (!Double.isNaN(other)) {
        result = other ;
      }
    } else {
      if (!Double.isNaN(other)) {
        result = Math.max(max, other) ;
      }
    }

    return result ;
  }

  /**
   * <zh>¡.</zh>
   * <en>Creates a clone of the axis.</en>
   *
   * @return A clone.
   *
   * @throws CloneNotSupportedException if some component of the axis does
   *         not support cloning.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>ԱǷһ.</zh>
   * <en>Tests this axis for equality with an arbitrary object.</en>
   *
   * @param obj <zh>ҪԵĶ (Ϊnull).</zh>
   * <en>The object (null permitted).</en>
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof OHLCValueAxis)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcode.</zh>
   * <en>Returns a hash code for this object.</en>
   *
   * @return <zh>hashcode.</zh>
   * <en>A hash code.</en>
   */
  public int hashCode() {
    int result = super.hashCode() ;

    return result ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return A string representation of the object.
   */
  public String toString() {
    return "OHLCValueAxis" ;
  }

}
