package com.ijchart.xychart.data ;

import java.io.Serializable ;

import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>(x,y)ֵͼݶ,ⲿǲ޸ĵ.</zh>
 * <en>The (x,y) Number ChartData object,this class is immutable.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class XYNumberChartData extends AbstractChartData implements IXYChartData, Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -1702439423962943708L ;

  /**
   * <zh>xֵ.</zh>
   * <en>The x number data.</en>
   */
  private Number x ;

  /**
   * <zh>yֵ.</zh>
   * <en>The y number data.</en>
   */
  private Number y ;

  /**
   * <zh>.</zh>
   * <en>Constructor.</en>
   *
   * @param x double <zh>xֵ.</zh>
   * <en>The x value.</en>
   * @param y double <zh>yֵ.</zh>
   * <en>The y value.</en>
   */
  public XYNumberChartData(double x, double y) {
    this(new Double(x), new Double(y)) ;
  }

  /**
   * <zh>.</zh>
   * <en>Constructor.</en>
   *
   * @param x Number <zh>xֵ (Ϊnull).</zh>
   * <en>The x Number object (null not permitted).</en>
   * @param y Number <zh>yֵ (Ϊnull).</zh>
   * <en>The y Number object (null not permitted).</en>
   */
  public XYNumberChartData(Number x, Number y) {
    if (x == null) {
      throw new IllegalArgumentException("Null 'x' argument.") ;
    }
    if (y == null) {
      throw new IllegalArgumentException("Null 'y' argument.") ;
    }

    this.x = x ;
    this.y = y ;
  }

  /**
   * <zh>xֶ.</zh>
   * <en>Returns the x Number data.</en>
   *
   * @return Number <zh>Ϊnull.</zh>
   * <en>Never null.</en>
   */
  public Number getXData() {
    return this.x ;
  }

  /**
   * <zh>yֶ.</zh>
   * <en>Returns the y Number data.</en>
   *
   * @return Number <zh>Ϊnull.</zh>
   * <en>Never null.</en>
   */
  public Number getYData() {
    return this.y ;
  }

  /**
   * <zh>õdouble͵xֵ.</zh>
   * <en>Returns the double x value.</en>
   *
   * @return double <zh>double͵ֵ.</zh>
   * <en>The double value.</en>
   */
  public double getX() {
    return this.x.doubleValue() ;
  }

  /**
   * <zh>õdouble͵yֵ.</zh>
   * <en>Returns the double y value.</en>
   *
   * @return double <zh>double͵ֵ.</zh>
   * <en>The double value.</en>
   */
  public double getY() {
    return this.y.doubleValue() ;
  }

  /**
   * <zh>ʵComparableӿڵcompareTo().</zh>
   * <en>Implements the compareTo() method of interface Comparable.</en>
   *
   * @param o Object <zh>ҪȽϵĶ.</zh>
   * <en>The Object to be compared.</en>
   * @return int <zh>ֵΪʾС,0ʾ,ʾ.</zh>
   * <en>A negative integer, zero, or a positive integer as this object	is less than, equal to, or
   * greater than the specified object.</en>
   *
   * @throws ClassCastException if the specified object's type prevents it
   *         from being compared to this Object.
   */
  public int compareTo(Object o) {
    if (o == null) {
      return -1 ;
    } else {
      if (!(o instanceof XYNumberChartData)) {
        throw new ClassCastException("Must be 'XYNumberChartData'.") ;
      }

      XYNumberChartData other = (XYNumberChartData) o ;
      double diff = this.x.doubleValue() - other.getX() ;
      return (diff < 0) ? -1 : ((diff == 0) ? 0 : 1) ;
    }
  }

  /**
   * <zh>رǩַ.</zh>
   * <en>Returns the label text.</en>
   *
   * @return String <zh>ǩַ (Ϊnull).</zh>
   * <en>The label text (never null).</en>
   */
  public String getLabelText() {
    return "(" + formatNumber(this.x) + "," + formatNumber(this.y) + ")" ;
  }

  /**
   * <zh>ͼȵareaĺ.</zh>
   * <en>Returns the function arguments of hotspot area.</en>
   *
   * @return String <zh> (Ϊnull).</zh>
   * <en>The function arguments (maybe null).</en>
   */
  public String getAreaFunctionArguments() {
    return formatNumberForFunction(this.x) + "," + formatNumberForFunction(this.y) ;
  }

  /**
   * <zh>ͼȵareaurl.</zh>
   * <en>Returns the url for hotspot area.</en>
   *
   * @return String <zh>url (Ϊnull).</zh>
   * <en>The url fragment (maybe null).</en>
   */
  public String getAreaURLFragment() {
    return "x=" + formatNumber(this.x) + "&y=" + formatNumber(this.y) ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof XYNumberChartData)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    XYNumberChartData that = (XYNumberChartData) obj ;
    if (EqualsUtil.equals(this.x, that.x)) {
      return false ;
    }
    if (EqualsUtil.equals(this.y, that.y)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int result = super.hashCode() ;
    result = 37 * result + HashcodeUtil.hashCode(this.x) ;
    result = 37 * result + HashcodeUtil.hashCode(this.y) ;

    return result ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[x=" + getX() + ",y=" + getY() + "]" ;
  }

}
