package com.ijchart.xychart.event ;

import java.io.Serializable ;
import java.util.EventObject ;

import com.ijchart.xychart.dataset.IDataset ;
import com.ijchart.xychart.util.EqualsUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ʾݼı¼.</zh>
 * <en>A change event that encapsulates information about a change to a dataset.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class DatasetChangeEvent extends EventObject implements Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -380402858131648399L ;

  /**
   * <zh>ݼ仯ݼ.</zh>
   * <en>The dataset that generated the change event.</en>
   */
  private IDataset dataset ;

  /**
   * <zh>һµݼ仯</zh>
   * <en>Constructs a new dataset changed event.</en>
   *
   * @param source Object <zh>¼Դ.</zh>
   * <en>The source of the event.</en>
   * @param dataset IDataset <zh>¼ݼ (Ϊnull).</zh>
   * <en>The dataset that generated the event (null permitted).</en>
   */
  public DatasetChangeEvent(Object source, IDataset dataset) {
    super(source) ;
    if (dataset == null) {
      throw new IllegalArgumentException("Null 'dataset' argument.") ;
    }
    this.dataset = dataset ;
  }

  /**
   * <zh>ز¼ݼ,Ϊnull.</zh>
   * <en>Returns the dataset that generated the event.  Note that the dataset maybe null.</en>
   *
   * @return <zh>¼ݼ,Ϊnull.</zh>
   * <en>The dataset (possibly null).</en>
   */
  public IDataset getDataset() {
    return this.dataset ;
  }

  /**
   * Returns a hash code value for the object.
   * @return int
   */
  public int hashCode() {
    int result = source.hashCode() ;
    result = 29 * result + ((dataset == null) ? 0 : dataset.hashCode()) ;

    return result ;
  }

  /**
   * Indicates whether some other object is "equal to" this one.
   * @param obj Object
   * @return boolean
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }

    if (!(obj instanceof DatasetChangeEvent)) {
      return false ;
    }

    DatasetChangeEvent that = (DatasetChangeEvent) obj ;

    return EqualsUtil.equals(this.source, that.source)
      && EqualsUtil.equals(this.dataset, that.dataset) ;
  }

}
