package com.ijchart.xychart.format ;

import java.io.Serializable ;

import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.StringUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ʽ͵Ķ,ʽַΪ"prefix+obj.toString()+suffix".</zh>
 * <en>A Formatter provides support for formatting an arbitary Object. The result string is
 * "prefix+obj.toString()+suffix".</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class AppendFormatter implements IFormatter, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -3368183880075138172L ;

  /**
   * <zh>ǰ׺ַ.</zh>
   * <en>A prefix string.</en>
   */
  private String prefix ;

  /**
   * <zh>׺ַ.</zh>
   * <en>A prefix string.</en>
   */
  private String suffix ;

  /**
   * <zh>һµĶ.</zh>
   * <en>Create a new instance.</en>
   *
   * @param prefix String <zh>ǰ׺ַ.</zh>
   * <en>A prefix string.</en>
   * @param suffix String <zh>׺ַ.</zh>
   * <en>A suffix string.</en>
   */
  public AppendFormatter(String prefix, String suffix) {
    this.prefix = StringUtil.nullToEmpty(prefix) ;
    this.suffix = StringUtil.nullToEmpty(suffix) ;
  }

  /**
   * <zh>ǰ׺ַ.</zh>
   * <en>Returns the prefix string.</en>
   *
   * @return String <zh>ǰ׺ַ.</zh>
   * <en>A prefix string.</en>
   */
  public String getPrefix() {
    return this.prefix ;
  }

  /**
   * <zh>غ׺ַ.</zh>
   * <en>Returns the prefix string.</en>
   *
   * @return String <zh>׺ַ.</zh>
   * <en>A prefix string.</en>
   */
  public String getSuffix() {
    return this.suffix ;
  }

  /**
   * <zh>ʽһΪַ.ĳֵһϵж(ϵıǩ
   * ̶ֵ),ʽʱԸindexֵĲͬʽΪַͬ.indexʱûʹ,Ҫʹ,ɼ̳д.
   * ʽַΪ"prefix+obj.toString()+suffix".</zh>
   * <en>Formatted an arbitary object to a string. If a value is composed of a
   * series of objects (such as the coordinates of the axis labels or scale value),
   * the format could be based on index.Parameter index do not have to use,to use,
   * extends this class.The result string is "prefix+obj.toString()+suffix".</en>
   *
   * @param obj Object <zh>ҪʽĶ (Ϊnull).</zh>
   * <en>The object to be formatted (null permitted).</en>
   * @param index int <zh>һϵֵ˳,0ʼ.˲ûʹ.</zh>
   * <en>The index value of a series objects.
   * The first is at index 0, the next is at 1, and so on.This parameter is not used.</en>
   *
   * @return String <zh>ʽַ.</zh>
   * <en>A formatted Number string.</en>
   */
  public String format(Object obj, int index) {
    if (obj == null) {
      return this.prefix + suffix ;
    } else {
      return this.prefix + obj.toString() + this.suffix ;
    }
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this object for equality with an arbitrary object.</en>
   *
   * @param obj <zh>ҪȽϵһ (Ϊnull).</zh>
   * <en>The object to test against (null permitted).</en>
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }
    if (!(obj instanceof AppendFormatter)) {
      return false ;
    }

    AppendFormatter that = (AppendFormatter) obj ;
    if (!EqualsUtil.equals(this.prefix, that.prefix)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.suffix, that.suffix)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcode.</zh>
   * <en>Returns a hash code.</en>
   *
   * @return A hash code.
   */
  public int hashCode() {
    return this.prefix.hashCode() + 29 * this.suffix.hashCode() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of this Range.</en>
   *
   * @return A String.
   */
  public String toString() {
    return "[prefix=" + this.prefix + ",suffix=" + this.suffix + "]" ;
  }

}
