package com.ijchart.xychart.render.category ;

import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;

import com.ijchart.xychart.axis.category.CategoryAxis ;
import com.ijchart.xychart.axis.value.AbstractValueAxis ;
import com.ijchart.xychart.chartinfo.RenderShapeCollection ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.LegendType ;
import com.ijchart.xychart.data.IChartData ;
import com.ijchart.xychart.data.NumberChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.shape.DualBreakLineChartShape ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>Ʋ(Category Step)͵ͼ.</zh>
 * <en>Line/Step item render for an {@link com.ijchart.xychart.plot.CategoryPlot}.  This class
 *  draws lines between data points, only allowing horizontal or vertical lines (steps).</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class StepRender extends AbstractCategoryRender implements Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -117708221320750599L ;

  /**
   * <zh>ȱʡʾʱǷӼ.</zh>
   * <en>A flag that controls whether or not the steps for multiple series are staggered.</en>
   */
  public static final boolean DEFAULT_STAGGER = true ;

  /**
   * <zh>ȱʡļĿ.</zh>
   * <en>The default stagger width.</en>
   */
  public static final int DEFAULT_STAGGER_WIDTH = 5 ;

  /**
   * <zh>ʾʱǷӼ.</zh>
   * <en>A flag that controls whether or not the steps for multiple series are staggered.</en>
   */
  private boolean stagger ;

  /**
   * <zh>Ŀ.</zh>
   * <en>The stagger width.</en>
   */
  public int staggerWidth ;

  /**
   * <zh>.</zh>
   * <en>constructor.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   */
  public StepRender(IMatrixDataset dataset) {
    super(dataset) ;

    this.stagger = DEFAULT_STAGGER ;
    this.staggerWidth = DEFAULT_STAGGER_WIDTH ;
  }

  /**
   * <zh>ʾʱǷӼ.</zh>
   * <en>Returns a flag that controls whether or not the steps for multiple series are
   * staggered.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isStagger() {
    return this.stagger ;
  }

  /**
   * <zh>ʾʱǷӼ,
   * м{@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets a flag that controls whether or not the steps for multiple series are staggered and
   * sends a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param stagger boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setStagger(boolean stagger) {
    if (this.stagger != stagger) {
      this.stagger = stagger ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ؼĿ.</zh>
   * <en>Returns the stagger width.</en>
   *
   * @return int <zh>.</zh>
   * <en>The width.</en>
   */
  public int getStaggerWidth() {
    return this.staggerWidth ;
  }

  /**
   * <zh>üĿ,
   * м{@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the stagger width and sends a
   *  {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param staggerWidth int <zh>.</zh>
   * <en>The width.</en>
   */
  public void setStaggerWidth(int staggerWidth) {
    if (this.staggerWidth != staggerWidth) {
      this.staggerWidth = staggerWidth ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>еͼ.</zh>
   * <en>Draws all data items.</en>
   *
   * @param categoryAxis CategoryAxis <zh>category (Ϊnull).</zh>
   * <en>The category axis (null not permitted).</en>
   * @param valueAxis AbstractValueAxis <zh>value (Ϊnull).</zh>
   * <en>The value axis (null not permitted).</en>
   * @param valueAxisEdge RectangleEdge <zh>valueλ (Ϊnull).</zh>
   * <en>The value axis edge (null not permitted).</en>
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param orientation ChartOrientation <zh>ͼķ (Ϊnull).</zh>
   * <en>The chart orientation (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   * @param alpha float <zh>alphaֵ,ڱ[0.0,1.0].</zh>
   * <en>The alpha transparency (in the range 0.0f to 1.0f, where 0.0f is fully transparent,
   * and 1.0f is fully opaque).</en>
   */
  public void drawAllItems(CategoryAxis categoryAxis, AbstractValueAxis valueAxis,
                           RectangleEdge valueAxisEdge, Rectangle2D dataArea,
                           ChartOrientation orientation, RenderShapeCollection renderShapes,
                           float alpha) {
    if (categoryAxis == null) {
      throw new IllegalArgumentException("Null 'categoryAxis' argument.") ;
    }
    if (valueAxis == null) {
      throw new IllegalArgumentException("Null 'valueAxis' argument.") ;
    }
    if (valueAxisEdge == null) {
      throw new IllegalArgumentException("Null 'valueAxisEdge' argument.") ;
    }
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (orientation == null) {
      throw new IllegalArgumentException("Null 'orientation' argument.") ;
    }
    if (renderShapes == null) {
      throw new IllegalArgumentException("Null 'renderShapes' argument.") ;
    }
    if (alpha < 0.0f || alpha > 1.0f) {
      throw new IllegalArgumentException("The 'alpha' value must be in the range 0.0f to 1.0f.") ;
    }

    IMatrixDataset dataset = getDataset() ;
    if (dataset.isEmpty()) {
      return ;
    }
    int columns = dataset.getColumnCount() ;
    int rows = dataset.getRowCount() ;

    double basePosition = valueAxis.valueToJava2D(dataArea, valueAxisEdge, getBaseValue(), true) ;
    if (basePosition == AbstractValueAxis.INVALID_COORDINATE_VALUE) { //ֵ᷶Χ
      return ;
    }

    this.fillPaints = getFillPaints(rows) ;
    getBorderPaints(rows) ;
    this.borderPaints = fillPaints ;
    this.borderStrokes = getBorderStrokes(rows) ;

    //״ͼ
    if (orientation.equals(ChartOrientation.VERTICAL)) { //ֱ
      double axisMargin = categoryAxis.getLowerMargin() + categoryAxis.getUpperMargin() ; //еĿհ
      double barItemWidth = dataArea.getWidth() * (1 - axisMargin) / columns ; //ÿϵеĿ

      for (int i = 0 ; i < rows ; i++) {
        double startX = dataArea.getX() + dataArea.getWidth() * categoryAxis.getLowerMargin() ;
        double xStagger = 0 ;
        if (isStagger()) {
          xStagger = i * this.staggerWidth ;
          if (xStagger >= barItemWidth) {
            xStagger = 0.5 * barItemWidth ;
          }
        }
        xStagger = Math.max(0, xStagger) ;

        double nextY = Double.NaN ; //һԪصyֵ
        int currRowCols = dataset.getColumnCount(i) ;
        for (int j = 0 ; j < currRowCols ; j++) {
          IChartData data = dataset.getData(i, j) ;
          if (data != null && data instanceof NumberChartData) {
            double value = ((NumberChartData) data).getValue() ;
            double currentY = (!Double.isNaN(nextY)) ? nextY :
                              valueAxis.valueToJava2D(dataArea, valueAxisEdge, value, false) ;
            //ֵ᷶Χֵ
            if (currentY == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
              renderShapes.addDataItemShape(null) ;
              nextY = Double.NaN ;
              continue ;
            }

            DualBreakLineChartShape dualLineShape = null ;
            Rectangle2D drawArea = null ;
            double x11 = startX + j * barItemWidth + ((j == 0) ? 0 : xStagger) ;
            double x12 = x11 + barItemWidth + ((j == 0 && columns > 1) ? xStagger : 0)
                         + ((j == columns - 1 && columns > 1) ? -xStagger : 0) ;
            if (j == columns - 1) { //һԪ
              drawArea = new Rectangle2D.Double(Math.min(x11, x12), currentY,
                                                Math.abs(x12 - x11), 1) ;
              dualLineShape = new DualBreakLineChartShape(drawArea, null, null, x11, currentY, x12,
                                                          currentY, x12, currentY) ;
            } else {
              IChartData nextData = dataset.getData(i, j + 1) ;
              if (nextData != null && nextData instanceof NumberChartData) {
                double nextValue = ((NumberChartData) nextData).getValue() ;
                nextY = valueAxis.valueToJava2D(dataArea, valueAxisEdge, nextValue, false) ;
                if (nextY == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
                  drawArea = new Rectangle2D.Double(Math.min(x11, x12), currentY,
                                                    Math.abs(x12 - x11), 1) ;
                  dualLineShape = new DualBreakLineChartShape(drawArea, null, null, x11, currentY,
                                                              x12, currentY, x12, currentY) ;
                } else {
                  drawArea = new Rectangle2D.Double(Math.min(x11, x12), currentY,
                                                    Math.abs(x12 - x11), 1) ;
                  dualLineShape = new DualBreakLineChartShape(drawArea, null, null, x11, currentY,
                                                              x12, currentY, x12, nextY) ;
                }
              } else {
                nextY = Double.NaN ;
                drawArea = new Rectangle2D.Double(Math.min(x11, x12), currentY,
                                                  Math.abs(x12 - x11), 1) ;
                dualLineShape = new DualBreakLineChartShape(drawArea, null, null, x11, currentY,
                                                            x12,
                                                            currentY, x12, currentY) ;
              }
            }

            IChartShape shape = createItemShape(orientation, valueAxis, i, j, null, fillPaints[i],
                                                borderPaints[i], borderStrokes[i], alpha,
                                                dualLineShape) ;
            renderShapes.addDataItemShape(shape) ;
          } else {
            renderShapes.addDataItemShape(null) ;
          }
        }
      }
    } else if (orientation.equals(ChartOrientation.HORIZONTAL)) { //ˮƽ
      double axisMargin = categoryAxis.getLowerMargin() + categoryAxis.getUpperMargin() ; //еĿհ
      double barItemWidth = dataArea.getHeight() * (1 - axisMargin) / columns ; //ÿϵеĿ

      for (int i = 0 ; i < rows ; i++) {
        double yStagger = 0 ;
        if (isStagger()) {
          yStagger = i * this.staggerWidth ;
          if (yStagger >= barItemWidth) {
            yStagger = 0.5 * barItemWidth ;
          }
        }
        yStagger = Math.max(0, yStagger) ;

        double startY = dataArea.getY() + dataArea.getWidth() * categoryAxis.getUpperMargin() ;
        double nextX = Double.NaN ; //һԪصxֵ
        int currRowCols = dataset.getColumnCount(i) ;
        for (int j = 0 ; j < currRowCols ; j++) {
          IChartData data = dataset.getData(i, j) ;
          if (data != null && data instanceof NumberChartData) {
            double value = ((NumberChartData) data).getValue() ;
            double currentX = (!Double.isNaN(nextX)) ? nextX :
                              valueAxis.valueToJava2D(dataArea, valueAxisEdge, value, false) ;
            //ֵ᷶Χֵ
            if (currentX == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
              renderShapes.addDataItemShape(null) ;
              nextX = Double.NaN ;
              continue ;
            }

            DualBreakLineChartShape dualLineShape = null ;
            Rectangle2D drawArea = null ;
            double y11 = startY + j * barItemWidth + ((j == 0) ? 0 : yStagger) ;
            double y12 = y11 + barItemWidth + ((j == 0 && columns > 1) ? yStagger : 0)
                         + ((j == columns - 1 && columns > 1) ? -yStagger : 0) ;
            if (j == columns - 1) { //һԪ
              drawArea = new Rectangle2D.Double(currentX, y11, 1, Math.abs(y12 - y11)) ;
              dualLineShape = new DualBreakLineChartShape(drawArea, null, null, currentX, y11,
                                                          currentX,
                                                          y12, currentX, y12) ;
            } else {
              IChartData nextData = dataset.getData(i, j + 1) ;
              if (nextData != null && nextData instanceof NumberChartData) {
                double nextValue = ((NumberChartData) nextData).getValue() ;
                nextX = valueAxis.valueToJava2D(dataArea, valueAxisEdge, nextValue, false) ;
                if (nextX == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
                  drawArea = new Rectangle2D.Double(currentX, y11, 1, Math.abs(y12 - y11)) ;
                  dualLineShape = new DualBreakLineChartShape(drawArea, null, null, currentX, y11,
                                                              currentX, y12, currentX, y12) ;
                } else {
                  drawArea = new Rectangle2D.Double(currentX, y11, 1, Math.abs(y12 - y11)) ;
                  dualLineShape = new DualBreakLineChartShape(drawArea, null, null, currentX, y11,
                                                              currentX, y12, nextX, y12) ;
                }
              } else {
                nextX = Double.NaN ;
                drawArea = new Rectangle2D.Double(currentX, y11, 1, Math.abs(y12 - y11)) ;
                dualLineShape = new DualBreakLineChartShape(drawArea, null, null, currentX, y11,
                                                            currentX, y12, currentX, y12) ;
              }
            }

            IChartShape shape = createItemShape(orientation, valueAxis, i, j, null, fillPaints[i],
                                                borderPaints[i], borderStrokes[i], alpha,
                                                dualLineShape) ;
            renderShapes.addDataItemShape(shape) ;
          } else {
            renderShapes.addDataItemShape(null) ;
          }
        }
      }
    }
  }

  /**
   * <zh>ֻͼ,ͼε.</zh>
   * <en>Create a shape with no attribute specified.</en>
   *
   * @param rect Rectangle2D <zh>ͼľ (Ϊnull).</zh>
   * <en>The bar rectangle (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   * @param appendParam Object <zh>ӵĲ,ڴݶĶ (Ϊnull).</zh>
   * <en>The append parameter,used for special propose (null permitted).</en>
   *
   * @return IChartShape <zh>ͼ.</zh>
   * <en>The shape.</en>
   */
  protected IChartShape createItemShapeOnly(Rectangle2D rect, String text,
                                            Object appendParam) {
    if (appendParam == null) {
      throw new IllegalArgumentException("Null 'appendParam' argument.") ;
    }

    return (IChartShape) appendParam ;
  }

  /**
   * <zh>ͼ.</zh>
   * <en>Return the legend type.</en>
   *
   * @return LegendType <zh>ͼ.</zh>
   * <en>The legend type.</en>
   */
  public LegendType getLegendType() {
    return LegendType.ROW ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof StepRender)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    StepRender that = (StepRender) obj ;
    if (this.stagger != that.stagger) {
      return false ;
    }
    if (this.staggerWidth != that.staggerWidth) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    double result = super.hashCode() ;
    result = 37 * result + HashcodeUtil.hashCode(this.stagger) ;
    result = 37 * result + this.staggerWidth ;

    return (int) result ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "StepRender" ;
  }

}
