package com.ijchart.xychart.render.noaxis ;

import java.awt.BasicStroke ;
import java.awt.Color ;
import java.awt.Font ;
import java.awt.GradientPaint ;
import java.awt.Paint ;
import java.awt.Polygon ;
import java.awt.Stroke ;
import java.awt.geom.Arc2D ;
import java.awt.geom.Area ;
import java.awt.geom.Ellipse2D ;
import java.awt.geom.GeneralPath ;
import java.awt.geom.Rectangle2D ;
import java.io.IOException ;
import java.io.ObjectInputStream ;
import java.io.ObjectOutputStream ;
import java.io.Serializable ;
import java.text.NumberFormat ;
import java.util.ArrayList ;
import java.util.Collections ;
import java.util.List ;

import com.ijchart.xychart.chartinfo.RenderShapeCollection ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.HorizontalAlignment ;
import com.ijchart.xychart.constant.MeterShapeType ;
import com.ijchart.xychart.constant.TextPosition ;
import com.ijchart.xychart.data.StartEndNumberChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.dataset.SingleRowDataset ;
import com.ijchart.xychart.range.NumberRange ;
import com.ijchart.xychart.render.INoAxisRender ;
import com.ijchart.xychart.shape.GeneralPathChartShape ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.shape.MeterArcChartShape ;
import com.ijchart.xychart.shape.RectangleTextChartShape ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.util.SerialUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>Ǳͼ.</zh>
 * <en>Draw meter chart shapes.</en>
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class MeterRender extends AbstractNoAxisRender implements INoAxisRender, Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 3605520744974137715L ;

  /**
   * <zh>ȱʡǱ̵ͼ.</zh>
   * <en>The default dial shape type.</en>
   */
  public static final MeterShapeType DEFAULT_METER_SHAPE_TYPE = MeterShapeType.CIRCLE ;

  /**
   * <zh>ȱʡĻָpaint.</zh>
   * <en>The default needle paint.</en>
   */
  public static final Paint DEFAULT_NEEDLE_PAINT = Color.BLACK ;

  /**
   * <zh>̶ֵǷɼ.</zh>
   * <en>A flag that controls whether or not the tick labels are visible.</en>
   */
  public static final boolean DEFAULT_TICK_LABEL_VISIBLE = true ;

  /**
   * <zh>ȱʡֵ.</zh>
   * <en>The default value font.</en>
   */
  public static final Font DEFAULT_VALUE_FONT = new Font("SansSerif", Font.BOLD, 12) ;

  /**
   * <zh>ȱʡĻֵpaint.</zh>
   * <en>The default value paint.</en>
   */
  public static final Paint DEFAULT_VALUE_PAINT = Color.BLACK ;

  /**
   * <zh>ȱʡǱĽǶ.</zh>
   * <en>The default meter angle.</en>
   */
  public static final int DEFAULT_METER_ANGLE = 270 ;

  /**
   * <zh>ȱʡǱ̵ݷΧ.</zh>
   * <en>The default overall range of data values on the dial.</en>
   */
  public static final NumberRange DEFAULT_RANGE = new NumberRange(0.0, 100.0) ;

  /**
   * <zh>ȱʡĿ̶ȴС.</zh>
   * <en>The default tick size.</en>
   */
  public static final double DEFAULT_TICK_SIZE = 10.0D ;

  /**
   * <zh>ȱʡĻƿ̶ȵpaint.</zh>
   * <en>The default paint used to draw the ticks.</en>
   */
  public static final Paint DEFAULT_TICK_PAINT = Color.lightGray ;

  /**
   * <zh>ȱʡĻƿ̶ȵstroke.</zh>
   * <en>The default stroke used to draw ticks.</en>
   */
  public static final Stroke DEFAULT_TICK_STROKE = new BasicStroke(2.5F) ;

  /**
   * <zh>ȱʡʾǱϵĵλ.</zh>
   * <en>The default units displayed on the dial.</en>
   */
  public static final String DEFAULT_UNITS = "Units" ;

  /**
   * <zh>ȱʡı߿ĴС.</zh>
   * <en>The default border size.</en>
   */
  public static final float DEFAULT_BORDER_SIZE = 3F ;

  /**
   * <zh>ȱʡָԲεĴС.</zh>
   * <en>The default circle size.</en>
   */
  public static final float DEFAULT_CIRCLE_SIZE = 10F ;

  /**
   * <zh>ȱʡıǩ.</zh>
   * <en>The default label font.</en>
   */
  public static final Font DEFAULT_LABEL_FONT = new Font("SansSerif", Font.BOLD, 10) ;

  /**
   * <zh>ƿ̶ֵpaint.</zh>
   * <en>The tick label paint.</en>
   */
  public static final Paint DEFAULT_TICK_LABEL_PAINT = Color.black ;

  /**
   * <zh>ȱʡļĴС.</zh>
   * <en>The default gap size.</en>
   */
  public static final double DEFAULT_GAP_SIZE = 4.0 ;

  /**
   * <zh>ûֵָʱʾֵ.</zh>
   * <en>The needle label when needle value is null.</en>
   */
  public static final String DEFAULT_NO_NEEDLE_VALUE_LABEL = "No value" ;

  /**
   * <zh>Ǳ̵ͼ.</zh>
   * <en>The dial shape type.</en>
   */
  private MeterShapeType meterShapeType ;

  /**
   * <zh>Ǳ̵ĽǶ.</zh>
   * <en>The dial extent (measured in degrees).</en>
   */
  private double meterAngle ;

  /**
   * <zh>Ǳ̵ݷΧ.</zh>
   * <en>The overall range of data values on the dial.</en>
   */
  private NumberRange range ;

  /**
   * <zh>̶ȵĴС.</zh>
   * <en>The tick size.</en>
   */
  private double tickSize ;

  /**
   * <zh>ƿ̶ȵpaint.</zh>
   * <en>The paint used to draw the ticks.</en>
   */
  private transient Paint tickPaint ;

  /**
   * <zh>ƿ̶ȵstroke.</zh>
   * <en>The stroke used to draw ticks.</en>
   */
  private transient Stroke tickStroke ;

  /**
   * <zh>ʾǱϵĵλ.</zh>
   * <en>The units displayed on the dial.</en>
   */
  private String units ;

  /**
   * <zh>߿ĴС.</zh>
   * <en>The border size.</en>
   */
  private double borderSize ;

  /**
   * <zh>ָԲεĴС.</zh>
   * <en>The circle size.</en>
   */
  private float circleSize ;

  /**
   * <zh>ʾֵʹõ.</zh>
   * <en>The font for the value displayed in the center of the dial.</en>
   */
  private Font valueFont ;

  /**
   * <zh>ʾֵʹõpaint.</zh>
   * <en>The paint for the value displayed in the center of the dial.</en>
   */
  private transient Paint valuePaint ;

  /**
   * <zh>ָpaint.</zh>
   * <en>The paint for the needle.</en>
   */
  private transient Paint needlePaint ;

  /**
   * <zh>̶ֵǷɼ.</zh>
   * <en>A flag that controls whether or not the tick labels are visible.</en>
   */
  private boolean tickLabelVisible ;

  /**
   * <zh>ƿ̶ֵ.</zh>
   * <en>The tick label font.</en>
   */
  private Font tickLabelFont ;

  /**
   * <zh>ƿ̶ֵpaint.</zh>
   * <en>The tick label paint.</en>
   */
  private transient Paint tickLabelPaint ;

  /**
   * <zh>ʽ̶ֵĶ.</zh>
   * <en>The tick label format.</en>
   */
  private NumberFormat tickLabelFormat ;

  /**
   * <zh>ĴС.</zh>
   * <en>The gap size.</en>
   */
  private double gapSize ;

  /**
   * <zh>ֵָ.</zh>
   * <en>The needle value.</en>
   */
  private double value ;

  /**
   * <zh>ûֵָʱʾֵ.</zh>
   * <en>The needle label when needle value is null.</en>
   */
  private String noNeedleValueLabel ;

  /**
   * <zh>.</zh>
   * <en>constructor.</en>
   *
   * @param dataset SingleRowDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   */
  public MeterRender(SingleRowDataset dataset) {
    this(dataset, Double.NaN) ;
  }

  /**
   * <zh>.</zh>
   * <en>constructor.</en>
   *
   * @param dataset SingleRowDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   * @param needleValue <zh>ֵָ.</zh>
   * <en>The needle value.</en>
   */
  public MeterRender(SingleRowDataset dataset, double needleValue) {
    super(dataset) ;

    this.meterShapeType = DEFAULT_METER_SHAPE_TYPE ;
    this.meterAngle = DEFAULT_METER_ANGLE ;
    this.range = DEFAULT_RANGE ;

    this.tickSize = DEFAULT_TICK_SIZE ;
    this.tickPaint = DEFAULT_TICK_PAINT ;
    this.tickStroke = DEFAULT_TICK_STROKE ;
    this.units = DEFAULT_UNITS ;

    this.value = needleValue ;
    this.valueFont = DEFAULT_VALUE_FONT ;
    this.valuePaint = DEFAULT_VALUE_PAINT ;

    this.borderSize = DEFAULT_BORDER_SIZE ;
    this.circleSize = DEFAULT_CIRCLE_SIZE ;
    this.circleSize = DEFAULT_CIRCLE_SIZE ;

    this.needlePaint = DEFAULT_NEEDLE_PAINT ;
    this.noNeedleValueLabel = DEFAULT_NO_NEEDLE_VALUE_LABEL ;

    this.tickLabelVisible = DEFAULT_TICK_LABEL_VISIBLE ;
    this.tickLabelFont = DEFAULT_LABEL_FONT ;
    this.tickLabelPaint = DEFAULT_TICK_LABEL_PAINT ;
    this.gapSize = DEFAULT_GAP_SIZE ;
    this.tickLabelFormat = NumberFormat.getInstance() ;
    this.gapSize = DEFAULT_GAP_SIZE ;

    setBackgroundPaint(null) ;
  }

  /**
   * <zh>Ǳ̵ͼ.</zh>
   * <en>Returns the dial shape type.</en>
   *
   * @return MeterShapeType <zh>Ǳ̵ͼ (Ϊnull).</zh>
   * <en>The dial shape type (never null).</en>
   */
  public MeterShapeType getMeterShapeType() {
    return this.meterShapeType ;
  }

  /**
   * <zh>Ǳ̵ͼ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the dial shape type and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param shapeType MeterShapeType <zh>Ǳ̵ͼ (Ϊnull).</zh>
   * <en>The dial shape type (null not permitted).</en>
   */
  public void setMeterShapeType(MeterShapeType shapeType) {
    if (shapeType == null) {
      throw new IllegalArgumentException("Null 'shapeType' argument.") ;
    }

    this.meterShapeType = shapeType ;
    fireRenderChanged() ;
  }

  /**
   * <zh>Ǳ̵ĽǶ.</zh>
   * <en>Returns the dial extent (measured in degrees).</en>
   *
   * @return double <zh>Ǳ̵ĽǶ.</zh>
   * <en>The dial extent.</en>
   */
  public double getMeterAngle() {
    return this.meterAngle ;
  }

  /**
   * <zh>Ǳ̵ĽǶ,м{@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the dial extent (measured in degrees) and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param angle double <zh>Ǳ̵ĽǶ.</zh>
   * <en>The dial extent.</en>
   */
  public void setMeterAngle(double angle) {
    if (this.meterAngle != angle) {
      this.meterAngle = angle ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ؿ̶ȵĴС.</zh>
   * <en>Returns the tick size.</en>
   *
   * @return double <zh>̶ȵĴС.</zh>
   * <en>The tick size.</en>
   */
  public double getTickSize() {
    return this.tickSize ;
  }

  /**
   * <zh>ÿ̶ȵĴС,м{@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the tick size and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param tickSize double <zh>̶ȵĴС.</zh>
   * <en>The tick size.</en>
   */
  public void setTickSize(double tickSize) {
    if (this.tickSize != tickSize) {
      this.tickSize = tickSize ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ػƿ̶ȵpaint.</zh>
   * <en>Returns the paint used to draw the ticks.</en>
   *
   * @return Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  public Paint getTickPaint() {
    return this.tickPaint ;
  }

  /**
   * <zh>ûƿ̶ȵpaint,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the paint used to draw the ticks and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (null not permitted).</en>
   */
  public void setTickPaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument.") ;
    }

    this.tickPaint = paint ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ػƿ̶ȵstroke.</zh>
   * <en>Returns the stroke used to draw ticks.</en>
   *
   * @return Stroke <zh> (Ϊnull).</zh>
   * <en>The stroke (never null).</en>
   */
  public Stroke getTickStroke() {
    return this.tickStroke ;
  }

  /**
   * <zh>ûƿ̶ȵstroke,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the stroke used to draw ticks,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param stroke Stroke <zh> (Ϊnull).</zh>
   * <en>The stroke (null not permitted).</en>
   */
  public void setTickStroke(Stroke stroke) {
    if (!(this.tickStroke == null && stroke == null)) {
      this.tickStroke = stroke ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ʾǱϵĵλ.</zh>
   * <en>Returns the units displayed on the dial.</en>
   *
   * @return String <zh>λ (Ϊnull).</zh>
   * <en>The unit name (maybe null).</en>
   */
  public String getUnits() {
    return this.units ;
  }

  /**
   * <zh>ʾǱϵĵλ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the units displayed on the dial and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param units String <zh>λ (Ϊnull).</zh>
   * <en>The unit name (null permitted).</en>
   */
  public void setUnits(String units) {
    if (!EqualsUtil.equals(this.units, units)) {
      this.units = units ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ر߿ĴС.</zh>
   * <en>Returns the border size.</en>
   *
   * @return double <zh>߿С.</zh>
   * <en>The border size.</en>
   */
  public double getBorderSize() {
    return this.borderSize ;
  }

  /**
   * <zh>ñ߿ĴС,м{@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the border size and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param size double <zh>߿С.</zh>
   * <en>The border size.</en>
   */
  public void setBorderSize(double size) {
    if (this.borderSize != size) {
      this.borderSize = size ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ָԲεĴС.</zh>
   * <en>Returns the circle size.</en>
   *
   * @return float <zh>С.</zh>
   * <en>The size.</en>
   */
  public float getCircleSize() {
    return this.circleSize ;
  }

  /**
   * <zh>ָԲεĴС,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the circle size and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param size float <zh>С.</zh>
   * <en>The size.</en>
   */
  public void setCircleSize(float size) {
    if (this.circleSize != size) {
      this.circleSize = size ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ʾֵʹõ.</zh>
   * <en>Returns the font for the value displayed in the center of the dial.</en>
   *
   * @return Font <zh> (Ϊnull).</zh>
   * <en>The font (never null).</en>
   */
  public Font getValueFont() {
    return this.valueFont ;
  }

  /**
   * <zh>ʾֵʹõ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the font for the value displayed in the center of the dial and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param font Font <zh> (Ϊnull).</zh>
   * <en>The font (null not permitted).</en>
   */
  public void setValueFont(Font font) {
    if (font == null) {
      throw new IllegalArgumentException("Null 'font' argument.") ;
    }

    this.valueFont = font ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ʾֵʹõpaint.</zh>
   * <en>Returns the paint for the value displayed in the center of the dial.</en>
   *
   * @return Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  public Paint getValuePaint() {
    return this.valuePaint ;
  }

  /**
   * <zh>ʾֵʹõpaint,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the paint for the value displayed in the center of the dial and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (null not permitted).</en>
   */
  public void setValuePaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument.") ;
    }

    this.valuePaint = paint ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ֵָ.</zh>
   * <en>Returns the needle value.</en>
   *
   * @return double <zh>ֵָ.</zh>
   * <en>The needle value.</en>
   */
  public double getValue() {
    return this.value ;
  }

  /**
   * <zh>ֵָ,м{@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the needle value and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param value double <zh>ֵָ.</zh>
   * <en>The needle value.</en>
   */
  public void setValue(double value) {
    if (this.value != value) {
      this.value = value ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ػָpaint.</zh>
   * <en>Returns the paint for the needle.</en>
   *
   * @return Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  public Paint getNeedlePaint() {
    return this.needlePaint ;
  }

  /**
   * <zh>ûָpaint,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the paint for the needle and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (null not permitted).</en>
   */
  public void setNeedlePaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument.") ;
    }

    this.needlePaint = paint ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ûֵָʱʾֵ.</zh>
   * <en>Returns the needle label when needle value is null.</en>
   *
   * @return String <zh>ǩֵ (Ϊnull).</zh>
   * <en>The label (maybe null).</en>
   */
  public String getNoNeedleValueLabel() {
    return this.noNeedleValueLabel ;
  }

  /**
   * <zh>ûֵָʱʾֵ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the needle label when needle value is null and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param label String <zh>ǩֵ.</zh>
   * <en>The label.</en>
   */
  public void setNoNeedleValueLabel(String label) {
    if (!(this.noNeedleValueLabel == null && label == null)) {
      this.noNeedleValueLabel = label ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ÿ̶ֵǷɼ.</zh>
   * <en>Sets the flag that controls whether or not the tick labels are visible.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isTickLabelVisible() {
    return this.tickLabelVisible ;
  }

  /**
   * <zh>ÿ̶ֵǷɼ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the flag that controls whether or not the tick labels are visible and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setTickLabelVisible(boolean flag) {
    if (this.tickLabelVisible != flag) {
      this.tickLabelVisible = flag ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ػƿ̶ֵ.</zh>
   * <en>Returns the tick label font.</en>
   *
   * @return Font <zh> (Ϊnull).</zh>
   * <en>The font (never null).</en>
   */
  public Font getTickLabelFont() {
    return this.tickLabelFont ;
  }

  /**
   * <zh>ûƿ̶ֵ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the tick label font and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param font Font <zh> (Ϊnull).</zh>
   * <en>The font (null not permitted).</en>
   */
  public void setTickLabelFont(Font font) {
    if (font == null) {
      throw new IllegalArgumentException("Null 'font' argument.") ;
    }

    this.tickLabelFont = font ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ػƿ̶ֵpaint.</zh>
   * <en>Returns the tick label paint.</en>
   *
   * @return Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  public Paint getTickLabelPaint() {
    return this.tickLabelPaint ;
  }

  /**
   * <zh>ûƿ̶ֵpaint,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the tick label paint and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (null not permitted).</en>
   */
  public void setTickLabelPaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument.") ;
    }

    this.tickLabelPaint = paint ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ظʽ̶ֵĶ.</zh>
   * <en>Returns the tick label format.</en>
   *
   * @return NumberFormat <zh>ʽֵĶ (Ϊnull).</zh>
   * <en>The number format object (never null).</en>
   */
  public NumberFormat getTickLabelFormat() {
    return this.tickLabelFormat ;
  }

  /**
   * <zh>øʽ̶ֵĶ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the tick label format and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param format NumberFormat <zh>ʽֵĶ (Ϊnull).</zh>
   * <en>The number format object (null not permitted).</en>
   */
  public void setTickLabelFormat(NumberFormat format) {
    if (format == null) {
      throw new IllegalArgumentException("Null 'format' argument.") ;
    }

    this.tickLabelFormat = format ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ؼĴС.</zh>
   * <en>Returns the gap size.</en>
   *
   * @return double <zh>ĴС.</zh>
   * <en>The gap size.</en>
   */
  public double getGapSize() {
    return this.gapSize ;
  }

  /**
   * <zh>üĴС,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the gap size and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param size double <zh>ĴС.</zh>
   * <en>The gap size.</en>
   */
  public void setGapSize(double size) {
    if (this.gapSize != size) {
      this.gapSize = size ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>еͼ.</zh>
   * <en>Draws all data items.</en>
   *
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   * @param alpha float <zh>alphaֵ,ڱ[0.0,1.0].</zh>
   * <en>The alpha transparency (in the range 0.0f to 1.0f, where 0.0f is fully transparent,
   * and 1.0f is fully opaque).</en>
   */
  public void drawAllItems(Rectangle2D dataArea, RenderShapeCollection renderShapes, float alpha) {
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (renderShapes == null) {
      throw new IllegalArgumentException("Null 'renderShapes' argument.") ;
    }
    if (alpha < 0.0f || alpha > 1.0f) {
      throw new IllegalArgumentException("The 'alpha' value must be in the range 0.0f to 1.0f.") ;
    }

    IMatrixDataset dataset = getDataset() ;
    if (dataset.isEmpty()) {
      return ;
    }
    calculateRange() ;

    int columns = dataset.getColumnCount() ;
    Rectangle2D newArea = new Rectangle2D.Double(dataArea.getX() + gapSize,
                                                 dataArea.getY() + gapSize,
                                                 dataArea.getWidth() - 2 * gapSize,
                                                 dataArea.getHeight() - 2 * gapSize) ;

    double meterX = newArea.getX() + this.borderSize ;
    double meterY = newArea.getY() + this.borderSize ;
    double meterW = newArea.getWidth() - this.borderSize * 2 ;
    double meterH = newArea.getHeight() - this.borderSize * 2
                    + ((this.meterAngle <= 180 && !this.meterShapeType.equals(MeterShapeType.CIRCLE))
                       ? newArea.getHeight() / 1.25 : 0) ;

    double min = Math.min(meterW, meterH) / 2 ;
    meterX = (meterX + meterX + meterW) / 2 - min ;
    meterY = (meterY + meterY + meterH) / 2 - min ;
    meterW = 2 * min ;
    meterH = 2 * min ;

    Rectangle2D meterArea = new Rectangle2D.Double(meterX, meterY, meterW, meterH) ;
    double meterMiddleX = meterArea.getCenterX() ;
    double meterMiddleY = meterArea.getCenterY() ;

    this.fillPaints = getFillPaints(columns) ;
    this.borderPaints = getBorderPaints(columns) ;
    this.borderStrokes = getBorderStrokes(columns) ;

    double tickStartAngle = valueToAngle(this.range.getLowerBound()) ;
    double tickAngle = valueToAngle(this.range.getLowerBound() + this.tickSize) ;
    double tickAngleSize = tickAngle - tickStartAngle ;

    SingleRowDataset singleDataset = (SingleRowDataset) dataset ;
    int cols = singleDataset.getColumnCount() ;
    for (int i = 0 ; i < cols ; i++) {
      StartEndNumberChartData aData = (StartEndNumberChartData) singleDataset.getData(0, i) ;
      if (aData != null) {
        double start = aData.getStartValue() ;
        double end = aData.getEndValue() ;
        if (Double.isNaN(start) || Double.isNaN(end)) {
          renderShapes.addDataItemShape(null) ;
          continue ;
        }

        double startValue = Math.max(start, end) ;
        double endValue = Math.min(start, end) ;
        double startAngle = valueToAngle(startValue) ;
        double endAngle = valueToAngle(endValue) ;
        double extent = endAngle - startAngle ;

        Arc2D.Double arc = new Arc2D.Double(meterX, meterY, meterW, meterH,
                                            startAngle, extent, Arc2D.PIE) ;
        double startTickAngle = calculateArcStartTickAngle(startValue, endValue) ;
        Number[] tickLabels = calculateArcTickLables(endValue, startValue, i) ;
        MeterArcChartShape arcShape = new MeterArcChartShape(arc, startTickAngle,
                                                             Math.abs(tickAngleSize), tickLabels) ;
        arcShape.setTickPaint(this.tickPaint) ;
        arcShape.setTickStroke(this.tickStroke) ;
        arcShape.setTickLabelFont(this.tickLabelFont) ;
        arcShape.setDrawTickLabel(this.tickLabelVisible) ;
        arcShape.setTickLabelFont(this.tickLabelFont) ;
        arcShape.setTickLabelPaint(this.tickLabelPaint) ;
        arcShape.setTickLabelFormat(this.tickLabelFormat) ;
        arcShape.setBorderSize(this.borderSize) ;

        //̶ֵ
        IChartShape chartShape = createItemShape(0, i, fillPaints[i], borderPaints[i],
                                                 borderStrokes[i], alpha, arcShape) ;
        renderShapes.addDataItemShape(chartShape) ;
      } else {
        renderShapes.addDataItemShape(null) ;
      }
    }

    //ƿ̶ֵ
    String valueStr = this.noNeedleValueLabel ;
    if (!Double.isNaN(this.value)) {
      valueStr = this.tickLabelFormat.format(this.value) ;
      valueStr += " " + this.units ;
    }

    float fontHalfSize = this.valueFont.getSize() * 0.5F ;
    Rectangle2D textRect = new Rectangle2D.Double(meterMiddleX, meterMiddleY + fontHalfSize, 1, 1) ;
    RectangleTextChartShape textShape = new RectangleTextChartShape(textRect, valueStr) ;
    textShape.setDrawBorder(false) ;
    textShape.setFillShape(false) ;
    textShape.setTextPaint(this.valuePaint) ;
    textShape.setTextFont(this.valueFont) ;
    textShape.setForceInsideTextNoWrap(true) ;
    textShape.setTextHorizontalAlign(HorizontalAlignment.CENTER) ;
    textShape.setTextPosition(TextPosition.OUTSIDE_BOTTOM) ;
    renderShapes.setMeterUnitShape(textShape) ;

    //ָͼ
    if (!Double.isNaN(this.value) && this.range.contains(value)) {
      double radius = meterArea.getWidth() / 2 + this.borderSize + 10 ;
      double valueAngle = valueToAngle(this.value) ;
      double valueP1 = meterMiddleX + radius * Math.cos(Math.PI * valueAngle / 180) ;
      double valueP2 = meterMiddleY - radius * Math.sin(Math.PI * valueAngle / 180) ;

      Polygon arrow = new Polygon() ;
      if ((valueAngle > 135 && valueAngle < 225) || (valueAngle < 45 && valueAngle > -45)) {
        double valueP3 = meterMiddleY - this.circleSize / 4 ;
        double valueP4 = meterMiddleY + this.circleSize / 4 ;
        arrow.addPoint((int) meterMiddleX, (int) valueP3) ;
        arrow.addPoint((int) meterMiddleX, (int) valueP4) ;

      } else {
        arrow.addPoint((int) (meterMiddleX - this.circleSize / 4), (int) meterMiddleY) ;
        arrow.addPoint((int) (meterMiddleX + this.circleSize / 4), (int) meterMiddleY) ;
      }
      arrow.addPoint((int) valueP1, (int) valueP2) ;

      Ellipse2D circle = new Ellipse2D.Double(meterMiddleX - this.circleSize / 2,
                                              meterMiddleY - this.circleSize / 2,
                                              this.circleSize, this.circleSize) ;

      GeneralPath arrowShape = new GeneralPath(arrow) ;
      arrowShape.append(circle, false) ;
      Area areaArrow = new Area(arrow) ;
      areaArrow.intersect(new Area(circle)) ;
      arrowShape.append(areaArrow, false) ;

      GeneralPathChartShape pathShape = new GeneralPathChartShape(arrowShape) ;
      pathShape.setDrawBorder(false) ;
      pathShape.setFillShape(true) ;
      pathShape.setFillAlpha(alpha) ;
      pathShape.setFillPaint(this.needlePaint) ;
      renderShapes.setMeterNeedleShape(pathShape) ;
    }

  }

  /**
   * <zh>εʼǶ.</zh>
   * <en>Calculate the first tick angle of arc.</en>
   *
   * @param start double <zh>ʼֵ.</zh>
   * <en>The start value.</en>
   * @param end double <zh>ֵ.</zh>
   * <en>The end value.</en>
   *
   * @return double <zh>Ƕ.</zh>
   * <en>The angle.</en>
   */
  protected double calculateArcStartTickAngle(double start, double end) {
    double reside = start % this.tickSize ;
    double tickStart = start ; //̶ȵĿʼֵ
    if (reside != 0) {
      tickStart = ((int) (start / this.tickSize) + 1) * this.tickSize ;
    }

    return valueToAngle(tickStart) ;
  }

  /**
   * <zh>εıǩ.</zh>
   * <en>Calculate the tick labels of arc.</en>
   * @param start double <zh>ʼֵ.</zh>
   * <en>The start value.</en>
   * @param end double <zh>ֵ.</zh>
   * <en>The end value.</en>
   * @param index int <zh>εĸ.</zh>
   * <en>The index of arc.</en>
   *
   * @return Number[] <zh>ǩֵ.</zh>
   * <en>The tick labels.</en>
   */
  protected Number[] calculateArcTickLables(double start, double end, int index) {
    List tickLabels = new ArrayList() ;
    double reside = start % this.tickSize ;
    double tickStart = start ; //̶ȵĿʼֵ
    if (reside != 0) {
      tickStart = ((int) (start / this.tickSize) + 1) * this.tickSize ;
    }

    double rangeEndValue = this.range.getUpperBound() ;
    while (tickStart <= end && tickStart <= rangeEndValue) {
      tickLabels.add(new Double(tickStart)) ;
      tickStart += this.tickSize ;
    }

    //ȥ˵һһԪ
    int size = tickLabels.size() ;
    if (size > 0) {
      if (index == 0) {
        for (int i = 1 ; i < size - 1 ; i++) {
          tickLabels.set(i, null) ;
        }
      } else {
        for (int i = 0 ; i < size - 1 ; i++) {
          tickLabels.set(i, null) ;
        }

      }
    }

    Collections.reverse(tickLabels) ;

    return (Number[]) tickLabels.toArray(new Number[0]) ;
  }

  /**
   * <zh>ֻͼ,ͼε.</zh>
   * <en>Create a shape with no attribute specified.</en>
   *
   * @param rect Rectangle2D <zh>ͼľ (Ϊnull).</zh>
   * <en>The bar rectangle (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   * @param appendParam Object <zh>ӵĲ,ڴݶĶ (Ϊnull).</zh>
   * <en>The append parameter,used for special propose (null permitted).</en>
   *
   * @return IChartShape <zh>ͼ.</zh>
   * <en>The shape.</en>
   */
  protected IChartShape createItemShapeOnly(Rectangle2D rect, String text,
                                            Object appendParam) {
    if (appendParam == null) {
      throw new IllegalArgumentException("Null 'appendParam' argument.") ;
    }

    return (IChartShape) appendParam ;
  }

  /**
   * <zh>Ǳ̵ݷΧ.</zh>
   * <en>Calculate the range of data values on the dial.</en>
   */
  protected void calculateRange() {
    SingleRowDataset singleDataset = (SingleRowDataset) getDataset() ;
    if (singleDataset.isEmpty()) {
      this.range = DEFAULT_RANGE ;
      return ;
    }

    double min = Double.NaN, max = Double.NaN ;
    int currRowCols = singleDataset.getColumnCount(0) ;
    for (int i = 0 ; i < currRowCols ; i++) {
      StartEndNumberChartData aData = (StartEndNumberChartData) singleDataset.getData(0, i) ;
      double start = aData.getStartValue() ;
      double end = aData.getEndValue() ;

      if (aData != null) {
        if (!Double.isNaN(start)) {
          if (Double.isNaN(min)) {
            min = start ;
          } else {
            min = Math.min(min, start) ;
          }

          if (Double.isNaN(max)) {
            max = start ;
          } else {
            max = Math.max(min, start) ;
          }
        }

        if (!Double.isNaN(end)) {
          if (Double.isNaN(min)) {
            min = end ;
          } else {
            min = Math.min(min, end) ;
          }

          if (Double.isNaN(max)) {
            max = end ;
          } else {
            max = Math.max(min, end) ;
          }
        }

      }
    }

    if (Double.isNaN(min) || Double.isNaN(max)) {
      this.range = DEFAULT_RANGE ;
    } else {
      this.range = new NumberRange(Math.min(min, max), Math.max(min, max)) ;
    }
  }

  /**
   * <zh>ֵתΪǶ.</zh>
   * <en>Translates a data value to an angle on the dial.</en>
   *
   * @param value <zh>ֵ.</zh>
   * <en>The value.</en>
   *
   * @return <zh>תĽǶ.</zh>
   * <en>The angle on the dial.</en>
   */
  public double valueToAngle(double value) {
    double v = value - this.range.getLowerBound() ;
    double baseAngle = 180 + ((this.meterAngle - 180) / 2) ;

    return baseAngle - ((v / this.range.getLength()) * this.meterAngle) ;
  }

  /**
   * <zh>Ʊɫ.</zh>
   * <en>Draws a background for the data area.</en>
   *
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param orientation ChartOrientation <zh>ͼƷ (Ϊnull).</zh>
   * <en>The chart draw orientation (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   */
  public void drawBackground(Rectangle2D dataArea, ChartOrientation orientation,
                             RenderShapeCollection renderShapes) {
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (orientation == null) {
      throw new IllegalArgumentException("Null 'orientation' argument.") ;
    }
    if (renderShapes == null) {
      throw new IllegalArgumentException("Null 'renderShapes' argument.") ;
    }

    Paint p = getBackgroundPaint() ;
    if (p == null) {
      return ;
    }

    if (p instanceof GradientPaint) {
      GradientPaint gp = (GradientPaint) p ;
      if (orientation.equals(ChartOrientation.VERTICAL)) {
        p = new GradientPaint((float) dataArea.getCenterX(),
                              (float) dataArea.getMaxY(), gp.getColor1(),
                              (float) dataArea.getCenterX(), (float) dataArea.getMinY(),
                              gp.getColor2()) ;
      } else if (orientation.equals(ChartOrientation.HORIZONTAL)) {
        p = new GradientPaint((float) dataArea.getMinX(),
                              (float) dataArea.getCenterY(), gp.getColor1(),
                              (float) dataArea.getMaxX(), (float) dataArea.getCenterY(),
                              gp.getColor2()) ;
      }
    }

    Rectangle2D newArea = new Rectangle2D.Double(dataArea.getX() + this.gapSize,
                                                 dataArea.getY() + this.gapSize,
                                                 dataArea.getWidth() - 2 * this.gapSize,
                                                 dataArea.getHeight() - 2 * this.gapSize) ;

    double meterX = newArea.getX() + this.borderSize ;
    double meterY = newArea.getY() + this.borderSize ;
    double meterW = newArea.getWidth() - this.borderSize * 2 ;
    double meterH = newArea.getHeight() - this.borderSize * 2
                    + ((this.meterAngle <= 180 && !this.meterShapeType.equals(MeterShapeType.CIRCLE))
                       ? newArea.getHeight() / 1.25 : 0) ;

    double min = Math.min(meterW, meterH) / 2 ;
    meterX = (meterX + meterX + meterW) / 2 - min ;
    meterY = (meterY + meterY + meterH) / 2 - min ;
    meterW = 2 * min ;
    meterH = 2 * min ;

    calculateRange() ;
    double dataMin = this.range.getLowerBound() ;
    double dataMax = this.range.getUpperBound() ;

    double startAngle = valueToAngle(dataMax) ;
    double endAngle = valueToAngle(dataMin) ;
    double extent = endAngle - startAngle ;

    int joinType = Arc2D.OPEN ;
    if (this.meterShapeType.equals(MeterShapeType.PIE)) {
      joinType = Arc2D.PIE ;
    } else if (this.meterShapeType.equals(MeterShapeType.CHORD)) {
      if (this.meterAngle > 180) {
        joinType = Arc2D.CHORD ;
      } else {
        joinType = Arc2D.PIE ;
      }
    } else if (this.meterShapeType.equals(MeterShapeType.CIRCLE)) {
      joinType = Arc2D.PIE ;
      extent = 360 ;
    }

    //ɫͼ
    double offX = meterW * 0.01 ;
    double offY = meterH * 0.01 ;
    Arc2D.Double arc = new Arc2D.Double(meterX - offX, meterY - offY, meterW + 2 * offX,
                                        meterH + 2 * offY, startAngle,
                                        extent, joinType) ;
    MeterArcChartShape arcShape = new MeterArcChartShape(arc) ;
    arcShape.setFillPaint(p) ;
    arcShape.setDrawBorder(false) ;
    arcShape.setFillShape(true) ;
    arcShape.setDrawTick(false) ;
    arcShape.setDrawRadius(false) ;

    renderShapes.setBackgroundShape(arcShape) ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof MeterRender)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    MeterRender that = (MeterRender) obj ;
    if (!EqualsUtil.equals(this.meterShapeType, that.meterShapeType)) {
      return false ;
    }
    if (this.meterAngle != that.meterAngle) {
      return false ;
    }
    if (!EqualsUtil.equals(this.range, that.range)) {
      return false ;
    }
    if (this.tickSize != that.tickSize) {
      return false ;
    }
    if (!EqualsUtil.equals(this.tickPaint, that.tickPaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.tickStroke, that.tickStroke)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.units, that.units)) {
      return false ;
    }
    if (this.borderSize != that.borderSize) {
      return false ;
    }
    if (this.circleSize != that.circleSize) {
      return false ;
    }
    if (!EqualsUtil.equals(this.valueFont, that.valueFont)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.valuePaint, that.valuePaint)) {
      return false ;
    }
    if (this.value != that.value) {
      return false ;
    }
    if (!EqualsUtil.equals(this.needlePaint, that.needlePaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.noNeedleValueLabel, that.noNeedleValueLabel)) {
      return false ;
    }
    if (this.tickLabelVisible != that.tickLabelVisible) {
      return false ;
    }
    if (!EqualsUtil.equals(this.tickLabelFont, that.tickLabelFont)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.tickLabelPaint, that.tickLabelPaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.tickLabelFormat, that.tickLabelFormat)) {
      return false ;
    }
    if (this.gapSize != that.gapSize) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    double result = super.hashCode() ;
    result = 37 * result + HashcodeUtil.hashCode(this.meterShapeType) ;
    result = 37 * result + this.meterAngle ;
    result = 37 * result + HashcodeUtil.hashCode(this.range) ;
    result = 37 * result + this.tickSize ;
    result = 37 * result + HashcodeUtil.hashCode(this.tickPaint) ;
    result = 37 * result + HashcodeUtil.hashCode(this.tickStroke) ;
    result = 37 * result + HashcodeUtil.hashCode(this.units) ;
    result = 37 * result + this.borderSize ;
    result = 37 * result + this.circleSize ;
    result = 37 * result + HashcodeUtil.hashCode(this.valueFont) ;
    result = 37 * result + HashcodeUtil.hashCode(this.valuePaint) ;
    result = 37 * result + this.value ;
    result = 37 * result + HashcodeUtil.hashCode(this.needlePaint) ;
    result = 37 * result + HashcodeUtil.hashCode(this.noNeedleValueLabel) ;
    result = 37 * result + HashcodeUtil.hashCode(this.tickLabelVisible) ;
    result = 37 * result + HashcodeUtil.hashCode(this.tickLabelFont) ;
    result = 37 * result + HashcodeUtil.hashCode(this.tickLabelPaint) ;
    result = 37 * result + HashcodeUtil.hashCode(this.tickLabelFormat) ;
    result = 37 * result + this.gapSize ;

    return (int) result ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "MeterRender" ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the output stream.
   *
   * @throws IOException  if there is an I/O error.
   */
  private void writeObject(ObjectOutputStream stream) throws IOException {
    stream.defaultWriteObject() ;

    SerialUtil.writePaint(stream, this.tickPaint) ;
    SerialUtil.writeStroke(stream, this.tickStroke) ;
    SerialUtil.writePaint(stream, this.valuePaint) ;
    SerialUtil.writePaint(stream, this.needlePaint) ;
    SerialUtil.writePaint(stream, this.tickLabelPaint) ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the input stream.
   *
   * @throws IOException  if there is an I/O error.
   * @throws ClassNotFoundException  if there is a classpath problem.
   */
  private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
    stream.defaultReadObject() ;

    this.tickPaint = SerialUtil.readPaint(stream) ;
    this.tickStroke = SerialUtil.readStroke(stream) ;
    this.valuePaint = SerialUtil.readPaint(stream) ;
    this.needlePaint = SerialUtil.readPaint(stream) ;
    this.tickLabelPaint = SerialUtil.readPaint(stream) ;
  }

}
