package com.ijchart.xychart.shape ;

import java.awt.AlphaComposite ;
import java.awt.Color ;
import java.awt.Graphics2D ;
import java.awt.Paint ;
import java.awt.Shape ;
import java.awt.geom.AffineTransform ;
import java.awt.geom.Ellipse2D ;
import java.awt.geom.GeneralPath ;
import java.awt.geom.Line2D ;
import java.awt.geom.Point2D ;
import java.awt.geom.Rectangle2D ;
import java.io.IOException ;
import java.io.ObjectInputStream ;
import java.io.ObjectOutputStream ;
import java.io.Serializable ;

import com.ijchart.xychart.constant.ShapeType ;
import com.ijchart.xychart.ui.Graphics2DState ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.GraphicsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.util.SerialUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>BoxAndWhiskerͼ.</zh>
 * <en>Draw a BoxAndWhiskerChartShape {@link com.ijchart.xychart.shape.IChartShape}.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class BoxAndWhiskerChartShape extends AbstractRectangleTextChartShape implements
  Serializable, Cloneable {

  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 414550107535246547L ;

  /**
   * <zh>ȱʡԲͻƺߵPaint.</zh>
   * <en>The default color used to paint the median line and average marker.</en>
   */
  public static final Paint DEFAULT_ARTIFACT_PAINT = Color.black ;

  /**
   * <zh>½ǵĵX.</zh>
   * <en>The x coordinate of left-hand corner.</en>
   */
  private double leftX ;

  /**
   * <zh>½ǵĵY.</zh>
   * <en>The y coordinate of left-hand corner.</en>
   */
  private double bottomY ;

  /**
   * <zh>εĿ.</zh>
   * <en>The width of I-shaped shape.</en>
   */
  private double width ;

  /**
   * <zh>εĸ߶.</zh>
   * <en>The height of I-shaped shape.</en>
   */
  private double height ;

  /**
   * <zh>ͼ֮ϵߵĳ.</zh>
   * <en>The length of top line.</en>
   */
  private double topLineLength ;

  /**
   * <zh>ͼ֮µߵĳ.</zh>
   * <en>The length of bottom line.</en>
   */
  private double bottomLineLength ;

  /**
   * <zh>Բͼ.</zh>
   * <en>The ellipse.</en>
   */
  private transient Ellipse2D avgEllipse ;

  /**
   * <zh>ֱߵʼy.</zh>
   * <en>The y-coordinate of line.</en>
   */
  private double lineY ;

  /**
   * <zh>ԲͻƺߵPaint.</zh>
   * <en>The color used to paint the median line and average marker.</en>
   */
  private transient Paint artifactPaint ;

  /**
   * <zh>һԲαʾһƽֵ򳬹һƽֵ,Բ.</zh>
   * <en>Draws one or two circles to represent the average value of a outlier or more than one
   * outlier.</en>
   */
  private transient Ellipse2D[] averageValueEllipse ;

  /**
   * <zh>ʾֵ.</zh>
   * <en> Draws a triangle to indicate the presence of far-out values.</en>
   */
  private transient Shape highFarBorderTriangle ;

  /**
   * <zh>ʾֵ.</zh>
   * <en> Draws a triangle to indicate the presence of low-out values.</en>
   */
  private transient Shape lowFarBorderTriangle ;

  /**
   * <zh>ƹεͼ.</zh>
   * <en>Draw a I-shaped chart shape.</en>
   *
   * @param leftX double <zh>½ǵĵX.</zh>
   * <en>The x coordinate of left-hand corner.</en>
   * @param bottomY double <zh>½ǵĵY.</zh>
   * <en>The y coordinate of left-hand corner.</en>
   * @param width double <zh>εĿ.</zh>
   * <en>The width of I-shaped shape.</en>
   * @param height double <zh>εĸ߶.</zh>
   * <en>The height of I-shaped shape.</en>
   * @param topLineLength double <zh>ͼ֮ϵߵĳ.</zh>
   * <en>The length of top line.</en>
   * @param bottomLineLength double <zh>ͼ֮µߵĳ.</zh>
   * <en>The length of bottom line.</en>
   * @param ellipse Ellipse2D <zh>Բͼ (Ϊnull).</zh>
   * <en>The ellipse (null permitted).</en>
   * @param lineY double <zh>ֱߵʼy,ֵΪDouble.NaN,򲻻ֱ.</zh>
   * <en>The y-coordinate of line,if this value is Double.Nan,no line will be draw.</en>
   * @param artifactPaint <zh>ԲͻƺߵPaint (Ϊnull).</zh>
   * <en>The color used to paint the median line and average marker (null permitted).</en>
   */
  public BoxAndWhiskerChartShape(double leftX, double bottomY, double width, double height,
                                 double topLineLength, double bottomLineLength,
                                 Ellipse2D ellipse, double lineY, Paint artifactPaint) {
    this(leftX, bottomY, width, height, topLineLength, bottomLineLength, ellipse, lineY,
         artifactPaint, null, null, null, null) ;
  }

  /**
   * <zh>ƹεͼ.</zh>
   * <en>Draw a I-shaped chart shape.</en>
   *
   * @param leftX double <zh>½ǵĵX.</zh>
   * <en>The x coordinate of left-hand corner.</en>
   * @param bottomY double <zh>½ǵĵY.</zh>
   * <en>The y coordinate of left-hand corner.</en>
   * @param width double <zh>εĿ.</zh>
   * <en>The width of I-shaped shape.</en>
   * @param height double <zh>εĸ߶.</zh>
   * <en>The height of I-shaped shape.</en>
   * @param topLineLength double <zh>ͼ֮ϵߵĳ.</zh>
   * <en>The length of top line.</en>
   * @param bottomLineLength double <zh>ͼ֮µߵĳ.</zh>
   * <en>The length of bottom line.</en>
   * @param ellipse Ellipse2D <zh>Բͼ (Ϊnull).</zh>
   * <en>The ellipse (null permitted).</en>
   * @param lineY double <zh>ֱߵʼy,ֵΪDouble.NaN,򲻻ֱ.</zh>
   * <en>The y-coordinate of line,if this value is Double.Nan,no line will be draw.</en>
   * @param artifactPaint <zh>ԲͻƺߵPaint (Ϊnull).</zh>
   * <en>The color used to paint the median line and average marker (null permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   */
  public BoxAndWhiskerChartShape(double leftX, double bottomY, double width, double height,
                                 double topLineLength, double bottomLineLength,
                                 Ellipse2D ellipse, double lineY, Paint artifactPaint, String text) {
    this(leftX, bottomY, width, height, topLineLength, bottomLineLength, ellipse, lineY,
         artifactPaint, text, null, null, null) ;
  }

  /**
   * <zh>ƹεͼ.</zh>
   * <en>Draw a I-shaped chart shape.</en>
   *
   * @param leftX double <zh>½ǵĵX.</zh>
   * <en>The x coordinate of left-hand corner.</en>
   * @param bottomY double <zh>½ǵĵY.</zh>
   * <en>The y coordinate of left-hand corner.</en>
   * @param width double <zh>εĿ.</zh>
   * <en>The width of I-shaped shape.</en>
   * @param height double <zh>εĸ߶.</zh>
   * <en>The height of I-shaped shape.</en>
   * @param topLineLength double <zh>ͼ֮ϵߵĳ.</zh>
   * <en>The length of top line.</en>
   * @param bottomLineLength double <zh>ͼ֮µߵĳ.</zh>
   * <en>The length of bottom line.</en>
   * @param ellipse Ellipse2D <zh>Բͼ (Ϊnull).</zh>
   * <en>The ellipse (null permitted).</en>
   * @param lineY double <zh>ֱߵʼy,ֵΪDouble.NaN,򲻻ֱ.</zh>
   * <en>The y-coordinate of line,if this value is Double.Nan,no line will be draw.</en>
   * @param artifactPaint <zh>ԲͻƺߵPaint (Ϊnull).</zh>
   * <en>The color used to paint the median line and average marker (null permitted).</en>
   * @param averageValueEllipse Ellipse2D[]
   * <zh>һԲαʾһƽֵ򳬹һƽֵ,Բ (Ϊnull).</zh>
   * <en>Draws one or two circles to represent the average value of a outlier or more than one
   * outlier (null permitted).</en>
   * @param highFarBorderTriangle Shape <zh>ʾֵ (Ϊnull).</zh>
   * <en> Draws a triangle to indicate the presence of far-out values (null permitted).</en>
   * @param lawFarBorder Shape <zh>ʾֵ (Ϊnull).</zh>
   * <en> Draws a triangle to indicate the presence of low-out values (null permitted).</en>
   */
  public BoxAndWhiskerChartShape(double leftX, double bottomY, double width, double height,
                                 double topLineLength, double bottomLineLength,
                                 Ellipse2D ellipse, double lineY, Paint artifactPaint,
                                 Ellipse2D[] averageValueEllipse, Shape highFarBorderTriangle,
                                 Shape lawFarBorder) {
    this(leftX, bottomY, width, height, topLineLength, bottomLineLength, ellipse, lineY,
         artifactPaint, null, averageValueEllipse, highFarBorderTriangle, lawFarBorder) ;
  }

  /**
   * <zh>ƹεͼ.</zh>
   * <en>Draw a I-shaped chart shape.</en>
   *
   * @param leftX double <zh>½ǵĵX.</zh>
   * <en>The x coordinate of left-hand corner.</en>
   * @param bottomY double <zh>½ǵĵY.</zh>
   * <en>The y coordinate of left-hand corner.</en>
   * @param width double <zh>εĿ.</zh>
   * <en>The width of I-shaped shape.</en>
   * @param height double <zh>εĸ߶.</zh>
   * <en>The height of I-shaped shape.</en>
   * @param topLineLength double <zh>ͼ֮ϵߵĳ.</zh>
   * <en>The length of top line.</en>
   * @param bottomLineLength double <zh>ͼ֮µߵĳ.</zh>
   * <en>The length of bottom line.</en>
   * @param ellipse Ellipse2D <zh>Բͼ (Ϊnull).</zh>
   * <en>The ellipse (null permitted).</en>
   * @param lineY double <zh>ֱߵʼy,ֵΪDouble.NaN,򲻻ֱ.</zh>
   * <en>The y-coordinate of line,if this value is Double.Nan,no line will be draw.</en>
   * @param artifactPaint <zh>ԲͻƺߵPaint (Ϊnull).</zh>
   * <en>The color used to paint the median line and average marker (null permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   * @param averageValueEllipse Ellipse2D[]
   * <zh>һԲαʾһƽֵ򳬹һƽֵ,Բ (Ϊnull).</zh>
   * <en>Draws one or two circles to represent the average value of a outlier or more than one
   * outlier (null permitted).</en>
   * @param highFarBorderTriangle Shape <zh>ʾֵ (Ϊnull).</zh>
   * <en> Draws a triangle to indicate the presence of far-out values (null permitted).</en>
   * @param lowFarBorder Shape <zh>ʾֵ (Ϊnull).</zh>
   * <en> Draws a triangle to indicate the presence of low-out values (null permitted).</en>
   */
  public BoxAndWhiskerChartShape(double leftX, double bottomY, double width, double height,
                                 double topLineLength, double bottomLineLength,
                                 Ellipse2D ellipse, double lineY, Paint artifactPaint, String text,
                                 Ellipse2D[] averageValueEllipse, Shape highFarBorderTriangle,
                                 Shape lowFarBorder) {
    super(new Rectangle2D.Double(leftX, bottomY - (height - topLineLength), width,
                                 height - topLineLength - bottomLineLength), text, null) ;
    if (topLineLength + bottomLineLength >= height) {
      throw new IllegalArgumentException("Must be topLineLength+bottomLineLength<height.") ;
    }
    if (averageValueEllipse != null && averageValueEllipse.length > 2) {
      throw new IllegalArgumentException(
        "The length of 'averageValueEllipse' can not be greater than 2.") ;
    }

    this.leftX = leftX ;
    this.bottomY = bottomY ;
    this.width = width ;
    this.height = height ;
    this.topLineLength = topLineLength ;
    this.bottomLineLength = bottomLineLength ;
    this.avgEllipse = ellipse ;
    this.lineY = lineY ;
    this.artifactPaint = artifactPaint ;
    this.averageValueEllipse = averageValueEllipse ;
    this.highFarBorderTriangle = highFarBorderTriangle ;
    this.lowFarBorderTriangle = lowFarBorder ;

    setFillShape(true) ;
    setDrawBorder(true) ;
  }

  /**
   * <zh>½ǵĵX.</zh>
   * <en>Returns the x coordinate of left-hand corner.</en>
   *
   * @return double
   */
  public double getLeftX() {
    return leftX ;
  }

  /**
   * <zh>½ǵĵX,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the x coordinate of left-hand corner,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param leftX double
   */
  public void setLeftX(double leftX) {
    if (this.leftX != leftX) {
      this.leftX = leftX ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>½ǵĵY.</zh>
   * <en>Returns the y coordinate of left-hand corner,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @return double
   */
  public double getBottomY() {
    return bottomY ;
  }

  /**
   * <zh>½ǵĵY,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the y coordinate of left-hand corner,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param bottomY double
   */
  public void setBottomY(double bottomY) {
    if (this.bottomY != bottomY) {
      this.bottomY = bottomY ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ùεĿ.</zh>
   * <en>Sets the width of I-shaped shape.</en>
   *
   * @return double
   */
  public double getWidth() {
    return width ;
  }

  /**
   * <zh>ùεĿ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the width of I-shaped shape,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param width double
   */
  public void setWidth(double width) {
    if (this.width != width) {
      this.width = width ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>عεĸ߶.</zh>
   * <en>Returns the height of I-shaped shape.</en>
   *
   * @return double
   */
  public double getHeight() {
    return height ;
  }

  /**
   * <zh>ùεĸ߶,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the height of I-shaped shape,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param height double
   */
  public void setHeight(double height) {
    if (this.height != height) {
      this.height = height ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ؾͼ֮ϵߵĳ.</zh>
   * <en>Returns the length of top line.</en>
   *
   * @return double <zh>.</zh>
   * <en>The length.</en>
   */
  public double getTopLineLength() {
    return this.topLineLength ;
  }

  /**
   * <zh>þͼ֮ϵߵĳ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the length of top line,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param length double <zh>.</zh>
   * <en>The length.</en>
   */
  public void setTopLineLength(double length) {
    if (this.topLineLength != length) {
      this.topLineLength = length ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ؾͼ֮µߵĳ.</zh>
   * <en>Returns the length of bottom line.</en>
   *
   * @return double <zh>.</zh>
   * <en>The length.</en>
   */
  public double getBottomLineLength() {
    return this.bottomLineLength ;
  }

  /**
   * <zh>þͼ֮µߵĳ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the length of bottom line,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param length double <zh>.</zh>
   * <en>The length.</en>
   */
  public void setBottomLineLength(double length) {
    if (this.bottomLineLength != length) {
      this.bottomLineLength = length ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>Բͼ.</zh>
   * <en>Returns the ellipse.</en>
   *
   * @return Ellipse2D <zh>Բͼ.</zh>
   * <en>The ellipse.</en>
   */
  public Ellipse2D getAvgEllipse() {

    return avgEllipse ;
  }

  /**
   * <zh>Բͼ,м{@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the ellipse,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param ellipse Ellipse2D <zh>Բͼ (Ϊnull).</zh>
   * <en>The ellipse (null permitted).</en>
   */
  public void setAvgEllipse(Ellipse2D ellipse) {
    if (ellipse == null) {
      throw new IllegalArgumentException("Null 'ellipse' argument.") ;
    }

    this.avgEllipse = ellipse ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>ֱߵʼy.</zh>
   * <en>Returns the y-coordinate of line.</en>
   *
   * @return double <zh>ֱߵʼy.</zh>
   * <en>The y-coordinate of line.</en>
   */
  public double getLineY() {
    return this.lineY ;
  }

  /**
   * <zh>ֱߵʼy,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the y-coordinate of line,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param lineY double <zh>ֱߵʼy.</zh>
   * <en>The y-coordinate of line.</en>
   */
  public void setLineY(double lineY) {
    if (this.lineY != lineY) {
      this.lineY = lineY ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ԲͻƺߵPaint.</zh>
   * <en>Returns the color used to paint the median line and average marker.</en>
   *
   * @return Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  public Paint getArtifactPaint() {
    return this.artifactPaint ;
  }

  /**
   * <zh>ԲͻƺߵPaint,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the color used to paint the median line and average marker,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (null permitted).</en>
   */
  public void setArtifactPaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument.") ;
    }

    this.artifactPaint = paint ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>رʾƽֵԲ.</zh>
   * <en>Returns the ellipses for represent the average value.</en>
   *
   * @return Ellipse2D[] <zh>Բμ,Ϊnull.</zh>
   * <en>The ellipses,maybe null.</en>
   */
  public Ellipse2D[] getAverageValueEllipse() {
    return this.averageValueEllipse ;
  }

  /**
   * <zh>ñʾƽֵԲ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the ellipses for represent the average value,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param ellipses Ellipse2D[] <zh>Բμ,Բ (Ϊnull).</zh>
   * <en>The ellipses,the length of ellipses must be less or equal to 2 (null permitted).</en>
   */
  public void setAverageValueEllipse(Ellipse2D[] ellipses) {
    if (averageValueEllipse != null && averageValueEllipse.length > 2) {
      throw new IllegalArgumentException(
        "The length of 'ellipses' can not be greater than 2.") ;
    }

    this.averageValueEllipse = ellipses ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>رʾֵ.</zh>
   * <en>Returns a triangle to indicate the presence of far-out values.</en>
   *
   * @return Shape <zh>ֱ߼ (Ϊnull).</zh>
   * <en>The lines (maybe null).</en>
   */
  public Shape getHighFarBorderTriangle() {
    return this.highFarBorderTriangle ;
  }

  /**
   * <zh>ñʾֵ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets a triangle to indicate the presence of far-out values,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param lines Shape <zh>ֱ߼ (Ϊnull).</zh>
   * <en>The lines (null permitted).</en>
   */
  public void setHighFarBorderTriangle(Shape lines) {
    this.highFarBorderTriangle = lines ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>رʾֵ..</zh>
   * <en>Returns a triangle to indicate the presence of low-out values.</en>
   *
   * @return Shape <zh>ֱ߼ (Ϊnull).</zh>
   * <en>The lines (maybe null).</en>
   */
  public Shape getLowFarBorderTriangle() {
    return this.lowFarBorderTriangle ;
  }

  /**
   * <zh>ñʾֵ.,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets a triangle to indicate the presence of far-out values,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param lines Shape <zh>ֱ߼ (Ϊnull).</zh>
   * <en>The lines (null permitted).</en>
   */
  public void setLowFarBorderTriangle(Shape lines) {
    this.lowFarBorderTriangle = lines ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻƴͼ</zh>
   * <en>Draw the ChartShape on a Java 2d graphics device(such as the screen or the printer).</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>The graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  public void draw(Graphics2D g2) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (!isVisible()) {
      return ;
    }

    Graphics2DState state = new Graphics2DState(g2) ;
    state.saveOldState() ;

    g2.translate(this.leftX, this.bottomY) ;
    g2.rotate(getRotate()) ;
    beforeDraw(g2) ;

    //ͼ
    GeneralPath borderPath = new GeneralPath() ;
    borderPath.moveTo(0, 0) ;
    borderPath.lineTo((float)this.width, 0) ;
    borderPath.lineTo((float) (0.5 * this.width), 0) ;
    borderPath.lineTo((float) (0.5 * this.width), (float) - this.bottomLineLength) ;
    borderPath.lineTo((float)this.width, (float) - this.bottomLineLength) ;
    borderPath.lineTo((float)this.width, (float) - (this.height - this.topLineLength)) ;
    borderPath.lineTo((float) (0.5 * this.width), (float) - (this.height - this.topLineLength)) ;
    borderPath.lineTo((float) (0.5 * this.width), (float) - this.height) ;
    borderPath.lineTo((float)this.width, (float) - this.height) ;
    borderPath.lineTo(0, (float) - this.height) ;
    borderPath.lineTo((float) (0.5 * this.width), (float) - this.height) ;
    borderPath.lineTo((float) (0.5 * this.width), (float) - (this.height - this.topLineLength)) ;
    borderPath.lineTo(0, (float) - (this.height - this.topLineLength)) ;
    borderPath.lineTo(0, (float) - this.bottomLineLength) ;
    borderPath.lineTo((float) (0.5 * this.width), (float) - this.bottomLineLength) ;
    borderPath.lineTo((float) (0.5 * this.width), 0) ;
    borderPath.lineTo(0, 0) ;
    borderPath.closePath() ;

    drawBackground(g2, borderPath) ;
    drawBorder(g2, borderPath) ;

    //Բ
    if (this.avgEllipse != null) {
      g2.setPaint((this.artifactPaint == null) ? DEFAULT_ARTIFACT_PAINT : this.artifactPaint) ;
      g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, getFillAlpha())) ;
      Ellipse2D newEllipse = new Ellipse2D.Double(this.avgEllipse.getX() - this.leftX,
                                                  this.avgEllipse.getY() - this.bottomY,
                                                  this.avgEllipse.getWidth(),
                                                  this.avgEllipse.getHeight()) ;
      g2.fill(newEllipse) ;
    }

    if (!Double.isNaN(this.lineY)) { //ֱ
      g2.setPaint((this.artifactPaint == null) ? DEFAULT_ARTIFACT_PAINT : this.artifactPaint) ;
      Line2D line = new Line2D.Double(0, this.lineY - this.bottomY, this.width,
                                      this.lineY - this.bottomY) ;
      g2.draw(line) ;
    }

    if (this.averageValueEllipse != null) {
      g2.setPaint((this.artifactPaint == null) ? DEFAULT_ARTIFACT_PAINT : this.artifactPaint) ;
      g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, getFillAlpha())) ;
      for (int i = 0 ; i < this.averageValueEllipse.length ; i++) {
        Ellipse2D newEllipse = new Ellipse2D.Double(this.averageValueEllipse[i].getX() - this.leftX,
                                                    this.averageValueEllipse[i].getY()
                                                    - this.bottomY,
                                                    this.averageValueEllipse[i].getWidth(),
                                                    this.averageValueEllipse[i].getHeight()) ;
        g2.draw(newEllipse) ;
      }
    }

    //draw high-out values
    if (this.highFarBorderTriangle != null) {
      g2.setPaint((this.artifactPaint == null) ? DEFAULT_ARTIFACT_PAINT : this.artifactPaint) ;
      g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, getFillAlpha())) ;
      g2.draw(moveShape(this.highFarBorderTriangle)) ;
    }

    // draw low-out values
    if (this.lowFarBorderTriangle != null) {
      g2.setPaint((this.artifactPaint == null) ? DEFAULT_ARTIFACT_PAINT : this.artifactPaint) ;
      g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, getFillAlpha())) ;
      g2.draw(moveShape(this.lowFarBorderTriangle)) ;
    }

    g2.translate(0, -(height - topLineLength)) ;
    Rectangle2D cloneArea = new Rectangle2D.Double(0, 0, getDrawArea().getWidth(),
                                                   getDrawArea().getHeight()) ;
    drawTextWithTrimMarginRect(g2, cloneArea) ;

    afterDraw(g2) ;
    g2.rotate( -getRotate()) ;
    state.restoreOldState() ;
    drawSelect(g2) ;
  }

  /**
   * <zh>ƶͼ.</zh>
   * <en>Translate shape.</en>
   * @param shape Shape
   * @return GeneralPath
   */
  private Shape moveShape(Shape shape) {
    if (shape == null) {
      return null ;
    }

    return GraphicsUtil.translateShapeByPath(shape, (float)this.leftX, (float)this.bottomY) ;
  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻͼεѡв,Ϊͼοܻص,ԻͼʱҪȻеͼκ,
   * ٵô˷ѡв.</zh>
   * <en>Draw the selected ChartShape on a Java 2d graphics device(such
   * as the screen or the printer).</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>the graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  protected void drawSelect(Graphics2D g2) {
    if (isSelected()) {
      double halfWidth = 0.5 * this.width ;
      boolean drawCenter = this.width > 2 * getSelectRectWidth() ;

      Point2D.Double[]
        points = new Point2D.Double[] {
                 new Point2D.Double(0, 0),
                 drawCenter ? new Point2D.Double(halfWidth, 0) : null,
                 new Point2D.Double(this.width, 0),

                 new Point2D.Double(0, -this.bottomLineLength),
                 drawCenter ? new Point2D.Double(halfWidth, -this.bottomLineLength) : null,
                 new Point2D.Double(this.width, -this.bottomLineLength),

                 new Point2D.Double(0, -(this.height - this.topLineLength)),
                 drawCenter ? new Point2D.Double(halfWidth, -(this.height - this.topLineLength)) : null,
                 new Point2D.Double(this.width, -(this.height - this.topLineLength)),

                 new Point2D.Double(0, -this.height),
                 drawCenter ? new Point2D.Double(halfWidth, -this.height) : null,
                 new Point2D.Double(this.width, -this.height)
      } ;

      drawAnchorByPoints(g2, this.leftX, this.bottomY, points) ;
    }
  }

  /**
   * <zh>ƶͼ,X᷽ƫtx,Y᷽ƫty,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Concatenates the current Graphics2D Transform with a translation transform.and sends
   * a {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.
   * Subsequent rendering is translated by the specified
   * distance relative to the previous position.
   * This is equivalent to calling transform(T), where T is an
   * AffineTransform represented by the following matrix:
   * <pre>
   *		[   1    0    tx  ]
   *		[   0    1    ty  ]
   *		[   0    0    1   ]
   * </pre>.</en>
   *
   * @param tx double <zh>X᷽ľ.</zh>
   * <en>The distance to translate along the x-axis.</en>
   * @param ty double <zh>Y᷽ľ.</zh>
   * <en>The distance to translate along the y-axis.</en>
   */
  public void translate(double tx, double ty) {
    if (!(tx == 0 && ty == 0)) {
      this.leftX += tx ;
      this.bottomY += ty ;

      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ͼϽǵ(ͼεת).</zh>
   * <en>Returns the top left corner coordinates of this shape (not consider rotation).</en>
   *
   * @return Point2D <zh>ֵ.</zh>
   * <en>The coordinate.</en>
   */
  public Point2D getTopLeftPoint() {
    return new Point2D.Double(this.leftX, this.bottomY - this.height) ;
  }

  /**
   * <zh>ʱͼεĵ.</zh>
   * <en>Returns the lock coordinates of this shape.</en>
   *
   * @return Point2D <zh>ֵ.</zh>
   * <en>The coordinate.</en>
   */
  public Point2D getLockPoint() {
    GeneralPath path = new GeneralPath() ;
    path.moveTo(0, (float) - this.bottomLineLength) ;
    path.lineTo((float)this.width, (float) - this.bottomLineLength) ;
    path.lineTo((float)this.width, (float) - (this.height - this.topLineLength)) ;
    path.lineTo(0, (float) - (this.height - this.topLineLength)) ;
    path.closePath() ;

    AffineTransform transform = AffineTransform.getTranslateInstance(this.leftX, this.bottomY) ;
    transform.rotate(getRotate()) ;
    Shape rotateShape = transform.createTransformedShape(path) ;
    Rectangle2D rotateBound = rotateShape.getBounds2D() ;

    return new Point2D.Double(rotateBound.getCenterX(), rotateBound.getCenterY()) ;
  }

  /**
   * <zh>任ͼ.</zh>
   * <en>Calculate the transformed shape.</en>
   */
  protected void calculateTransformedShape() {
    if (transformedShape == null) {
      float halfWidth = (float) (0.5 * this.width) ;
      float fWidth = (float)this.width ;
      float fHeight = (float)this.height ;
      final float expandWidth = 1.5F ;

      GeneralPath path = new GeneralPath() ;
      path.moveTo(0, -expandWidth) ;
      path.lineTo(halfWidth - expandWidth, -expandWidth) ;
      path.lineTo(halfWidth - expandWidth, (float) - this.bottomLineLength + expandWidth) ;
      path.lineTo(0, (float) ( -this.bottomLineLength + expandWidth)) ;
      path.lineTo(0, (float) - (fHeight - this.topLineLength + expandWidth)) ;
      path.lineTo(halfWidth - expandWidth, (float) - (fHeight - this.topLineLength + expandWidth)) ;
      path.lineTo(halfWidth - expandWidth, -(fHeight - expandWidth)) ;
      path.lineTo(0, -(fHeight - expandWidth)) ;
      path.lineTo(0, -(fHeight + expandWidth)) ;
      path.lineTo(fWidth, -(fHeight + expandWidth)) ;
      path.lineTo(fWidth, -(fHeight - expandWidth)) ;
      path.lineTo(halfWidth + expandWidth, -(fHeight - expandWidth)) ;
      path.lineTo(halfWidth + expandWidth, (float) - (fHeight - this.topLineLength + expandWidth)) ;
      path.lineTo(fWidth, (float) - (fHeight - this.topLineLength + expandWidth)) ;
      path.lineTo(fWidth, (float) - (this.bottomLineLength - expandWidth)) ;
      path.lineTo(halfWidth + expandWidth, (float) - (this.bottomLineLength - expandWidth)) ;
      path.lineTo(halfWidth + expandWidth, -expandWidth) ;
      path.lineTo(fWidth, -expandWidth) ;
      path.lineTo(fWidth, expandWidth) ;
      path.lineTo(0, expandWidth) ;
      path.closePath() ;

      //Բ
      if (this.avgEllipse != null) {
        Ellipse2D newEllipse = new Ellipse2D.Double(this.avgEllipse.getX() - this.leftX,
                                                    this.avgEllipse.getY() - this.bottomY,
                                                    this.avgEllipse.getWidth(),
                                                    this.avgEllipse.getHeight()) ;
        path.append(newEllipse, false) ;
      }

      if (this.averageValueEllipse != null) {
        for (int i = 0 ; i < this.averageValueEllipse.length ; i++) {
          Ellipse2D newEllipse = new Ellipse2D.Double(this.averageValueEllipse[i].getX()
                                                      - this.leftX,
                                                      this.averageValueEllipse[i].getY()
                                                      - this.bottomY,
                                                      this.averageValueEllipse[i].getWidth(),
                                                      this.averageValueEllipse[i].getHeight()) ;
          path.append(newEllipse, false) ;
        }
      }

      //draw high-out values
      if (this.highFarBorderTriangle != null) {
        path.append(this.highFarBorderTriangle, false) ;
      }

      // draw low-out values
      if (this.lowFarBorderTriangle != null) {
        path.append(this.lowFarBorderTriangle, false) ;
      }

      AffineTransform transform = new AffineTransform() ;
      transform.translate(this.leftX, this.bottomY) ;
      transform.rotate(getRotate()) ;
      transformedShape = transform.createTransformedShape(path) ;
    }
  }

  /**
   * <zh>ȵͼ.</zh>
   * <en>Returns the hotspot shape.</en>
   *
   * @return Shape <zh>ȵͼ (Ϊnull).</zh>
   * <en>The hotspot shape (maybe null).</en>
   */
  protected Shape getHotspotShape() {
    AffineTransform transform = AffineTransform.getTranslateInstance(this.leftX, this.bottomY) ;
    transform.rotate(getRotate()) ;

    Rectangle2D rect = new Rectangle2D.Double(0, -(height - topLineLength), width,
                                              height - topLineLength - bottomLineLength) ;
    return transform.createTransformedShape(rect) ;
  }

  /**
   * <zh>ͼε.</zh>
   * <en>The type for describe this shape.</en>
   *
   * @return ShapeType <zh>{@link ShapeType}ඨĳ (Ϊnull).</zh>
   * <en>The constant value in class {@link ShapeType}.</en>
   */
  public ShapeType getShapeType() {
    return ShapeType.SHAPE_BOXANDWHISKER ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates whether some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument; false
   *  otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    if (!(obj instanceof BoxAndWhiskerChartShape)) {
      return false ;
    }

    final double theta = 0.00001 ;
    BoxAndWhiskerChartShape that = (BoxAndWhiskerChartShape) obj ;
    if (!EqualsUtil.equals(this.leftX, that.leftX, theta)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.bottomY, that.bottomY, theta)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.width, that.width, theta)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.height, that.height, theta)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.topLineLength, that.topLineLength, theta)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.bottomLineLength, that.bottomLineLength, theta)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.avgEllipse, that.avgEllipse)) {
      return false ;
    }
    if (this.lineY != that.lineY) {
      return false ;
    }
    if (!EqualsUtil.equals(this.artifactPaint, that.artifactPaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.averageValueEllipse, that.averageValueEllipse)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.highFarBorderTriangle, that.highFarBorderTriangle)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.lowFarBorderTriangle, that.lowFarBorderTriangle)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    double hashCode = super.hashCode() ;
    hashCode = 37 * hashCode + this.leftX ;
    hashCode = 37 * hashCode + this.bottomY ;
    hashCode = 37 * hashCode + this.width ;
    hashCode = 37 * hashCode + this.height ;
    hashCode = 37 * hashCode + this.topLineLength ;
    hashCode = 37 * hashCode + this.bottomLineLength ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.avgEllipse) ;
    hashCode = 37 * hashCode + this.lineY ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.artifactPaint) ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.averageValueEllipse) ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.highFarBorderTriangle) ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.lowFarBorderTriangle) ;

    return (int) hashCode ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object. </en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[leftX=" + this.leftX
      + ",bottomY=" + this.bottomY
      + ",width=" + this.width
      + ",height=" + this.height
      + ",topLineLength=" + this.topLineLength
      + ",bottomLineLength=" + this.bottomLineLength
      + ",lineY=" + this.lineY
      + "]" ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the output stream.
   *
   * @throws IOException  if there is an I/O error.
   */
  private void writeObject(ObjectOutputStream stream) throws IOException {
    stream.defaultWriteObject() ;
    SerialUtil.writeShape(stream, this.avgEllipse) ;

    if (this.averageValueEllipse == null) {
      stream.writeBoolean(true) ;
    } else {
      stream.write(this.averageValueEllipse.length) ;
      for (int i = 0 ; i < this.averageValueEllipse.length ; i++) {
        SerialUtil.writeShape(stream, this.averageValueEllipse[i]) ;
      }
    }

    SerialUtil.writeShape(stream, this.highFarBorderTriangle) ;
    SerialUtil.writeShape(stream, this.lowFarBorderTriangle) ;

    SerialUtil.writePaint(stream, this.artifactPaint) ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the input stream.
   *
   * @throws IOException  if there is an I/O error.
   * @throws ClassNotFoundException  if there is a classpath problem.
   */
  private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
    stream.defaultReadObject() ;
    this.avgEllipse = (Ellipse2D) SerialUtil.readShape(stream) ;

    boolean isNull = stream.readBoolean() ;
    if (isNull) {
      this.averageValueEllipse = null ;
    } else {
      int count = stream.readInt() ;
      this.averageValueEllipse = new Ellipse2D[count] ;
      for (int i = 0 ; i < count ; i++) {
        this.averageValueEllipse[i] = (Ellipse2D) SerialUtil.readShape(stream) ;
      }
    }

    this.highFarBorderTriangle = SerialUtil.readShape(stream) ;
    this.lowFarBorderTriangle = SerialUtil.readShape(stream) ;

    this.artifactPaint = SerialUtil.readPaint(stream) ;
  }
}
