package com.ijchart.xychart.shape ;

import java.awt.Font ;
import java.awt.Graphics2D ;
import java.awt.Polygon ;
import java.awt.Shape ;
import java.awt.geom.AffineTransform ;
import java.awt.geom.GeneralPath ;
import java.awt.geom.Line2D ;
import java.awt.geom.Point2D ;
import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;

import com.ijchart.xychart.constant.ShapeType ;
import com.ijchart.xychart.ui.Graphics2DState ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.StrokeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ͼ.</zh>
 * <en>Draw a line {@link com.ijchart.xychart.shape.IChartShape}.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class DualBreakLineChartShape extends AbstractRectangleTextChartShape implements
  Serializable, Cloneable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -8310766581350360197L ;

  /**
   * <zh>һx.</zh>
   * <en>The first point's x coordinate.</en>
   */
  private double x1 ;

  /**
   * <zh>һy.</zh>
   * <en>The first point's y coordinate.</en>
   */
  private double y1 ;

  /**
   * <zh>ڶx.</zh>
   * <en>The second point's x coordinate.</en>
   */
  private double x2 ;

  /**
   * <zh>ڶy.</zh>
   * <en>The second point's y coordinate.</en>
   */
  private double y2 ;

  /**
   * <zh>x.</zh>
   * <en>The third point's x coordinate.</en>
   */
  private double x3 ;

  /**
   * <zh>y.</zh>
   * <en>The third point's y coordinate.</en>
   */
  private double y3 ;

  /**
   * <zh>һӵ(x1,y1)(x2,y2)ٵ(x3,y3).</zh>
   * <en>Draw a break line from point (x1,y1) to point (x2,y2) to point (x3,y3).</en>
   *
   * @param drawArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The draw area (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   * @param font <zh> (Ϊnull),fontΪnull,
   * Ϊ{@link com.ijchart.xychart.shape.IChartShape#DEFAULT_TEXT_FONT}.</zh>
   * <en>The font (null permitted).</en>
   * @param x1 double <zh>һx.</zh>
   * <en>The first point's x coordinate.</en>
   * @param y1 double <zh>һy.</zh>
   * <en>The first point's y coordinate.</en>
   * @param x2 double <zh>ڶx.</zh>
   * <en>The second point's x coordinate.</en>
   * @param y2 double <zh>ڶy.</zh>
   * <en>The second point's y coordinate.</en>
   * @param x3 double <zh>x.</zh>
   * <en>The third point's x coordinate.</en>
   * @param y3 double <zh>y.</zh>
   * <en>The third point's y coordinate.</en>
   */
  public DualBreakLineChartShape(Rectangle2D drawArea, String text, Font font, double x1,
                                 double y1, double x2, double y2, double x3, double y3) {
    super(drawArea, text, font) ;
    checkPointsValid(x1, y1, x2, y2, x3, y3) ;
    this.x1 = x1 ;
    this.y1 = y1 ;
    this.x2 = x2 ;
    this.y2 = y2 ;
    this.x3 = x3 ;
    this.y3 = y3 ;
  }

  /**
   * <zh>һӵp1p2.</zh>
   * <en>Draw a break line from point (p1) to point (p2).</en>
   *
   * @param drawArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The draw area (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   * @param font <zh> (Ϊnull),fontΪnull,
   * Ϊ{@link com.ijchart.xychart.shape.IChartShape#DEFAULT_TEXT_FONT}.</zh>
   * <en>The font (null permitted).</en>
   * @param p1 Point2D <zh>һ (Ϊnull).</zh>
   * <en>The first point (null not permitted).</en>
   * @param p2 Point2D <zh>ڶ (Ϊnull).</zh>
   * <en>The second point (null not permitted).</en>
   * @param p3 Point2D <zh> (Ϊnull).</zh>
   * <en>The third point (null not permitted).</en>
   */
  public DualBreakLineChartShape(Rectangle2D drawArea, String text, Font font, Point2D p1,
                                 Point2D p2, Point2D p3) {
    super(drawArea, text, font) ;
    if (p1 == null) {
      throw new IllegalArgumentException("Null 'p1' point.") ;
    }
    if (p2 == null) {
      throw new IllegalArgumentException("Null 'p2' point.") ;
    }
    if (p3 == null) {
      throw new IllegalArgumentException("Null 'p3' point.") ;
    }
    checkPointsValid(p1.getX(), p1.getY(), p2.getX(), p2.getY(), p3.getX(), p3.getY()) ;

    this.x1 = p1.getX() ;
    this.y1 = p1.getY() ;
    this.x2 = p2.getX() ;
    this.y2 = p2.getY() ;
    this.x3 = p3.getX() ;
    this.y3 = p3.getY() ;
    setForceShowAllText(true) ;
  }

  /**
   * <zh>Ƿ,ɵǴֱ.</zh>
   * <en>Check whether or not the three point is valid.</en>
   * @param x1 double
   * @param y1 double
   * @param x2 double
   * @param y2 double
   * @param x3 double
   * @param y3 double
   * @throws IllegalArgumentException
   */
  private static void checkPointsValid(double x1, double y1, double x2, double y2, double x3,
                                       double y3) throws
    IllegalArgumentException {

    if (!(x2 == x1 || y2 == y1)) {
      throw new IllegalArgumentException("Must be x2=x1 or y2=y1.") ;
    }
    if (!(x2 == x3 || y2 == y3)) {
      throw new IllegalArgumentException("Must be x2=x3 or y2=y3.") ;
    }
  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻƴͼ</zh>
   * <en>Draw the ChartShape on a Java 2d graphics device(such as the screen or the printer).</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull).</zh>
   * <en>The graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  public void draw(Graphics2D g2) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (!isVisible()) {
      return ;
    }

    Graphics2DState state = new Graphics2DState(g2) ;
    state.saveOldState() ;

    g2.translate(this.x1, this.y1) ;
    g2.rotate(getRotate()) ;
    beforeDraw(g2) ;

    g2.setStroke(getBorderStroke()) ;
    g2.setPaint(getBorderPaint()) ;

    Polygon polygon = new Polygon() ;
    polygon.addPoint(0, 0) ;
    polygon.addPoint((int) (this.x2 - this.x1), (int) (this.y2 - this.y1)) ;
    polygon.addPoint((int) (this.x3 - this.x1), (int) (this.y3 - this.y1)) ;
    polygon.addPoint((int) (this.x2 - this.x1), (int) (this.y2 - this.y1)) ;
    polygon.addPoint(0, 0) ;
    g2.draw(polygon) ;

    Rectangle2D drawArea = getDrawArea() ;
    Rectangle2D cloneArea = new Rectangle2D.Double(drawArea.getX() - this.x1,
                                                   drawArea.getY() - this.y1, drawArea.getWidth(),
                                                   drawArea.getHeight()) ;
    drawTextWithTrimMarginRect(g2, cloneArea) ;

    afterDraw(g2) ;
    g2.rotate( -getRotate()) ;
    state.restoreOldState() ;
    drawSelect(g2) ;
  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻͼεѡв,Ϊͼοܻص,ԻͼʱҪȻеͼκ,
   * ٵô˷ѡв.</zh>
   * <en>Draw the selected ChartShape on a Java 2d graphics device(such
   * as the screen or the printer).</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>the graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  protected void drawSelect(Graphics2D g2) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }

    if (isSelected()) {
      Point2D.Double[] points = new Point2D.Double[] {
                                new Point2D.Double(0, 0),
                                new Point2D.Double(this.x2 - this.x1, this.y2 - this.y1),
                                new Point2D.Double(this.x3 - this.x1, this.y3 - this.y1)} ;
      drawAnchorByPoints(g2, this.x1, this.y1, points) ;
    }
  }

  /**
   * <zh>ƶͼ,X᷽ƫtx,Y᷽ƫty,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Concatenates the current Graphics2D Transform with a translation transform.and sends
   * a {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.
   * Subsequent rendering is translated by the specified
   * distance relative to the previous position.
   * This is equivalent to calling transform(T), where T is an
   * AffineTransform represented by the following matrix:
   * <pre>
   *		[   1    0    tx  ]
   *		[   0    1    ty  ]
   *		[   0    0    1   ]
   * </pre>.</en>
   *
   * @param tx double <zh>X᷽ľ.</zh>
   * <en>The distance to translate along the x-axis.</en>
   * @param ty double <zh>Y᷽ľ.</zh>
   * <en>The distance to translate along the y-axis.</en>
   */
  public void translate(double tx, double ty) {
    if (!(tx == 0 && ty == 0)) {
      this.x1 += tx ;
      this.y1 += ty ;
      this.x2 += tx ;
      this.y2 += ty ;
      this.x3 += tx ;
      this.y3 += ty ;

      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ͼϽǵ(ͼεת).</zh>
   * <en>Returns the top left corner coordinates of this shape (not consider rotation).</en>
   *
   * @return Point2D <zh>ֵ.</zh>
   * <en>The coordinate.</en>
   */
  public Point2D getTopLeftPoint() {
    double minX = Math.min(Math.min(this.x1, this.x2), this.x3) ;
    double minY = Math.min(Math.min(this.y1, this.y2), this.y3) ;

    return new Point2D.Double(minX, minY) ;
  }

  /**
   * <zh>ʱͼεĵ.</zh>
   * <en>Returns the lock coordinates of this shape.</en>
   *
   * @return Point2D <zh>ֵ.</zh>
   * <en>The coordinate.</en>
   */
  public Point2D getLockPoint() {
    Line2D line = null ;
    if (this.x1 == this.x2) {
      line = new Line2D.Double(this.x2 - this.x1, this.y2 - this.y1,
                               this.x3 - this.x1, this.y3 - this.y1) ;
    } else {
      line = new Line2D.Double(0, 0, this.x2 - this.x1, this.y2 - this.y1) ;
    }

    AffineTransform transform = AffineTransform.getTranslateInstance(this.x1, this.y1) ;
    transform.rotate(getRotate()) ;
    Shape rotateShape = transform.createTransformedShape(line) ;
    Rectangle2D rotateBound = rotateShape.getBounds2D() ;

    return new Point2D.Double(rotateBound.getCenterX(), rotateBound.getCenterY()) ;
  }

  /**
   * <zh>任ͼ.</zh>
   * <en>Calculate the transformed shape.</en>
   */
  protected void calculateTransformedShape() {
    if (transformedShape == null) {
      float strokeWidth = StrokeUtil.getStrokeWidth(getBorderStroke()) ;
      if (strokeWidth < 3) { //ʿС3
        strokeWidth = 3 ;
      }
      float halfStrokeWidth = (float) (strokeWidth * 0.5) ;
      final float fX1 = 0 ;
      final float fY1 = 0 ;
      float fX2 = (float) (this.x2 - this.x1) ;
      float fY2 = (float) (this.y2 - this.y1) ;
      float fX3 = (float) (this.x3 - this.x1) ;
      float fY3 = (float) (this.y3 - this.y1) ;

      GeneralPath path = new GeneralPath() ;
      path.moveTo(0, 0) ;
      if (fX2 == fX1) { //ڶ͵һ㴹ֱ
        if (fY2 <= fY1) { //ڶڵһ֮
          int sign = (fX3 >= fX2) ? -1 : 1 ;
          path.moveTo(sign * halfStrokeWidth, 0) ;
          path.lineTo(sign * halfStrokeWidth, fY2 - halfStrokeWidth) ;
          path.lineTo(fX3, fY3 - halfStrokeWidth) ;
          path.lineTo(fX3, fY3 + halfStrokeWidth) ;
          path.lineTo(fX2 - sign * halfStrokeWidth, fY2 + halfStrokeWidth) ;
          path.lineTo(fX1 - sign * halfStrokeWidth, fY1) ;
        } else { //ڶڵһ֮
          int sign = (fX3 >= fX2) ? -1 : 1 ;
          path.moveTo(sign * halfStrokeWidth, 0) ;
          path.lineTo(sign * halfStrokeWidth, fY2 + halfStrokeWidth) ;
          path.lineTo(fX3, fY3 + halfStrokeWidth) ;
          path.lineTo(fX3, fY3 - halfStrokeWidth) ;
          path.lineTo(fX2 - sign * halfStrokeWidth, fY2 - halfStrokeWidth) ;
          path.lineTo(fX1 - sign * halfStrokeWidth, fY1) ;
        }
      }

      if (fY2 == fY1) { //ڶ͵һƽ
        if (fX2 >= fX1) { //ڶڵһұ
          int sign = (fY3 <= fY2) ? 1 : -1 ;
          path.moveTo(0, sign * halfStrokeWidth) ;
          path.lineTo(fX2 + halfStrokeWidth, sign * halfStrokeWidth) ;
          path.lineTo(fX3 + halfStrokeWidth, fY3) ;
          path.lineTo(fX3 - halfStrokeWidth, fY3) ;
          path.lineTo(fX2 - halfStrokeWidth, fY2 - sign * halfStrokeWidth) ;
          path.lineTo(0, -sign * halfStrokeWidth) ;
        } else { //ڶڵһ
          int sign = (fY3 <= fY2) ? 1 : -1 ;
          path.moveTo(0, sign * halfStrokeWidth) ;
          path.lineTo(fX2 - halfStrokeWidth, sign * halfStrokeWidth) ;
          path.lineTo(fX3 - halfStrokeWidth, fY3) ;
          path.lineTo(fX3 + halfStrokeWidth, fY3) ;
          path.lineTo(fX2 + halfStrokeWidth, fY2 - sign * halfStrokeWidth) ;
          path.lineTo(0, -sign * halfStrokeWidth) ;
        }
      }
      path.closePath() ;

      AffineTransform transform = new AffineTransform() ;
      transform.translate(this.x1, this.y1) ;
      transform.rotate(getRotate()) ;
      transformedShape = transform.createTransformedShape(path) ;
    }

  }

  /**
   * <zh>õһx,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the first point's x coordinate,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param x1 double
   */
  public void setX1(double x1) {
    if (this.x1 != x1) {
      this.x1 = x1 ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>صһx.</zh>
   * <en>Returns the first point's x coordinate.</en>
   *
   * @return double
   */
  public double getX1() {
    return this.x1 ;
  }

  /**
   * <zh>õһy,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the first point's y coordinate,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param y1 double
   */
  public void setY1(double y1) {
    if (this.y1 != y1) {
      this.y1 = y1 ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>صһy.</zh>
   * <en>Returns the first point's y coordinate.</en>
   *
   * @return double
   */
  public double getY1() {
    return this.y1 ;
  }

  /**
   * <zh>õڶx,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the second point's x coordinate,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param x2 double
   */
  public void setX2(double x2) {
    if (this.x2 != x2) {
      this.x2 = x2 ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>صڶx.</zh>
   * <en>Returns the second point's x coordinate.</en>
   *
   * @return double
   */
  public double getX2() {
    return this.x2 ;
  }

  /**
   * <zh>õڶy,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the second point's y coordinate,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param y2 double
   */
  public void setY2(double y2) {
    if (this.y2 != y2) {
      this.y2 = y2 ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>صڶy.</zh>
   * <en>Returns the second point's y coordinate.</en>
   * @return double
   */
  public double getY2() {
    return this.y2 ;
  }

  /**
   * <zh>õx,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the third point's x coordinate,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param x3 double
   */
  public void setX3(double x3) {
    if (this.x3 != x3) {
      this.x3 = x3 ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>صx.</zh>
   * <en>Returns the third point's x coordinate.</en>
   *
   * @return double
   */
  public double getX3() {
    return this.x3 ;
  }

  /**
   * <zh>õy,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the third point's y coordinate,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param y3 double
   */
  public void setY3(double y3) {
    if (this.y3 != y3) {
      this.y3 = y3 ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>صy.</zh>
   * <en>Returns the third point's y coordinate.</en>
   * @return double
   */
  public double getY3() {
    return this.y3 ;
  }

  /**
   * <zh>õһ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the first point coordinate,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param x double <zh>һx.</zh>
   * <en>The first point's x coordinate.</en>
   * @param y double <zh>һy.</zh>
   * <en>The first point's y coordinate.</en>
   */
  public void setPoint1(double x, double y) {
    this.x1 = x ;
    this.y1 = y ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>õڶ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the second point coordinate,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param x double <zh>ڶx.</zh>
   * <en>The second point's x coordinate.</en>
   * @param y double <zh>ڶy.</zh>
   * <en>The second point's y coordinate.</en>
   */
  public void setPoint2(double x, double y) {
    this.x2 = x ;
    this.y2 = y ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>õ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the third point coordinate,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param x double <zh>x.</zh>
   * <en>The third point's x coordinate.</en>
   * @param y double <zh>y.</zh>
   * <en>The third point's y coordinate.</en>
   */
  public void setPoint3(double x, double y) {
    this.x3 = x ;
    this.y3 = y ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>õһ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the first point coordinate,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param p Point2D <zh>һ (Ϊnull).</zh>
   * <en>The first point coordinate (null not permitted).</en>
   */
  public void setPoint1(Point2D p) {
    if (p == null) {
      throw new IllegalArgumentException("Null 'p' argument.") ;
    }

    this.x1 = p.getX() ;
    this.y1 = p.getY() ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>صһ.</zh>
   * <en>Returns the first point coordinate.</en>
   *
   * @return Point2D <zh>һ (Ϊnull).</zh>
   * <en>Returns the first point coordinate (never null).</en>
   */
  public Point2D getPoint1() {
    return new Point2D.Double(this.x1, this.y1) ;
  }

  /**
   * <zh>õڶ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the second point coordinate,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param p Point2D <zh>ڶ (Ϊnull).</zh>
   * <en>The second point coordinate (null not permitted).</en>
   */
  public void setPoint2(Point2D p) {
    if (p == null) {
      throw new IllegalArgumentException("Null 'p' argument.") ;
    }

    this.x2 = p.getX() ;
    this.y2 = p.getY() ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>صڶ.</zh>
   * <en>Returns the second point coordinate.</en>
   *
   * @return Point2D <zh>ڶ (Ϊnull).</zh>
   * <en>Returns the second point coordinate (never null).</en>
   */
  public Point2D getPoint2() {
    return new Point2D.Double(this.x2, this.y2) ;
  }

  /**
   * <zh>õ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the third point coordinate,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param p Point2D <zh> (Ϊnull).</zh>
   * <en>The third point coordinate (null not permitted).</en>
   */
  public void setPoint3(Point2D p) {
    if (p == null) {
      throw new IllegalArgumentException("Null 'p' argument.") ;
    }

    this.x3 = p.getX() ;
    this.y3 = p.getY() ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>ص.</zh>
   * <en>Returns the third point coordinate.</en>
   *
   * @return Point2D <zh> (Ϊnull).</zh>
   * <en>Returns the third point coordinate (never null).</en>
   */
  public Point2D getPoint3() {
    return new Point2D.Double(this.x3, this.y3) ;
  }

  /**
   * <zh>ȵͼ.</zh>
   * <en>Returns the hotspot shape.</en>
   *
   * @return Shape <zh>ȵͼ (Ϊnull).</zh>
   * <en>The hotspot shape (maybe null).</en>
   */
  protected Shape getHotspotShape() {
    calculateTransformedShape() ;
    return transformedShape ;
  }

  /**
   * <zh>ͼε.</zh>
   * <en>The type for describe this shape.</en>
   *
   * @return ShapeType <zh>{@link ShapeType}ඨĳ (Ϊnull).</zh>
   * <en>The constant value in class {@link ShapeType}.</en>
   */
  public ShapeType getShapeType() {
    return ShapeType.SHAPE_DUAL_BREAK_LINE ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates whether some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument; false
   *  otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    if (!(obj instanceof DualBreakLineChartShape)) {
      return false ;
    }

    final double theta = 0.00001 ;
    DualBreakLineChartShape that = (DualBreakLineChartShape) obj ;
    if (!EqualsUtil.equals(this.x1, that.x1, theta)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.y1, that.y1, theta)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.x2, that.x2, theta)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.y2, that.y2, theta)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.x3, that.x3, theta)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.y3, that.y3, theta)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    double hashCode = super.hashCode() ;
    hashCode = 37 * hashCode + this.x1 ;
    hashCode = 37 * hashCode + this.y1 ;
    hashCode = 37 * hashCode + this.x2 ;
    hashCode = 37 * hashCode + this.y2 ;
    hashCode = 37 * hashCode + this.x3 ;
    hashCode = 37 * hashCode + this.y3 ;

    return (int) hashCode ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object. </en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[x1=" + this.x1 + ",y1=" + this.y1 + ",x2=" + this.x2 + ",y2=" + this.y2 + ",x3="
      + this.x3 + ",y3=" + this.y3 + "]" ;
  }
}
