package com.ijchart.xychart.shape ;

import java.awt.Font ;
import java.awt.Graphics2D ;
import java.awt.Shape ;
import java.awt.geom.AffineTransform ;
import java.awt.geom.Arc2D ;
import java.awt.geom.Area ;
import java.awt.geom.Ellipse2D ;
import java.awt.geom.GeneralPath ;
import java.awt.geom.Point2D ;
import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;

import com.ijchart.xychart.constant.HorizontalAlignment ;
import com.ijchart.xychart.constant.ShapeType ;
import com.ijchart.xychart.constant.TransformShapeType ;
import com.ijchart.xychart.ui.Graphics2DState ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.GeometryUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>һԲڲⲿʾ.</zh>
 * <en>Draw a text inside or outside a eclipse,with or not have a border.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class EllipseTextChartShape extends AbstractRectangleTextChartShape implements Cloneable,
  Serializable {

  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -1395248184199467210L ;

  /**
   * <zh>ȱʡͼεı任.</zh>
   * <en>The default transform type.</en>
   */
  public static final TransformShapeType DEFAULT_TRANSFORM_SHAPE_TYPE = TransformShapeType.INSIDE ;

  /**
   * <zh>ͼεı任.</zh>
   * <en>The transform type.</en>
   */
  private TransformShapeType transformShapeType ;

  /**
   * <zh>drawAreaʾtext.</zh>
   * <en>Draw text in area drawArea.</en>
   *
   * @param drawArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The draw area (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   */
  public EllipseTextChartShape(Rectangle2D drawArea, String text) {
    this(drawArea, text, null) ;
  }

  /**
   * <zh>drawAreaʾtext.</zh>
   * <en>Draw text in area drawArea.</en>
   *
   * @param drawArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The draw area (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   * @param font <zh> (Ϊnull),fontΪnull,
   * Ϊ{@link com.ijchart.xychart.shape.IChartShape#DEFAULT_TEXT_FONT}.</zh>
   * <en>The specified font (null permitted).If font is null,
   * {@link com.ijchart.xychart.shape.IChartShape#DEFAULT_TEXT_FONT} is used.</en>
   */
  public EllipseTextChartShape(Rectangle2D drawArea, String text, Font font) {
    super(drawArea, text, font) ;

    this.transformShapeType = DEFAULT_TRANSFORM_SHAPE_TYPE ;
    setTextHorizontalAlign(HorizontalAlignment.CENTER) ;
  }

  /**
   * <zh>ͼεı任.</zh>
   * <en>Returns the transform type.</en>
   *
   * @return TransformShapeType <zh>任 (Ϊnull).</zh>
   * <en>The transform type (never null).</en>
   */
  public TransformShapeType getTransformShapeType() {
    return this.transformShapeType ;
  }

  /**
   * <zh>ͼεı任,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the transform type and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param type TransformShapeType <zh>任 (Ϊnull).</zh>
   * <en>The transform type (null not permitted).</en>
   */
  public void setTransformShapeType(TransformShapeType type) {
    if (type == null) {
      throw new IllegalArgumentException("Null 'type' argument") ;
    }

    if (!this.transformShapeType.equals(type)) {
      this.transformShapeType = type ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻƴͼ</zh>
   * <en>Draw the ChartShape on a Java 2d graphics device(such as the screen or the printer).</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>The graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  public void draw(Graphics2D g2) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (!isVisible()) {
      return ;
    }

    Graphics2DState state = new Graphics2DState(g2) ;
    state.saveOldState() ;

    Rectangle2D drawArea = getDrawArea() ;
    g2.translate(drawArea.getX(), drawArea.getY()) ;
    g2.rotate(getRotate()) ;
    beforeDraw(g2) ;
//    g2.setClip(cloneArea) ;

    Rectangle2D cloneArea = new Rectangle2D.Double(0, 0, drawArea.getWidth(), drawArea.getHeight()) ;
    Ellipse2D ellipse = new Ellipse2D.Double(0, 0, drawArea.getWidth(), drawArea.getHeight()) ;
    drawBackground(g2, ellipse) ;
    drawBorder(g2, ellipse) ;

    drawTextWithTrimMarginRect(g2, cloneArea) ;
    afterDraw(g2) ;
    g2.rotate( -getRotate()) ;

    state.restoreOldState() ;
    drawSelect(g2) ;
  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻͼεѡв,Ϊͼοܻص,ԻͼʱҪȻеͼκ,
   * ٵô˷ѡв.</zh>
   * <en>Draw the selected ChartShape on a Java 2d graphics device(such as the screen or the
   * printer).</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>the graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  protected void drawSelect(Graphics2D g2) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }

    if (isSelected()) {
      Rectangle2D drawArea = getDrawArea() ;
      double areaWidth = drawArea.getWidth() ;
      double areaHeight = drawArea.getHeight() ;
      Point2D.Double[] points = new Point2D.Double[] {new Point2D.Double(areaWidth / 2, 0),
                                new Point2D.Double(0, areaHeight / 2),
                                new Point2D.Double(areaWidth, areaHeight / 2),
                                new Point2D.Double(areaWidth / 2, areaHeight)} ;

      drawAnchorByPoints(g2, drawArea.getX(), drawArea.getY(), points) ;
    }

  }

  /**
   * <zh>任ͼ.</zh>
   * <en>Calculate the transformed shape.</en>
   */
  protected void calculateTransformedShape() {
    if (transformedShape == null) {
      AffineTransform transform = new AffineTransform() ;
      Rectangle2D area = getDrawArea() ;
      transform.translate(area.getX(), area.getY()) ;
      transform.rotate(getRotate()) ;

      if (this.transformShapeType.equals(TransformShapeType.INSIDE)) {
        Ellipse2D newArea = new Ellipse2D.Double(0, 0, area.getWidth(), area.getHeight()) ;
        transformedShape = transform.createTransformedShape(newArea) ;
      } else if (this.transformShapeType.equals(TransformShapeType.OUTLINE)) {
        final float gapSize = 1.0F, doubleGapSize = 2.0F ;
        Ellipse2D outline = new Ellipse2D.Double( -gapSize, -gapSize,
                                                 area.getWidth() + doubleGapSize,
                                                 area.getHeight() + doubleGapSize) ;
        Ellipse2D inside = new Ellipse2D.Double(gapSize, gapSize, area.getWidth() - doubleGapSize,
                                                area.getHeight() - doubleGapSize) ;
        Area outlineArea = new Area(outline) ;
        Area insideArea = new Area(inside) ;
        outlineArea.subtract(insideArea) ;
        transformedShape = transform.createTransformedShape(outlineArea) ;
      }
    }
  }

  /**
   * <zh>ȵͼ.</zh>
   * <en>Returns the hotspot shape.</en>
   *
   * @return Shape <zh>ȵͼ (Ϊnull).</zh>
   * <en>The hotspot shape (maybe null).</en>
   */
  protected Shape getHotspotShape() {
    Rectangle2D drawArea = getDrawArea() ;
    AffineTransform transform = AffineTransform.getTranslateInstance(drawArea.getX(), drawArea.getY()) ;
    transform.rotate(getRotate()) ;

    Arc2D arc = new Arc2D.Double(0, 0, drawArea.getWidth(), drawArea.getHeight(), 0, 360, Arc2D.PIE) ;
    GeneralPath arcPath = GeometryUtil.getArcPath(arc) ;

    return transform.createTransformedShape(arcPath) ;
  }

  /**
   * <zh>ͼε.</zh>
   * <en>The type for describe this shape.</en>
   * @return ShapeType <zh>{@link com.ijchart.xychart.constant.ShapeType}ඨĳ
   * (Ϊnull).</zh>
   * <en>The constant value in class {@link com.ijchart.xychart.constant.ShapeType}.</en>
   */
  public ShapeType getShapeType() {
    return ShapeType.SHAPE_TEXT_IN_ELLIPSE ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates whether some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument;
   * false otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return false ;
    }
    if (!(obj instanceof EllipseTextChartShape)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    EllipseTextChartShape that = (EllipseTextChartShape) obj ;
    if (!EqualsUtil.equals(this.transformShapeType, that.transformShapeType)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int hashCode = super.hashCode() ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.transformShapeType) ;

    return hashCode ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object. </en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[text=" + getText() + "]" ;
  }

}
