package com.ijchart.xychart.shape ;

import java.awt.AlphaComposite ;
import java.awt.Font ;
import java.awt.Graphics2D ;
import java.awt.Paint ;
import java.awt.Shape ;
import java.awt.geom.AffineTransform ;
import java.awt.geom.Point2D ;
import java.awt.geom.Rectangle2D ;
import java.io.IOException ;
import java.io.ObjectInputStream ;
import java.io.ObjectOutputStream ;
import java.io.Serializable ;

import com.ijchart.xychart.constant.HorizontalAlignment ;
import com.ijchart.xychart.constant.ShapeType ;
import com.ijchart.xychart.ui.Graphics2DState ;
import com.ijchart.xychart.ui.Rectangle ;
import com.ijchart.xychart.ui.RectangleInsets ;
import com.ijchart.xychart.util.AnchorUtil ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.util.SerialUtil ;
import com.ijchart.xychart.util.StringUtil ;
import com.ijchart.xychart.util.TextUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ʾͼΧڵͼ.ÿͼͼκֱǩ.ע:еĳЩԱͼûӰ.</zh>
 * <en>A legend item chart shape,composite by a chart shape and a label.
 * Note:Some super class method does not affect graphics</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class LegendItemChartShape extends AbstractChartShape implements Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -5715021675582103546L ;

  /**
   * <zh>ȱʡǷʾ.</zh>
   * <en>A default flag that controls whether or not the label is drawn.</en>
   */
  public static final boolean DEFAULT_DRAW_LABEL = true ;

  /**
   * <zh>ȱʡͼκ֮Ŀհ.</zh>
   * <en>The default space between shape and label.</en>
   */
  public static final double DEFAULT_GAP_SIZE = 5 ;

  /**
   * <zh>ͼͼ.</zh>
   * <en>The chart shape.</en>
   */
  private IChartShape shape ;

  /**
   * <zh>Ƿʾ.</zh>
   * <en>A flag that controls whether or not the label is drawn.</en>
   */
  private boolean drawLabel ;

  /**
   * <zh>ڲʹõPaint</zh>
   * <en>The data Paint</en>
   */
  private transient Paint labelPaint ;

  /**
   * <zh>ʾ.</zh>
   * <en>The label text.</en>
   */
  private String label ;

  /**
   * <zh>ֵ.</zh>
   * <en>The label font.</en>
   */
  private Font labelFont ;

  /**
   * <zh>ͼκ֮Ŀհ.</zh>
   * <en>The space between shape and label.</en>
   */
  private double gapSize ;

  /**
   * <zh>ͼͼεĻ.</zh>
   * <en>The area of draw this legend shape.</en>
   */
  private transient Rectangle2D drawArea ;

  /**
   * <zh>ͼͼκͱǩͼͼ.</zh>
   * <en>Construct a legend shape by chart shape and label.</en>
   *
   * @param shape IChartShape <zh>ͼͼ (Ϊnull).</zh>
   * <en>The chart shape (null not permitted).</en>
   * @param label String <zh>ǩ (Ϊnull).</zh>
   * <en>The label (null permitted).</en>
   */
  public LegendItemChartShape(IChartShape shape, String label) {
    if (shape == null) {
      throw new IllegalArgumentException("Null 'shape' argument.") ;
    }
    if (shape instanceof LegendItemChartShape) {
      throw new IllegalArgumentException("'shape' can not be LegendItemChartShape.") ;
    }

    this.shape = shape ;

    this.drawLabel = true ;
    this.label = label ;
    this.labelPaint = DEFAULT_CONTENT_PAINT ;
    this.labelFont = DEFAULT_TEXT_FONT ;

    this.gapSize = DEFAULT_GAP_SIZE ;
    this.drawArea = null ;
  }

  /**
   * <zh>ͼͼ.</zh>
   * <en>Returns the chart shape.</en>
   *
   * @return IChartShape <zh>ͼͼ (Ϊnull).</zh>
   * <en>The chart shape (never null).</en>
   */
  public IChartShape getShape() {
    return this.shape ;
  }

  /**
   * <zh>ͼͼ,м{@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the chart shape and sends
   * a {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   * @param shape IChartShape <zh>ͼͼ (Ϊnull).</zh>
   * <en>The chart shape (null not permitted).</en>
   */
  public void setShape(IChartShape shape) {
    if (shape == null) {
      throw new IllegalArgumentException("Null 'shape' argument.") ;
    }
    if (shape instanceof LegendItemChartShape) {
      throw new IllegalArgumentException("'shape' can not be LegendItemChartShape.") ;
    }

    this.shape = shape ;
  }

  /**
   * <zh>Ƿʾ,
   * м{@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets a flag that controls whether or not the label is drawn and sends
   * a {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param drawLabel boolean
   */
  public void setDrawLabel(boolean drawLabel) {
    if (this.drawLabel != drawLabel) {
      this.drawLabel = drawLabel ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>Ƿʾ.</zh>
   * <en>Returns a flag that controls whether or not the label is drawn.</en>
   *
   * @return boolean
   */
  public boolean isDrawLabel() {
    return this.drawLabel ;
  }

  /**
   * <zh>ͼֵPaint,
   * м{@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼</zh>
   * <en>Sets the shape label Paint and sends
   * a {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>ͼPaint(Ϊnull)</zh>
   *  <en>The ChartShape label Paint(null not permitted).</en>
   */
  public void setLabelPaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument.") ;
    }

    this.labelPaint = paint ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>Paint</zh>
   * <en>Get the ChartShape label Paint</en>
   *
   * @return Paint <zh>Paint</zh>
   * <en>The label Paint</en>
   */
  public Paint getLabelPaint() {
    return this.labelPaint ;
  }

  /**
   * <zh>ʾ.</zh>
   * <en>Returns the label text.</en>
   *
   * @return String <zh>ʾ,Ϊnull.</zh>
   * <en>The label text,maybe null.</en>
   */
  public String getLabel() {
    return this.label ;
  }

  /**
   * <zh>ʾ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the label text,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners..</en>
   * @param label String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The label text (null permitted).</en>
   */
  public void setLabel(String label) {
    if (!EqualsUtil.equals(this.label, label)) {
      this.label = label ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ֵ.</zh>
   * <en>Returns the label font.</en>
   *
   * @return Font
   */
  public Font getLabelFont() {
    return this.labelFont ;
  }

  /**
   * <zh>ֵ,
   * м{@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the label font and sends
   * a {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param font Font <zh>ֵ (Ϊnull).</zh>
   * <en>The label font (null not permitted).</en>
   */
  public void setLabelFont(Font font) {
    if (font == null) {
      throw new IllegalArgumentException("Null 'font' argument.") ;
    }

    if (!this.labelFont.equals(font)) {
      this.labelFont = font ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ͼκ֮Ŀհ.</zh>
   * <en>Returns the space between shape and label.</en>
   *
   * @return double <zh>ͼκ֮Ŀհ.</zh>
   * <en>The space between shape and label.</en>
   */
  public double getGapSize() {
    return this.gapSize ;
  }

  /**
   * <zh>ͼκ֮Ŀհ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the space between shape and label and sends
   * a {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param gap double <zh>ͼκ֮Ŀհ.</zh>
   * <en>The space between shape and label.</en>
   */
  public void setGapSize(double gap) {
    if (this.gapSize != gap) {
      this.gapSize = gap ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ͼͼεĻ.</zh>
   * <en>Returns the area of draw this legend shape.</en>
   *
   * @return Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The draw area (maybe null).</en>
   */
  public Rectangle2D getDrawArea() {
    return this.drawArea ;
  }

  /**
   * <zh>ͼͼεĻ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the area of draw this legend shape and sends
   * a {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param drawArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The draw area (null permitted).</en>
   */
  public void setDrawArea(Rectangle2D drawArea) {
    if (!(this.drawArea == null && drawArea == null)) {
      this.drawArea = drawArea ;
      if (this.shape instanceof PaintScaleLegendChartShape) {
        ((PaintScaleLegendChartShape)this.shape).setDrawArea(drawArea) ;
      }
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ƻҪ{@link Rectangle}.</zh>
   * <en>Estimate the {@link Rectangle} of draw this shape.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>The graphics device (such as the screen or the printer) (null not permitted).</en>
   * @return Rectangle <zh>Ҫ.</zh>
   * <en>The Rectangle of draw this shape.</en>
   */
  public Rectangle estimateDrawBound(Graphics2D g2) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }

    double width = 0, height = 0 ;

    Rectangle2D rect = this.shape.getBound() ;
    width = rect.getWidth() ;
    height = rect.getHeight() ;

    if (this.drawLabel && !StringUtil.isEmptyWithTrim(this.label)) {
      Graphics2DState state = new Graphics2DState(g2) ;
      state.saveOldState() ;

      g2.setFont(this.labelFont) ;
      Rectangle2D textBound = TextUtil.getStringBound(g2, this.label) ;
      width += textBound.getWidth() ;
      width += this.gapSize ;

      height = Math.max(height, textBound.getHeight()) ;

      state.restoreOldState() ;
    }

    RectangleInsets margin = getMargin() ; //հ״С
    width += margin.getLeft() + margin.getRight() ;

    return new Rectangle(width, height) ;
  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻƴͼ.ʱshapeλ.</zh>
   * <en>Draw the ChartShape on a Java 2d graphics device(such as the screen or the printer).
   * The shape's coordinate is ignored.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>The graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  public void draw(Graphics2D g2) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (!isVisible()) {
      return ;
    }

    if (this.drawArea == null) {
      return ;
    }

    Rectangle2D shapeBound = this.shape.getBound() ;

    Rectangle2D cloneArea = (Rectangle2D)this.drawArea.clone() ;
    RectangleInsets margin = getMargin() ; //հ״С
    margin.trim(cloneArea) ;
    if (shapeBound.getWidth() > cloneArea.getWidth() //̫С
        || shapeBound.getHeight() > cloneArea.getHeight()) {
      return ;
    }

    //ͼ
    this.shape.translate(cloneArea.getX(), cloneArea.getY()) ;
    Point2D leftTopOfShape = this.shape.getTopLeftPoint() ;

    double offsetY = 0 ;
    if (shapeBound.getHeight() < cloneArea.getHeight()) { //ͼθ߶ȱַ߶С,ͼδֱж뵽
      offsetY = 0.5 * (cloneArea.getHeight() - shapeBound.getHeight()) ;
    }
    this.shape.translate(cloneArea.getX() - leftTopOfShape.getX(),
                         cloneArea.getY() - leftTopOfShape.getY() + offsetY) ; //ƶͼεĿλ
    beforeDraw(g2) ;
    this.shape.draw(g2) ;

    cloneArea.setRect(cloneArea.getX() + shapeBound.getWidth() + this.gapSize, cloneArea.getY(),
                      cloneArea.getWidth() - shapeBound.getWidth() - this.gapSize,
                      cloneArea.getHeight()) ; //ȥshape

    if (cloneArea.getWidth() < 0) { //ûʾַĿռ
      return ;
    }

    //ʾ
    RectangleTextChartShape textShape = new RectangleTextChartShape(cloneArea, this.label) ;
    textShape.setTextFont(getLabelFont()) ;
    textShape.setFillShape(false) ;
    textShape.setDrawBorder(false) ;
    textShape.setTextMaxShowLine(1) ;
    textShape.setTextHorizontalAlign(HorizontalAlignment.LEFT) ;
    textShape.draw(g2) ;

    afterDraw(g2) ;

    drawSelect(g2) ;
  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻͼεѡв,Ϊͼοܻص,ԻͼʱҪȻеͼκ,
   * ٵô˷ѡв.</zh>
   * <en>Draw the selected ChartShape on a Java 2d graphics device(such as the screen or the
   * printer).</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>the graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  protected void drawSelect(Graphics2D g2) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (this.drawArea == null) {
      return ;
    }

    if (isSelected()) {
      Graphics2DState state = new Graphics2DState(g2) ;
      state.saveOldState() ;
      g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, getSelectRectAlpha())) ;

      double rectWidth = getSelectRectWidth() ;
      g2.translate(this.drawArea.getX(), this.drawArea.getY()) ;
      g2.setPaint(getSelectAnchorPaint()) ;
      //ê
      AnchorUtil.drawAnchor(g2, this.drawArea, rectWidth) ;

      state.restoreOldState() ;
    }

  }

  /**
   * <zh>任ͼ.</zh>
   * <en>Calculate the transformed shape.</en>
   */
  protected void calculateTransformedShape() {
    if (transformedShape == null && this.drawArea != null) {
      AffineTransform transform = new AffineTransform() ;
      transform.translate(this.drawArea.getX(), this.drawArea.getY()) ;
      transform.rotate(getRotate()) ;
      Rectangle2D newArea = new Rectangle2D.Double(0, 0, this.drawArea.getWidth(),
                                                   this.drawArea.getHeight()) ;

      transformedShape = transform.createTransformedShape(newArea) ;
    }
  }

  /**
   * <zh>ƶͼ,X᷽ƫtx,Y᷽ƫty,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Concatenates the current Graphics2D Transform with a translation transform.and sends
   * a {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.
   * Subsequent rendering is translated by the specified
   * distance relative to the previous position.
   * This is equivalent to calling transform(T), where T is an
   * AffineTransform represented by the following matrix:
   * <pre>
   *		[   1    0    tx  ]
   *		[   0    1    ty  ]
   *		[   0    0    1   ]
   * </pre>.</en>
   *
   * @param tx double <zh>X᷽ľ.</zh>
   * <en>The distance to translate along the x-axis.</en>
   * @param ty double <zh>Y᷽ľ.</zh>
   * <en>The distance to translate along the y-axis.</en>
   */
  public void translate(double tx, double ty) {
    if (!(tx == 0 && ty == 0)) {
      this.drawArea.setRect(drawArea.getX() + tx, drawArea.getY() + ty,
                            drawArea.getWidth(), drawArea.getHeight()) ;

      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ͼϽǵ.</zh>
   * <en>Returns the top left corner coordinates of this shape.</en>
   *
   * @return Point2D <zh>ֵ.</zh>
   * <en>The coordinate.</en>
   */
  public Point2D getTopLeftPoint() {
    return new Point2D.Double(this.drawArea.getX(), this.drawArea.getY()) ;
  }

  /**
   * <zh>γȵ.</zh>
   * <en>Returns a string containing the coordinates for a given shape.This
   * string is intended for use in an image map.</en>
   *
   * @return String <zh>ȵ.</zh>
   * <en>The coordinates of hotspot.</en>
   */
  public String getHotspotPolyCoords() {
    if (this.drawArea != null) {
      return super.getHotspotPolyCoords(null) ;
    } else {
      return this.shape.getHotspotPolyCoords(null) ;
    }
  }

  /**
   * <zh>ȵͼ.</zh>
   * <en>Returns the hotspot shape.</en>
   *
   * @return Shape <zh>ȵͼ (Ϊnull).</zh>
   * <en>The hotspot shape (maybe null).</en>
   */
  protected Shape getHotspotShape() {
    if (this.drawArea != null) {
      AffineTransform transform = AffineTransform.getTranslateInstance(this.drawArea.getX(),
                                                                       this.drawArea.getY()) ;
      transform.rotate(getRotate()) ;

      Rectangle2D rect = new Rectangle2D.Double(0, 0, drawArea.getWidth(), drawArea.getHeight()) ;
      return transform.createTransformedShape(rect) ;
    }

    return null ;
  }

  /**
   * <zh>ͼε.</zh>
   * <en>The type for describe this shape.</en>
   * @return ShapeType <zh>{@link com.ijchart.xychart.constant.ShapeType}ඨĳ
   * (Ϊnull).</zh>
   * <en>The constant value in class {@link com.ijchart.xychart.constant.ShapeType}.</en>
   */
  public ShapeType getShapeType() {
    return ShapeType.SHAPE_LEGEND ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int result = super.hashCode() ;
    result = 37 * result + HashcodeUtil.hashCode(this.shape) ;
    result = 37 * result + HashcodeUtil.hashCode(this.drawLabel) ;
    result = 37 * result + HashcodeUtil.hashCode(this.labelPaint) ;
    result = 37 * result + HashcodeUtil.hashCode(this.label) ;
    result = 37 * result + HashcodeUtil.hashCode(this.labelFont) ;
    result = 37 * result + (int)this.gapSize ;
    result = 37 * result + HashcodeUtil.hashCode(this.drawArea) ;

    return result ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates whether some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument; false otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }
    if (!(obj instanceof LegendItemChartShape)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    LegendItemChartShape that = (LegendItemChartShape) obj ;
    if (!EqualsUtil.equals(this.shape, that.shape)) {
      return false ;
    }
    if (this.drawLabel != that.drawLabel) {
      return false ;
    }
    if (!EqualsUtil.equals(this.labelPaint, that.labelPaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.label, that.label)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.labelFont, that.labelFont)) {
      return false ;
    }
    if (this.gapSize != that.gapSize) {
      return false ;
    }
    if (!EqualsUtil.equals(this.drawArea, that.drawArea)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>л</zh>
   * <en>Provides serialization support</en>
   *
   * @param stream <zh></zh>
   * <en>the output stream</en>
   * @throws IOException  if there is an I/O error.
   */
  private void writeObject(ObjectOutputStream stream) throws IOException {
    stream.defaultWriteObject() ;

    SerialUtil.writePaint(stream, this.labelPaint) ;
    SerialUtil.writeShape(stream, this.drawArea) ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the input stream.
   *
   * @throws IOException  if there is an I/O error.
   * @throws ClassNotFoundException  if there is a classpath problem.
   */
  private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
    stream.defaultReadObject() ;

    this.labelPaint = SerialUtil.readPaint(stream) ;
    this.drawArea = (Rectangle2D) SerialUtil.readShape(stream) ;
  }

}
