package com.ijchart.xychart.shape ;

import java.awt.AlphaComposite ;
import java.awt.BasicStroke ;
import java.awt.Color ;
import java.awt.Font ;
import java.awt.Graphics2D ;
import java.awt.Paint ;
import java.awt.Shape ;
import java.awt.Stroke ;
import java.awt.geom.AffineTransform ;
import java.awt.geom.Arc2D ;
import java.awt.geom.GeneralPath ;
import java.awt.geom.Line2D ;
import java.awt.geom.Point2D ;
import java.awt.geom.Rectangle2D ;
import java.io.IOException ;
import java.io.ObjectInputStream ;
import java.io.ObjectOutputStream ;
import java.io.Serializable ;
import java.text.NumberFormat ;
import java.util.ArrayList ;
import java.util.Collection ;
import java.util.Collections ;
import java.util.List ;

import com.ijchart.xychart.constant.ShapeType ;
import com.ijchart.xychart.ui.Graphics2DState ;
import com.ijchart.xychart.util.AngleUtil ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.GeometryUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.util.SerialUtil ;
import com.ijchart.xychart.util.StringUtil ;
import com.ijchart.xychart.util.TextUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>Ǳͼεͼ.ע:еĳЩԱͼûӰ.</zh>
 * <en>A arc shape for meter chart.Note:Some super class method does not affect graphics</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class MeterArcChartShape extends AbstractChartShape implements Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 4599910853361252882L ;

  /**
   * <zh>ȱʡǷư뾶.</zh>
   * <en>A default boolean flag controls whether or not draw arc radius.</en>
   */
  public static final boolean DEFAULT_DRAW_RADIUS = false ;

  /**
   * <zh>ȱʡǷʾ̶ֵ.</zh>
   * <en>A default flag that controls whether or not the tick label is drawn.</en>
   */
  public static final boolean DEFAULT_DRAW_TICK_LABEL = true ;

  /**
   * <zh>ȱʡǷʾ̶.</zh>
   * <en>A default flag that controls whether or not the tick is drawn.</en>
   */
  public static final boolean DEFAULT_DRAW_TICK = true ;

  /**
   * <zh>ȱʡĿ̶ȵĳ.</zh>
   * <en>The default tick length.</en>
   */
  public static final double DEFAULT_TICK_LENGTH = 8 ;

  /**
   * <zh>ȱʡĻƿ̶ȵpaint.</zh>
   * <en>The default paint used to draw ticks.</en>
   */
  public static final Paint DEFAULT_TICK_PAINT = Color.lightGray ;

  /**
   * <zh>ȱʡĻƿ̶ȵstroke.</zh>
   * <en>The default stroke used to draw ticks.</en>
   */
  public static final Stroke DEFAULT_TICK_STROKE = new BasicStroke(2.5F) ;

  /**
   * <zh>ƿ̶ȱǩ.</zh>
   * <en>The font used to draw tick label.</en>
   */
  public static final Font DEFAULT_TICK_LABEL_FONT = new Font("SansSerif", Font.BOLD, 10) ;

  /**
   * <zh>ƿ̶ȱǩpaint.</zh>
   * <en>The paint used to draw tick label.</en>
   */
  public static final Paint DEFAULT_TICK_LABEL_PAINT = Color.lightGray ;

  /**
   * <zh>ȱʡı߽С.</zh>
   * <en>The default border size.</en>
   */
  public static final double DEFAULT_BORDER_SIZE = 2F ;

  /**
   * <zh>ȱʡĸʽ̶ֵ.</zh>
   * <en>The default tick label format.</en>
   */
  public static final NumberFormat DEFAULT_TICK_LABEL_FORMAT = NumberFormat.getInstance() ;

  /**
   * <zh>Ʊ߿paint.</zh>
   * <en>The paint used to draw border.</en>
   */
  public static final Paint DEFAULT_ARC_BORDER_PAINT = Color.lightGray ;

  /**
   * <zh>ͼ.</zh>
   * <en>The arc shape.</en>
   */
  private Arc2D arc ;

  /**
   * <zh>Ƿư뾶.</zh>
   * <en>A boolean flag controls whether or not draw arc radius.</en>
   */
  private boolean drawRadius ;

  /**
   * <zh>Ƿʾ̶ֵ.</zh>
   * <en>A flag that controls whether or not the tick label is drawn.</en>
   */
  private boolean drawTickLabel ;

  /**
   * <zh>Ƿʾ̶.</zh>
   * <en>A flag that controls whether or not the tick is drawn.</en>
   */
  private boolean drawTick ;

  /**
   * <zh>ʼƿ̶ȵĽǶ.</zh>
   * <en>The start angle for draw tick.</en>
   */
  private double startTickAngle ;

  /**
   * <zh>̶ȵĴС(λΪǶ).</zh>
   * <en>The tick size (measured in degree).</en>
   */
  private double tickSizeAngle ;

  /**
   * <zh>̶ȵĳ.</zh>
   * <en>The tick length.</en>
   */
  private double tickLength ;

  /**
   * <zh>ƿ̶ȵpaint.</zh>
   * <en>The paint used to draw ticks.</en>
   */
  private transient Paint tickPaint ;

  /**
   * <zh>ƿ̶ȵstroke.</zh>
   * <en>The stroke used to draw ticks.</en>
   */
  private transient Stroke tickStroke ;

  /**
   * <zh>߽С.</zh>
   * <en>The border size.</en>
   */
  private double borderSize ;

  /**
   * <zh>ƿ̶ȱǩ.</zh>
   * <en>The font used to draw tick label.</en>
   */
  private Font tickLabelFont ;

  /**
   * <zh>ƿ̶ȱǩpaint.</zh>
   * <en>The paint used to draw tick label.</en>
   */
  private transient Paint tickLabelPaint ;

  /**
   * <zh>ʽ̶ֵ.</zh>
   * <en>The tick label format.</en>
   */
  private NumberFormat tickLabelFormat ;

  /**
   * <zh>̶ֵ.</zh>
   * <en>The array of tick labels.</en>
   */
  private Number[] tickLabels ;

  /**
   * <zh>̶ֵͼμ.</zh>
   * <en>The list of tick line shapes.</en>
   */
  private List tickLineShapes ;

  /**
   * <zh>̶ȱǩͼμ.</zh>
   * <en>The list of tick label shapes.</en>
   */
  private List tickLabelShapes ;

  /**
   * <zh>.</zh>
   * <en>A constructor.</en>
   *
   * @param arc Arc2D <zh>ͼ (Ϊnull).</zh>
   * <en>A arc shape (null not permitted).</en>
   */
  public MeterArcChartShape(Arc2D arc) {
    this(arc, 0, 1, null) ;
  }

  /**
   * <zh>.</zh>
   * <en>A constructor.</en>
   *
   * @param arc Arc2D <zh>ͼ (Ϊnull).</zh>
   * <en>A arc shape (null not permitted).</en>
   * @param startTickAngle double <zh>ʼƿ̶ȵĽǶ.</zh>
   * <en>The start angle for draw tick (measured in degree).</en>
   * @param tickAngleSize double <zh>̶ȵĴС(λΪǶ).</zh>
   * <en>The tick size (measured in degree).</en>
   * @param tickLabels Number[] <zh>̶ֵ (Ϊnull).</zh>
   * <en>The array of tick labels (null permitted).</en>
   */
  public MeterArcChartShape(Arc2D arc, double startTickAngle, double tickAngleSize,
                            Number[] tickLabels) {
    if (arc == null) {
      throw new IllegalArgumentException("Null 'arc' argument.") ;
    }
    if (tickAngleSize <= 0) {
      throw new IllegalArgumentException("Argument 'tickAngleSize' must be >=0.") ;
    }

    this.arc = arc ;
    this.drawRadius = DEFAULT_DRAW_RADIUS ;
    this.drawTickLabel = DEFAULT_DRAW_TICK_LABEL ;
    this.drawTick = DEFAULT_DRAW_TICK ;

    this.startTickAngle = startTickAngle ;
    this.tickSizeAngle = tickAngleSize ;
    this.tickLength = DEFAULT_TICK_LENGTH ;
    this.tickPaint = DEFAULT_TICK_PAINT ;
    this.tickStroke = DEFAULT_TICK_STROKE ;
    this.borderSize = DEFAULT_BORDER_SIZE ;

    this.tickLabels = tickLabels ;
    this.tickLabelFormat = DEFAULT_TICK_LABEL_FORMAT ;
    this.tickLabelFont = DEFAULT_TICK_LABEL_FONT ;
    this.tickLabelPaint = DEFAULT_TICK_LABEL_PAINT ;

    this.tickLineShapes = new ArrayList() ;
    this.tickLabelShapes = new ArrayList() ;

    setFillShape(true) ;
    setDrawBorder(true) ;
    setBorderPaint(DEFAULT_ARC_BORDER_PAINT) ;
  }

  /**
   * <zh>ͼ.</zh>
   * <en>Returns the arc shape.</en>
   *
   * @return Arc2D <zh>ͼ (Ϊnull).</zh>
   * <en>The arc shape (never null).</en>
   */
  public Arc2D getArc() {
    return this.arc ;
  }

  /**
   * <zh>ͼ,м{@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the arc shape,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param arc Arc2D <zh>ͼ (Ϊnull).</zh>
   * <en>The arc shape (null not permitted).</en>
   */
  public void setArc(Arc2D arc) {
    if (arc == null) {
      throw new IllegalArgumentException("Null 'arc' argument.") ;
    }

    this.arc = arc ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>ر߽С.</zh>
   * <en>Returns the border size.</en>
   *
   * @return double <zh>߽С.</zh>
   * <en>The border size.</en>
   */
  public double getBorderSize() {
    return this.borderSize ;
  }

  /**
   * <zh>ñ߽С,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the border size,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param size double <zh>߽С.</zh>
   * <en>The border size.</en>
   */
  public void setBorderSize(double size) {
    if (this.borderSize != size) {
      this.borderSize = size ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>Ƿư뾶.</zh>
   * <en>A boolean flag controls whether or not draw arc radius.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isDrawRadius() {
    return this.drawRadius ;
  }

  /**
   * <zh>Ƿư뾶,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>A boolean flag controls whether or not draw arc radius,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setDrawRadius(boolean flag) {
    if (this.drawRadius != flag) {
      this.drawRadius = flag ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>Ƿʾ̶.</zh>
   * <en>A flag that controls whether or not the tick is drawn.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isDrawTick() {
    return this.drawTick ;
  }

  /**
   * <zh>Ƿʾ̶,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>A flag that controls whether or not the tick is drawn,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setDrawTick(boolean flag) {
    if (this.drawTick != flag) {
      this.drawTick = flag ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>Ƿʾ̶ֵ.</zh>
   * <en>A flag that controls whether or not the tick label is drawn.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isDrawTickLabel() {
    return this.drawTickLabel ;
  }

  /**
   * <zh>Ƿʾ̶ֵ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>A flag that controls whether or not the tick label is drawn,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setDrawTickLabel(boolean flag) {
    if (this.drawTickLabel != flag) {
      this.drawTickLabel = flag ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ؿʼƿ̶ȵĽǶ.</zh>
   * <en>Returns the start angle for draw tick.</en>
   *
   * @return double <zh>Ƕ.</zh>
   * <en>The angle.</en>
   */
  public double getStartTickAngle() {
    return this.startTickAngle ;
  }

  /**
   * <zh>ÿʼƿ̶ȵĽǶ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the start angle for draw tick,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param angle double <zh>Ƕ.</zh>
   * <en>The angle.</en>
   */
  public void setStartTickAngle(double angle) {
    if (this.startTickAngle != angle) {
      this.startTickAngle = angle ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ػƿ̶ȱǩ.</zh>
   * <en>Returns the font used to draw tick label.</en>
   *
   * @return Font <zh>.</zh>
   * <en>The font.</en>
   */
  public Font getTickLabelFont() {
    return this.tickLabelFont ;
  }

  /**
   * <zh>ûƿ̶ȱǩ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the font used to draw tick label,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param font Font <zh> (Ϊnull).</zh>
   * <en>The font (null not permitted).</en>
   */
  public void setTickLabelFont(Font font) {
    if (font == null) {
      throw new IllegalArgumentException("Null 'font' argument.") ;
    }

    this.tickLabelFont = font ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>ظʽ̶ֵ.</zh>
   * <en>Returns the tick label format.</en>
   *
   * @return NumberFormat <zh>ʽ.</zh>
   * <en>The number format object.</en>
   */
  public NumberFormat getTickLabelFormat() {
    return this.tickLabelFormat ;
  }

  /**
   * <zh>øʽ̶ֵ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the tick label format,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param format NumberFormat <zh>ʽ (Ϊnull).</zh>
   * <en>The number format object (null permitted).</en>
   */
  public void setTickLabelFormat(NumberFormat format) {
    if (!(this.tickLabelFormat == null && format == null)) {
      this.tickLabelFormat = format ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ػƿ̶ȱǩpaint.</zh>
   * <en>Returns the paint used to draw tick label.</en>
   *
   * @return Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  public Paint getTickLabelPaint() {
    return this.tickLabelPaint ;
  }

  /**
   * <zh>ûƿ̶ȱǩpaint,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the paint used to draw tick label,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (null not permitted).</en>
   */
  public void setTickLabelPaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument.") ;
    }

    this.tickLabelPaint = paint ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>ؿ̶ֵ.</zh>
   * <en>Returns the array of tick labels.</en>
   *
   * @return Number[] <zh>̶ֵ (Ϊnull).</zh>
   * <en>The array of tick labels (maybe null).</en>
   */
  public Number[] getTickLabels() {
    return this.tickLabels ;
  }

  /**
   * <zh>ÿ̶ֵ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the array of tick labels,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param labels Number[] <zh>̶ֵ (Ϊnull).</zh>
   * <en>The array of tick labels (null permitted).</en>
   */
  public void setTickLabels(Number[] labels) {
    if (!(this.tickLabels == null && labels == null)) {
      this.tickLabels = labels ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ؿ̶ȵĳ.</zh>
   * <en>Returns the tick length.</en>
   *
   * @return double <zh>.</zh>
   * <en>The tick length.</en>
   */
  public double getTickLength() {
    return this.tickLength ;
  }

  /**
   * <zh>ÿ̶ȵĳ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the tick length,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param length double <zh> (0).</zh>
   * <en>The tick length (must be greater than 0).</en>
   */
  public void setTickLength(double length) {
    if (length <= 0) {
      throw new IllegalArgumentException("Must be 'length'>0.") ;
    }

    if (this.tickLength != length) {
      this.tickLength = length ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ػƿ̶ȵpaint.</zh>
   * <en>Returns the paint used to draw ticks.</en>
   *
   * @return Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  public Paint getTickPaint() {
    return this.tickPaint ;
  }

  /**
   * <zh>ûƿ̶ȵpaint,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the paint used to draw ticks,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (null not permitted).</en>
   */
  public void setTickPaint(Paint paint) {
    if (!(this.tickPaint == null && paint == null)) {
      this.tickPaint = paint ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ؿ̶ȵĴС(λΪǶ).</zh>
   * <en>Returns the tick size (measured in degree).</en>
   *
   * @return double <zh>̶ȴС.</zh>
   * <en>The tick size.</en>
   */
  public double getTickSizeAngle() {
    return tickSizeAngle ;
  }

  /**
   * <zh>ÿ̶ȵĴС(λΪǶ),м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the tick size (measured in degree),and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param size double <zh>̶ȴС (0).</zh>
   * <en>The tick size (must be greater than 0).</en>
   */
  public void setTickSizeAngle(double size) {
    if (size <= 0) {
      throw new IllegalArgumentException("Must be 'size'>0.") ;
    }

    if (this.tickSizeAngle != size) {
      this.tickSizeAngle = size ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ػƿ̶ȵstroke.</zh>
   * <en>Returns the stroke used to draw ticks.</en>
   *
   * @return Stroke <zh> (Ϊnull).</zh>
   * <en>The stroke (never null).</en>
   */
  public Stroke getTickStroke() {
    return this.tickStroke ;
  }

  /**
   * <zh>ûƿ̶ȵstroke,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the stroke used to draw ticks,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param stroke Stroke <zh> (Ϊnull).</zh>
   * <en>The stroke (null not permitted).</en>
   */
  public void setTickStroke(Stroke stroke) {
    if (!(this.tickStroke == null && stroke == null)) {
      this.tickStroke = stroke ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ؿ̶ֵͼμ,˼ǲ޸ĵ.</zh>
   * <en>Returns the list of tick line shapes and is unmodifiable.</en>
   *
   * @return Collection <zh>ͼμ (Ϊnull).</zh>
   * <en>The collection of shapes (never null).</en>
   */
  public Collection getTickLineShapes() {
    return Collections.unmodifiableCollection(this.tickLineShapes) ;
  }

  /**
   * <zh>ؿ̶ȱǩͼμ,˼ǲ޸ĵ.</zh>
   * <en>Returns the list of tick label shapes and is unmodifiable.</en>
   *
   * @return Collection <zh>ͼμ (Ϊnull).</zh>
   * <en>The collection of shapes (never null).</en>
   */
  public Collection getTickLabelShapes() {
    return Collections.unmodifiableCollection(this.tickLabelShapes) ;
  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻƴͼ.ʱshapeλ.</zh>
   * <en>Draw the ChartShape on a Java 2d graphics device(such as the screen or the printer).
   * The shape's coordinate is ignored.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>The graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  public void draw(Graphics2D g2) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (!isVisible()) {
      return ;
    }

    this.tickLineShapes.clear() ;
    this.tickLabelShapes.clear() ;

    Graphics2DState state = new Graphics2DState(g2) ;
    state.saveOldState() ;
    beforeDraw(g2) ;

    double angleStart = this.arc.getAngleStart() ; //ʼǶ
    double angleExtent = this.arc.getAngleExtent() ; //
    double arcX = this.arc.getX() ;
    double arcY = this.arc.getY() ;
    double arcWidth = this.arc.getWidth() ;
    double arcHeight = this.arc.getHeight() ;
    Arc2D newArc = new Arc2D.Double(0.0D, 0.0D, arcWidth, arcHeight,
                                    angleStart, angleExtent, this.arc.getArcType()) ;

    g2.translate(arcX, arcY) ;
    g2.rotate(getRotate()) ;
//    g2.setClip(newArc) ;
    drawBackground(g2, newArc) ;
    drawBorder(g2, newArc) ;

    //Ʊ߿,ҪǷư뾶
    if (this.drawRadius) {
      newArc.setArcType(Arc2D.PIE) ;
      drawBorder(g2, newArc) ;
    } else {
      newArc.setArcType(Arc2D.OPEN) ;
      drawBorder(g2, newArc) ;
    }

    //ƿ̶
    double halfArcWidth = 0.5 * arcWidth ;
    double halfArcHeight = 0.5 * arcHeight ;

    double radiusWidth = halfArcWidth + this.borderSize ;
    double radiusHeight = halfArcHeight + this.borderSize ;

    double drawAngle = this.startTickAngle % 360 ;
    int index = -1 ; //

    double actualMaxAngle = GeometryUtil.calculateArcEndPointAngle(newArc) ;
    while (drawAngle <= actualMaxAngle + 0.001 && this.tickSizeAngle != 0) { //ڿ̶֮
      if (drawAngle < angleStart) {
        drawAngle += this.tickSizeAngle ;
        continue ;
      }

      double radians = drawAngle / 180 * Math.PI ;
      double tanAngle = Math.tan(radians) ;
      int quadrant = AngleUtil.getQuadrantByDegreeAngle(drawAngle) ;

      double x = Math.sqrt(1 / (1 / Math.pow(radiusWidth, 2)
                                + Math.pow(tanAngle, 2) / Math.pow(radiusHeight, 2))) ;
      double y = Math.abs(x * tanAngle) ;

      if (this.drawTick) { //Իƿ̶
        g2.setPaint(this.tickPaint) ;
        g2.setStroke(this.tickStroke) ;

        double offsetX = Math.abs(this.tickLength * Math.cos(radians)) ; //xƫƳ
        double offsetY = Math.abs(this.tickLength * Math.sin(radians)) ; //yƫƳ
        double startX = 0, startY = 0, endX = 0, endY = 0 ; //ʼͽ
        if (quadrant == 1 || quadrant == 4) {
          endX = halfArcWidth + x ;
          startX = endX - offsetX ;
        } else {
          endX = halfArcWidth - x ;
          startX = endX + offsetX ;
        }
        if (quadrant == 1 || quadrant == 2) {
          endY = halfArcHeight - y ;
          startY = endY + offsetY ;
        } else {
          endY = halfArcHeight + y ;
          startY = endY - offsetY ;
        }

        Line2D tickLine = new Line2D.Double(startX, startY, endX, endY) ;
        Line2D beforeTranslateTickLine = new Line2D.Double(arcX + startX, arcY + startY,
                                                           arcX + endX, arcY + endY) ;
        LineChartShape lineShape = new LineChartShape(beforeTranslateTickLine) ;
        this.tickLineShapes.add(lineShape) ;

        //ƿ̶ȱǩ
        index++ ;
        if (this.drawTickLabel && this.tickLabels != null && index < this.tickLabels.length) {
          g2.setFont(this.tickLabelFont) ;
          g2.setPaint(this.tickLabelPaint) ;

          String label = null ;
          Number value = this.tickLabels[index] ;
          if (value != null) {
            if (this.tickLabelFormat != null) {
              label = this.tickLabelFormat.format(value.doubleValue()) ;
            } else {
              label = value.toString() ;
            }
          }

          if (!StringUtil.isEmpty(label)) {
            Rectangle2D textRect = TextUtil.getStringBound(g2, label) ;
            double textWidth = textRect.getWidth() ;
            double textHeight = textRect.getHeight() ;

            double labelX = startX, labelY = startY ;
            if (drawAngle == 90 || drawAngle == 270) {
              labelX -= textWidth * 0.5 ;
            } else if (drawAngle < 90 || drawAngle > 270) {
              labelX -= textWidth ;
            }

            final double xGap = 2 ;
            if (drawAngle <= 45 || drawAngle >= 315) {
              labelX -= xGap ;
            } else if (drawAngle >= 135 && drawAngle <= 225) {
              labelX += xGap ;
            }

            if ((drawAngle > 135 && drawAngle < 225) || drawAngle > 315 || drawAngle < 45) {
              labelY -= textHeight * 0.5 ;
            } else if (drawAngle >= 225 && drawAngle <= 315) {
              labelY -= textHeight ;
            }

            Rectangle2D rect = new Rectangle2D.Double(labelX + arcX, labelY + arcY,
                                                      textWidth, textHeight) ;
            RectangleTextChartShape textShape =
              new RectangleTextChartShape(rect, label, this.tickLabelFont) ;
            textShape.setTextFont(this.tickLabelFont) ;
            textShape.setTextPaint(this.tickLabelPaint) ;
            textShape.setForceShowAllText(true) ;
            this.tickLabelShapes.add(textShape) ;
          }
        }

      }

      drawAngle += this.tickSizeAngle ;
    }

    afterDraw(g2) ;
    g2.rotate( -getRotate()) ;
    state.restoreOldState() ;
    drawSelect(g2) ;
  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻͼεѡв,Ϊͼοܻص,ԻͼʱҪȻеͼκ,
   * ٵô˷ѡв.</zh>
   * <en>Draw the selected ChartShape on a Java 2d graphics device(such as the screen or the
   * printer).</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>The graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  protected void drawSelect(Graphics2D g2) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }

    if (isSelected()) {
      Graphics2DState state = new Graphics2DState(g2) ;
      state.saveOldState() ;

      double angleStart = this.arc.getAngleStart() ; //ʼǶ
      double angleExtent = this.arc.getAngleExtent() ; //
      double arcX = this.arc.getX() ;
      double arcY = this.arc.getY() ;
      double arcWidth = this.arc.getWidth() ;
      double arcHeight = this.arc.getHeight() ;
      Arc2D newArc = new Arc2D.Double(0.0D, 0.0D, arcWidth, arcHeight,
                                      angleStart, angleExtent, this.arc.getArcType()) ;

      g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, getSelectRectAlpha())) ;
      g2.setPaint(getSelectAnchorPaint()) ;

      g2.translate(arcX, arcY) ;
      g2.rotate(getRotate()) ;
//      g2.setClip(newArc) ;

      double rectWidth = getSelectRectWidth() ;
      final double halfRect = rectWidth / 2 ;

      //ê
      double halfArcWidth = 0.5 * this.arc.getWidth() ;
      double halfArcHeight = 0.5 * this.arc.getHeight() ;

      double drawAngle = this.arc.getAngleStart() ;
      final double anchorAngle = 30 ;

      int ratio = 1 ; //ϵ
      double actualMaxAngle = GeometryUtil.calculateArcEndPointAngle(newArc) ;
      if (actualMaxAngle - angleStart > Math.abs(angleExtent)) {
        actualMaxAngle -= 360 ;
        ratio = -1 ;
      }

      int anchorCount = 0 ; //êĸ
      Point2D startPoint = newArc.getStartPoint() ;
      Point2D endPoint = newArc.getEndPoint() ;
      boolean drawEnd = false ; //ǷendPointڵê
      double endX = 0, endY = 0 ; //
      while (drawAngle <= actualMaxAngle + 0.001 && this.tickSizeAngle != 0) { //ڿ̶֮
        double radians = drawAngle / 180 * Math.PI ;
        double tanAngle = Math.tan(radians) ;
        int quadrant = AngleUtil.getQuadrantByDegreeAngle(drawAngle) ;

        if (anchorCount == 0) {
          endX = startPoint.getX() ;
          endY = startPoint.getY() ;
        } else {
          double x = Math.sqrt(1 / (1 / Math.pow(halfArcWidth, 2)
                                    + Math.pow(tanAngle, 2) / Math.pow(halfArcHeight, 2))) ;
          double y = Math.abs(x * tanAngle) ;

          if (quadrant == 1 || quadrant == 4) {
            endX = halfArcWidth + x ;
          } else {
            endX = halfArcWidth - x ;
          }
          if (quadrant == 1 || quadrant == 2) {
            endY = halfArcHeight - y ;
          } else {
            endY = halfArcHeight + y ;
          }
        }

        g2.fill(new Rectangle2D.Double(endX - halfRect, endY - halfRect, rectWidth, rectWidth)) ;

        drawAngle += ratio * anchorAngle ;
        anchorCount++ ;

        if (ratio == 1 && drawAngle > actualMaxAngle) {
          if (anchorCount >= 2) {
            break ;
          } else {
            drawAngle = actualMaxAngle ;
            drawEnd = true ;
          }
        } else if (ratio == -1 && drawAngle < actualMaxAngle) {
          if (anchorCount >= 2) {
            break ;
          } else {
            drawAngle = actualMaxAngle ;
            drawEnd = true ;
          }
        }
      }
      ;

      if (!drawEnd) { //һêյľ̫,ûбҪê
        double distince = Math.sqrt((endX - endPoint.getX()) * (endX - endPoint.getX())
                                    + (endY - endPoint.getY()) * (endY - endPoint.getY())) ;
        if (distince > rectWidth * 3) {
          g2.fill(new Rectangle2D.Double(endPoint.getX() - halfRect, endPoint.getY() - halfRect,
                                         rectWidth, rectWidth)) ;
        }
      }

      g2.rotate( -getRotate()) ;
      state.restoreOldState() ;
    }
  }

  /**
   * <zh>任ͼ.</zh>
   * <en>Calculate the transformed shape.</en>
   */
  protected void calculateTransformedShape() {
    if (transformedShape == null) {
      double angleStart = this.arc.getAngleStart() ; //ʼǶ
      double angleExtent = this.arc.getAngleExtent() ; //
      double arcX = this.arc.getX() ;
      double arcY = this.arc.getY() ;
      Arc2D newArc = new Arc2D.Double(0.0D, 0.0D, this.arc.getWidth(), this.arc.getHeight(),
                                      angleStart, angleExtent, this.arc.getArcType()) ;

      AffineTransform transform = new AffineTransform() ;
      transform.translate(arcX, arcY) ;
      transform.rotate(getRotate()) ;
      transformedShape = transform.createTransformedShape(newArc) ;
    }
  }

  /**
   * <zh>ƶͼ,X᷽ƫtx,Y᷽ƫty,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Concatenates the current Graphics2D Transform with a translation transform.and sends
   * a {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.
   * Subsequent rendering is translated by the specified
   * distance relative to the previous position.
   * This is equivalent to calling transform(T), where T is an
   * AffineTransform represented by the following matrix:
   * <pre>
   *		[   1    0    tx  ]
   *		[   0    1    ty  ]
   *		[   0    0    1   ]
   * </pre>.</en>
   *
   * @param tx double <zh>X᷽ľ.</zh>
   * <en>The distance to translate along the x-axis.</en>
   * @param ty double <zh>Y᷽ľ.</zh>
   * <en>The distance to translate along the y-axis.</en>
   */
  public void translate(double tx, double ty) {
    if (!(tx == 0 && ty == 0)) {
      this.arc.setArc(this.arc.getX() + tx, this.arc.getY() + ty,
                      this.arc.getWidth(), this.arc.getHeight(),
                      this.arc.getAngleStart(), this.arc.getAngleExtent(),
                      this.arc.getArcType()) ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ͼϽǵ.</zh>
   * <en>Returns the top left corner coordinates of this shape.</en>
   *
   * @return Point2D <zh>ֵ.</zh>
   * <en>The coordinate.</en>
   */
  public Point2D getTopLeftPoint() {
    return new Point2D.Double(this.arc.getX(), this.arc.getY()) ;
  }

  /**
   * <zh>ʱͼεĵ.</zh>
   * <en>Returns the lock coordinates of this shape.</en>
   *
   * @return Point2D <zh>ֵ.</zh>
   * <en>The coordinate.</en>
   */
  public Point2D getLockPoint() {
    if (this.arc == null) {
      return null ;
    }

    double angleStart = this.arc.getAngleStart() ; //ʼǶ
    double angleExtent = this.arc.getAngleExtent() ; //
    double arcX = this.arc.getX() ;
    double arcY = this.arc.getY() ;
    double arcWidth = this.arc.getWidth() ;
    double arcHeight = this.arc.getHeight() ;
    Arc2D newArc = new Arc2D.Double(0.0D, 0.0D, arcWidth, arcHeight,
                                    angleStart, angleExtent, this.arc.getArcType()) ;

    AffineTransform transform = AffineTransform.getTranslateInstance(arcX, arcY) ;
    transform.rotate(getRotate()) ;
    Shape rotateShape = transform.createTransformedShape(newArc) ;
    Rectangle2D rotateBound = rotateShape.getBounds2D() ;

    return new Point2D.Double(rotateBound.getCenterX(), rotateBound.getCenterY()) ;
  }

  /**
   * <zh>ȵͼ.</zh>
   * <en>Returns the hotspot shape.</en>
   *
   * @return Shape <zh>ȵͼ (Ϊnull).</zh>
   * <en>The hotspot shape (maybe null).</en>
   */
  protected Shape getHotspotShape() {
    GeneralPath arcPath = GeometryUtil.getArcPath(this.arc) ;
    Arc2D newArc = new Arc2D.Double(0.0D, 0.0D, this.arc.getWidth(), this.arc.getHeight(),
                                    this.arc.getAngleStart(), this.arc.getAngleExtent(),
                                    this.arc.getArcType()) ;
    arcPath.lineTo((float) newArc.getCenterX(), (float) newArc.getCenterY()) ;
    arcPath.closePath() ;

    AffineTransform transform = AffineTransform.getTranslateInstance(this.arc.getX(), this.arc.getY()) ;
    transform.rotate(getRotate()) ;

    return transform.createTransformedShape(arcPath) ;
  }

  /**
   * <zh>ͼε.</zh>
   * <en>The type for describe this shape.</en>
   * @return ShapeType <zh>{@link com.ijchart.xychart.constant.ShapeType}ඨĳ
   * (Ϊnull).</zh>
   * <en>The constant value in class {@link com.ijchart.xychart.constant.ShapeType}.</en>
   */
  public ShapeType getShapeType() {
    return ShapeType.SHAPE_METER_ARC ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    double result = super.hashCode() ;
    result = 37 * result + HashcodeUtil.hashCode(this.arc) ;
    result = 37 * result + HashcodeUtil.hashCode(this.drawRadius) ;
    result = 37 * result + HashcodeUtil.hashCode(this.drawTickLabel) ;
    result = 37 * result + HashcodeUtil.hashCode(this.drawTick) ;
    result = 37 * result + this.startTickAngle ;
    result = 37 * result + this.tickSizeAngle ;
    result = 37 * result + this.tickLength ;
    result = 37 * result + HashcodeUtil.hashCode(this.tickPaint) ;
    result = 37 * result + HashcodeUtil.hashCode(this.tickStroke) ;
    result = 37 * result + this.borderSize ;
    result = 37 * result + HashcodeUtil.hashCode(this.tickLabels) ;
    result = 37 * result + HashcodeUtil.hashCode(this.tickLabelFormat) ;
    result = 37 * result + HashcodeUtil.hashCode(this.tickLabelFont) ;
    result = 37 * result + HashcodeUtil.hashCode(this.tickLabelPaint) ;

    return (int) result ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates whether some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument; false otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }
    if (!(obj instanceof MeterArcChartShape)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    MeterArcChartShape that = (MeterArcChartShape) obj ;
    if (this.startTickAngle != that.startTickAngle) {
      return false ;
    }
    if (this.tickSizeAngle != that.tickSizeAngle) {
      return false ;
    }
    if (this.tickLength != that.tickLength) {
      return false ;
    }
    if (this.borderSize != that.borderSize) {
      return false ;
    }
    if (!EqualsUtil.equals(this.arc, that.arc)) {
      return false ;
    }
    if (this.drawRadius != that.drawRadius) {
      return false ;
    }
    if (this.drawTickLabel != that.drawTickLabel) {
      return false ;
    }
    if (this.drawTick != that.drawTick) {
      return false ;
    }
    if (!EqualsUtil.equals(this.tickPaint, that.tickPaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.tickStroke, that.tickStroke)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.tickLabels, that.tickLabels)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.tickLabelFormat, that.tickLabelFormat)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.tickLabelFont, that.tickLabelFont)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.tickLabelPaint, that.tickLabelPaint)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>л</zh>
   * <en>Provides serialization support</en>
   *
   * @param stream <zh></zh>
   * <en>the output stream</en>
   * @throws IOException  if there is an I/O error.
   */
  private void writeObject(ObjectOutputStream stream) throws IOException {
    stream.defaultWriteObject() ;

    SerialUtil.writePaint(stream, this.tickPaint) ;
    SerialUtil.writeStroke(stream, this.tickStroke) ;
    SerialUtil.writePaint(stream, this.tickLabelPaint) ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the input stream.
   *
   * @throws IOException  if there is an I/O error.
   * @throws ClassNotFoundException  if there is a classpath problem.
   */
  private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
    stream.defaultReadObject() ;

    this.tickPaint = SerialUtil.readPaint(stream) ;
    this.tickStroke = SerialUtil.readStroke(stream) ;
    this.tickLabelPaint = SerialUtil.readPaint(stream) ;
  }

}
