package com.ijchart.xychart.tickunit ;

import java.io.Serializable ;
import java.text.DateFormat ;
import java.text.SimpleDateFormat ;
import java.util.Calendar ;
import java.util.TimeZone ;

import com.ijchart.xychart.constant.DateTickUnitType ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ڼ͵ݵ̶ֵ.</zh>
 * <en>Calculate the tick unit for date data.</en>
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class DateTickUnit implements INumberTickUnit, Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -6741996622656274726L ;

  /**
   * <zh>1Сʱĺֵ.</zh>
   * <en>The millisecond of 1 hour.</en>
   */
  public static final long ONE_HOUR_MILLSECONDS = 60 * 60 * 1000 ;

  /**
   * <zh>ڵĵλ.</zh>
   * <en>The unit type of date.</en>
   */
  private DateTickUnitType unitType ;

  /**
   * <zh>ʽڵ.</zh>
   * <en>The date formatter.</en>
   */
  private DateFormat dateFormatter ;

  /**
   * <zh>ʱ.</zh>
   * <en>The time zone.</en>
   */
  private TimeZone timeZone ;

  /**
   * <zh>.</zh>
   * <en>Constructor.</en>
   *
   * @param unitType DateTickUnitType <zh>ڵĵλ,Ϊnull,Զѡ.(Ϊnull).</zh>
   * <en>The unit type of date,the unit type will be auto select for null value.(null permitted).</en>
   */
  public DateTickUnit(DateTickUnitType unitType) {
    this.unitType = unitType ;
  }

  /**
   * <zh>.</zh>
   * <en>Constructor.</en>
   *
   * @param unitType DateTickUnitType <zh>ڵĵλ,Ϊnull,Զѡ.(Ϊnull).</zh>
   * <en>The unit type of date,the unit type will be auto select for null value.(null permitted).</en>
   * @param formatter DateFormat <zh>ʽڵ,Ϊnull,Զѡ.(Ϊnull).</zh>
   * <en>The date formatter,the unit type will be auto select for null value.(null not permitted).</en>
   */
  public DateTickUnit(DateTickUnitType unitType, DateFormat formatter) {
    this.unitType = unitType ;
    this.dateFormatter = formatter ;
  }

  /**
   * <zh>ڵĵλ.</zh>
   * <en>Returns the unit type of date.</en>
   *
   * @return DateTickUnitType <zh>ڵĵλ (Ϊnull).</zh>
   * <en>The unit type of date (maybe null).</en>
   */
  public DateTickUnitType getUnitType() {
    return this.unitType ;
  }

  /**
   * <zh>ڵĵλ.</zh>
   * <en>Sets the unit type of date.</en>
   *
   * @param unitType DateTickUnitType <zh>ڵĵλ (Ϊnull).</zh>
   * <en>The unit type of date (maybe null).</en>
   */
  public void setUnitType(DateTickUnitType unitType) {
    this.unitType = unitType ;
  }

  /**
   * <zh>ظʽڵ.</zh>
   * <en>Returns the date formatter.</en>
   *
   * @return DateFormat <zh>ʽڵ (Ϊnull).</zh>
   * <en>The date formatter (never null).</en>
   */
  public DateFormat getDateFormatter() {
    return this.dateFormatter ;
  }

  /**
   * <zh>øʽڵ.</zh>
   * <en>Sets the date formatter.</en>
   *
   * @param dateFormatter DateFormat <zh>ʽڵ (Ϊnull).</zh>
   * <en>The date formatter (never null).</en>
   */
  public void setDateFormatter(DateFormat dateFormatter) {
    this.dateFormatter = dateFormatter ;
  }

  /**
   * <zh>ʱ.</zh>
   * <en>Returns the time zone.</en>
   *
   * @return TimeZone <zh>ʱ.</zh>
   * <en>The time zone.</en>
   */
  public TimeZone getTimeZone() {
    return this.timeZone ;
  }

  /**
   * <zh>ʱ.</zh>
   * <en>Sets the time zone.</en>
   *
   * @param timeZone TimeZone <zh>ʱ.</zh>
   * <en>The time zone.</en>
   */
  public void setTimeZone(TimeZone timeZone) {
    this.timeZone = timeZone ;
  }

  /**
   * <zh>СֵֵݵȷϵĿ̶ֵ,̶ֵ.</zh>
   * <en>According to the minimum and maximum data and the largest number of coordinates
   * to determine the scale of axis,all values are integer.</en>
   *
   * @param minValue double <zh>Сֵ.</zh>
   * <en>The minimum value.</en>
   * @param maxValue double <zh>ֵ.</zh>
   * <en>The max value.</en>
   * @param maxCount int <zh>̶ֵ,0.</zh>
   * <en>The maximum count of tick units (must great than 0).</en>
   * @param includeZero boolean <zh>ֵ,Ƿ0ֵ.˲ûʹ.</zh>
   * <en>A boolean flag indicates whether or not include zero value.This parameter is not used.</en>
   *
   * @return String[] <zh>̶ֵǿ,ֵС˳,ÿԪض,(Ϊnull).</zh>
   * <en>An array of tick units,the values are sorted ascending.
   * All elements in array is number (never null).</en>
   */
  public String[] getTickUnits(double minValue, double maxValue, int maxCount, boolean includeZero) {
    if (minValue > maxValue) {
      throw new IllegalArgumentException("Must 'minValue'<='maxValue'.") ;
    }
    if (!(minValue >= Long.MIN_VALUE && minValue <= Long.MAX_VALUE)) {
      throw new IllegalArgumentException("'minValue' must be a long value.") ;
    }
    if (!(maxValue >= Long.MIN_VALUE && maxValue <= Long.MAX_VALUE)) {
      throw new IllegalArgumentException("'maxValue' must be a long value.") ;
    }
    if (maxCount <= 0) {
      throw new IllegalArgumentException("Argument 'maxCount' must be >=0.") ;
    }

    //´Ļ˼·Ǹݿ̶ͰѺֵתΪӦ͵ʱ
    DateTickUnitType calUnitType = this.unitType ;
    if (calUnitType == null) {
      calUnitType = calculateDateTickUnitType(minValue, maxValue, maxCount) ;
    }

    double minConverted = 0 ; //תСֵ
    double maxConverted = 0 ; //תĴֵ
    long oneTickUnit = 0 ; //̶ֵСλ
    if (calUnitType.equals(DateTickUnitType.DATE_UNIT_MILLISECOND)) {
      oneTickUnit = 1 ;
      minConverted = minValue ;
      maxConverted = maxValue - minValue ;
    } else if (calUnitType.equals(DateTickUnitType.DATE_UNIT_SECOND)) {
      final int second = 1000 ;
      oneTickUnit = second ;
      minConverted = minValue - minValue % second ;
      maxConverted = (maxValue - minValue) / second ;
    } else if (calUnitType.equals(DateTickUnitType.DATE_UNIT_MINUTE)) {
      final int minute = 60 * 1000 ;
      oneTickUnit = minute ;
      minConverted = minValue - minValue % minute ;
      maxConverted = (maxValue - minValue) / minute ;
    } else if (calUnitType.equals(DateTickUnitType.DATE_UNIT_HOUR)) {
      final int hour = 60 * 60 * 1000 ;
      oneTickUnit = hour ;
      minConverted = minValue - minValue % hour ;
      maxConverted = (maxValue - minValue) / hour ;
    } else if (calUnitType.equals(DateTickUnitType.DATE_UNIT_DAY)) {
      final int day = 24 * 60 * 60 * 1000 ;
      oneTickUnit = day ;
      minConverted = minValue - minValue % day ;
      maxConverted = (maxValue - minValue) / day ;
    } else if (calUnitType.equals(DateTickUnitType.DATE_UNIT_MONTH)) {
      Calendar minCalendar = Calendar.getInstance() ;
      minCalendar.setTimeInMillis((long) minValue) ;
      int minMonth = minCalendar.get(Calendar.YEAR) * 12 + minCalendar.get(Calendar.MONTH) + 1 ;
      minConverted = minMonth ;

      Calendar maxCalendar = Calendar.getInstance() ;
      maxCalendar.setTimeInMillis((long) maxValue) ;
      int maxMonth = maxCalendar.get(Calendar.YEAR) * 12 + maxCalendar.get(Calendar.MONTH) + 1 ;

      maxConverted = maxMonth - minMonth ;
    } else if (calUnitType.equals(DateTickUnitType.DATE_UNIT_YEAR)) {
      Calendar minCalendar = Calendar.getInstance() ;
      minCalendar.setTimeInMillis((long) minValue) ;
      int minYear = minCalendar.get(Calendar.YEAR) ;
      minConverted = minYear ;

      Calendar maxCalendar = Calendar.getInstance() ;
      maxCalendar.setTimeInMillis((long) maxValue) ;
      int maxYear = maxCalendar.get(Calendar.YEAR) ;

      maxConverted = maxYear - minYear ;
    }

    LongTickUnit longTickUnit = new LongTickUnit() ;
    String[] tickLabels = longTickUnit.getTickUnits(0, maxConverted, maxCount, false) ;
    if (tickLabels != null) {
      long step = 0 ; //̶ȵĲ
      if (tickLabels.length > 1) {
        step = Math.abs(Long.parseLong(tickLabels[1], 10) - Long.parseLong(tickLabels[0], 10)) ;
      }

      long firstTickValue = Long.parseLong(tickLabels[0], 10) ;
      String[] newTickLabels = new String[tickLabels.length + 1] ; //
      if (calUnitType.equals(DateTickUnitType.DATE_UNIT_MONTH)) { //̶ȵλΪ
        Calendar minCalendar = Calendar.getInstance() ;
        minCalendar.setTimeInMillis((long) minValue) ;
        int minMonth = minCalendar.get(Calendar.YEAR) * 12 + minCalendar.get(Calendar.MONTH) + 1 ;

        long[] units = new long[tickLabels.length + 1] ; //Ŵһ̶ֵ
        for (int i = 0 ; i < units.length ; i++) { //ÿһλ
          units[i] = firstTickValue + step * i ;
        }

        for (int i = 0 ; i < newTickLabels.length ; i++) { //ת·Ϊֵ
          newTickLabels[i] = String.valueOf(convertMonthsToMilliseconds(minMonth + units[i])) ;
        }
      } else if (calUnitType.equals(DateTickUnitType.DATE_UNIT_YEAR)) { //̶ȵλΪ
        Calendar minCalendar = Calendar.getInstance() ;
        minCalendar.setTimeInMillis((long) minValue) ;
        int minYear = minCalendar.get(Calendar.YEAR) ;

        long[] units = new long[tickLabels.length + 1] ;
        for (int i = 0 ; i < units.length ; i++) {
          units[i] = firstTickValue + step * i ;
        }

        for (int i = 0 ; i < newTickLabels.length ; i++) { //תΪֵ
          newTickLabels[i] = String.valueOf(convertYearsToMilliseconds(minYear + units[i])) ;
        }
      } else {
        step *= oneTickUnit ;

        //ݲÿһ̶ȵʵֵ
        firstTickValue = (long) minConverted - ((step == 0) ? 0 : (long) minConverted % step) ;
        newTickLabels[0] = String.valueOf(firstTickValue) ;
        for (int i = 1 ; i < newTickLabels.length ; i++) {
          newTickLabels[i] = String.valueOf(firstTickValue + i * step) ;
        }
      }

      //ڶֵѾֵ,ȥһֵ
      if (newTickLabels.length > 2) {
        long lastSecond = Long.parseLong(newTickLabels[newTickLabels.length - 2], 10) ;
        if (lastSecond >= maxValue) { //Ӧȥһֵ
          System.arraycopy(newTickLabels, 0, tickLabels, 0, tickLabels.length) ;
        } else {
          tickLabels = newTickLabels ;
        }
      } else {
        tickLabels = newTickLabels ;
      }
    }

    //01979-1-1Ӧʱ,Ҫ⴦
    if (tickLabels != null) { //̶,,,ȥʱСʱ
      if (calUnitType.equals(DateTickUnitType.DATE_UNIT_DAY)) {
        Calendar cal = null ;
        if (this.timeZone != null) {
          cal = Calendar.getInstance(this.timeZone) ;
        } else {
          cal = Calendar.getInstance() ;
        }
        cal.setTimeInMillis(0) ;
        int hours = cal.get(Calendar.HOUR_OF_DAY) ;

        long[] longArr = new long[tickLabels.length] ;
        int[] flagArr = new int[tickLabels.length] ;
        for (int i = 0 ; i < tickLabels.length ; i++) {
          longArr[i] = Long.parseLong(tickLabels[i]) - hours * ONE_HOUR_MILLSECONDS ;
        }

        int removeCount = 0 ; //ҪɾԪظ,´ȥΧĿ̶ֵ
        for (int i = 0 ; i < longArr.length - 1 ; i++) {
          if (longArr[i] < minValue && longArr[i + 1] <= minValue) {
            flagArr[i] = 1 ;
            removeCount++ ;
          }
        }
        for (int i = longArr.length - 1 ; i > 0 ; i--) {
          if (longArr[i] >= maxValue && longArr[i - 1] > maxValue) {
            flagArr[i] = 1 ;
            removeCount++ ;
          }
        }

        boolean addMax = false ;
        long addMaxValue = 0 ;
        if (longArr[longArr.length - 1] < maxValue) { //̶ֵСmaxValue
          if (longArr.length > 1) {
            addMax = true ;
            long step = longArr[longArr.length - 1] - longArr[longArr.length - 2] ;
            addMaxValue = longArr[longArr.length - 1] + step ;
          }
        }

        tickLabels = new String[tickLabels.length - removeCount + (addMax ? 1 : 0)] ;
        int index = 0 ;
        for (int i = 0 ; i < longArr.length ; i++) {
          if (flagArr[i] == 0) {
            tickLabels[index++] = String.valueOf(longArr[i]) ;
          }
        }

        if (addMax) {
          tickLabels[tickLabels.length - 1] = String.valueOf(addMaxValue) ;
        }

      }
    }

    return tickLabels ;
  }

  /**
   * <zh>ֵĿ̶,ֵΪ{@link DateTickUnitType}ĳֵ.</zh>
   * <en>Calculate the tick unit type of date value,the unit is constant value defined in class
   * {@link DateTickUnitType}.</en>
   *
   * @param minMillis double <zh>Сֵ.</zh>
   * <en>The minimum value.</en>
   * @param maxMillis double <zh>ֵ.</zh>
   * <en>The maximum value.</en>
   * @param maxCount int <zh>̶ֵ,0.</zh>
   * <en>The maximum count of tick units (must great than 0).</en>
   *
   * @return int <zh>̶ֵ.</zh>
   * <en>The tick unit type.</en>
   */
  public static DateTickUnitType calculateDateTickUnitType(double minMillis, double maxMillis,
                                                           int maxCount) {
    if (minMillis > maxMillis) {
      throw new IllegalArgumentException("Must be 'minMillis'<='maxMillis'.") ;
    }
    if (maxCount <= 0) {
      throw new IllegalArgumentException("Argument 'maxCount' must be >=0.") ;
    }

    double diff = (maxMillis - minMillis) / maxCount ;
    if (diff <= 1000) { // 1000 milliseconds
      return DateTickUnitType.DATE_UNIT_MILLISECOND ;
    } else if (diff <= 60 * 1000) { // 60 seconds
      return DateTickUnitType.DATE_UNIT_SECOND ;
    } else if (diff <= 60 * 60 * 1000) { // 60 minutes
      return DateTickUnitType.DATE_UNIT_MINUTE ;
    } else if (diff <= 24 * 60 * 60 * 1000) { // 24 hours
      return DateTickUnitType.DATE_UNIT_HOUR ;
    } else if (diff <= 31 * 24 * 60 * 60 * 1000L) { // 31 days
      return DateTickUnitType.DATE_UNIT_DAY ;
    } else if (diff <= 2 * 31 * 24 * 60 * 60 * 1000L) { // two months
      return DateTickUnitType.DATE_UNIT_MONTH ;
    } else {
      return DateTickUnitType.DATE_UNIT_YEAR ;
    }
  }

  /**
   * <zh>ڵͷӦ{@link DateFormat}.</zh>
   * <en>Returns the {@link DateFormat} by date unit type.</en>
   *
   * @param unitType DateTickUnitType <zh>ڵĿ̶ (Ϊnull).</zh>
   * <en>The date tick unit type (null not permitted).</en>
   *
   * @return DateFormat <zh>ʽ.</zh>
   * <en>The date formatter.</en>
   */
  public static DateFormat getDataFormatter(DateTickUnitType unitType) {
    if (unitType == null) {
      throw new IllegalArgumentException("Null 'unitType' argument.") ;
    }

    String pattern = "yyyy-MM-dd HH:mm:ss" ;
    if (unitType.equals(DateTickUnitType.DATE_UNIT_MILLISECOND)) {
      pattern = "HH:mm:ss.SSS" ;
    } else if (unitType.equals(DateTickUnitType.DATE_UNIT_SECOND)) {
      pattern = "HH:mm:ss" ;
    } else if (unitType.equals(DateTickUnitType.DATE_UNIT_MINUTE)) {
      pattern = "HH:mm" ;
    } else if (unitType.equals(DateTickUnitType.DATE_UNIT_HOUR)) {
      pattern = "MM-dd HH" ;
    } else if (unitType.equals(DateTickUnitType.DATE_UNIT_DAY)) {
      pattern = "MM-dd" ;
    } else if (unitType.equals(DateTickUnitType.DATE_UNIT_MONTH)) {
      pattern = "yyyy-MM" ;
    } else if (unitType.equals(DateTickUnitType.DATE_UNIT_YEAR)) {
      pattern = "yyyy" ;
    }

    return new SimpleDateFormat(pattern) ;
  }

  /**
   * <zh>תΪֵ.</zh>
   * <en>Convert year to milliseconds.</en>
   *
   * @param years long <zh>.</zh>
   * <en>The year.</en>
   *
   * @return long <zh>ֵ.</zh>
   * <en>The milliseconds.</en>
   */
  private static long convertYearsToMilliseconds(long years) {
    Calendar calendar = Calendar.getInstance() ;
    calendar.set((int) years, 0, 1, 0, 0, 0) ;
    calendar.set(Calendar.MILLISECOND, 0) ;

    return calendar.getTimeInMillis() ;
  }

  /**
   * <zh>ת·Ϊֵ.</zh>
   * <en>Convert month to milliseconds.</en>
   *
   * @param months long <zh>·.</zh>
   * <en>The month.</en>
   *
   * @return long <zh>ֵ.</zh>
   * <en>The milliseconds.</en>
   */
  private static long convertMonthsToMilliseconds(long months) {
    Calendar calendar = Calendar.getInstance() ;
    int year = (int) (months / 12) ;
    int month = (int) (months % 12) ;
    if (month == 0) {
      year -= 1 ;
      month = 11 ;
    } else {
      month -= 1 ;
    }
    calendar.set(year, month, 1, 0, 0, 0) ;
    calendar.set(Calendar.MILLISECOND, 0) ;

    return calendar.getTimeInMillis() ;
  }

  /**
   * <zh>ԱǷһ.</zh>
   * <en>Tests the axis for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return true or false.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    if (!(obj instanceof DateTickUnit)) {
      return false ;
    }

    DateTickUnit that = (DateTickUnit) obj ;
    if (!EqualsUtil.equals(this.unitType, that.unitType)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.dateFormatter, that.dateFormatter)) {
      return false ;
    }

    return true ;
  }

  /**
   * Returns a clone of the object.
   *
   * @return A clone.
   *
   * @throws CloneNotSupportedException if some component of the axis does
   *         not support cloning.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>hashCode.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int result = super.hashCode() ;
    result = result * 37 + HashcodeUtil.hashCode(this.unitType) ;
    result = result * 37 + HashcodeUtil.hashCode(this.dateFormatter) ;

    return result ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return A string representation of the object.
   */
  public String toString() {
    return "DateTickUnit" ;
  }

}
