package com.ijchart.xychart.tickunit ;

import java.io.Serializable ;
import java.text.DecimalFormat ;

import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ڼ͵ݵ̶ֵ,̶ֵΪdouble͵ĸ.</zh>
 * <en>Calculate the tick unit for number data and all values is double.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class DoubleTickUnit implements INumberTickUnit, Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -5778222236112511491L ;

  /**
   * <zh>ʽdouble͵.</zh>
   * <en>Formatter for double value.</en>
   */
  private DecimalFormat format ;

  /**
   * <zh>.</zh>
   * <en>The step.</en>
   */
  private double step ;

  /**
   * <zh>ȱʡ.</zh>
   * <en>A default constructor.</en>
   */
  public DoubleTickUnit() {
    this(Double.NaN) ;
  }

  /**
   * <zh>.</zh>
   * <en>A constructor.</en>
   *
   * @param step double <zh>.</zh>
   * <en>The step.</en>
   */
  public DoubleTickUnit(double step) {
    this.step = step ;
  }

  /**
   * <zh>ز.</zh>
   * <en>Returns the step.</en>
   *
   * @return double <zh>.</zh>
   * <en>The step.</en>
   */
  public double getStep() {
    return this.step ;
  }

  /**
   * <zh>СֵֵݵȷϵĿ̶ֵ,̶ֵdouble͵ĸ.</zh>
   * <en>According to the minimum and maximum data and the largest number of coordinates
   * to determine the scale of axis,all values are double.</en>
   *
   * @param minValue double <zh>Сֵ.</zh>
   * <en>The minimum value.</en>
   * @param maxValue double <zh>ֵ.</zh>
   * <en>The max value.</en>
   * @param maxCount int <zh>̶ֵ,0.</zh>
   * <en>The maximum count of tick units (must great than 0).</en>
   * @param includeZero boolean <zh>ֵ,Ƿ0ֵ.</zh>
   * <en>A boolean flag indicates whether or not include zero value.</en>
   *
   * @return String[] <zh>̶ֵǿ,ֵС˳,ÿԪض,(Ϊnull).</zh>
   * <en>An array of tick units,the values are sorted ascending.
   * All elements in array is number (never null).</en>
   */
  public String[] getTickUnits(double minValue, double maxValue, int maxCount, boolean includeZero) {
    if (minValue > maxValue) {
      throw new IllegalArgumentException("Must be 'minValue'<='maxValue'.") ;
    }
    if (maxCount < 1) {
      throw new IllegalArgumentException("Must be 'maxCount'>0.") ;
    }

    if (includeZero) {
      if (maxValue < 0) {
        maxValue = 0 ;
      } else if (minValue > 0) {
        minValue = 0 ;
      }
    }

    if (minValue == maxValue) {
      this.format = new DecimalFormat("#.#") ;
      return new String[] {formatValue(minValue)} ;
    }

    String[] result = new String[0] ;
    if (minValue >= 0) { //minValue>=0 maxValue>=0
      if (includeZero) { //0
        double estimateStep = (maxValue - 0) / maxCount ;

        int count = 0 ;
        while (true && count++ < 5) {
          double calcStep = this.step ;
          if (Double.isNaN(calcStep)) {
            calcStep = calculateStep(estimateStep) ;
          }

          int unitCount = (int) (maxValue / calcStep) + (needAddOne(maxValue, calcStep) ? 1 : 0)
                          + 1 ;
          if (unitCount > maxCount) {
            estimateStep = calcStep ;
            continue ;
          } else {
            result = new String[unitCount] ;

            for (int i = 0 ; i < unitCount ; i++) {
              result[i] = formatValue(i * calcStep) ;
            }
            break ;
          }
        }
      } else { //0
        double minStartValue = 0 ;
        if (!includeZero) {
          minStartValue = minValue ;
        }

        double estimateStep = (maxValue - minStartValue) / maxCount ;

        int count = 0 ;
        while (true && count++ < 5) {
          double calcStep = this.step ;
          if (Double.isNaN(calcStep)) {
            calcStep = calculateStep(estimateStep) ;
          }

          double firstValue = Math.floor(minValue / calcStep) * calcStep ;
          int unitCount = (int) ((maxValue - firstValue) / calcStep)
                          + ((needAddOne(maxValue - firstValue, calcStep)) ? 1 : 0) + 1 ;

          if (unitCount > maxCount) {
            estimateStep = calcStep ;
            continue ;
          } else {
            result = new String[unitCount] ;

            result[0] = formatValue(firstValue) ;
            for (int i = 1 ; i < unitCount ; i++) {
              double tmpValue = firstValue + i * calcStep ;
              result[i] = formatValue(tmpValue) ;
            }

            break ;
          }
        }
      }
    } else if (minValue < 0 && maxValue >= 0) {
      double estimateStep = (maxValue - minValue) / maxCount ;

      int count = 0 ;
      while (true && count++ < 5) {
        double calcStep = this.step ;
        if (Double.isNaN(calcStep)) {
          calcStep = calculateStep(estimateStep) ;
        }

        int postiveCount = (int) (maxValue / calcStep) + (needAddOne(maxValue, calcStep) ? 1 : 0)
                           + 1 ;
        int negativeCount = (int) ( -minValue / calcStep)
                            + (needAddOne( -minValue, calcStep) ? 1 : 0) ;
        if (negativeCount + postiveCount > maxCount) {
          estimateStep = calcStep ;
          continue ;
        } else {
          result = new String[negativeCount + postiveCount] ;

          for (int i = 0 ; i < negativeCount ; i++) { //
            result[i] = formatValue( -(negativeCount - i) * calcStep) ;
          }
          for (int i = 0 ; i < postiveCount ; i++) { //
            result[negativeCount + i] = formatValue(i * calcStep) ;
          }

          break ;
        }
      }
    } else if (minValue < 0 && maxValue < 0) {
      if (includeZero) { //0
        double estimateStep = ( -minValue - 0) / maxCount ;

        int count = 0 ;
        while (true && count++ < 5) {
          double calcStep = this.step ;
          if (Double.isNaN(calcStep)) {
            calcStep = calculateStep(estimateStep) ;
          }

          int unitCount = (int) ( -minValue / calcStep) + (needAddOne( -minValue, calcStep) ? 1 : 0)
                          + 1 ;
          if (unitCount > maxCount) {
            estimateStep = calcStep ;
            continue ;
          } else {
            result = new String[unitCount] ;

            for (int i = 0 ; i < unitCount ; i++) {
              result[i] = formatValue( -calcStep * (unitCount - i - 1)) ;
            }
            break ;
          }
        }
      } else { //0
        double tmpMinValue = -maxValue ;
        double tmpMaxValue = -minValue ;
        double minStartValue = tmpMinValue ;

        double estimateStep = (tmpMaxValue - minStartValue) / maxCount ;

        int count = 0 ;
        while (true && count++ < 5) {
          double calcStep = this.step ;
          if (Double.isNaN(calcStep)) {
            calcStep = calculateStep(estimateStep) ;
          }

          double firstValue = Math.floor(tmpMinValue / calcStep) * calcStep ;
          int unitCount = (int) ((tmpMaxValue - firstValue) / calcStep)
                          + (needAddOne(tmpMaxValue - firstValue, calcStep) ? 1 : 0) + 1 ;

          if (unitCount > maxCount) {
            estimateStep = calcStep ;
            continue ;
          } else {
            result = new String[unitCount] ;

            result[unitCount - 1] = formatValue((firstValue != 0) ? -firstValue : firstValue) ;
            for (int i = 1 ; i < unitCount ; i++) {
              double tmpValue = firstValue + i * calcStep ;
              result[unitCount - 1 - i] = formatValue((tmpValue != 0) ? -tmpValue : tmpValue) ;
            }

            break ;
          }
        }

      }
    }

    return result ;
  }

  /**
   * <zh>ǷҪһֵ.</zh>
   * <en>Whether or not need add a value.</en>
   * @param value double
   * @param step double
   * @return boolean
   */
  private static boolean needAddOne(double value, double step) {
    return Math.abs(value - ((int) (value / step)) * step) > 0.0000000001 ;
  }

  /**
   * <zh>㲽</zh>
   * <en>Calculate step size.</en>
   *
   * @param nStep double
   * @return double
   */
  private double calculateStep(double nStep) {
    if (nStep == 0.0) { //Ѿ0
      createFormatter(1, false) ;
      return 0 ;
    }

    double a1 = 0, a2 = 0, a3 = 0 ; //ԱԺ
    double baseVal = 1 ; //

    if (nStep >= 1) {
      while (true) {
        a1 = 1 * baseVal ;
        a2 = 2.5 * baseVal ;
        a3 = 5 * baseVal ;
        baseVal *= 10 ;

        if (nStep < a1) {
          createFormatter(baseVal, false) ;
          return a1 ;
        } else if (nStep < a2) {
          createFormatter(baseVal, true) ;
          return a2 ;
        } else if (nStep < a3) {
          createFormatter(baseVal, false) ;
          return a3 ;
        }
      }
    } else {
      if (nStep >= 0.5) {
        createFormatter(1, false) ;
        return 1 ;
      }

      while (true) {
        a1 = 0.1 * baseVal ;
        a2 = 0.25 * baseVal ;
        a3 = 0.5 * baseVal ;
        baseVal *= 0.1 ;

        if (nStep >= a1 && nStep < a2) {
          createFormatter(baseVal, true) ;
          return a2 ;
        } else if (nStep >= a2 && nStep < a3) {
          createFormatter(baseVal, false) ;
          return a3 ;
        } else if (nStep >= a1 * 0.5 && nStep < a1) {
          createFormatter(baseVal, false) ;
          return a1 ;
        }
      }
    }

  }

  /**
   * <zh>ֵĲͬѡͬDecimalFormat.</zh>
   * <en>According to the value of different options for different DecimalFormat.</en>
   *
   * @param baseVal double
   * @param has5 <zh>ֵǷ0.252.5</zh>
   * <en>Whether or not multiply 0.25 or 2.5.</en>
   */
  private void createFormatter(double baseVal, boolean has5) {
    if (Math.abs(baseVal) > 1) {
      if (has5 && Math.abs(baseVal) <= 10) {
        this.format = new DecimalFormat("#.0") ;
      } else {
        this.format = new DecimalFormat("#") ;
      }
    } else {
      double value = baseVal ;
      String pattern = "#." ;
      do { //Сÿһλ0
        value *= 10 ;
        pattern += "0" ;
      } while (value < 1) ;

      if (has5) {
        pattern += "0" ;
      }

      this.format = new DecimalFormat(pattern) ;
    }
  }

  /**
   * <zh>ʽ,ֶ0,0;Сǵһλ,ǰ׺0.</zh>
   * <en>Digital format, if all the figures are 0, return 0;
   * if the decimal point is the first one, then prefix-0.</en>
   *
   * @param value double
   * @return String
   */
  private String formatValue(double value) {
    String str = null ;
    if (this.format == null) {
      str = (new DecimalFormat("#.#")).format(value) ;
    } else {
      str = this.format.format(value) ;
    }

    if (value >= 0) {
      int dotIndex = str.indexOf(".") ;
      if (dotIndex == 0) {
        boolean isZero = true ;
        for (int i = 1 ; i < str.length() ; i++) {
          if (str.charAt(i) != '0') {
            isZero = false ;
          }
        }

        if (isZero) { //valueֵΪ0
          return "0" ;
        } else {
          return "0" + str ;
        }
      }

      if (dotIndex == 1 && str.indexOf("-") == 0) { //-.00
        boolean isZero = true ;
        for (int i = 2 ; i < str.length() ; i++) {
          if (str.charAt(i) != '0') {
            isZero = false ;
          }
        }

        if (isZero) {
          return "0" ;
        }
      }
    } else {
      int dotIndex = str.indexOf(".") ;
      if (dotIndex == 1) {
        return "-0." + str.substring(dotIndex + 1) ;
      }

    }

    if (str.equals("-0")) {
      return "0" ;
    }

//    int dotIndex = str.indexOf(".") ;
//    if (dotIndex > 0) {
//      int zeroCount = 0 ;
//      for (int i = str.length() - 1 ; i >= 0 ; i--) {
//        if (str.charAt(i) == '0') {
//          zeroCount++ ;
//        } else {
//          break ;
//        }
//      }
//
//      if (zeroCount > 0) {
//        str = str.substring(0, str.length() - zeroCount) ;
//        if (str.endsWith(".")) {
//          str = str.substring(0, str.length() - 1) ;
//        }
//      }
//    }

    return str ;
  }

  /**
   * <zh>ԱǷһ.</zh>
   * <en>Tests the axis for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return true or false.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }

    if (!(obj instanceof DoubleTickUnit)) {
      return false ;
    }

    DoubleTickUnit that = (DoubleTickUnit) obj ;
    if (!EqualsUtil.equals(this.format, that.format)) {
      return false ;
    }
    if (this.step != that.step) {
      return false ;
    }

    return true ;
  }

  /**
   * Returns a clone of the object.
   *
   * @return A clone.
   *
   * @throws CloneNotSupportedException if some component of the axis does
   *         not support cloning.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>hashCode.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    double result = super.hashCode() ;
    result = result * 37 + HashcodeUtil.hashCode(this.format) ;
    result = result * 37 + this.step ;

    return (int) result ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return A string representation of the object.
   */
  public String toString() {
    return "DoubleTickUnit" ;
  }

}
