package com.ijchart.xychart.title ;

import java.awt.Graphics2D ;
import java.awt.Image ;
import java.awt.geom.Rectangle2D ;
import java.io.IOException ;
import java.io.ObjectInputStream ;
import java.io.ObjectOutputStream ;
import java.io.Serializable ;

import com.ijchart.xychart.chartinfo.ChartShapeCollection ;
import com.ijchart.xychart.chartinfo.TitleShapeCollection ;
import com.ijchart.xychart.constant.IAngle ;
import com.ijchart.xychart.shape.RectangleImageChartShape ;
import com.ijchart.xychart.ui.Rectangle ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.util.SerialUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ʾͼƬı,ֱǷǳõ,ʾ˾Logo.</zh>
 * <en>A chart title that displays an image.  This is useful, for example, if you
 * have an image of your corporate logo and want to use as a footnote or part
 * of a title in a chart you create.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class ImageTitle extends AbstractTitle implements Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 4325932975796078834L ;

  /**
   * <zh>ȱʡǷͼƬtitle.</zh>
   * <en>A default flag controls whether or not to fill the title area by image.</en>
   */
  public static final boolean DEFAULT_IMAGE_FILL_TITLE_AREA = false ;

  /**
   * <zh>ͼƬ.</zh>
   * <en>The title image.</en>
   */
  private transient Image image ;

  /**
   * <zh>ͼƬĻƿ.</zh>
   * <en>The width used to draw the image.</en>
   */
  private int imageDrawWidth ;

  /**
   * <zh>ͼƬĻƸ߶.</zh>
   * <en>The height used to draw the image.</en>
   */
  private int imageDrawHeight ;

  /**
   * <zh>ǷԶͼƬtitle.</zh>
   * <en>A boolean flag controls whether or not fill title area by image.</en>
   */
  private boolean imageFill ;

  /**
   * <zh>.</zh>
   * <en>Creates a new image title.</en>
   *
   * @param image Image <zh>ͼƬ,Ѿ.</zh>
   * <en>The image,you must have already loaded this image from its source.</en>
   */
  public ImageTitle(Image image) {
    this(image, (image != null) ? image.getWidth(null) : 0,
         (image != null) ? image.getHeight(null) : 0) ;
  }

  /**
   * <zh>image,ָͼƬĻƿȺ͸߶.</zh>
   * <en>Creates a new image title with the given image scaled to the given
   * width and height in the given location.</en>
   *
   * @param image Image <zh>ͼƬ,Ѿ.</zh>
   * <en>The image,you must have already loaded this image from its source.</en>
   * @param imageDrawWidth int <zh>ͼƬʵʻƿ.</zh>
   * <en>The width used to draw the image.</en>
   * @param imageDrawHeight int <zh>ͼƬʵʻƸ߶.</zh>
   * <en>The height used to draw the image.</en>
   */
  public ImageTitle(Image image, int imageDrawWidth, int imageDrawHeight) {
    this.image = image ;
    this.imageDrawWidth = imageDrawWidth ;
    this.imageDrawHeight = imageDrawHeight ;

    this.imageFill = DEFAULT_IMAGE_FILL_TITLE_AREA ;
  }

  /**
   * <zh>ͼƬ.</zh>
   * <en>Returns the image for the title.</en>
   *
   * @return Image <zh>ͼƬ (Ϊnull).</zh>
   * <en>The image for the title (maybe null).</en>
   */
  public Image getImage() {
    return this.image ;
  }

  /**
   * <zh>ͼƬ,м{@link com.ijchart.xychart.event.TitleChangeEvent}¼.</zh>
   * <en>Sets the image for the title and notifies registered listeners that the
   * title has been modified.</en>
   *
   * @param image Image <zh>ͼƬ (Ϊnull).</zh>
   * <en>The new image (null permitted).</en>
   */
  public void setImage(Image image) {
    if (!(this.image == null && image == null)) {
      this.image = image ;
      fireTitleChanged() ;
    }
  }

  /**
   * <zh>ͼƬʵʻƿ.</zh>
   * <en>Returns the width used to draw the image.</en>
   *
   * @return int <zh>ͼƬʵʻƿ.</zh>
   * <en>The width used to draw the image.</en>
   */
  public int getImageDrawWidth() {
    return this.imageDrawWidth ;
  }

  /**
   * <zh>ͼƬʵʻƿ,м
   * {@link com.ijchart.xychart.event.TitleChangeEvent}¼.</zh>
   * <en>Sets the width used to draw the image,and sends a
   *  {@link com.ijchart.xychart.event.TitleChangeEvent} to all registered listeners.</en>
   *
   * @param imageDrawWidth int <zh>ͼƬʵʻƿ.</zh>
   * <en>The width used to draw the image.</en>
   */
  public void setImageDrawWidth(int imageDrawWidth) {
    if (this.imageDrawWidth != imageDrawWidth) {
      this.imageDrawWidth = imageDrawWidth ;
      fireTitleChanged() ;
    }
  }

  /**
   * <zh>ͼƬʵʻƸ߶.</zh>
   * <en>Returns the height used to draw the image.</en>
   *
   * @return int <zh>ͼƬʵʻƸ߶.</zh>
   * <en>The height used to draw the image.</en>
   */
  public int getImageDrawHeight() {
    return this.imageDrawHeight ;
  }

  /**
   * <zh>ͼƬʵʻƸ߶,м
   * {@link com.ijchart.xychart.event.TitleChangeEvent}¼.</zh>
   * <en>Sets the height used to draw the image,and sends a
   *  {@link com.ijchart.xychart.event.TitleChangeEvent} to all registered listeners.</en>
   *
   * @param imageDrawHeight int <zh>ͼƬʵʻƸ߶.</zh>
   * <en>The height used to draw the image.</en>
   */
  public void setImageDrawHeight(int imageDrawHeight) {
    if (this.imageDrawHeight != imageDrawHeight) {
      this.imageDrawHeight = imageDrawHeight ;
      fireTitleChanged() ;
    }
  }

  /**
   * <zh>ǷͼƬԶtitle.</zh>
   * <en>Returns whether or not fill title area by image.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isImageFill() {
    return this.imageFill ;
  }

  /**
   * <zh>ǷͼƬԶtitle,м
   * {@link com.ijchart.xychart.event.TitleChangeEvent}¼.</zh>
   * <en>Sets whether or not fill title area by image,and sends a
   *  {@link com.ijchart.xychart.event.TitleChangeEvent} to all registered listeners.</en>
   *
   * @param fill boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setImageFill(boolean fill) {
    if (this.imageFill != fill) {
      this.imageFill = fill ;
      fireTitleChanged() ;
    }
  }

  /**
   * <zh>ͼchartAreaƱⲿ.ͼеͼδshapeCollection.</zh>
   * <en>Draw the title on chart area(chartArea).All title shape is stored in shapeCollection.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   * @param chartArea Rectangle2D <zh>ͼ.</zh>
   * <en>The area within which the chart should be drawn.</en>
   * @param shapeCollection ChartShapeCollection <zh>ͼͼμ.</zh>
   * <en>Collects information about the plot (null permitted).</en>
   *
   * @return Rectangle <zh>.</zh>
   * <en>The area within which the title should be drawn.</en>
   */
  public Rectangle draw(Graphics2D g2, Rectangle2D chartArea, ChartShapeCollection shapeCollection) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (chartArea == null) {
      throw new IllegalArgumentException("Null 'chartArea' argument.") ;
    }
    if (shapeCollection == null) {
      throw new IllegalArgumentException("Null 'shapeCollection' argument.") ;
    }

    if (this.image == null) {
      return Rectangle.RECTANGLE_ZERO ;
    }

    RectangleEdge position = getPosition() ;

    Image drawImage = scaleImage() ;
    if (drawImage == null) {
      return Rectangle.RECTANGLE_ZERO ;
    }
    int drawImageHeight = drawImage.getHeight(null) ; //ͼƬĻƸ߶

    RectangleImageChartShape imageShape = null ;
    Rectangle2D drawArea = null ;
    if (RectangleEdge.isTopOrBottom(position)) {
      //֤Խ
      double drawAreaWidth = chartArea.getWidth() ;
      double drawAreaHeight = Math.min(drawImageHeight, chartArea.getHeight()) ;

      if (position.equals(RectangleEdge.TOP)) { //
        drawArea = new Rectangle2D.Double(chartArea.getX(), chartArea.getY(),
                                          drawAreaWidth, drawAreaHeight) ;
      } else if (position.equals(RectangleEdge.BOTTOM)) { //ײ
        drawArea = new Rectangle2D.Double(chartArea.getX(), chartArea.getMaxY() - drawAreaHeight,
                                          drawAreaWidth, drawAreaHeight) ;
      }

      imageShape = new RectangleImageChartShape(drawArea, drawImage) ;
    } else if (RectangleEdge.isLeftOrRight(position)) {
      //֤Խ
      double drawAreaWidth = chartArea.getHeight() ;
      double drawAreaHeight = Math.min(drawImageHeight, chartArea.getWidth()) ;

      if (position.equals(RectangleEdge.LEFT)) { //
        drawArea = new Rectangle2D.Double(chartArea.getX(), chartArea.getMaxY(),
                                          drawAreaWidth, drawAreaHeight) ;
      } else if (position.equals(RectangleEdge.RIGHT)) { //ұ
        drawArea = new Rectangle2D.Double(chartArea.getMaxX() - drawAreaHeight, chartArea.getMaxY(),
                                          drawAreaWidth, drawAreaHeight) ;
      }

      imageShape = new RectangleImageChartShape(drawArea, drawImage) ;
      imageShape.setRotate(IAngle.ANGLE_UP_90) ;
    }

    imageShape.setMargin(getPadding()) ;
    imageShape.setImageHorizontalAlign(getHorizontalAlignment()) ;
    imageShape.setImageVerticalAlign(getVerticalAlignment()) ;
    imageShape.setMargin(getPadding()) ;
    imageShape.setTooltipText(getToolTipText()) ;
    imageShape.setURL(getURLText()) ;
    imageShape.setDrawBorder(isBorderVisible()) ;
    imageShape.setBorderStroke(getBorderStroke()) ;
    imageShape.setBorderPaint(getBorderPaint()) ;
    imageShape.setFillShape(isFillBackground()) ;
    imageShape.setFillPaint(getBackgroundPaint()) ;
    imageShape.setFillAlpha(getBackgroundAlpha()) ;
    imageShape.setImageFill(isImageFill()) ;

    TitleShapeCollection titleShapes = new TitleShapeCollection() ;
    titleShapes.setDrawArea(drawArea) ;
    titleShapes.addTitleShape(imageShape) ;

    shapeCollection.addTitleShapes(titleShapes) ;

    return new Rectangle(drawArea.getWidth(), drawArea.getHeight()) ;
  }

  /**
   * <zh>ͼƬ.</zh>
   * <en>Creates a scaled version of this image.</en>
   *
   * @return Image
   */
  protected Image scaleImage() {
    if (this.image == null) {
      return null ;
    }

    if (this.image.getWidth(null) == this.imageDrawWidth
        && this.image.getHeight(null) == this.imageDrawHeight) { //Ҫ
      return this.image ;
    }

    return this.image.getScaledInstance(this.imageDrawWidth, this.imageDrawHeight,
                                        Image.SCALE_SMOOTH) ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this title for equality with another object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return true or false.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof ImageTitle)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    ImageTitle that = (ImageTitle) obj ;
    if (!EqualsUtil.equals(this.image, that.image)) {
      return false ;
    }
    if (this.imageDrawWidth != that.imageDrawWidth) {
      return false ;
    }
    if (this.imageDrawHeight != that.imageDrawHeight) {
      return false ;
    }
    if (this.imageFill != that.imageFill) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code.</en>
   *
   * @return A hash code.
   */
  public int hashCode() {
    int result = super.hashCode() ;
    result = 29 * result + HashcodeUtil.hashCode(this.image) ;
    result = 29 * result + this.imageDrawWidth ;
    result = 29 * result + this.imageDrawHeight ;
    result = 29 * result + HashcodeUtil.hashCode(this.imageFill) ;

    return result ;
  }

  /**
   * <zh>¡.</zh>
   * <en>Returns a clone of this object.</en>
   *
   * @return A clone.
   *
   * @throws CloneNotSupportedException never.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the output stream.
   *
   * @throws IOException  if there is an I/O error.
   */
  private void writeObject(ObjectOutputStream stream) throws IOException {
    stream.defaultWriteObject() ;

    SerialUtil.writeImage(stream, this.image) ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the input stream.
   *
   * @throws IOException  if there is an I/O error.
   * @throws ClassNotFoundException  if there is a classpath problem.
   */
  private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
    stream.defaultReadObject() ;

    this.image = SerialUtil.readImage(stream) ;
  }

}
