package com.ijchart.xychart.title ;

import java.awt.Graphics2D ;
import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;
import java.util.Collection ;
import java.util.Iterator ;
import java.util.List ;

import com.ijchart.xychart.chartinfo.ChartShapeCollection ;
import com.ijchart.xychart.chartinfo.LegendShapeCollection ;
import com.ijchart.xychart.render.AbstractRender ;
import com.ijchart.xychart.shape.LegendItemChartShape ;
import com.ijchart.xychart.shape.PaintScaleLegendChartShape ;
import com.ijchart.xychart.shape.RectangleTextChartShape ;
import com.ijchart.xychart.ui.Rectangle ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.ui.RectangleInsets ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ͼ͵ı.</zh>
 * <en>A chart title that displays a legend for the data in the chart.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class LegendTitle extends AbstractTitle implements Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -5483594814444393524L ;

  /**
   * <zh>render.</zh>
   * <en>The collection of renders.</en>
   */
  private Collection renders ;

  /**
   * <zh>.</zh>
   * <en>Constructor.</en>
   *
   * @param renders Collection <zh>render,еÿһԪ
   * {@link AbstractRender} (Ϊnull).</zh>
   * <en>The collection of renders,all elements in this collection
   *  is {@link AbstractRender} (null not permitted).</en>
   */
  public LegendTitle(Collection renders) {
    if (renders == null) {
      throw new IllegalArgumentException("Null 'renders' argument.") ;
    }

    this.renders = renders ;
    setBorderVisible(true) ;
  }

  /**
   * <zh>render.</zh>
   * <en>Returns the collection of renders.</en>
   *
   * @return Collection <zh>render (Ϊnull).</zh>
   * <en>The collection of renders (never null).</en>
   */
  public Collection getRenders() {
    return this.renders ;
  }

  /**
   * <zh>render,м{@link com.ijchart.xychart.event.TitleChangeEvent}¼.</zh>
   * <en>Sets the collection of renders,and
   * sends a {@link com.ijchart.xychart.event.TitleChangeEvent} to all registered listeners.</en>
   *
   * @param renders Collection <zh>render,еÿһԪ
   * {@link AbstractRender} (Ϊnull).</zh>
   * <en>The collection of renders,all elements in this collection
   *  is {@link AbstractRender} (null not permitted).</en>
   */
  public void setRenders(Collection renders) {
    if (renders == null) {
      throw new IllegalArgumentException("Null 'renders' argument.") ;
    }

    this.renders = renders ;
    fireTitleChanged() ;
  }

  /**
   * <zh>˲֧.</zh>
   * <en>.</en>
   *
   * @param tooltip String <zh>tooltipַ.</zh>
   * <en>The text (null permitted).</en>
   */
  public void setToolTipText(String tooltip) {
    throw new UnsupportedOperationException("Not supported.") ;
  }

  /**
   * <zh>URLַ,м{@link com.ijchart.xychart.event.TitleChangeEvent}¼.</zh>
   * <en>Sets the URL text to the specified text and sends a
   * {@link com.ijchart.xychart.event.TitleChangeEvent} to all registered listeners.</en>
   *
   * @param url String <zh>URLַ.</zh>
   * <en>The URL string (null permitted).</en>
   */
  public void setURLText(String url) {
    throw new UnsupportedOperationException("Not supported.") ;
  }

  /**
   * <zh>ͼchartAreaƱⲿ.ͼеͼδshapeCollection.</zh>
   * <en>Draw the title on chart area(chartArea).All title shape is stored in shapeCollection.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   * @param chartArea Rectangle2D <zh>ͼ.</zh>
   * <en>The area within which the chart should be drawn.</en>
   * @param shapeCollection ChartShapeCollection <zh>ͼͼμ.</zh>
   * <en>Collects information about the plot (null permitted).</en>
   *
   * @return Rectangle <zh>.</zh>
   * <en>The area within which the title should be drawn.</en>
   */
  public Rectangle draw(Graphics2D g2, Rectangle2D chartArea, ChartShapeCollection shapeCollection) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (chartArea == null) {
      throw new IllegalArgumentException("Null 'chartArea' argument.") ;
    }
    if (shapeCollection == null) {
      throw new IllegalArgumentException("Null 'shapeCollection' argument.") ;
    }

    final double rowGap = 2, columnGap = 2 ; //֮Ĳ

    Rectangle result = null ;
    RectangleInsets padding = getPadding() ;
    LegendShapeCollection legendShapes = new LegendShapeCollection() ;
    shapeCollection.setLegendShapes(legendShapes) ;
    RectangleEdge position = getPosition() ;
    int totalShapeCount = 0 ;
    if (RectangleEdge.isLeftOrRight(position)) { //߻ұ
      double currColumnMaxWidth = 0 ; //һͼε
      double totalWidth = 0 ; //еĿ֮
      double maxHeight = 0 ; //е߶
      double currColumnHeight = 0 ; //ǰеͼεĸ߶֮
      int currColumnCount = 0 ; //ǰеͼεĸ
      Iterator iter = this.renders.iterator() ;
      int renderIndex = 0 ; //render
      while (iter.hasNext()) {
        Object obj = iter.next() ;
        if (obj != null && obj instanceof AbstractRender) {
          AbstractRender render = (AbstractRender) obj ;
          render.setChartArea(chartArea) ;
          render.setLegendPositon(position) ;
          if (renderIndex++ == 0) {
            render.getDrawingSupplier().reset() ;
          }

          List legendItems = render.calculateLegendItemShapes() ;
          for (int i = 0 ; i < legendItems.size() ; i++) {
            Object legendObj = legendItems.get(i) ;
            if (legendObj != null && legendObj instanceof LegendItemChartShape) {
              totalShapeCount++ ;
              currColumnCount++ ;
              LegendItemChartShape legendItem = (LegendItemChartShape) legendObj ;
              legendShapes.addLegendShape(legendItem) ;
              Rectangle bound = legendItem.estimateDrawBound(g2) ;
              currColumnMaxWidth = Math.max(currColumnMaxWidth, bound.getWidth()) ;
              double shapeHeight = bound.getHeight() ;
              //ÿʾһͼ,߶
              if (currColumnHeight + shapeHeight + rowGap > chartArea.getHeight()
                  && currColumnCount > 1) {
                totalWidth += currColumnMaxWidth + columnGap ;
                maxHeight = Math.max(maxHeight, currColumnHeight) ;
                currColumnMaxWidth = bound.getWidth() ;
                currColumnCount = 1 ;

                Rectangle2D shapeDrawArea = new Rectangle2D.Double(totalWidth, 0, bound.getWidth(),
                                                                   bound.getHeight()) ;
                legendItem.setDrawArea(shapeDrawArea) ;
                currColumnHeight = shapeHeight + rowGap ;
              } else {
                Rectangle2D shapeDrawArea = null ;
                if (legendItem.getShape() instanceof PaintScaleLegendChartShape) {
                  shapeDrawArea = new Rectangle2D.Double(totalWidth, currColumnHeight,
                                                         bound.getWidth(), bound.getHeight() - 7) ;
                } else {
                  shapeDrawArea = new Rectangle2D.Double(totalWidth, currColumnHeight,
                                                         bound.getWidth(), bound.getHeight()) ;
                }
                legendItem.setDrawArea(shapeDrawArea) ;
                currColumnHeight += shapeHeight + rowGap ;
              }
            }
          }

        }
      }

      if (totalShapeCount == 0) {
        shapeCollection.setLegendShapes(null) ;
        return null ;
      }

      totalWidth += currColumnMaxWidth ;
      maxHeight = Math.max(maxHeight, currColumnHeight) ;
      legendShapes.translate(padding.getLeft(), padding.getTop()) ; //padding,ƫͼ

      //borderͼ
      double borderRectWidth = totalWidth + padding.getLeft() + padding.getRight() ;
      double borderRectHeight = Math.min(maxHeight + padding.getTop() + padding.getBottom(),
                                         chartArea.getHeight()) - 2 ;
      Rectangle2D borderRect = new Rectangle2D.Double(0, 0, borderRectWidth, borderRectHeight) ;
      RectangleTextChartShape borderShape = createBorderShape(borderRect) ;
      legendShapes.setBorderShape(borderShape) ;

      result = new Rectangle(borderRect.getWidth() + 2, borderRect.getHeight()) ;

      //λελ
      double offsetX = 0, offsetY = 0 ; //ƫλ
      if (position.equals(RectangleEdge.LEFT)) {
        offsetX = 2 ;
        offsetY = chartArea.getY() + (chartArea.getHeight() - maxHeight) * 0.5 ;
        legendShapes.translate(offsetX, offsetY) ;
      } else if (position.equals(RectangleEdge.RIGHT)) {
        offsetX = chartArea.getX() + chartArea.getWidth() - borderRect.getWidth() - 2 ;
        offsetY = chartArea.getY() + (chartArea.getHeight() - maxHeight) * 0.5 ;
        legendShapes.translate(offsetX, offsetY) ;
      }
      legendShapes.setDrawArea(new Rectangle2D.Double(offsetX, offsetY, borderRectWidth,
                                                      borderRectHeight)) ;
    } else if (RectangleEdge.isTopOrBottom(position)) { //
      double currRowMaxHeight = 0 ; //һͼε߶
      double totalHeight = 0 ; //еĸ߶֮
      double maxWidth = 0 ; //е߶
      double currRowWidth = 0 ; //ǰеͼεĿ֮
      int currRowCount = 0 ; //ǰеͼεĸ
      double maxShapeWidth = chartArea.getWidth() - padding.getLeft() - padding.getRight() - 2 ; //ÿͼεȲܳchartArea
      int renderIndex = 0 ; //render
      Iterator iter = this.renders.iterator() ;
      while (iter.hasNext()) {
        Object obj = iter.next() ;
        if (obj != null && obj instanceof AbstractRender) {
          AbstractRender render = (AbstractRender) obj ;
          render.setChartArea(chartArea) ;
          render.setLegendPositon(position) ;
          if (renderIndex++ == 0) {
            render.getDrawingSupplier().reset() ;
          }

          List legendItems = render.calculateLegendItemShapes() ;
          if (legendItems != null) {
            for (int i = 0 ; i < legendItems.size() ; i++) {
              Object legendObj = legendItems.get(i) ;
              if (legendObj != null && legendObj instanceof LegendItemChartShape) {
                totalShapeCount++ ;
                currRowCount++ ;
                LegendItemChartShape legendItem = (LegendItemChartShape) legendObj ;
                legendShapes.addLegendShape(legendItem) ;
                Rectangle bound = legendItem.estimateDrawBound(g2) ;
                currRowMaxHeight = Math.max(currRowMaxHeight, bound.getHeight()) ;
                double shapeWidth = Math.min(bound.getWidth(), maxShapeWidth) ;
                //ÿʾһͼ,
                if (currRowWidth + shapeWidth + columnGap > chartArea.getWidth()
                    && currRowCount > 1) {
                  totalHeight += currRowMaxHeight + columnGap ;
                  maxWidth = Math.max(maxWidth, currRowWidth) ;
                  currRowMaxHeight = bound.getHeight() ;
                  currRowCount = 1 ;

                  Rectangle2D shapeDrawArea =
                    new Rectangle2D.Double(0, totalHeight, Math.min(bound.getWidth(), maxShapeWidth),
                                           bound.getHeight()) ;

                  legendItem.setDrawArea(shapeDrawArea) ;
                  currRowWidth = shapeWidth + columnGap ;
                } else {
                  Rectangle2D shapeDrawArea = new Rectangle2D.Double(currRowWidth, totalHeight,
                                                                     Math.min(bound.getWidth(),
                                                                              maxShapeWidth),
                                                                     bound.getHeight()) ;
                  legendItem.setDrawArea(shapeDrawArea) ;
                  currRowWidth += shapeWidth + columnGap ;
                }
              }
            }
          }
        }
      }

      if (totalShapeCount == 0) {
        shapeCollection.setLegendShapes(null) ;
        return null ;
      }

      totalHeight += currRowMaxHeight ;
      maxWidth = Math.max(maxWidth, currRowWidth) ;
      legendShapes.translate(padding.getLeft(), padding.getTop()) ; //padding,ƫͼ

      //borderͼ
      double borderRectWidth = maxWidth + padding.getLeft() + padding.getRight() - 2 ;
      double borderRectHeight = Math.min(totalHeight + padding.getTop() + padding.getBottom(),
                                         chartArea.getHeight() - 2) ;
      Rectangle2D borderRect = new Rectangle2D.Double(0, 0, borderRectWidth, borderRectHeight) ;
      RectangleTextChartShape borderShape = createBorderShape(borderRect) ;
      legendShapes.setBorderShape(borderShape) ;

      result = new Rectangle(borderRect.getWidth(), borderRect.getHeight()) ;

      //λελ
      double offsetX = 0, offsetY = 0 ; //ƫλ
      if (position.equals(RectangleEdge.TOP)) {
        offsetX = chartArea.getX() + (chartArea.getWidth() - maxWidth) * 0.5 ;
        offsetY = chartArea.getY() + 2 ;
        legendShapes.translate(offsetX, offsetY) ;
      } else if (position.equals(RectangleEdge.BOTTOM)) {
        offsetX = chartArea.getX() + (chartArea.getWidth() - maxWidth) * 0.5 ;
        offsetY = chartArea.getY() + (chartArea.getHeight() - borderRect.getHeight()) - 2 ;
        legendShapes.translate(offsetX, offsetY) ;
      }
      legendShapes.setDrawArea(new Rectangle2D.Double(offsetX, offsetY, borderRectWidth,
                                                      borderRectHeight)) ;
    }

    return result ;
  }

  /**
   * <zh>߿ͼ.</zh>
   * <en>Create the border shape.</en>
   *
   * @param borderRect Rectangle2D
   * @return RectangleTextChartShape
   */
  private RectangleTextChartShape createBorderShape(Rectangle2D borderRect) {
    RectangleTextChartShape borderShape = new RectangleTextChartShape(borderRect, null) ;
    borderShape.setTextVerticalAlign(getVerticalAlignment()) ;
    borderShape.setMargin(getPadding()) ;
    borderShape.setDrawBorder(isBorderVisible()) ;
    borderShape.setBorderStroke(getBorderStroke()) ;
    borderShape.setBorderPaint(getBorderPaint()) ;
    borderShape.setFillShape(isFillBackground()) ;
    borderShape.setFillPaint(getBackgroundPaint()) ;
    borderShape.setFillAlpha(getBackgroundAlpha()) ;

    return borderShape ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    return HashcodeUtil.hashCode(this.renders) ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates whether some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument; false otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }
    if (!(obj instanceof LegendTitle)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    LegendTitle that = (LegendTitle) obj ;
    if (!EqualsUtil.equals(this.renders, that.renders)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

}
