package com.ijchart.xychart.ui ;

import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ʾҷ.</zh>
 * <en>Used to indicate the edge of a rectangle.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public final class RectangleEdge implements Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 6096123118256421534L ;

  /**
   * <zh>ϱ߽.</zh>
   * <en>Top.</en>
   */
  public static final RectangleEdge TOP = new RectangleEdge("RectangleEdge.TOP") ;

  /**
   * <zh>±߽.</zh>
   * <en>Bottom.</en>
   */
  public static final RectangleEdge BOTTOM = new RectangleEdge("RectangleEdge.BOTTOM") ;

  /**
   * <zh>߽.</zh>
   * <en>Left.</en>
   */
  public static final RectangleEdge LEFT = new RectangleEdge("RectangleEdge.LEFT") ;

  /**
   * <zh>ұ߽.</zh>
   * <en>Right.</en>
   */
  public static final RectangleEdge RIGHT = new RectangleEdge("RectangleEdge.RIGHT") ;

  /**
   * <zh></zh>
   * <en>The name.</en>
   */
  private String name ;

  /**
   * <zh>˽йֹʵ.</zh>
   * <en>Private constructor to prevent instance.</en>
   *
   * @param name <zh>.</zh>
   * <en>the name.</en>
   */
  private RectangleEdge(final String name) {
    this.name = name ;
  }

  /**
   * <zh>hashcode.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return <zh>hashcodeֵ.</zh>
   * <en>the hashcode</en>
   */
  public int hashCode() {
    return this.name.hashCode() ;
  }

  /**
   * <zh>Ƿϱ߽±߽.</zh>
   * <en>Returns true if the edge is TOP or BOTTOM, and false otherwise.</en>
   *
   * @param edge <zh>߽</zh>
   * <en>the edge.</en>
   * @return A boolean.
   */
  public static boolean isTopOrBottom(final RectangleEdge edge) {
    if (edge != null) {
      return edge.equals(RectangleEdge.TOP) || edge.equals(RectangleEdge.BOTTOM) ;
    } else {
      return false ;
    }
  }

  /**
   * <zh>Ƿ߽ұ߽.</zh>
   * <en>Returns true if the edge is LEFT or RIGHT, and false otherwise.</en>
   *
   * @param edge <zh>߽</zh>
   * <en>the edge.</en>
   * @return A boolean.
   */
  public static boolean isLeftOrRight(final RectangleEdge edge) {
    if (edge != null) {
      return edge.equals(RectangleEdge.LEFT) || edge.equals(RectangleEdge.RIGHT) ;
    } else {
      return false ;
    }
  }

  /**
   * <zh>෴ıֵ߽.</zh>
   * <en>Returns the opposite edge.</en>
   *
   * @param edge <zh>߽</zh>
   * <en>the edge.</en>
   * @return <zh>෴ıֵ߽.</zh>
   * <en>The opposite edge.</en>
   */
  public static RectangleEdge opposite(final RectangleEdge edge) {
    if (edge == null) {
      return null ;
    }

    RectangleEdge result = null ;
    if (edge.equals(RectangleEdge.TOP)) {
      result = RectangleEdge.BOTTOM ;
    } else if (edge.equals(RectangleEdge.BOTTOM)) {
      result = RectangleEdge.TOP ;
    } else if (edge.equals(RectangleEdge.LEFT)) {
      result = RectangleEdge.RIGHT ;
    } else if (edge.equals(RectangleEdge.RIGHT)) {
      result = RectangleEdge.LEFT ;
    }

    return result ;
  }

  /**
   * <zh>ݱֵ߽XYֵ.</zh>
   * <en>Returns the x or y coordinate of the specified edge.</en>
   *
   * @param rectangle <zh> (Ϊnull).</zh>
   * <en>The rectangle (null not permitted).</en>
   * @param edge <zh>ֵ߽ (Ϊnull).</zh>
   * <en>The edge (null not permitted).</en>
   *
   * @return <zh>XY.</zh>
   * <en>The coordinate.</en>
   */
  public static double coordinate(final Rectangle2D rectangle, final RectangleEdge edge) {
    if (rectangle == null) {
      throw new IllegalArgumentException("Null 'rectangle' argument.") ;
    }
    if (edge == null) {
      throw new IllegalArgumentException("Null 'edge' argument.") ;
    }

    double result = 0.0 ;
    if (edge.equals(RectangleEdge.TOP)) {
      result = rectangle.getMinY() ;
    } else if (edge.equals(RectangleEdge.BOTTOM)) {
      result = rectangle.getMaxY() ;
    } else if (edge.equals(RectangleEdge.LEFT)) {
      result = rectangle.getMinX() ;
    } else if (edge.equals(RectangleEdge.RIGHT)) {
      result = rectangle.getMaxX() ;
    }

    return result ;
  }

  /**
   * <zh>edgeȥεĳһlen.</zh>
   * <en> Shrinks the given rectangle by len.</en>
   * @param rect Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The rectangle (null not permitted).</en>
   * @param len double <zh>,ڵ0.</zh>
   * <en>The length,must be greater than or equal to 0</en>
   */
  public void trim(Rectangle2D rect, double len) {
    if (rect == null) {
      throw new IllegalArgumentException("Null 'rect' argument") ;
    }
    if (len < 0) {
      throw new IllegalArgumentException("Must be 'len'>=0.") ;
    }

    double x = rect.getX() ;
    double y = rect.getY() ;
    double width = rect.getWidth() ;
    double height = rect.getHeight() ;

    if (this.equals(RectangleEdge.TOP)) {
      y += len ;
      height -= len ;
    } else if (this.equals(RectangleEdge.BOTTOM)) {
      height -= len ;
    } else if (this.equals(RectangleEdge.LEFT)) {
      x += len ;
      width -= len ;
    } else if (this.equals(RectangleEdge.RIGHT)) {
      width -= len ;
    }

    rect.setFrame(x, y, width, height) ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representing the object.</en>
   *
   * @return The string.
   */
  public String toString() {
    return this.name ;
  }

  /**
   * <zh>equals.</zh>
   * <en>Returns true if this object is equal to the specified object, and false otherwise.</en>
   *
   * @param o  the other object.
   * @return A boolean.
   */
  public boolean equals(final Object o) {
    if (this == o) {
      return true ;
    }

    if (!(o instanceof RectangleEdge)) {
      return false ;
    }

    RectangleEdge that = (RectangleEdge) o ;
    if (!this.name.equals(that.name)) {
      return false ;
    }

    return true ;

  }

}
