package com.ijchart.xychart.ui ;

import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;

import com.ijchart.xychart.constant.LengthAdjustmentType ;
import com.ijchart.xychart.constant.UnitType ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ʾǶ,ָǾֵֵ(ٷֱ).ǲⲿı.</zh>
 * <en>Represents the insets for a rectangle, specified in absolute or relative terms.
 * This class is immutable.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class RectangleInsets implements Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -950552416576835243L ;

  /**
   * <zh>ߡײұ߶ΪRectangleInsets.</zh>
   * <en>A useful constant representing zero insets.</en>
   */
  public static final RectangleInsets INSETS_ZERO = new RectangleInsets(
    UnitType.ABSOLUTE, 0.0, 0.0, 0.0, 0.0) ;

  /**
   * <zh>ߡײұ߶Ϊ1RectangleInsets.</zh>
   * <en>A useful constant representing one insets.</en>
   */
  public static final RectangleInsets INSETS_ONE = new RectangleInsets(
    UnitType.ABSOLUTE, 1.0, 1.0, 1.0, 1.0) ;

  /**
   * <zh>ߡײұ߶Ϊ2RectangleInsets.</zh>
   * <en>A useful constant representing two insets.</en>
   */
  public static final RectangleInsets INSETS_TWO = new RectangleInsets(
    UnitType.ABSOLUTE, 2.0, 2.0, 2.0, 2.0) ;

  /**
   * <zh>ߡײұ߶Ϊ3RectangleInsets.</zh>
   * <en>A useful constant representing three insets.</en>
   */
  public static final RectangleInsets INSETS_THREE = new RectangleInsets(
    UnitType.ABSOLUTE, 3.0, 3.0, 3.0, 3.0) ;

  /**
   * <zh>.</zh>
   * <en>Absolute or relative units.</en>
   */
  private UnitType unitType ;

  /**
   * <zh>.</zh>
   * <en>The top insets.</en>
   */
  private double top ;

  /**
   * <zh>߳.</zh>
   * <en>The left insets.</en>
   */
  private double left ;

  /**
   * <zh>ײ.</zh>
   * <en>The bottom insets. </en>
   */
  private double bottom ;

  /**
   * <zh>ұ߳.</zh>
   * <en>The right insets. </en>
   */
  private double right ;

  /**
   * <zh>,Ϊֵ.</zh>
   * <en>Creates a new instance with the specified insets (as 'absolute' units).</en>
   *
   * @param top <zh>.</zh>
   * <en>the top insets.</en>
   * @param left <zh>߳.</zh>
   * <en>the left insets.</en>
   * @param bottom <zh>ײ.</zh>  <en>the bottom insets.</en>
   * @param right <zh>ұ߳.</zh>   <en>the right insets.</en>
   */
  public RectangleInsets(final double top, final double left,
                         final double bottom, final double right) {
    this(UnitType.ABSOLUTE, top, left, bottom, right) ;
  }

  /**
   * <zh>ָUnitType.</zh>
   * <en>Creates a new instance.</en>
   * @param unitType <zh>ֵ.</zh>
   * <en>absolute or relative units (null not permitted).</en>
   *
   * @param top <zh>.</zh>
   * <en>the top insets.</en>
   * @param left <zh>߳.</zh>
   * <en>the left insets.</en>
   * @param bottom <zh>ײ.</zh>
   * <en>the bottom insets.</en>
   * @param right <zh>ұ߳.</zh>
   * <en>the right insets.</en>
   */
  public RectangleInsets(final UnitType unitType,
                         final double top, final double left,
                         final double bottom, final double right) {
    if (unitType == null) {
      throw new IllegalArgumentException("Null 'unitType' argument.") ;
    }
    this.unitType = unitType ;
    this.top = top ;
    this.bottom = bottom ;
    this.left = left ;
    this.right = right ;
  }

  /**
   * <zh>ȵλ,ֵֵ(ٷֱ).</zh>
   * <en>Returns the unit type (absolute or relative).This specifies whether the insets are measured
   * as Java2D units or percentages.</en>
   *
   * @return <zh>ȵλ,Ϊnull.</zh>
   * <en>The unit type (never null).</en>
   */
  public UnitType getUnitType() {
    return this.unitType ;
  }

  /**
   * <zh>ض.</zh>
   * <en>Returns the top insets.</en>
   *
   * @return <zh>.</zh>
   * <en>The top insets.</en>
   */
  public double getTop() {
    return this.top ;
  }

  /**
   * <zh>صײ.</zh>
   * <en>Returns the bottom insets.</en>
   *
   * @return <zh>ײ.</zh>
   * <en>The bottom insets.</en>
   */
  public double getBottom() {
    return this.bottom ;
  }

  /**
   * <zh>߳.</zh>
   * <en>Returns the left insets.</en>
   *
   * @return <zh>߳.</zh>
   * <en>The left insets.</en>
   */
  public double getLeft() {
    return this.left ;
  }

  /**
   * <zh>ұ߳.</zh>
   * <en>Returns the right insets.</en>
   *
   * @return <zh>ұ߳.</zh>
   * <en>The right insets.</en>
   */
  public double getRight() {
    return this.right ;
  }

  /**
   * <zh>ݱˮƽʹֱĳȵͼµľ.</zh>
   * <en>Creates an adjusted rectangle using the supplied rectangle, the insets specified by this
   * instance, and the horizontal and vertical adjustment types.</en>
   *
   * @param base <zh>Ҫľ.</zh>
   * <en>The base rectangle (null not permitted).</en>
   * @param horizontal <zh>ˮƽĳȵ (Ϊnull).</zh>
   * <en>The horizontal adjustment type (null not permitted).</en>
   * @param vertical <zh>ֱĳȵ.</zh>
   * <en>The vertical adjustment type (null not permitted).</en>
   * @return <zh>ľ.</zh>
   * <en>The inset rectangle.</en>
   */
  public Rectangle2D createAdjustedRectangle(final Rectangle2D base,
                                             final LengthAdjustmentType horizontal,
                                             final LengthAdjustmentType vertical) {
    if (base == null) {
      throw new IllegalArgumentException("Null 'base' argument.") ;
    }
    if (horizontal == null) {
      throw new IllegalArgumentException("Null 'horizontal' argument.") ;
    }
    if (vertical == null) {
      throw new IllegalArgumentException("Null 'vertical' argument.") ;
    }

    double x = base.getX() ;
    double y = base.getY() ;
    double w = base.getWidth() ;
    double h = base.getHeight() ;

    if (horizontal.equals(LengthAdjustmentType.EXPAND)) {
      final double leftOutset = calculateLeftOutset(w) ;
      x -= leftOutset ;
      w = w + leftOutset + calculateRightOutset(w) ;
    } else if (horizontal.equals(LengthAdjustmentType.CONTRACT)) {
      final double leftMargin = calculateLeftInset(w) ;
      x += leftMargin ;
      w = w - leftMargin - calculateRightInset(w) ;
    }

    if (vertical.equals(LengthAdjustmentType.EXPAND)) {
      final double topMargin = calculateTopOutset(h) ;
      y -= topMargin ;
      h = h + topMargin + calculateBottomOutset(h) ;
    } else if (vertical.equals(LengthAdjustmentType.CONTRACT)) {
      final double topMargin = calculateTopInset(h) ;
      y += topMargin ;
      h = h - topMargin - calculateBottomInset(h) ;
    }

    return new Rectangle2D.Double(x, y, w, h) ;
  }

  /**
   * <zh>һµľζ.</zh>
   * <en>Creates an 'inset' rectangle.</en>
   *
   * @param base <zh>µľζĻ (Ϊnull).</zh>
   * <en>The base rectangle (null not permitted).</en>
   *
   * @return <zh>µľζ.</zh>
   * <en>The inset rectangle.</en>
   */
  public Rectangle2D createInsetRectangle(final Rectangle2D base) {
    return createInsetRectangle(base, true, true) ;
  }

  /**
   * <zh>һµľζ.</zh>
   * <en>Creates an 'inset' rectangle.</en>
   *
   * @param base <zh>µľζĻ (Ϊnull).</zh>
   * <en>The base rectangle (null not permitted).</en>
   * @param horizontal <zh>ǷӦˮƽinsert.</zh>
   * <en>Whether or not apply horizontal insets.</en>
   * @param vertical <zh>ǷӦôֱinsert.</zh>
   * <en>Whether or not apply vertical insets.</en>
   * @return <zh>µľζ.</zh>
   * <en>The inset rectangle.</en>
   */
  public Rectangle2D createInsetRectangle(final Rectangle2D base,
                                          final boolean horizontal,
                                          final boolean vertical) {
    if (base == null) {
      throw new IllegalArgumentException("Null 'base' argument.") ;
    }

    double topMargin = 0.0 ;
    double bottomMargin = 0.0 ;
    if (vertical) {
      topMargin = calculateTopInset(base.getHeight()) ;
      bottomMargin = calculateBottomInset(base.getHeight()) ;
    }

    double leftMargin = 0.0 ;
    double rightMargin = 0.0 ;
    if (horizontal) {
      leftMargin = calculateLeftInset(base.getWidth()) ;
      rightMargin = calculateRightInset(base.getWidth()) ;
    }

    return new Rectangle2D.Double(
      base.getX() + leftMargin,
      base.getY() + topMargin,
      base.getWidth() - leftMargin - rightMargin,
      base.getHeight() - topMargin - bottomMargin
      ) ;
  }

  /**
   * <zh>һµľζ.</zh>
   * <en>Creates an 'outset' rectangle.</en>
   *
   * @param base <zh>µľζĻ (Ϊnull).</zh>
   * <en>The base rectangle (null not permitted).</en>
   *
   * @return <zh>µľζ.</zh>
   * <en>The inset rectangle.</en>
   */
  public Rectangle2D createOutsetRectangle(final Rectangle2D base) {
    return createOutsetRectangle(base, true, true) ;
  }

  /**
   * <zh>һµľζ.</zh>
   * <en>Creates an 'outset' rectangle.</en>
   *
   * @param base <zh>µľζĻ (Ϊnull).</zh>
   * <en>The base rectangle (null not permitted).</en>
   * @param horizontal <zh>ǷӦˮƽinsert.</zh>
   * <en>Whether or not apply horizontal insets.</en>
   * @param vertical <zh>ǷӦôֱinsert.</zh>
   * <en>Whether or not apply vertical insets.</en>
   * @return <zh>µľζ.</zh>
   * <en>The inset rectangle.</en>
   */
  public Rectangle2D createOutsetRectangle(final Rectangle2D base,
                                           final boolean horizontal,
                                           final boolean vertical) {
    if (base == null) {
      throw new IllegalArgumentException("Null 'base' argument.") ;
    }
    double topMargin = 0.0 ;
    double bottomMargin = 0.0 ;
    if (vertical) {
      topMargin = calculateTopOutset(base.getHeight()) ;
      bottomMargin = calculateBottomOutset(base.getHeight()) ;
    }
    double leftMargin = 0.0 ;
    double rightMargin = 0.0 ;
    if (horizontal) {
      leftMargin = calculateLeftOutset(base.getWidth()) ;
      rightMargin = calculateRightOutset(base.getWidth()) ;
    }

    return new Rectangle2D.Double(base.getX() - leftMargin, base.getY() - topMargin,
                                  base.getWidth() + leftMargin + rightMargin,
                                  base.getHeight() + topMargin + bottomMargin) ;
  }

  /**
   * <zh>ض˿հ.</zh>
   * <en>Returns the top margin.</en>
   *
   * @param height <zh>εĵ.</zh>
   * <en>The height of the base rectangle.</en>
   *
   * @return <zh>հ.</zh>
   * <en>The top margin (in Java2D units).</en>
   */
  public double calculateTopInset(final double height) {
    double result = this.top ;
    if (this.unitType.equals(UnitType.RELATIVE)) {
      result = (this.top * height) ;
    }

    return result ;
  }

  /**
   * Returns the top margin.
   *
   * @param height  the height of the base rectangle.
   *
   * @return The top margin (in Java2D units).
   */
  public double calculateTopOutset(final double height) {
    double result = this.top ;
    if (this.unitType.equals(UnitType.RELATIVE)) {
      result = (height / (1 - this.top - this.bottom)) * this.top ;
    }
    return result ;
  }

  /**
   * Returns the bottom margin.
   *
   * @param height  the height of the base rectangle.
   *
   * @return The bottom margin (in Java2D units).
   */
  public double calculateBottomInset(final double height) {
    double result = this.bottom ;
    if (this.unitType.equals(UnitType.RELATIVE)) {
      result = (this.bottom * height) ;
    }
    return result ;
  }

  /**
   * Returns the bottom margin.
   *
   * @param height  the height of the base rectangle.
   *
   * @return The bottom margin (in Java2D units).
   */
  public double calculateBottomOutset(final double height) {
    double result = this.bottom ;
    if (this.unitType.equals(UnitType.RELATIVE)) {
      result = (height / (1 - this.top - this.bottom)) * this.bottom ;
    }

    return result ;
  }

  /**
   * Returns the left margin.
   *
   * @param width  the width of the base rectangle.
   *
   * @return The left margin (in Java2D units).
   */
  public double calculateLeftInset(final double width) {
    double result = this.left ;
    if (this.unitType.equals(UnitType.RELATIVE)) {
      result = (this.left * width) ;
    }

    return result ;
  }

  /**
   * Returns the left margin.
   *
   * @param width  the width of the base rectangle.
   *
   * @return The left margin (in Java2D units).
   */
  public double calculateLeftOutset(final double width) {
    double result = this.left ;
    if (this.unitType.equals(UnitType.RELATIVE)) {
      result = (width / (1 - this.left - this.right)) * this.left ;
    }

    return result ;
  }

  /**
   * Returns the right margin.
   *
   * @param width  the width of the base rectangle.
   *
   * @return The right margin (in Java2D units).
   */
  public double calculateRightInset(final double width) {
    double result = this.right ;
    if (this.unitType.equals(UnitType.RELATIVE)) {
      result = (this.right * width) ;
    }

    return result ;
  }

  /**
   * Returns the right margin.
   *
   * @param width  the width of the base rectangle.
   *
   * @return The right margin (in Java2D units).
   */
  public double calculateRightOutset(final double width) {
    double result = this.right ;
    if (this.unitType.equals(UnitType.RELATIVE)) {
      result = (width / (1 - this.left - this.right)) * this.right ;
    }

    return result ;
  }

  /**
   * Trims the given width to allow for the insets.
   *
   * @param width  the width.
   *
   * @return The trimmed width.
   */
  public double trimWidth(final double width) {
    return width - calculateLeftInset(width) - calculateRightInset(width) ;
  }

  /**
   * Extends the given width to allow for the insets.
   *
   * @param width  the width.
   *
   * @return The extended width.
   */
  public double extendWidth(final double width) {
    return width + calculateLeftOutset(width) + calculateRightOutset(width) ;
  }

  /**
   * Trims the given height to allow for the insets.
   *
   * @param height  the height.
   *
   * @return The trimmed height.
   */
  public double trimHeight(final double height) {
    return height - calculateTopInset(height) - calculateBottomInset(height) ;
  }

  /**
   * Extends the given height to allow for the insets.
   *
   * @param height  the height.
   *
   * @return The extended height.
   */
  public double extendHeight(final double height) {
    return height + calculateTopOutset(height) + calculateBottomOutset(height) ;
  }

  /**
   * <zh>չεĸ߶.</zh>
   * <en>Expansion rectangle height.</en>
   *
   * @param rect Rectangle2D <zh>Ҫչľ (Ϊnull).</zh>
   * <en>The rectangle (null not permitted).</en>
   */
  public void expandRectHeight(Rectangle2D rect) {
    if (rect == null) {
      throw new IllegalArgumentException("Null 'rect' argument.") ;
    }

    rect.setRect(rect.getX(), rect.getY(), rect.getWidth(), rect.getHeight() + top + bottom) ;
  }

  /**
   * <zh>չεĿ.</zh>
   * <en>Expansion rectangle width.</en>
   *
   * @param rect Rectangle2D <zh>Ҫչľ (Ϊnull).</zh>
   * <en>The rectangle (null not permitted).</en>
   */
  public void expandRectWidth(Rectangle2D rect) {
    if (rect == null) {
      throw new IllegalArgumentException("Null 'rect' argument.") ;
    }

    rect.setRect(rect.getX(), rect.getY(), rect.getWidth() + left + right, rect.getHeight()) ;
  }

  /**
   * <zh>չεĿȺ͸߶.</zh>
   * <en>Rectangular extension of the width and height.</en>
   *
   * @param rect Rectangle2D <zh>Ҫչľ (Ϊnull).</zh>
   * <en>The rectangle (null not permitted).</en>
   */
  public void expandRect(Rectangle2D rect) {
    if (rect == null) {
      throw new IllegalArgumentException("Null 'rect' argument.") ;
    }

    rect.setRect(rect.getX(), rect.getY(),
                 rect.getWidth() + left + right, rect.getHeight() + top + bottom) ;
  }

  /**
   * <zh>areaĿհȥ.</zh>
   * <en>Shrinks the given rectangle by the amount of these insets.</en>
   *
   * @param area Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The area (null not permitted).</en>
   */
  public void trim(final Rectangle2D area) {
    if (area == null) {
      throw new IllegalArgumentException("Null 'area' argument.") ;
    }

    final double w = area.getWidth() ;
    final double h = area.getHeight() ;
    final double l = calculateLeftInset(w) ;
    final double r = calculateRightInset(w) ;
    final double t = calculateTopInset(h) ;
    final double b = calculateBottomInset(h) ;

    area.setRect(area.getX() + l, area.getY() + t, w - l - r, h - t - b) ;
  }

  /**
   * <zh>areaȥհ׺ͼ.</zh>
   * <en>Shrinks the given rectangle by the amount of these insets.</en>
   *
   * @param area Rectangle2D <zh>Ҫľ (Ϊnull).</zh>
   * <en>The area (null not permitted).</en>
   *
   * @return Rectangle2D
   */
  public Rectangle2D getTrimArea(Rectangle2D area) {
    if (area == null) {
      throw new IllegalArgumentException("Null 'area' argument.") ;
    }

    final double w = area.getWidth() ;
    final double h = area.getHeight() ;
    final double l = calculateLeftInset(w) ;
    final double r = calculateRightInset(w) ;
    final double t = calculateTopInset(h) ;
    final double b = calculateBottomInset(h) ;

    return new Rectangle2D.Double(area.getX() + l, area.getY() + t, w - l - r, h - t - b) ;
  }

  /**
   * <zh>ĳ߽ĳСlen,len.</zh>
   * <en>Expansion edge if the edge of this object is least than len.</en>
   *
   * @param len double <zh>,ڵ0.</zh>
   * <en>The length (must greater or equal than len).</en>
   * @return RectangleInsets <zh>µRectangleInsets.</zh>
   * <en>The new RectangleInsets object.</en>
   */
  public RectangleInsets ensureAtLeast(double len) {
    if (len < 0) {
      throw new IllegalArgumentException("Must be len>=0.") ;
    }

    return new RectangleInsets(Math.max(this.top, len), Math.max(this.left, len),
                               Math.max(this.bottom, len), Math.max(this.right, len)) ;
  }

  /**
   * <zh>equals.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj <zh>ҪȽϵĶ.</zh>
   * <en>the object (null permitted).</en>
   *
   * @return <zh>򷵻true,򷵻false.</zh>
   * <en>A boolean.</en>
   */
  public boolean equals(final Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof RectangleInsets)) {
      return false ;
    }

    RectangleInsets that = (RectangleInsets) obj ;

    return this.unitType.equals(that.unitType)
      && this.left == that.left
      && this.right == that.right
      && this.top == that.top
      && this.bottom == that.bottom ;
  }

  /**
   * <zh>hasCode,hascodeֵ.</zh>
   * <en>Returns a hash code for the object.</en>
   *
   * @return A hash code.
   */
  public int hashCode() {
    int result = (this.unitType != null) ? this.unitType.hashCode() : 0 ;
    long temp = (this.top != +0.0d) ? Double.doubleToLongBits(this.top) : 0L ;
    result = 29 * result + (int) (temp ^ (temp >>> 32)) ;
    temp = (this.bottom != +0.0d) ? Double.doubleToLongBits(this.bottom) : 0L ;
    result = 29 * result + (int) (temp ^ (temp >>> 32)) ;
    temp = (this.left != +0.0d) ? Double.doubleToLongBits(this.left) : 0L ;
    result = 29 * result + (int) (temp ^ (temp >>> 32)) ;
    temp = (this.right != +0.0d) ? Double.doubleToLongBits(this.right) : 0L ;
    result = 29 * result + (int) (temp ^ (temp >>> 32)) ;

    return result ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a textual representation of this instance, useful for debugging purposes.</en>
   *
   * @return A string representing this instance.
   */
  public String toString() {
    return "[unitType=" + this.unitType
      + ",top=" + this.top
      + ",left=" + this.left
      + ",bottom=" + this.bottom
      + ",right=" + this.right
      + "]" ;
  }

}
