package com.ijchart.xychart.ui.supplier ;

import java.awt.Paint ;
import java.awt.Shape ;
import java.awt.Stroke ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ṩһϵеPaint,Stroke,Shape,ظĶ.</zh>
 * <en>A supplier of Paint,Stroke and Shape objects for use by plots and renders.By providing a
 * central place for obtaining these items, we can ensure that duplication is avoided.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public interface IDrawingSupplier {

  /**
   * <zh>һPaint.</zh>
   * <en>Returns the next paint in a sequence maintained by the supplier.</en>
   *
   * @return Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  Paint getNextPaint() ;

  /**
   * <zh>һ߿Paint.</zh>
   * <en>Returns the next border paint in a sequence maintained by the supplier.</en>
   *
   * @return Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  Paint getNextBorderPaint() ;

  /**
   * <zh>һͼεPaint.</zh>
   * <en>Returns the next fill paint in a sequence maintained by the supplier.</en>
   *
   * @return Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  Paint getNextFillPaint() ;

  /**
   * <zh>һStroke.</zh>
   * <en>Returns the next Stroke object in a sequence maintained by the supplier.</en>
   *
   * @return Stroke <zh>Stroke (Ϊnull).</zh>
   * <en>The Stroke (never null).</en>
   */
  Stroke getNextStroke() ;

  /**
   * <zh>һƱ߿Stroke.</zh>
   * <en>Returns the next Stroke object in a sequence maintained by the supplier.</en>
   *
   * @return Stroke <zh>Stroke (Ϊnull).</zh>
   * <en>The Stroke (never null).</en>
   */
  Stroke getNextBorderStroke() ;

  /**
   * <zh>һShape.</zh>
   * <en>Returns the next Shape object in a sequence maintained by the supplier.</en>
   *
   * @return Shape <zh>Shape (Ϊnull).</zh>
   * <en>The shape (never null).</en>
   */
  Shape getNextShape() ;

  /**
   * <zh>ñൽʼ״̬.</zh>
   * <en>Reset this class to first initiate state.</en>
   */
  void reset() ;

  /**
   * <zh>Ƿñൽʼ״̬.</zh>
   * <en>Whether or not can reset.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  boolean isCanReset() ;

  /**
   * <zh>Ƿñൽʼ״̬.</zh>
   * <en>Sets whether or not can reset.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  void setCanReset(boolean flag) ;

  /**
   * <zh>浱ǰ״̬.</zh>
   * <en>Save current state.</en>
   */
  void saveState() ;

  /**
   * <zh>ָǰ״̬Ϊɵ״̬.</zh>
   * <en>Restore state to old.</en>
   */
  void restoreState() ;

}
